Package: iCOBRA
Type: Package
Title: Comparison and Visualization of Ranking and Assignment Methods
Version: 1.38.0
Date: 2025-07-17
Authors@R: c(person(given = "Charlotte", family = "Soneson", 
    role = c("aut", "cre"),
    email = "charlottesoneson@gmail.com",
    comment = c(ORCID = "0000-0003-3833-2169")))
Description: This package provides functions for calculation and visualization
    of performance metrics for evaluation of ranking and binary classification
    (assignment) methods. Various types of performance plots can be generated
    programmatically. The package also contains a shiny application for 
    interactive exploration of results.
License: GPL (>=2)
LazyData: TRUE
Collate: 'AllGenerics.R' 'COBRAData.R' 'COBRAPerformance.R'
        'COBRAPlot.R' 'calculate_performance.R' 'cobradata_example.R'
        'cobradata_example_sval.R' 'helpers_general.R' 'plot_methods.R'
        'printHead.R' 'shiny.R'
Depends: R (>= 4.4.0)
Imports: shiny (>= 0.9.1.9008), shinydashboard, shinyBS, reshape2,
        ggplot2 (>= 3.4.0), scales, ROCR, dplyr, DT, limma, methods,
        UpSetR, utils, rlang
Suggests: knitr, markdown, rmarkdown, testthat
VignetteBuilder: knitr
biocViews: Classification, Visualization
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://github.com/csoneson/iCOBRA
BugReports: https://github.com/csoneson/iCOBRA/issues
git_url: https://git.bioconductor.org/packages/iCOBRA
git_branch: RELEASE_3_22
git_last_commit: 60e06fc
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 04:31:44 UTC; biocbuild
Author: Charlotte Soneson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3833-2169>)
Maintainer: Charlotte Soneson <charlottesoneson@gmail.com>

# Generated by roxygen2: do not edit by hand

export("corr<-")
export("deviation<-")
export("facetted<-")
export("fdrnbr<-")
export("fdrnbrcurve<-")
export("fdrtpr<-")
export("fdrtprcurve<-")
export("fpc<-")
export("fpr<-")
export("fsrnbr<-")
export("fsrnbrcurve<-")
export("maxsplit<-")
export("onlyshared<-")
export("overlap<-")
export("padj<-")
export("plotcolors<-")
export("pval<-")
export("roc<-")
export("scatter<-")
export("score<-")
export("splv<-")
export("sval<-")
export("tpr<-")
export("truth<-")
export(.COBRAData)
export(.COBRAPerformance)
export(.COBRAPlot)
export(COBRAData)
export(COBRAData_from_text)
export(COBRAData_to_text)
export(COBRAPerformance)
export(COBRAPlot)
export(COBRAapp)
export(basemethods)
export(calculate_adjp)
export(calculate_performance)
export(coerce)
export(corr)
export(deviation)
export(facetted)
export(fdrnbr)
export(fdrnbrcurve)
export(fdrtpr)
export(fdrtprcurve)
export(fpc)
export(fpr)
export(fsrnbr)
export(fsrnbrcurve)
export(maxsplit)
export(onlyshared)
export(overlap)
export(padj)
export(plot_corr)
export(plot_deviation)
export(plot_fdrnbrcurve)
export(plot_fdrtprcurve)
export(plot_fpc)
export(plot_fpr)
export(plot_fsrnbrcurve)
export(plot_overlap)
export(plot_roc)
export(plot_scatter)
export(plot_tpr)
export(plot_upset)
export(plotcolors)
export(prepare_data_for_plot)
export(pval)
export(reorder_levels)
export(roc)
export(scatter)
export(score)
export(splv)
export(stratiflevels)
export(sval)
export(tpr)
export(truth)
export(update_cobradata)
export(update_cobraperformance)
exportClasses(COBRAData)
exportClasses(COBRAPerformance)
exportClasses(COBRAPlot)
exportMethods("[")
exportMethods("corr<-")
exportMethods("deviation<-")
exportMethods("facetted<-")
exportMethods("fdrnbr<-")
exportMethods("fdrnbrcurve<-")
exportMethods("fdrtpr<-")
exportMethods("fdrtprcurve<-")
exportMethods("fpc<-")
exportMethods("fsrnbr<-")
exportMethods("fsrnbrcurve<-")
exportMethods("maxsplit<-")
exportMethods("onlyshared<-")
exportMethods("overlap<-")
exportMethods("padj<-")
exportMethods("plotcolors<-")
exportMethods("pval<-")
exportMethods("roc<-")
exportMethods("scatter<-")
exportMethods("score<-")
exportMethods("splv<-")
exportMethods("sval<-")
exportMethods("tpr<-")
exportMethods("truth<-")
exportMethods(basemethods)
exportMethods(corr)
exportMethods(deviation)
exportMethods(facetted)
exportMethods(fdrnbr)
exportMethods(fdrnbrcurve)
exportMethods(fdrtpr)
exportMethods(fdrtprcurve)
exportMethods(fpc)
exportMethods(fpr)
exportMethods(fsrnbr)
exportMethods(fsrnbrcurve)
exportMethods(maxsplit)
exportMethods(onlyshared)
exportMethods(overlap)
exportMethods(padj)
exportMethods(plotcolors)
exportMethods(pval)
exportMethods(roc)
exportMethods(scatter)
exportMethods(score)
exportMethods(splv)
exportMethods(stratiflevels)
exportMethods(sval)
exportMethods(tpr)
exportMethods(truth)
import(UpSetR)
import(dplyr)
import(ggplot2)
import(limma)
import(methods)
import(shiny)
import(shinydashboard)
importFrom(DT,datatable)
importFrom(ROCR,performance)
importFrom(ROCR,prediction)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(scales,brewer_pal)
importFrom(scales,hue_pal)
importFrom(shinyBS,bsTooltip)
importFrom(utils,packageVersion)

# iCOBRA - Interactive benchmarking of ranking and assignment methods

`iCOBRA` is a package to calculate and visualize performance metrics for 
ranking and binary assignment methods. A typical use case could be, 
for example, comparing methods calling differential expression in 
gene expression experiments, which could be seen as either a ranking 
problem (estimating the correct effect size and ordering the genes by 
significance) or a binary assignment problem (classifying the genes 
into differentially expressed and non-differentially expressed).

`iCOBRA` can be used either directly from the console, or via the 
interactive shiny application (see the function `COBRAapp()`). It can also 
be accessed from the web server [http://imlspenticton.uzh.ch:3838/iCOBRA/](http://imlspenticton.uzh.ch:3838/iCOBRA/)

We have also collected a set of benchmarking data sets, addressing different aspects of genomic data analysis. The collection is reachable via the following link: [http://imlspenticton.uzh.ch/robinson_lab/benchmark_collection/](http://imlspenticton.uzh.ch/robinson_lab/benchmark_collection/)

## Installation

`iCOBRA` can be installed from `Bioconductor` using `BiocManager`:

```
install.packages("BiocManager")
BiocManager::install("iCOBRA")
```

or, optionally, 

```
BiocManager::install("markrobinsonuzh/iCOBRA")
```


## Quick start guide

The `iCOBRA` workflow starts from an object of class `COBRAData`, 
containing (adjusted) p-values and/or scores for a set of features as 
well as the true status of the features. An example data set is provided in 
the package

```
library(iCOBRA)
data(cobradata_example)
```

The function `calculate_performance()` calculates the different performance 
metrics for a `COBRAData` object. By default, all performance metrics are 
calculated, but a subset can be selected using the `aspects` argument.

```
cobraperf <- calculate_performance(cobradata_example, binary_truth = "status",
                                   cont_truth = "logFC", 
                                   aspects = c("fdrtpr", "fdrtprcurve", 
                                               "corr"))
```

Next, the performance metrics are prepared for plotting using the 
`prepare_for_plot()` function. This function defines colors for the 
various methods and can also be used for selecting only a subset of the 
methods for visualization, without having to recalculate the performance metrics.

```
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Set2",
                                   keepmethods = c("voom", "edgeR"))
```

The resulting object can then be used to generate plots of the selected aspects.

```
plot_fdrtprcurve(cobraplot)
plot_corr(cobraplot, corrtype = "spearman")
```

## Vignette

The vignette can be found in the `vignettes/` directory. Further 
information is also available in the 'Instructions' tab of the shiny app. 
After installation, the vignette can be accessed from the R console by typing

```
browseVignettes("iCOBRA")
```

## Benchmarking data set collection

To facilitate future benchmarking studies, we have collected a set of benchmarking 
data sets on [http://imlspenticton.uzh.ch/robinson_lab/benchmark_collection/](http://imlspenticton.uzh.ch/robinson_lab/benchmark_collection/). The page provides 
links to raw data as well as evaluation results suitable for import into `iCOBRA`. 

## This function is almost verbatim taken from limma. The only change is on
## lines 35 and 43, adding drop = FALSE, and removal of some unncessary lines.
.printHead <- function (x) {
  if (is.atomic(x)) {
    d <- dim(x)
    if (length(d) < 2)
      which <- "OneD"
    if (length(d) == 2)
      which <- "TwoD"
#     if (length(d) > 2)
#       which <- "Array"
  } else {
    if (inherits(x, "data.frame")) {
      d <- dim(x)
      which <- "TwoD"
    } else {
#       if (is.call(x))
#         which <- "Call"
#       else {
        if (is.recursive(x))
          which <- "Recursive"
        else which <- "Other"
      # }
    }
  }
  switch(which, OneD = {
    n <- length(x)
    if (n > 20) {
      print(x[seq_len(5)])
      cat(n - 5, "more elements ...\n")
    } else print(x)
  }, TwoD = {
    n <- d[1]
    if (n > 10) {
      print(x[seq_len(5), , drop = FALSE])
      cat(n - 5, "more rows ...\n")
    } else print(x)
#   }, Array = {
#     n <- d[1]
#     if (n > 10) {
#       dn <- dimnames(x)
#       dim(x) <- c(d[1], prod(d[-1]))
#       x <- x[1:5, , drop = FALSE]
#       dim(x) <- c(5, d[-1])
#       if (!is.null(dn[[1]])) dn[[1]] <- dn[[1]][1:5]
#       dimnames(x) <- dn
#       print(x)
#       cat(n - 5, "more rows ...\n")
#     } else print(x)
  }, Recursive = {
    n <- length(x)
    if (n) {
      i <- names(x)
      if (is.null(i)) i <- seq_len(n)
      for (what in i) {
        y <- x[[what]]
        cat("$", what, "\n", sep = "")
        Recall(y)
        cat("\n")
      }
    }
  },
# Call = ,
Other = print(x))
}

## Function to perform some checks on the input (result) files.
## Return TRUE if everything seems ok,
## otherwise print a message and return FALSE.
res_check <- function(dfr) {
  if (ncol(dfr) == 1) {
    message("the data frame contains only one column")
    return(FALSE)
  }
  return(TRUE)
}

get_keepfeatures <- function(truth, df, method, colm, onlyshared) {
  if (!isTRUE(onlyshared)) {
    allg <- rownames(truth[which(!is.na(truth[, colm])), , drop = FALSE])
  } else {
    allg <- intersect(rownames(truth[which(!is.na(truth[, colm])), ,
                                     drop = FALSE]),
                      rownames(df)[which(!is.na(df[method]))])
  }
  allg
}

is_plottable <- function(obj) {
  if (is.null(obj))
    FALSE
  else if (is(obj, "data.frame") && nrow(obj) == 0)
    FALSE
  else if (length(obj) == 0)
    FALSE
  else
    TRUE
}

#' @import dplyr
#' @importFrom rlang .data
get_keeplevels <- function(truth, splv, binary_truth, maxsplit) {
  if (splv != "none") {
    if (!is.finite(maxsplit)) {
      keeplevels <- levels(as.factor(truth[[splv]]))
    } else {
      if (!is.null(binary_truth)) {
        nbrtrulydiff <-
          as.data.frame(truth %>% group_by(.data[[splv]]) %>%
                          summarise(nbrdiff = length(which(.data[[binary_truth]] == 1))),
                          # summarise_(nbrdiff = paste0("length(which(",
                          #                             binary_truth, "== 1))")),
                        stringsAsFactors = FALSE)
        tokeep <- nbrtrulydiff[nbrtrulydiff$nbrdiff > 0, splv]
        tbl <- table(truth[[splv]])
        tbl <- tbl[as.character(tokeep)]
      } else {
        tbl <- table(truth[[splv]])
      }
      tbl <- sort(tbl, decreasing = TRUE)
      keeplevels <- names(tbl)[seq_len(min(maxsplit, length(tbl)))]
    }
  } else {
    keeplevels <- "overall"
  }
  keeplevels
}

plot_theme <- function(stripsize, titlecol) {
  theme_grey() +
    theme(legend.position = "right",
          panel.background = element_rect(fill = "white", colour = "black"),
          panel.grid.minor.x = element_blank(),
          panel.grid.minor.y = element_blank(),
          strip.text = element_text(size = stripsize),
          strip.background = element_rect(fill = NA, colour = "black"),
          axis.text.x = element_text(angle = 90, vjust = 0.5,
                                     hjust = 1, size = 15),
          axis.text.y = element_text(size = 15),
          axis.title.x = element_text(size = 20),
          axis.title.y = element_text(size = 20),
          plot.title = element_text(colour = titlecol))
}

get_coltype <- function(col_name) {
  if (substr(col_name, nchar(col_name) - 1, 
             nchar(col_name)) == ":P") {
    coltype <- "pval"
  } else if (substr(col_name, nchar(col_name) - 4,
                    nchar(col_name)) == ":adjP") {
    coltype <- "padj"
  } else if (substr(col_name, nchar(col_name) - 1,
                    nchar(col_name)) == ":S") {
    coltype <- "sval"
  } else if (substr(col_name, nchar(col_name) - 5,
                    nchar(col_name)) == ":score") {
    coltype <- "score"
  } else {
    coltype <- NULL
  }
  coltype
}

fix_duplicates <- function(res_df, feature_id, method_name) {
  dp <-
    unique(as.character(res_df[, feature_id][duplicated(res_df[, feature_id])]))
  if (length(dp) > 0) {
    message("found duplicate gene name(s) for ", method_name)
    for (j in dp) {
      idx <- which(res_df[, feature_id] == j)
      kp <- which.min(res_df[idx, method_name])[1]
      if (is.na(kp)) kp <- 1
      rem <- idx[-kp]
      res_df <- res_df[-rem, , drop = FALSE]
    }
    if (sum(duplicated(res_df[, feature_id])) == 0)
      message("fixed the duplication")
  }
  res_df
}

#' Calculate adjusted p-values
#'
#' Calculate adjusted p-values for methods where only nominal p-values are
#' available in a \code{COBRAData} object.
#'
#' @param cobradata A \code{COBRAData} object.
#' @param method A character string giving the method (selected from
#'   \code{p.adjust.methods()}) that will be used to perform the adjustment.
#'
#' @return A \code{COBRAData} object, extended with the calculated adjusted
#'   p-values.
#'
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobradata_example <- calculate_adjp(cobradata_example, method = "BH")
calculate_adjp <- function(cobradata, method = "BH") {
  sf <- setdiff(colnames(pval(cobradata)), colnames(padj(cobradata)))
  if (length(sf) > 0) {
      pa <- as.data.frame(apply(pval(cobradata)[, sf, drop = FALSE], 2,
                                stats::p.adjust, method = method))
      missing_genes <- setdiff(rownames(pa), rownames(padj(cobradata)))
      if (length(padj(cobradata)) != 0) {
        tmpadd <-
          as.data.frame(matrix(NA, length(missing_genes),
                               ncol(padj(cobradata)),
                               dimnames = list(missing_genes,
                                               colnames(padj(cobradata)))))
        padj(cobradata) <- rbind(padj(cobradata), tmpadd)
        padj(cobradata) <- cbind(padj(cobradata),
                                 pa[match(rownames(padj(cobradata)),
                                          rownames(pa)), , drop = FALSE])
      } else {
        padj(cobradata) <- pa
      }
  }
  cobradata
}

fixcolname <- function(df, prevv, newv) {
  idx <- which(colnames(df) == prevv)
  if (length(idx) > 0)
    colnames(df)[idx] <- newv
  df
}

isinvalidpalette <- function(palette) {
  bnm <- strsplit(palette, " \\(")[[1]][1]
  if (!(bnm %in% c("hue_pal", "rainbow", "heat", "terrain", "topo",
                   "cm", "Accent", "Dark2", "Paired", "Pastel1",
                   "Pastel2", "Set1", "Set2", "Set3")))
    TRUE
  else
    FALSE
}

#' @importFrom scales hue_pal brewer_pal
define_colors <- function(cobraperf, palette, facetted, incloverall, 
                          conditionalfill) {
  levs <- basemethod <- NULL

  basem <- basemethods(cobraperf)
  strl <- paste0("_", stratiflevels(cobraperf))
  inp_methods <- expand.grid(basemethod = basem, levs = c("", strl))
  inp_methods$fullmethod <- paste0(inp_methods$basemethod, inp_methods$levs)

  if (!isTRUE(incloverall))
    inp_methods <- subset(inp_methods, levs != "_overall")

  ## Calculate number of required colors
  if (isTRUE(facetted)) {
    tmp_methods <- unique(inp_methods$basemethod)
    ncolors <- length(unique(tmp_methods))
  } else {
    tmp_methods <- unique(subset(inp_methods, levs != "" &
                                   basemethod != "truth")$fullmethod)
    if ("truth" %in% inp_methods$basemethod)
      tmp_methods <- c(tmp_methods, "truth")
    ncolors <- length(tmp_methods)
  }
  tmp_methods <- as.character(tmp_methods)

  ## Define colors
  if (length(palette) > 1 || 
      (length(palette) == 1 && isinvalidpalette(palette))) {
    ## User-specified color vector
    
    ## First get the pre-defined colors right
    use_colors0 <- c()
    if (any(tmp_methods %in% names(palette))) {
      use_colors0 <- palette[intersect(tmp_methods, names(palette))]
      tmp_methods <- setdiff(tmp_methods, names(use_colors0))
      ncolors <- ncolors - length(use_colors0)
      palette <- palette[!(names(palette) %in% names(use_colors0))]
    }
    ## Then exclude any color assigned to irrelevant method
    excl <- which(names(palette) != "" & !(names(palette) %in% tmp_methods))
    if (length(excl) > 0) {
      palette <- palette[-excl]
    }
    
    if (length(palette) > ncolors) {
      warning("too many colors supplied, only a subset will be used")
      use_colors1 <- palette[seq_len(ncolors)]
    } else if (length(palette) < ncolors) {
      warning("too few colors provided, ", ncolors - length(palette),
              " random colors will be added")
      use_colors1 <- c(palette,
                       setdiff(sample(grDevices::colors(),
                                      length(grDevices::colors())),
                               c("white",
                                 palette))[seq_len(ncolors - length(palette))])
    } else {
      use_colors1 <- palette
    }
    names(use_colors1) <- tmp_methods
    use_colors1 <- c(use_colors0, use_colors1)
  } else {
    bnm <- strsplit(palette, " \\(")[[1]][1]
    ## Check that the palette is valid, otherwise consider it a color
    if (!(bnm %in% c("hue_pal", "rainbow", "heat", "terrain", "topo",
                     "cm", "Accent", "Dark2", "Paired", "Pastel1",
                     "Pastel2", "Set1", "Set2", "Set3"))) {
      warning("invalid palette, will consider it a color")
      use_colors1 <- c(palette,
                       setdiff(sample(grDevices::colors(),
                                      length(grDevices::colors())),
                               c("white",
                                 palette))[seq_len(ncolors - length(palette))])
    } else {
      ## Pre-defined palette
      maxnbr <- c(Accent = 8, Dark2 = 8, Paired = 12, Pastel1 = 9,
                  Pastel2 = 8, Set1 = 9, Set2 = 8, Set3 = 12)
      if (palette == "hue_pal") use_colors1 <- scales::hue_pal()(ncolors)
      else if (palette == "rainbow") use_colors1 <- grDevices::rainbow(ncolors)
      else if (palette == "heat") use_colors1 <- grDevices::heat.colors(ncolors)
      else if (palette == "terrain") use_colors1 <-
          grDevices::terrain.colors(ncolors)
      else if (palette == "topo") use_colors1 <- grDevices::topo.colors(ncolors)
      else if (palette == "cm") use_colors1 <- grDevices::cm.colors(ncolors)
      else {
        if (ncolors <= maxnbr[bnm])
          use_colors1 <- scales::brewer_pal(palette = bnm)(ncolors)
        else
          use_colors1 <- scales::hue_pal()(ncolors)
      }
    }
    names(use_colors1) <- tmp_methods
  }

  if (isTRUE(facetted)) {
    inp_methods$color <- use_colors1[match(inp_methods$basemethod,
                                           names(use_colors1))]
  } else {
    inp_methods$color <- use_colors1[match(inp_methods$fullmethod,
                                           names(use_colors1))]
    inp_methods$color[inp_methods$basemethod == "truth"] <-
      use_colors1["truth"]
    if ("_overall" %in% inp_methods$levs) {
      for (m in unique(inp_methods$basemethod)) {
        inp_methods$color[inp_methods$fullmethod == m] <-
          inp_methods$color[inp_methods$fullmethod == paste0(m, "_overall")]
      }
    }
  }

  use_colors1 <- inp_methods$color
  names(use_colors1) <- inp_methods$fullmethod

  ## Get colors for "FDR satisfied/not satisfied" (white or method color), if
  ## conditionalfill = TRUE. If not, points will always be white.
  if (conditionalfill) {
    use_colors2 <- c(use_colors1, rep("white", length(use_colors1)))
  } else {
    use_colors2 <- rep("white", 2 * length(use_colors1))
  }
  names(use_colors2) <- c(paste0(names(use_colors1), "yes"),
                          paste0(names(use_colors1), "no"))
  
  use_colors <- c(use_colors1, use_colors2)
  return(use_colors)
}

symdiff <- function(x, y) { 
  setdiff(union(x, y), intersect(x, y))
}

check_agreement <- function(x, y, method, expsgn) {
  x <- x[method]
  y <- y[method]
  ok <- length(symdiff(rownames(x[!is.na(x[[method]]), , drop = FALSE]), 
                       rownames(y[!is.na(y[[method]]), , drop = FALSE]))) == 0
  nm <- intersect(rownames(x), rownames(y))
  x <- x[match(nm, rownames(x)), method]
  y <- y[match(nm, rownames(y)), method]
  sgn <- sign(diff(x[order(y)]))
  ## Check that x and y are monotonically related, with the right sign
  all(sgn[sgn != 0 & !is.na(sgn)] == expsgn) && ok
}

select_measure <- function(cobradata, method, asp, prefer_pval) {
  cobradata <- update_cobradata(cobradata, quiet = TRUE)
  ret <- NULL
  if (asp %in% c("nbr", "fpr", "fdr", "tpr")) {
    if (method %in% names(padj(cobradata))) ret <- "padj"
    else ret <- NULL
  } else if (asp %in% c("fsrnbr")) {
    if (method %in% names(score(cobradata)) && 
        method %in% names(sval(cobradata))) ret <- "sval"
    else ret <- NULL
  } else if (asp %in% c("fdrtpr", "fdrnbr")) {
    if (method %in% names(pval(cobradata))) {
      if (method %in% names(padj(cobradata))) {
        ## Check that pval is monotonically increasing with increasing padj
        if (check_agreement(padj(cobradata), pval(cobradata), method, 1)) {
          ret <- "pval"
        } else if (method %in% names(score(cobradata))) {
          ## Check that score is monotonically increasing with decreasing padj
          if (check_agreement(padj(cobradata), score(cobradata), method, -1)) {
            ret <- "score"
          } else { 
            ret <- "padj"
          }
        } else { 
          ret <- "padj"
        }
      } else ret <- "pval"
    } else if (method %in% names(score(cobradata))) {
      if (method %in% names(padj(cobradata))) {
        ## Check that score is monotonically increasing with decreasing padj
        if (check_agreement(padj(cobradata), score(cobradata), method, -1)) {
          ret <- "score"
        } else ret <- "padj"
      } else ret <- "score"
    } else if (method %in% names(padj(cobradata))) {
      ret <- "padj"
    } else ret <- NULL
  } else if (asp %in% c("roc", "fpc")) {
    if (prefer_pval) {
      if (method %in% names(pval(cobradata))) ret <- "pval"
      else if (method %in% names(padj(cobradata))) ret <- "padj"
      else if (method %in% names(score(cobradata))) ret <- "score"
      else ret <- NULL
    } else {
      if (method %in% names(score(cobradata))) ret <- "score"
      else if (method %in% names(pval(cobradata))) ret <- "pval"
      else if (method %in% names(padj(cobradata))) ret <- "padj"
      else ret <- NULL
    }
  } else if (asp %in% c("corr", "scatter", "deviation")) {
    if (method %in% names(score(cobradata))) ret <- "score"
    else ret <- NULL
  }
  ret
}

#' @importFrom reshape2 melt
extend_resulttable <- function(df, splv, keeplevels, valuename,
                               basemethod, domelt = TRUE) {
  if (isTRUE(domelt)) {
    df <- reshape2::melt(df, varnames = c("thr", "method"),
                         value.name = valuename)
    df <- fixcolname(df, prevv = "value", newv = valuename)
  }
  df$basemethod <- basemethod[match(df$method, names(basemethod))]
  df$meas <- sapply(df$method, function(w) {
    a <- strsplit(as.character(w), "__")[[1]]
    paste0("__", a[length(a)])
  })
  df$method <- gsub("__sval$", "", gsub("__padj$", "", 
                                        gsub("__pval$", "",
                                             gsub("__score", "", df$method))))

  df$fullmethod <- df$method
  if (splv == "none") {
    df <- droplevels(df[grep("_overall$", df$method), ])
    df$splitval <- "overall"
    df$method <- sapply(as.character(df$fullmethod), function(w) {
      gsub("_overall$", "", w)
    })
  } else {
    df$splitval <-
      factor(sapply(as.character(df$fullmethod),
                    function(w) {
                      if (length(grep("overall$", w)) > 0) {
                        "overall"
                      } else {
                        a <- unlist(strsplit(w, paste0("_", splv, ":")))
                        paste0(splv, ":", a[2])
                      }
                    }), levels = c("overall", paste0(splv, ":", keeplevels)))

    df$method <-
      sapply(as.character(df$fullmethod),
             function(w) {
               if (length(grep("_overall$", w)) > 0) {
                 unlist(strsplit(w, "_overall$")[1])
               } else {
                 a <- unlist(strsplit(w, paste0("_", splv, ":")))
                 a[1]
               }
             })
  }
  df
}

#' @importFrom DT datatable
fix_res <- function(res, methodcol, aspcts, tabtype = "large") {
  res$dist_ <- round(res$dist_, 1)
  if (tabtype != "scatter") {
    if (!("thr" %in% colnames(res))) {
      um <- unique(res[, methodcol])
      res <- res[match(um, res[, methodcol]), ]
    }
  }
  idx <- match(c("thr", methodcol, "dist_", "NBR", "CUTOFF", "FPC_CUTOFF",
                 "ROC_CUTOFF", "topN", "OBSERVATION", "TRUTH", "POSSIGN", 
                 "NEGSIGN", "ZEROSIGN"), colnames(res))
  colnames(res)[idx[!is.na(idx)]] <-
    c("Threshold", "Method", "Distance from cursor", "Number of detections",
      "Cutoff", "Cutoff", "Cutoff", "Number of detections", "observation",
      "truth", "Number of pos sign", "Number of neg sign", 
      "Number of zero sign")[!is.na(idx)]

  if ("Threshold" %in% colnames(res)) {
    res$Threshold <- gsub("thr", "", res$Threshold)
  }
  res$Method_m <- paste0(res$Method, res$meas)
  if (tabtype == "large") {
    res[, c("TP", "FP", "FN", "TN", "TOT_CALLED", "DIFF", "NONDIFF")] <-
      round(res[, c("TP", "FP", "FN", "TN", "TOT_CALLED", "DIFF", "NONDIFF")])
    DT::datatable(res[, c(ifelse("Threshold" %in% colnames(res),
                                 "Threshold", "Cutoff"),
                          "Method_m", aspcts, "TP", "FP",
                          "FN", "TN", "TOT_CALLED", "DIFF", "NONDIFF",
                          "Distance from cursor")])
  } else if (tabtype == "fsrnbr") {
    DT::datatable(res[, c(ifelse("Threshold" %in% colnames(res),
                                 "Threshold", "Cutoff"),
                          "Method_m", aspcts, "TS", "FS", "TOT_CALLED", 
                          "Number of pos sign", 
                          "Number of neg sign", "Number of zero sign", 
                          "Distance from cursor")])
  } else if (tabtype == "corr") {
    DT::datatable(res[, c("Method_m", aspcts, "Distance from cursor")])
  } else if (tabtype == "scatter") {
    DT::datatable(res[, c("Method_m", "feature", aspcts,
                          "Distance from cursor")])
  } else if (tabtype == "deviation") {
    res[, "Horizontal distance from cursor"] <-
      res[, "Distance from cursor"]
    DT::datatable(res[, c("Method_m", "feature", aspcts,
                          "Horizontal distance from cursor")])
  } else {
    DT::datatable(res[, c("Method_m", aspcts, "Cutoff",
                          "Distance from cursor")])
  }
}

## Test that calculate_adjp is doing the right thing

library(iCOBRA)
context("Check that calculate_adjp is working properly")

test_that("calculated adjusted p-values are correct", {
  pval <- c(0.0058, 0.771, 0.024, 0.741, 0.247)
  ib <- COBRAData(pval = data.frame(m1 = pval, row.names = paste0("F", 1:5)),
                 truth = data.frame(status = c(0, 1, 0, 1, 0),
                                    row.names = paste0("F", 1:5)))
  ib2 <- calculate_adjp(ib)
  expect_equal(padj(ib2)[, "m1"], p.adjust(pval, method = "BH"))
  ib2 <- calculate_adjp(ib, method = "holm")
  expect_equal(padj(ib2)[, "m1"], p.adjust(pval, method = "holm"))
})

test_that("adjusted p-values are added correctly", {
  pval <- c(0.0058, 0.771, 0.024, 0.741, 0.247)
  padj <- p.adjust(pval, method = "BH")
  ib <- COBRAData(pval = data.frame(m1 = pval, m2 = pval, row.names = paste0("F", 1:5)),
                 padj = data.frame(m1 = padj, row.names = paste0("F", 1:5)),
                 truth = data.frame(status = c(0, 1, 0, 1, 0),
                                    row.names = paste0("F", 1:5)))
  ib2 <- calculate_adjp(ib, method = "BH")
  expect_equal(padj(ib2)$m1, padj(ib2)$m2)
})

## Test helper functions

library(iCOBRA)
context("Check helper functions")

test_that("is_plottable works", {
  expect_false(iCOBRA:::is_plottable(NULL))
  expect_false(iCOBRA:::is_plottable(data.frame()))
  expect_false(iCOBRA:::is_plottable(logical(0)))
})

test_that("get_coltype works", {
  expect_equal(iCOBRA:::get_coltype("Method1:P"), "pval")
  expect_equal(iCOBRA:::get_coltype("Method2:adjP"), "padj")
  expect_equal(iCOBRA:::get_coltype("Method3:score"), "score")
  expect_equal(iCOBRA:::get_coltype("Method3:S"), "sval")
  expect_is(iCOBRA:::get_coltype("Method2"), "NULL")
})

test_that("fix_res works", {
  cobraperf <- calculate_performance(cobradata_example_sval, binary_truth = "status",
                                    cont_truth = "logFC")
  cobraplot <- prepare_data_for_plot(cobraperf)

  res <- fsrnbr(cobraplot)
  res$dist_ <- runif(nrow(res))
  dt <- iCOBRA:::fix_res(res, methodcol = "fullmethod", aspcts = "FSR", tabtype = "fsrnbr")
  expect_is(dt, "datatables")
  
  res <- tpr(cobraplot)
  res$dist_ <- runif(nrow(res))
  dt <- iCOBRA:::fix_res(res, methodcol = "fullmethod", aspcts = "TPR")
  expect_is(dt, "datatables")

  res <- corr(cobraplot)
  res$dist_ <- runif(nrow(res))
  dt <- iCOBRA:::fix_res(res, methodcol = "fullmethod", aspcts = "PEARSON",
                       tabtype = "corr")
  expect_is(dt, "datatables")

  res <- scatter(cobraplot)
  res$dist_ <- runif(nrow(res))
  dt <- iCOBRA:::fix_res(res, methodcol = "fullmethod", aspcts = c("observation", "truth"),
                       tabtype = "scatter")
  expect_is(dt, "datatables")

  res <- deviation(cobraplot)
  res$dist_ <- runif(nrow(res))
  dt <- iCOBRA:::fix_res(res, methodcol = "fullmethod", aspcts = "DEVIATION",
                       tabtype = "deviation")
  expect_is(dt, "datatables")

  res <- roc(cobraplot)
  res$dist_ <- runif(nrow(res))
  dt <- iCOBRA:::fix_res(res, methodcol = "fullmethod", aspcts = c("FPR", "TPR"),
                       tabtype = "small")
  expect_is(dt, "datatables")
})

test_that("res_check works", {
  expect_false(iCOBRA:::res_check(pval(cobradata_example_sval)[, 1, drop = FALSE]))
  expect_true(iCOBRA:::res_check(pval(cobradata_example_sval)))
})

test_that("fix_duplicates works", {
  df <- data.frame(gene = c("A", "A", "B", "C", "D"), "m1:P" = c(0.2, 0.1, 0.5, 0.3, 0.01), check.names = FALSE, stringsAsFactors = FALSE)
  expect_equal(nrow(iCOBRA:::fix_duplicates(df, "gene", "m1:P")), 4)
  expect_equal(iCOBRA:::fix_duplicates(df, "gene", "m1:P")[, "m1:P"], c(0.1, 0.5, 0.3, 0.01))
})

test_that("get_keeplevels works", {
  set.seed(42L)
  tmptruth <- data.frame(status = sample(c(0, 1), size = 100, replace = TRUE),
                         splitvar = sample(LETTERS[seq_len(10)], size = 100, replace = TRUE))
  expect_equal(get_keeplevels(truth = tmptruth, splv = "splitvar", 
                              binary_truth = "status", maxsplit = 3), 
               c("A", "J", "B"))
  expect_equal(get_keeplevels(truth = tmptruth, splv = "splitvar", 
                              binary_truth = "status", maxsplit = 6), 
               c("A", "J", "B", "F", "I", "E"))
  
  tmptruth$status[tmptruth$splitvar == "B"] <- 0
  tmptruth$status[tmptruth$splitvar == "J"] <- 1
  expect_equal(get_keeplevels(truth = tmptruth, splv = "splitvar", 
                              binary_truth = "status", maxsplit = 4), 
               c("A", "J", "F", "I"))
  
  expect_equal(get_keeplevels(truth = tmptruth, splv = "none", 
                              binary_truth = "status", maxsplit = 4), 
               "overall")
  
  expect_equal(get_keeplevels(truth = tmptruth, splv = "splitvar", 
                              binary_truth = NULL, maxsplit = 4), 
               c("A", "J", "B", "F"))
})

#' Interactive shiny app to visualize results
#'
#' Interactive shiny app for visualization of results. The app can be
#' initialized with a \code{COBRAData} object. If no object is provided, truth
#' and results are loaded into the app from text files (see the Instructions tab
#' of the app for formatting instructions). Properly formatted text files can
#' also be obtained using the function \code{\link{COBRAData_to_text}}.
#'
#' @param cobradata An (optional) \code{COBRAData} object. If not given, the
#'   user can load results from text files.
#' @param autorun A logical indicating whether the app calculations should start
#'   automatically on launch, or wait for the user to press the 'Start
#'   calculation!' button.
#' @param addStopButton Logical scalar. If \code{TRUE} (default), will add a
#'   button to stop the app (by calling \code{shiny::stopApp}).
#' @author Charlotte Soneson
#' @return Returns (and runs) an object representing the shiny app.
#' @import shiny
#' @import shinydashboard
#' @importFrom shinyBS bsTooltip
#' @importFrom utils packageVersion
#' @export
#' @examples
#' data(cobradata_example)
#' app <- COBRAapp(cobradata_example)
#' if (interactive()) {
#'   shiny::runApp(app)
#' }
COBRAapp <- function(cobradata = NULL, autorun = FALSE, 
                     addStopButton = TRUE) {
  ## ------------------------------------------------------------------ ##
  ##                          Define UI                                 ##
  ## ------------------------------------------------------------------ ##

  p_layout <-
    shinydashboard::dashboardPage(
      skin = "blue",

      shinydashboard::dashboardHeader(
        title = paste0("iCOBRA - interactive COmparative evaluation of ",
                       "Binary classification and RAnking methods (v", 
                       packageVersion("iCOBRA"), ")"),
        titleWidth = 950),

      shinydashboard::dashboardSidebar(
        width = 350,
        ## Settings and inputs for the truth
        shinydashboard::menuItem("Truth", icon = icon("database"),
                 ## Load the file containing the truth.
                 uiOutput("choose_truth_file"),
                 shinyBS::bsTooltip(
                   "truth", paste0("Select the file containing the true status",
                                   " of each feature. See the Instructions ",
                                   "tab for formatting instructions."),
                   "right", options = list(container = "body")),

                 ## Define the column containing the feature identifier
                 ## (for both truth and results)
                 uiOutput("choose_feature_id"),

                 ## Define the column containing the (binary) truth
                 uiOutput("choose_binary_truth"),

                 ## Define the column containing the (continuous) truth
                 uiOutput("choose_continuous_truth"),

                 ## Define the variable used to stratify the results,
                 ## based on the columns available in the truth file.
                 uiOutput("splitvar"),
                 shinyBS::bsTooltip(
                   "splitvar",
                   paste0("Select a feature attribute by which to stratify ",
                          "the result representations."),
                           "right", options = list(container = "body")),

                 ## Define the maximal number of categories to retain
                 ## in stratification.
                 uiOutput("choosemaxsplit"),
                 shinyBS::bsTooltip(
                   "choosemaxsplit",
                   paste0("Set the number of categories to show if the results",
                          " are stratified by a variable annotation. The most ",
                          "frequent categories with both positive and negative",
                          " instances will be retained"),
                   "right", options = list(container = "body")),

                 ## Decide whether or not to include the "overall" category when
                 ## stratifying the results.
                 uiOutput("chooseincludeoverall"),
                 shinyBS::bsTooltip(
                   "includeoverall",
                   paste0("Select whether or not to include the 'overall' ",
                          "class when showing stratified results."),
                   "right", options = list(container = "body"))),

        ## Settings and inputs for results
        shinydashboard::menuItem("Results", icon = icon("folder-open"),
                 ## Load the file containing the results.
                 uiOutput("choose_result_file"),
                 shinyBS::bsTooltip(
                   "file1", paste0("Select file containing results from one or",
                                   " multiple methods. See the Instructions ",
                                   "tab for formatting instructions."),
                   "right", options = list(container = "body")),

                 ## Decide which methods to include in the results.
                 ## Depends on the loaded result files.
                 uiOutput("columns"),
                 shinyBS::bsTooltip(
                   "columns",
                   paste0("Select the methods for which to show the results."),
                   "right", options = list(container = "body")),

                 ## Decide whether to base evaluations on all genes in
                 ## truth table, or only on variables for which both a truth
                 ## value and a result is provided
                 checkboxInput(
                   inputId = "onlyshared",
                   label = paste0("Calculate performance based only on ",
                                  "features shared between truth and result ",
                                  "tables."),
                   value = FALSE),
                 shinyBS::bsTooltip(
                   "onlyshared",
                   paste0("Calculate performance based only on ",
                          "features shared between truth and result ",
                          "tables. Otherwise, all features in the ",
                          "truth table will be used."),
                   "right", options = list(container = "body")),

                 actionButton("goButton", "Start calculation!",
                              icon = icon("plane"))),

        ## Plot settings
        shinydashboard::menuItem("Plot settings", icon = icon("paint-brush"),
                 ## Choose color scheme
                 selectInput(
                   "colorscheme", "Select color palette",
                   c("hue_pal", "Accent (max 7 methods)",
                     "Dark2 (max 7 methods)", "Paired (max 11 methods)",
                     "Pastel1 (max 8 methods)", "Pastel2 (max 7 methods)",
                     "Set1 (max 8 methods)", "Set2 (max 7 methods)",
                     "Set3 (max 11 methods)", "rainbow", "heat", "terrain",
                     "topo", "cm"),
                   selectize = TRUE),
                 shinyBS::bsTooltip(
                   "colorscheme",
                   paste0("Choose color palette. Some palettes are only ",
                          "applicable if the number of methods ",
                          "(or method/stratification level combinations) is ",
                          "below a certain threshold. If this threshold is ",
                          "exceeded, the colorscheme will default to hue_pal."),
                   "right", options = list(container = "body")),

                 ## Decide what to include in FDR/TPR plots.
                 checkboxGroupInput(
                   "plottype",
                   paste0("Display full curve and/or points in FDR/TPR, ",
                          "FDR/NBR and FSR/NBR plots"),
                   c("curve", "points"), c("points")),

                 ## Decide whether or not to facet plots.
                 checkboxGroupInput(
                   "facet_opt",
                   "Split plots into panels by stratifying variable",
                   "split", "split"),

                 ## Define the q-value thresholds to use in the plots.
                 textInput(inputId = "fdrthresholds", label = "FDR thresholds",
                           value = "0.01, 0.05, 0.1"),
                 shinyBS::bsTooltip("fdrthresholds",
                           paste0("Specific FDR thresholds at which the ",
                                  "performance will be evaluated. ",
                                  "Separate multiple values with comma"),
                           "right", options = list(container = "body")),

                 ## Define the s-value thresholds to use in the plots.
                 textInput(inputId = "svalthresholds", 
                           label = "s-value thresholds",
                           value = "0.01, 0.05, 0.1"),
                 shinyBS::bsTooltip("svalthresholds",
                                    paste0("Specific s-value thresholds at ", 
                                           "which the performance will be ", 
                                           "evaluated. Separate multiple ", 
                                           "values with comma"),
                                    "right", 
                                    options = list(container = "body")),

                 ## Define the plot height (in pixels).
                 numericInput(inputId = "plotheight",
                              label = "Plot height (numeric, in pixels)",
                              value = 600, min = 200, max = 2000, step = 10),
                 shinyBS::bsTooltip("plotheight",
                           paste0("The height of the plots (in pixels). ",
                                  "Default 800."),
                           "right", options = list(container = "body")),

                 ## Define the pointsize used in the plots.
                 numericInput(inputId = "pointsize", label = "Point size",
                              value = 5),
                 shinyBS::bsTooltip("pointsize",
                           paste0("The point size used in the plots."),
                           "right", options = list(container = "body")),

                 ## Define the fontsize used in the panel headers.
                 numericInput(inputId = "stripsize",
                              label = "Font size for panel headers",
                              value = 15),
                 shinyBS::bsTooltip("stripsize",
                           paste0("The font size used for panel headers in",
                                  " facetted plots."),
                           "right", options = list(container = "body"))),
        
        ## Button to close app
        shiny::uiOutput("close_app_ui")
      ),

      ## Outputs
      shinydashboard::dashboardBody(fluidRow(
        shinydashboard::tabBox(
          width = 12,

          ## Define output size of error messages
          tags$head(
            tags$style(HTML("
                            .shiny-output-error-validation {
                            font-size: 17px;
                            }
                            "))
            ),

          tabPanel("About",
                   includeMarkdown(system.file("extdata", "about.md",
                                               package = "iCOBRA")),
                   value = "about"),

          tabPanel("Instructions",
                   includeMarkdown(system.file("extdata", "instructions.md",
                                               package = "iCOBRA")),
                   value = "instructions"),
          
          tabPanel("Data preview",
                   h1("Truth"), 
                   DT::dataTableOutput("table.truth"),
                   h1("p-values"), 
                   DT::dataTableOutput("table.pval"),
                   h1("adjusted p-values"), 
                   DT::dataTableOutput("table.padj"),
                   h1("s-values"), 
                   DT::dataTableOutput("table.sval"),
                   h1("scores"), 
                   DT::dataTableOutput("table.score"),
                   value = "datapreview"),

          tabPanel("TPR vs FDR",
                   uiOutput("plot.fdrtprcurve"),
                   fluidRow(
                     column(3, sliderInput(inputId = "xrange_fdrtpr",
                                           label = "x-axis limits",
                                           min = 0, max = 1, value = c(0, 1),
                                           step = 0.01)),
                     column(3, sliderInput(inputId = "yrange_fdrtpr",
                                           label = "y-axis limits",
                                           min = 0, max = 1, value = c(0, 1),
                                           step = 0.01)),
                     column(2, br(), downloadButton("export.fdrtprcurve",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.fdrtpr.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), uiOutput("fdrtpr.df.tsv.button"))
                   ),
                   DT::dataTableOutput("fdrtprcurve_click_info"),
                   value = "fdrtprcurve"),

          tabPanel("NBR vs FDR",
                   uiOutput("plot.fdrnbrcurve"),
                   fluidRow(
                     column(4, sliderInput(inputId = "xrange_fdrnbr",
                                           label = "x-axis limits",
                                           min = 0, max = 1, value = c(0, 1),
                                           step = 0.01)),
                     column(2, br(), downloadButton("export.fdrnbrcurve",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.fdrnbr.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), uiOutput("fdrnbr.df.tsv.button"))
                   ),
                   DT::dataTableOutput("fdrnbrcurve_click_info"),
                   value = "fdrnbrcurve"),

          tabPanel("TPR", uiOutput("plot.tpr"),
                   fluidRow(
                     column(4, sliderInput(inputId = "xrange_tpr",
                                           label = "x-axis limits",
                                           min = 0, max = 1, value = c(0, 1),
                                           step = 0.01)),
                     column(2, br(), downloadButton("export.tpr",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.tpr.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.tpr.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   DT::dataTableOutput("tpr_click_info"), value = "tpr"),

          tabPanel("FPR", uiOutput("plot.fpr"),
                   fluidRow(
                     column(4, sliderInput(inputId = "xrange_fpr",
                                           label = "x-axis limits",
                                           min = 0, max = 1, value = c(0, 1),
                                           step = 0.01)),
                     column(2, br(), downloadButton("export.fpr",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.fpr.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.fpr.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   DT::dataTableOutput("fpr_click_info"), value = "fpr"),

          tabPanel("ROC", uiOutput("plot.roc"),
                   fluidRow(
                     column(3, sliderInput(inputId = "xrange_roc",
                                           label = "x-axis limits",
                                           min = 0, max = 1, value = c(0, 1),
                                           step = 0.01)),
                     column(3, sliderInput(inputId = "yrange_roc",
                                           label = "y-axis limits",
                                           min = 0, max = 1, value = c(0, 1),
                                           step = 0.01)),
                     column(2, br(), downloadButton("export.roc",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.roc.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.roc.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   DT::dataTableOutput("roc_click_info"), value = "roc"),

          tabPanel("False discovery curves",
                   uiOutput("plot.fpc"),
                   fluidRow(
                     column(3, numericInput(inputId = "maxnfdc",
                                            label = "Maximal rank to display",
                                            value = 500, min = 10, max = 10000,
                                            step = 10)),
                     column(2, br(), downloadButton("export.fpc",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.fpc.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.fpc.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   DT::dataTableOutput("fpc_click_info"), value = "fpc"),

          tabPanel("Correlation",
                   uiOutput("plot.corr"),
                   fluidRow(
                     column(3, sliderInput(inputId = "xrange_corr",
                                           label = "x-axis limits",
                                           min = -1, max = 1, value = c(-1, 1),
                                           step = 0.01)),
                     column(2, radioButtons(inputId = "corrtype",
                                            label = "Correlation measure",
                                            choices = c("pearson", "spearman"),
                                            selected = "spearman")),
                     column(2, br(), downloadButton("export.corr",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.corr.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.corr.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   DT::dataTableOutput("corr_click_info"), value = "corr"),

          tabPanel("Scatter",
                   uiOutput("plot.scatter"),
                   fluidRow(
                     column(2, checkboxInput("doflip", "Flip axes", FALSE),
                            checkboxInput("dolog", "Log-transform", FALSE)),
                     column(2, br(), downloadButton("export.scatter",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.scatter.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.scatter.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   DT::dataTableOutput("scatter_click_info"),
                   value = "scatter"),

          tabPanel("Deviations",
                   uiOutput("plot.deviation"),
                   fluidRow(
                     column(2, checkboxInput("dojitter",
                                             "Include jittered points", FALSE),
                            radioButtons(inputId = "devtrans",
                                         label = "Transformation",
                                         choices = c("raw", "absolute",
                                                     "squared"),
                                         selected = "raw")),
                     column(1, radioButtons(
                       inputId = "devtype", label = "Plot type",
                       choices = c("boxplot", "violinplot"),
                       selected = "violinplot")),
                     column(3, uiOutput("axislimitsdeviation")),
                     column(2, br(), downloadButton("export.deviation",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.deviation.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.deviation.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   DT::dataTableOutput("deviation_click_info"),
                   value = "deviation"),

          tabPanel("Venn diagram",
                   uiOutput("plot.overlap"),
                   fluidRow(
                     column(1, radioButtons(inputId = "incltruth",
                                            label = "Include truth",
                                            choices = c("yes", "no"),
                                            selected = "yes")),
                     shinyBS::bsTooltip(
                       "incltruth",
                       paste0("Whether or nor to include the truth as a ",
                              "(perfect) method in the Venn diagrams. Note ",
                              "that maximally five methods (including the ",
                              "truth) can be included."),
                       "bottom", options = list(container = "body")),
                     column(1, radioButtons(inputId = "vennType",
                                            label = "Type of threshold",
                                            choices = c("adjp", "rank"),
                                            selected = "adjp")),
                     shinyBS::bsTooltip(
                       "vennType",
                       paste0("How to determine the set of features to ",
                              "compare between methods. "),
                       "bottom", options = list(container = "body")),
                     column(2, numericInput(
                       inputId = "adjpVenn",
                       label = "Adjusted p-value threshold",
                       value = 0.05, min = 0, max = 1, step = 0.01)),
                     shinyBS::bsTooltip(
                       "adjpVenn",
                       paste0("The adjusted p-value threshold used to extract ",
                              "the sets of significant variables to use for ",
                              "the Venn diagram. "),
                       "bottom", options = list(container = "body")),
                     column(2, numericInput(
                       inputId = "rankVenn",
                       label = "Rank threshold",
                       value = 100, min = 0, max = 1e10, step = 1)),
                     shinyBS::bsTooltip(
                       "rankVenn",
                       paste0("The rank used to extract the top-ranked ",
                              "features to compare in the Venn diagram. "),
                       "bottom", options = list(container = "body")),
                     column(2, br(), downloadButton("export.overlap",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.overlap.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.overlap.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   value = "overlap"),
          
          tabPanel("UpSet plot",
                   uiOutput("plot.upset"),
                   fluidRow(
                     column(2, uiOutput("upset.stratum")),
                     column(2, radioButtons(inputId = "order.upset.by",
                                            label = "Order intersections by",
                                            choices = c("degree", "freq"),
                                            selected = "freq")),
                     column(2, radioButtons(inputId = "upset.order",
                                            label = "Order intersections",
                                            choices = c("decreasing", 
                                                        "increasing"),
                                            selected = "decreasing")),
                     column(2, br(), downloadButton("export.upset",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.upset.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), downloadButton("export.upset.df.tsv",
                                                    label = "Download tsv"))
                   ),
                   value = "upset"),
          
          tabPanel("NBR vs FSR",
                   uiOutput("plot.fsrnbrcurve"),
                   fluidRow(
                     column(4, sliderInput(inputId = "xrange_fsrnbr",
                                           label = "x-axis limits",
                                           min = 0, max = 1, value = c(0, 1),
                                           step = 0.01)),
                     column(2, br(), downloadButton("export.fsrnbrcurve",
                                                    label = "Download plot")),
                     column(2, br(), downloadButton("export.fsrnbr.df.rdata",
                                                    label = "Download Rdata")),
                     column(2, br(), uiOutput("fsrnbr.df.tsv.button"))
                   ),
                   DT::dataTableOutput("fsrnbrcurve_click_info"),
                   value = "fsrnbrcurve"),
          
          selected = "fdrtprcurve"
        )
      ))
    )

  options(shiny.maxRequestSize = 15*1024^2)
  ## ------------------------------------------------------------------ ##
  ##                          Define server                             ##
  ## ------------------------------------------------------------------ ##

  #nocov start
  server_function <- function(input, output, session) {
    values <- reactiveValues()
    values$my_cobradata <- COBRAData()

    output$choose_truth_file <- renderUI({
      if (!is.null(cobradata)) {
        NULL
      } else {
        return(fileInput(inputId = "truth", label = "Load truth file",
                         accept = c("text/csv", "text/comma-separated-values",
                                    "text/tab-separated-values", "text/plain",
                                    ".csv", ".tsv", ".tab", ".txt"),
                         multiple = FALSE))
      }
    })

    truthFile <- reactive({
      if (is.null(input$truth))
        return(NULL)
      trf <- utils::read.delim(input$truth$datapath, header = TRUE,
                               as.is = TRUE, sep = "\t", quote = "",
                               check.names = FALSE)
      isolate(values$my_cobradata <-
                COBRAData(truth = trf,
                          object_to_extend = values$my_cobradata))
      return(trf)
    })

    ## Render the UI element to choose the column containing the feature ID
    output$choose_feature_id <- renderUI({
      if (is.null(truthFile()))
        return(NULL)
      selectInput("feature_id", "Select column containing feature identifiers",
                  colnames(truthFile()), selectize = TRUE)
    })

    ## Render the UI element to choose the binary truth column
    output$choose_binary_truth <- renderUI({
      if (!is.null(cobradata)) {
        ccl <- sapply(truth(cobradata), class)
        only01 <- sapply(truth(cobradata), function(i) {
          if (all(i %in% c(0, 1, NA))) "binary"
          else "notbinary"
        })
      } else {
        if (is.null(truthFile()))
          return(NULL)
        ccl <- sapply(truthFile(), class)
        only01 <- sapply(truthFile(), function(i) {
          if (all(i %in% c(0, 1, NA))) "binary"
          else "notbinary"
        })
      }
      keepcols <- intersect(which(ccl %in% c("numeric", "integer")),
                            which(only01 == "binary"))
      if (!is.null(cobradata)) {
        selectInput("binary_truth", "Select column containing binary truth",
                    c(colnames(truth(cobradata))[keepcols], "none"),
                    selectize = TRUE)
      } else {
        selectInput("binary_truth", "Select column containing binary truth",
                    c(colnames(truthFile())[keepcols], "none"),
                    selectize = TRUE)
      }
    })

    ## Render the UI element to choose the continuous truth column
    output$choose_continuous_truth <- renderUI({
      if (!is.null(cobradata)) {
        ccl <- sapply(truth(cobradata), class)
      } else {
        if (is.null(truthFile()))
          return(NULL)
        ccl <- sapply(truthFile(), class)
      }
      keepcols <- which(ccl %in% c("numeric", "integer"))
      if (!is.null(cobradata)) {
        selectInput("cont_truth", "Select column containing continuous truth",
                    c("none", colnames(truth(cobradata))[keepcols]),
                    selectize = TRUE)
      } else {
        selectInput("cont_truth", "Select column containing continuous truth",
                    c("none", colnames(truthFile())[keepcols]),
                    selectize = TRUE)
      }
    })

    ## Render the UI element to choose which attribute (from the truth file)
    ## to split panels by.
    output$splitvar <- renderUI({
      if (!is.null(cobradata)) {
        selectInput("splv", "Select variable to stratify by",
                    c("none", colnames(truth(cobradata))), selectize = TRUE)
      } else {
        trf <- truthFile()
        if (is.null(trf))
          return(NULL)
        selectInput("splv", "Select variable to stratify by",
                    c("none", colnames(trf)), selectize = TRUE)
      }
    })

    ## Render the UI element to choose how many categories to include.
    output$choosemaxsplit <- renderUI({
      numericInput(inputId = "maxsplit",
                   label = "Maximum number of levels to show when stratifying",
                   value = 3, min = 1)
    })

    ## Render the UI element to determine whether the overall category should
    ## be included when stratifying
    output$chooseincludeoverall <- renderUI({
      radioButtons(inputId = "includeoverall",
                   label = "Include 'overall' class when stratifying",
                   choices = c("yes", "no"), selected = "yes")
    })

    ## Render the UI element to upload result file(s)
    output$choose_result_file <- renderUI({
      if (!is.null(cobradata)) {
        NULL
      } else {
        return(fileInput(inputId = "file1", label = "Add file with results",
                         accept = c("text/csv", "text/comma-separated-values",
                                    "text/tab-separated-values", "text/plain",
                                    ".csv", ".tsv"), multiple = FALSE))
      }
    })

    ## Render the UI object for selecting methods to include, based on the
    ## uploaded result files.
    output$columns <- renderUI({
      if (!is.null(cobradata)) {
        allm <- unique(c(colnames(pval(cobradata)), colnames(padj(cobradata)),
                         colnames(sval(cobradata)), colnames(score(cobradata))))
        isolate(values$my_cobradata <- cobradata)
        isolate(values$all_methods <- allm)
        isolate(values$my_cobradata <- calculate_adjp(values$my_cobradata))

        ## Create the UI element for selecting methods to include.
        checkboxGroupInput("cols", "Select methods",
                           isolate(values$all_methods),
                           isolate(values$all_methods))
      } else {
        rownames(truth(values$my_cobradata)) <-
          truth(values$my_cobradata)[, input$feature_id]

        ## Read file
        inFile <- input$file1
        if (is.null(inFile))
          return(NULL)
        v <- utils::read.delim(inFile$datapath, header = TRUE, as.is = TRUE,
                               sep = "\t", quote = "", check.names = FALSE)

        ## Perform some checks on the file, if ok then continue
        if (res_check(v)) {
          ## Extract all columns corresponding to methods
          ii <- 1

          while(ii <= ncol(v)) {
            i <- colnames(v)[ii]
            coltype <- get_coltype(i)

            if (!(is.null(coltype))) {
              ## Fix duplicates
              tmp1 <- v[, c(input$feature_id, i)]
              tmp1 <- fix_duplicates(tmp1, input$feature_id, i)

              ## Rename column to only method name
              tmp <- tmp1[i]
              rownames(tmp) <- tmp1[, input$feature_id]
              i <- gsub(":S", "", gsub(":score$", "", gsub(":adjP$", "", 
                                                           gsub(":P$", "", i))))
              colnames(tmp) <- i

              ## Add values to data collection
              isolate(values$all_methods <- unique(c(values$all_methods, i)))

              if (coltype == "pval") {
                isolate(values$my_cobradata <-
                          COBRAData(pval = tmp,
                                   object_to_extend = values$my_cobradata))
              } else if (coltype == "padj") {
                isolate(values$my_cobradata <-
                          COBRAData(padj = tmp,
                                   object_to_extend = values$my_cobradata))
              } else if (coltype == "sval") {
                isolate(values$my_cobradata <-
                          COBRAData(sval = tmp,
                                    object_to_extend = values$my_cobradata))
              } else if (coltype == "score") {
                isolate(values$my_cobradata <-
                          COBRAData(score = tmp,
                                   object_to_extend = values$my_cobradata))
              }
            }
            ii <- ii + 1
          }
          ## Calculate adjusted p-values if needed
          isolate(values$my_cobradata <- calculate_adjp(values$my_cobradata))

          ## Render the UI element
          checkboxGroupInput("cols", "Select methods",
                             isolate(values$all_methods),
                             isolate(values$all_methods))
        }
      }
    })

    plotvalues_TPR <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$binary_truth != "none") {
        thrs <- sort(unique(as.numeric(gsub(" ", "",
                                            unlist(strsplit(input$fdrthresholds,
                                                            ","))))))
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = input$binary_truth,
                                     cont_truth = NULL,
                                     aspects = "tpr", 
                                     thrs = thrs,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues_FDR <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$binary_truth != "none") {
        thrs <- sort(unique(as.numeric(gsub(" ", "",
                                            unlist(strsplit(input$fdrthresholds,
                                                            ","))))))
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = input$binary_truth,
                                     cont_truth = NULL,
                                     aspects = "fdrtpr", 
                                     thrs = thrs,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues_FPR <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$binary_truth != "none") {
        thrs <- sort(unique(as.numeric(gsub(" ", "",
                                            unlist(strsplit(input$fdrthresholds,
                                                            ","))))))
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = input$binary_truth,
                                     cont_truth = NULL,
                                     aspects = "fpr", 
                                     thrs = thrs,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues_ROC <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$binary_truth != "none") {
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = input$binary_truth,
                                     cont_truth = NULL,
                                     aspects = "roc",
                                     thrs = NULL,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues_FPC <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$binary_truth != "none") {
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = input$binary_truth,
                                     cont_truth = NULL,
                                     aspects = "fpc", 
                                     thrs = thrs,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues_FDRTPR <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$binary_truth != "none") {
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = input$binary_truth,
                                     cont_truth = NULL,
                                     aspects = "fdrtprcurve", 
                                     thrs = thrs,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues_FSR <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$cont_truth != "none") {
        thrs <- 
          sort(unique(as.numeric(gsub(" ", "",
                                      unlist(strsplit(input$svalthresholds,
                                                      ","))))))
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = NULL,
                                     cont_truth = input$cont_truth,
                                     aspects = "fsrnbr", 
                                     svalthrs = thrs,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })
    
    plotvalues_FSRNBR <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$cont_truth != "none") {
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = NULL,
                                     cont_truth = input$cont_truth,
                                     aspects = "fsrnbrcurve", 
                                     svalthrs = thrs,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })
    
    plotvalues_overlap <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun))) {
        if (input$binary_truth == "none") {
          bt <- NULL
        } else {
          bt <- input$binary_truth
        }
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = bt,
                                     cont_truth = NULL,
                                     aspects = "overlap", 
                                     thrs = NULL,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = input$adjpVenn,
                                     type_venn = input$vennType,
                                     topn_venn = input$rankVenn))
      } else {
        return(COBRAPerformance())
      }
    })
    
    plotvalues_corr <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$cont_truth != "none") {
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = NULL,
                                     cont_truth = input$cont_truth,
                                     aspects = "corr", 
                                     thrs = NULL,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues_scatter <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$cont_truth != "none") {
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = NULL,
                                     cont_truth = input$cont_truth,
                                     aspects = "scatter", 
                                     thrs = NULL,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues_deviation <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun)) &
          input$cont_truth != "none") {
        return(calculate_performance(values$my_cobradata,
                                     binary_truth = NULL,
                                     cont_truth = input$cont_truth,
                                     aspects = "deviation", 
                                     thrs = NULL,
                                     splv = input$splv,
                                     maxsplit = input$maxsplit,
                                     onlyshared = input$onlyshared,
                                     thr_venn = NULL,
                                     type_venn = "adjp",
                                     topn_venn = NULL))
      } else {
        return(COBRAPerformance())
      }
    })

    plotvalues <- reactive({
      if ((input$goButton > 0 | isTRUE(autorun))) {
        ## Put together results for chosen methods
        if (length(input$cols) != 0) {
          withProgress(message = "Calculating...", value = 0, {
            all_vals <-
              COBRAPerformance(tpr = tpr(plotvalues_TPR()),
                               fdrtpr = fdrtpr(plotvalues_FDR()),
                               fdrnbr = fdrtpr(plotvalues_FDR()),
                               fdrtprcurve = fdrtprcurve(plotvalues_FDRTPR()),
                               fdrnbrcurve = fdrnbrcurve(plotvalues_FDRTPR()),
                               fsrnbr = fsrnbr(plotvalues_FSR()),
                               fsrnbrcurve = fsrnbrcurve(plotvalues_FSRNBR()),
                               fpr = fpr(plotvalues_FPR()),
                               roc = roc(plotvalues_ROC()),
                               fpc = fpc(plotvalues_FPC()),
                               overlap = overlap(plotvalues_overlap()),
                               maxsplit = input$maxsplit,
                               splv = input$splv,
                               corr = corr(plotvalues_corr()),
                               scatter = scatter(plotvalues_scatter()),
                               deviation = deviation(plotvalues_deviation()))
            
            all_vals <- prepare_data_for_plot(
              cobraperf = all_vals,
              keepmethods = input$cols,
              incloverall = (input$includeoverall == "yes"),
              colorscheme = input$colorscheme,
              facetted = ("split" %in% input$facet_opt),
              incltruth = (input$incltruth == "yes"))

            ## This is a bit of a hack, just to make the plots directly
            ## depend on the truth input. Otherwise, they will not be
            ## automatically updated if the truth file is changed
            ## (not until something else that directly affects the plots
            ## is changed as well, like the FDR thresholds or plot size etc).
            ## The title is rendered in white, so it is not visible.
            title <- input$truth$name

            list(all_vals = all_vals, title = title)
          })
        }
      } else {
        NULL
      }
    })

    
    ## -------------------------- OVERLAP ------------------------------ ##
    ## Generate Venn diagram plot
    output$plot.overlap <- renderUI({
      plotOutput("overlap", width = "100%",
                 height = paste0(input$plotheight, "px"))
    })

    ## Figure for exporting
    output$export.overlap <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, height = 12, width = 12)
        if (length(values$all_methods) == 0 | length(input$cols) == 0)
          return(NULL)
        plot_overlap(plotvalues()$all_vals)
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.overlap.df.rdata <- downloadHandler(
      filename = "overlap-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$export.overlap.df.tsv <- downloadHandler(
      filename = "overlap-data.tsv",
      content = function(file) {
        overlap_df <- overlap(plotvalues()$all_vals)
        if (is(overlap_df, "list")) {
          overlap_df <- lapply(overlap_df,
                               function(w) cbind(feature = rownames(w), w))
          for (i in seq_len(length(overlap_df))) {
             overlap_df[[i]]$featureclass <- names(overlap_df)[i]
          }
          overlap_df <- do.call(rbind, overlap_df)
        }
        else {
          overlap_df <- cbind(feature = rownames(overlap_df), overlap_df)
        }
        utils::write.table(overlap_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$overlap <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(need(is_plottable(overlap(plotvalues()$all_vals)),
                    paste0("Venn diagrams can not be displayed. Check that ",
                           "binary_truth is not 'none', and that adjusted ",
                           "p-values are provided."))
      )
      validate(
        need((is(overlap(plotvalues()$all_vals), "list") &&
                ncol(overlap(plotvalues()$all_vals)[[1]]) <= 5) |
               (is(overlap(plotvalues()$all_vals), "data.frame") &&
                  ncol(overlap(plotvalues()$all_vals)) <= 5),
             paste0("Venn diagrams can not be constructed if more than five ",
                    "methods are selected (including the truth, ",
                    "if applicable)."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun))) {
          return(NULL)
        }

        plot_overlap(plotvalues()$all_vals)
      })
    })

    ## --------------------------- UPSET ------------------------------- ##
    ## Choose category
    output$upset.stratum <- renderUI({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(overlap(plotvalues()$all_vals)))
        return(NULL)
      else {
        if (!is(overlap(plotvalues()$all_vals), "list"))
          selectInput("upset.stratum", "Select category",
                      "overall", selectize = TRUE)
        else
          selectInput("upset.stratum", "Select category",
                      names(overlap(plotvalues()$all_vals)),
                      selectize = TRUE)
      }
    })
    
    ## Generate UpSet plot
    output$plot.upset <- renderUI({
      plotOutput("upset", width = "100%",
                 height = paste0(input$plotheight, "px"))
    })
    
    ## Figure for exporting
    output$export.upset <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, height = 12, width = 12)
        if (length(values$all_methods) == 0 | length(input$cols) == 0)
          return(NULL)
        plot_upset(plotvalues()$all_vals, order.by = input$order.upset.by,
                   decreasing = (input$upset.order == "decreasing"),
                   stratum = input$upset.stratum)
        grDevices::dev.off()
      })
    
    ## Data for exporting
    output$export.upset.df.rdata <- downloadHandler(
      filename = "upset-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })
    
    output$export.upset.df.tsv <- downloadHandler(
      filename = "upset-data.tsv",
      content = function(file) {
        overlap_df <- overlap(plotvalues()$all_vals)
        if (is(overlap_df, "list")) {
          overlap_df <- lapply(overlap_df,
                               function(w) cbind(feature = rownames(w), w))
          for (i in seq_len(length(overlap_df))) {
            overlap_df[[i]]$featureclass <- names(overlap_df)[i]
          }
          overlap_df <- do.call(rbind, overlap_df)
        }
        else {
          overlap_df <- cbind(feature = rownames(overlap_df), overlap_df)
        }
        utils::write.table(overlap_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })
    
    output$upset <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(need(is_plottable(overlap(plotvalues()$all_vals)),
                    paste0("UpSet plots can not be displayed. Check that ",
                           "binary_truth is not 'none', and that adjusted ",
                           "p-values are provided."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun))) {
          return(NULL)
        }
        
        plot_upset(plotvalues()$all_vals, order.by = input$order.upset.by,
                   decreasing = (input$upset.order == "decreasing"),
                   stratum = input$upset.stratum)
      })
    })
    
    ## ---------------------------- TPR -------------------------------- ##
    output$plot.tpr <- renderUI({
      plotOutput("tpr", width = "100%", height = paste0(input$plotheight, "px"),
                 hover = "tpr_plot_click")
    })

    ## Figure for exporting
    output$export.tpr <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(tpr(plotvalues()$all_vals)))
          return(NULL)
        print(plot_tpr(cobraplot = plotvalues()$all_vals,
                       title = plotvalues()$title,
                       stripsize = input$stripsize, titlecol = "white",
                       pointsize = input$pointsize,
                       xaxisrange = input$xrange_tpr))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.tpr.df.rdata <- downloadHandler(
      filename = "tpr-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$export.tpr.df.tsv <- downloadHandler(
      filename = "tpr-data.tsv",
      content = function(file) {
        tpr_df <- isolate(tpr(plotvalues()$all_vals))
        utils::write.table(tpr_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$tpr <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(
        need(is_plottable(tpr(plotvalues()$all_vals)),
             paste0("TPR can not be displayed. Check that ",
                    "binary_truth is not 'none' and that adjusted ",
                    "p-values are provided."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(tpr(plotvalues()$all_vals))) {
          return(NULL)
        }

        plot_tpr(cobraplot = plotvalues()$all_vals, title = plotvalues()$title,
                 stripsize = input$stripsize, titlecol = "white",
                 pointsize = input$pointsize, xaxisrange = input$xrange_tpr)
      })
    })

    output$tpr_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(tpr(plotvalues()$all_vals)))
        return(NULL)
      all_data <- isolate(tpr(plotvalues()$all_vals))
      if ("split" %in% isolate(input$facet_opt)) {
        res <- nearPoints(all_data, input$tpr_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          yvar = "num_method", xvar = "TPR")
      } else {
        res <- nearPoints(all_data, input$tpr_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, yvar = "num_method", xvar = "TPR")
      }
      fix_res(res, methodcol = "fullmethod", aspcts = c("TPR"),
              tabtype = "large")
    })

    ## ------------------------ DATA PREVIEW --------------------------- ##
    output$table.truth <- DT::renderDataTable({
      if (length(values$all_methods) == 0)
        return(NULL)
      DT::datatable(truth(values$my_cobradata))
    })
    
    output$table.pval <- DT::renderDataTable({
      if (length(values$all_methods) == 0)
        return(NULL)
      DT::datatable(pval(values$my_cobradata))
    })
    
    output$table.padj <- DT::renderDataTable({
      if (length(values$all_methods) == 0)
        return(NULL)
      DT::datatable(padj(values$my_cobradata))
    })
    
    output$table.sval <- DT::renderDataTable({
      if (length(values$all_methods) == 0)
        return(NULL)
      DT::datatable(sval(values$my_cobradata))
    })
    
    output$table.score <- DT::renderDataTable({
      if (length(values$all_methods) == 0)
        return(NULL)
      DT::datatable(score(values$my_cobradata))
    })
    
    ## ---------------------------- CORR ------------------------------- ##
    output$plot.corr <- renderUI({
      plotOutput("corr", width = "100%",
                 height = paste0(input$plotheight, "px"),
                 hover = "corr_plot_click")
    })

    ## Figure for exporting
    output$export.corr <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(corr(plotvalues()$all_vals)))
          return(NULL)
        print(plot_corr(cobraplot = plotvalues()$all_vals,
                        title = plotvalues()$title,
                        stripsize = input$stripsize, titlecol = "white",
                        pointsize = input$pointsize,
                        xaxisrange = input$xrange_corr,
                        corrtype = input$corrtype))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.corr.df.rdata <- downloadHandler(
      filename = "corr-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$export.corr.df.tsv <- downloadHandler(
      filename = "corr-data.tsv",
      content = function(file) {
        tpr_df <- isolate(corr(plotvalues()$all_vals))
        utils::write.table(tpr_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$corr <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(
        need(is_plottable(corr(plotvalues()$all_vals)),
             paste0("Correlation plots can not be displayed. Check that ",
                    "cont_truth is not 'none' and that scores ",
                    "are provided."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(corr(plotvalues()$all_vals)))
          return(NULL)

        plot_corr(cobraplot = plotvalues()$all_vals, title = plotvalues()$title,
                  stripsize = input$stripsize, titlecol = "white",
                  pointsize = input$pointsize, xaxisrange = input$xrange_corr,
                  corrtype = input$corrtype)
      })
    })

    output$corr_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(corr(plotvalues()$all_vals)))
        return(NULL)
      all_data <- isolate(corr(plotvalues()$all_vals))
      if ("split" %in% isolate(input$facet_opt)) {
        res <- nearPoints(all_data, input$corr_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          yvar = "num_method", xvar = toupper(input$corrtype))
      } else {
        res <- nearPoints(all_data, input$corr_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, yvar = "num_method",
                          xvar = toupper(input$corrtype))
      }
      fix_res(res, methodcol = "fullmethod", aspcts = toupper(input$corrtype),
              tabtype = "corr")
    })

    ## -------------------------- DEVIATION ---------------------------- ##
    output$axislimitsdeviation <- renderUI({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(deviation(plotvalues()$all_vals)))
        return(NULL)
      else {
        if (input$devtrans == "raw")
          tmp <- (deviation(plotvalues()$all_vals))$DEVIATION
        else if (input$devtrans == "absolute")
          tmp <- abs((deviation(plotvalues()$all_vals))$DEVIATION)
        else
          tmp <- (deviation(plotvalues()$all_vals))$DEVIATION^2
        mn <- signif(min(tmp[is.finite(tmp)]), 3)
        mx <- signif(max(tmp[is.finite(tmp)]), 3)
        sliderInput(inputId = "xrange_deviation", label = "x-axis limits",
                    min = mn, max = mx, value = c(mn, mx))
      }
    })

    output$plot.deviation <- renderUI({
      plotOutput("deviation", width = "100%",
                 height = paste0(input$plotheight, "px"),
                 hover = "deviation_plot_click")
    })

    ## Figure for exporting
    output$export.deviation <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(deviation(plotvalues()$all_vals)))
          return(NULL)
        print(plot_deviation(cobraplot = plotvalues()$all_vals,
                             title = plotvalues()$title,
                             stripsize = input$stripsize, titlecol = "white",
                             xaxisrange = input$xrange_deviation,
                             plottype = input$devtype,
                             dojitter = input$dojitter,
                             transf = input$devtrans))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.deviation.df.rdata <- downloadHandler(
      filename = "deviation-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$export.deviation.df.tsv <- downloadHandler(
      filename = "deviation-data.tsv",
      content = function(file) {
        deviation_df <- isolate(deviation(plotvalues()$all_vals))
        utils::write.table(deviation_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$deviation <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(
        need(is_plottable(deviation(plotvalues()$all_vals)),
             paste0("Deviation plots can not be displayed. Check that ",
                    "cont_truth is not 'none' and that scores ",
                    "are provided."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(deviation(plotvalues()$all_vals)))
          return(NULL)
        plot_deviation(cobraplot = plotvalues()$all_vals,
                       title = plotvalues()$title,
                       stripsize = input$stripsize, titlecol = "white",
                       xaxisrange = input$xrange_deviation,
                       plottype = input$devtype, dojitter = input$dojitter,
                       transf = input$devtrans)
      })
    })

    output$deviation_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(deviation(plotvalues()$all_vals)))
        return(NULL)
      all_data <- isolate(deviation(plotvalues()$all_vals))

      if (input$devtrans == "absolute")
        all_data$absDEVIATION <- abs(all_data$DEVIATION)
      if (input$devtrans == "squared")
        all_data$sqDEVIATION <- all_data$DEVIATION^2

      tmp_click <- input$deviation_plot_click
      if (!is.null(tmp_click))
        tmp_click$y <- round(tmp_click$y)
      if ("split" %in% isolate(input$facet_opt)) {
        res <- nearPoints(all_data, tmp_click,
                          threshold = 50, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          yvar = "num_method",
                          xvar = ifelse(input$devtrans == "raw", "DEVIATION",
                                        ifelse(input$devtrans == "absolute",
                                               "absDEVIATION", "sqDEVIATION")))
      } else {
        res <- nearPoints(all_data, tmp_click,
                          threshold = 50, maxpoints = 100,
                          addDist = TRUE, yvar = "num_method",
                          xvar = ifelse(input$devtrans == "raw", "DEVIATION",
                                        ifelse(input$devtrans == "absolute",
                                               "absDEVIATION", "sqDEVIATION")))
      }
      fix_res(res, methodcol = "fullmethod",
              aspcts = ifelse(input$devtrans == "raw", "DEVIATION",
                              ifelse(input$devtrans == "absolute",
                                     "absDEVIATION", "sqDEVIATION")),
              tabtype = "deviation")
    })

    ## ---------------------------- FPR -------------------------------- ##
    output$plot.fpr <- renderUI({
      plotOutput("fpr", width = "100%", height = paste0(input$plotheight, "px"),
                 hover = "fpr_plot_click")
    })

    ## Figure for exporting
    output$export.fpr <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(fpr(plotvalues()$all_vals)))
          return(NULL)
        print(plot_fpr(cobraplot = plotvalues()$all_vals,
                       title = plotvalues()$title,
                       stripsize = input$stripsize, titlecol = "white",
                       pointsize = input$pointsize,
                       xaxisrange = input$xrange_fpr))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.fpr.df.rdata <- downloadHandler(
      filename = "fpr-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$export.fpr.df.tsv <- downloadHandler(
      filename = "fpr-data.tsv",
      content = function(file) {
        fpr_df <- isolate(fpr(plotvalues()$all_vals))
        utils::write.table(fpr_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$fpr <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(
        need(is_plottable(fpr(plotvalues()$all_vals)),
             paste0("FPR can not be displayed. Check that ",
                    "binary_truth is not 'none' and that adjusted ",
                    "p-values are provided."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(fpr(plotvalues()$all_vals)))
          return(NULL)
        plot_fpr(cobraplot = plotvalues()$all_vals, title = plotvalues()$title,
                 stripsize = input$stripsize, titlecol = "white",
                 pointsize = input$pointsize, xaxisrange = input$xrange_fpr)
      })
    })

    output$fpr_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(fpr(plotvalues()$all_vals)))
        return(NULL)
      all_data <- isolate(fpr(plotvalues()$all_vals))
      if ("split" %in% isolate(input$facet_opt)) {
        res <- nearPoints(all_data, input$fpr_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          yvar = "num_method", xvar = "FPR")
      } else {
        res <- nearPoints(all_data, input$fpr_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, yvar = "num_method", xvar = "FPR")
      }
      fix_res(res, methodcol = "fullmethod", aspcts = c("FPR"),
              tabtype = "large")
    })

    ## ---------------------------- ROC -------------------------------- ##
    output$plot.roc <- renderUI({
      plotOutput("roc", width = "100%", height = paste0(input$plotheight, "px"),
                 hover = "roc_plot_click")
    })

    ## Figure for exporting
    output$export.roc <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(roc(plotvalues()$all_vals)))
          return(NULL)
        print(plot_roc(cobraplot = plotvalues()$all_vals,
                       title = plotvalues()$title,
                       stripsize = input$stripsize, titlecol = "white",
                       xaxisrange = input$xrange_roc,
                       yaxisrange = input$yrange_roc,
                       linewidth = 1))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.roc.df.rdata <- downloadHandler(
      filename = "roc-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$export.roc.df.tsv <- downloadHandler(
      filename = "roc-data.tsv",
      content = function(file) {
        roc_df <- isolate(roc(plotvalues()$all_vals))
        utils::write.table(roc_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$roc <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(
        need(is_plottable(roc(plotvalues()$all_vals)),
             paste0("ROC curves can not be displayed. Check that ",
                    "binary_truth is not 'none'."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(roc(plotvalues()$all_vals)))
          return(NULL)
        plot_roc(cobraplot = plotvalues()$all_vals, title = plotvalues()$title,
                 stripsize = input$stripsize, titlecol = "white",
                 xaxisrange = input$xrange_roc, yaxisrange = input$yrange_roc,
                 linewidth = 1)
      })
    })

    output$roc_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(roc(plotvalues()$all_vals)))
        return(NULL)

      all_data <- isolate(roc(plotvalues()$all_vals))
      if ("split" %in% isolate(input$facet_opt)) {
        res <- nearPoints(all_data, input$roc_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          xvar = "FPR", yvar = "TPR")
      } else {
        res <- nearPoints(all_data, input$roc_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, xvar = "FPR", yvar = "TPR")
      }
      fix_res(res, methodcol = "fullmethod", aspcts = c("FPR", "TPR"),
              tabtype = "small")
    })

    ## -------------------------- SCATTER ------------------------------ ##
    output$plot.scatter <- renderUI({
      plotOutput("scatter", width = "100%",
                 height = paste0(input$plotheight, "px"),
                 hover = "scatter_plot_click")
    })

    ## Figure for exporting
    output$export.scatter <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(scatter(plotvalues()$all_vals)))
          return(NULL)
        print(plot_scatter(cobraplot = plotvalues()$all_vals,
                           title = plotvalues()$title,
                           stripsize = input$stripsize, titlecol = "white",
                           pointsize = input$pointsize, doflip = input$doflip,
                           dolog = input$dolog))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.scatter.df.rdata <- downloadHandler(
      filename = "scatter-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$export.scatter.df.tsv <- downloadHandler(
      filename = "scatter-data.tsv",
      content = function(file) {
        scatter_df <- isolate(scatter(plotvalues()$all_vals))
        utils::write.table(scatter_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$scatter <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(
        need(is_plottable(scatter(plotvalues()$all_vals)),
             paste0("Scatter plots can not be displayed. Check that ",
                    "cont_truth is not 'none' and that scores ",
                    "are provided."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(scatter(plotvalues()$all_vals)))
          return(NULL)
        plot_scatter(cobraplot = plotvalues()$all_vals,
                     title = plotvalues()$title,
                     stripsize = input$stripsize, titlecol = "white",
                     pointsize = input$pointsize, doflip = input$doflip,
                     dolog = input$dolog)
      })
    })

    output$scatter_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(scatter(plotvalues()$all_vals)))
        return(NULL)

      all_data <- isolate(scatter(plotvalues()$all_vals))
      if ("split" %in% isolate(input$facet_opt)) {
        res <- nearPoints(all_data, input$scatter_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, xvar = "OBSERVATION",
                          yvar = "TRUTH")
      } else {
        res <- nearPoints(all_data, input$scatter_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, xvar = "OBSERVATION", yvar = "TRUTH")
      }
      fix_res(res, methodcol = "fullmethod", aspcts = c("observation", "truth"),
              tabtype = "scatter")
    })

    ## ---------------------------- FPC -------------------------------- ##
    output$plot.fpc <- renderUI({
      plotOutput("fpc", width = "100%", height = paste0(input$plotheight, "px"),
                 hover = "fpc_plot_click")
    })

    ## Figure for exporting
    output$export.fpc <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(fpc(plotvalues()$all_vals)))
          return(NULL)
        print(plot_fpc(cobraplot = plotvalues()$all_vals,
                       title = plotvalues()$title,
                       stripsize = input$stripsize, titlecol = "white",
                       maxnfdc = input$maxnfdc,
                       linewidth = 1))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.fpc.df.rdata <- downloadHandler(
      filename = "fpc-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$export.fpc.df.tsv <- downloadHandler(
      filename = "fpc-data.tsv",
      content = function(file) {
        fpc_df <- isolate(fpc(plotvalues()$all_vals))
        utils::write.table(fpc_df, file = file, quote = FALSE,
                           row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$fpc <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(
        need(is_plottable(fpc(plotvalues()$all_vals)),
             paste0("False discovery curves can not be displayed. Check that ",
                    "binary_truth is not 'none'."))
      )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(fpc(plotvalues()$all_vals)))
          return(NULL)

        plot_fpc(cobraplot = plotvalues()$all_vals, title = plotvalues()$title,
                 stripsize = input$stripsize, titlecol = "white",
                 maxnfdc = input$maxnfdc,
                 linewidth = 1)
      })
    })

    output$fpc_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)) ||
          !is_plottable(fpc(plotvalues()$all_vals)))
        return(NULL)
      all_data <- isolate(fpc(plotvalues()$all_vals))
      if ("split" %in% isolate(input$facet_opt)) {
        res <- nearPoints(all_data, input$fpc_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          xvar = "topN", yvar = "FP")
      } else {
        res <- nearPoints(all_data, input$fpc_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, xvar = "topN", yvar = "FP")
      }
      fix_res(res, methodcol = "fullmethod",
              aspcts = c("Number of detections", "FP"), tabtype = "small")
    })

    ## -------------------------- FDRTPR ------------------------------- ##
    output$plot.fdrtprcurve <- renderUI({
      plotOutput("fdrtprcurve", width = "100%",
                 height = paste0(input$plotheight, "px"),
                 hover = "fdrtprcurve_plot_click")
    })

    ## Figure for exporting
    output$export.fdrtprcurve <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(fdrtprcurve(plotvalues()$all_vals)))
          return(NULL)
        print(plot_fdrtprcurve(cobraplot = plotvalues()$all_vals,
                               title = plotvalues()$title,
                               stripsize = input$stripsize, titlecol = "white",
                               pointsize = input$pointsize,
                               xaxisrange = input$xrange_fdrtpr,
                               yaxisrange = input$yrange_fdrtpr,
                               plottype = input$plottype,
                               linewidth = 1))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.fdrtpr.df.rdata <- downloadHandler(
      filename = "fdrtpr-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$fdrtpr.df.tsv.button <- renderUI({
      if ("curve" %in% input$plottype)
        downloadButton("export.fdrtpr.df.tsv", label = "Download tsv (curve)")
      else
        downloadButton("export.fdrtpr.df.tsv", label = "Download tsv (points)")
    })

    output$export.fdrtpr.df.tsv <- downloadHandler(
      filename = "fdrtpr-data.tsv",
      content = function(file) {
        fdrtpr_curve_df <- isolate(fdrtprcurve(plotvalues()$all_vals))
        fdrtpr_point_df <- isolate(fdrtpr(plotvalues()$all_vals))
        if ("curve" %in% input$plottype)
          utils::write.table(fdrtpr_curve_df, file = file, quote = FALSE,
                             row.names = FALSE, col.names = TRUE, sep = "\t")
        else if ("points" %in% input$plottype)
          utils::write.table(fdrtpr_point_df, file = file, quote = FALSE,
                             row.names = FALSE, col.names = TRUE, sep = "\t")
      })

    output$fdrtprcurve <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(need(any(c("points", "curve") %in% input$plottype),
                    "No plot type selected.")
      )
      if ("curve" %in% input$plottype)
        validate(
          need(is_plottable(fdrtprcurve(plotvalues()$all_vals)),
               paste0("FDR/TPR curves can not be displayed. Check that ",
                      "binary_truth is not 'none'."))
        )
      if ("points" %in% input$plottype)
        validate(
          need(is_plottable(fdrtpr(plotvalues()$all_vals)),
               paste0("FDR/TPR at specified adjusted p-value thresholds ",
                      "can not be displayed. Check that binary_truth is not ",
                      "'none' and that adjusted p-values are provided."))
        )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)))
          return(NULL)
        if ("curve" %in% input$plottype &
            !is_plottable(fdrtprcurve(plotvalues()$all_vals)))
          return(NULL)
        if ("points" %in% input$plottype &
            !is_plottable(fdrtpr(plotvalues()$all_vals)))
          return(NULL)
        if (!any(c("curve", "points") %in% input$plottype))
          return(NULL)
        plot_fdrtprcurve(cobraplot = plotvalues()$all_vals,
                         title = plotvalues()$title,
                         stripsize = input$stripsize, titlecol = "white",
                         pointsize = input$pointsize,
                         xaxisrange = input$xrange_fdrtpr,
                         yaxisrange = input$yrange_fdrtpr,
                         plottype = input$plottype,
                         linewidth = 1)
      })
    })

    output$fdrtprcurve_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)))
        return(NULL)
      if ("curve" %in% input$plottype &
          !is_plottable(fdrtprcurve(plotvalues()$all_vals)))
        return(NULL)
      if ("points" %in% input$plottype &
          !is_plottable(fdrtpr(plotvalues()$all_vals)))
        return(NULL)
      if ("curve" %in% input$plottype)
        tab_data <- isolate(fdrtprcurve(plotvalues()$all_vals))
      else tab_data <- isolate(fdrtpr(plotvalues()$all_vals))
      if ("split" %in% isolate(input$facet_opt)) {
        res <- nearPoints(tab_data, input$fdrtprcurve_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          xvar = "FDR", yvar = "TPR")
      } else {
        res <- nearPoints(tab_data, input$fdrtprcurve_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, xvar = "FDR", yvar = "TPR")
      }
      fix_res(res, methodcol = "fullmethod", aspcts = c("FDR", "TPR"),
              tabtype = "large")
    })

    ## -------------------------- FDRNBR ------------------------------- ##
    output$plot.fdrnbrcurve <- renderUI({
      plotOutput("fdrnbrcurve", width = "100%",
                 height = paste0(input$plotheight, "px"),
                 hover = "fdrnbrcurve_plot_click")
    })

    ## Figure for exporting
    output$export.fdrnbrcurve <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(fdrnbrcurve(plotvalues()$all_vals)))
          return(NULL)
        print(plot_fdrnbrcurve(cobraplot = plotvalues()$all_vals,
                               title = plotvalues()$title,
                               stripsize = input$stripsize, titlecol = "white",
                               pointsize = input$pointsize,
                               xaxisrange = input$xrange_fdrnbr,
                               plottype = input$plottype,
                               linewidth = 1))
        grDevices::dev.off()
      })

    ## Data for exporting
    output$export.fdrnbr.df.rdata <- downloadHandler(
      filename = "fdrnbr-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })

    output$fdrnbr.df.tsv.button <- renderUI({
      if ("curve" %in% input$plottype)
        downloadButton("export.fdrnbr.df.tsv", label = "Download tsv (curve)")
      else
        downloadButton("export.fdrnbr.df.tsv", label = "Download tsv (points)")
    })

    output$export.fdrnbr.df.tsv <- downloadHandler(
      filename = "fdrnbr-data.tsv",
      content = function(file) {
        fdrnbr_curve_df <- isolate(fdrnbrcurve(plotvalues()$all_vals))
        fdrnbr_point_df <- isolate(fdrnbr(plotvalues()$all_vals))
        if ("curve" %in% input$plottype)
          utils::write.table(fdrnbr_curve_df, file = file, quote = FALSE,
                             row.names = FALSE, col.names = TRUE, sep = "\t")
        else if ("points" %in% input$plottype)
          utils::write.table(fdrnbr_point_df, file = file, quote = FALSE,
                             row.names = FALSE, col.names = TRUE, sep = "\t")
      })


    output$fdrnbrcurve <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(need(any(c("points", "curve") %in% input$plottype),
                    "No plot type selected.")
      )
      if ("curve" %in% input$plottype)
        validate(
          need(is_plottable(fdrnbrcurve(plotvalues()$all_vals)),
               paste0("FDR/NBR curves can not be displayed. Check that ",
                      "binary_truth is not 'none'."))
        )
      if ("points" %in% input$plottype)
        validate(
          need(is_plottable(fdrnbr(plotvalues()$all_vals)),
               paste0("FDR/NBR at specified adjusted p-value thresholds ",
                      "can not be displayed. Check that binary_truth is not ",
                      "'none' and that adjusted p-values are provided."))
        )

      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)))
          return(NULL)
        if ("curve" %in% input$plottype &
            !is_plottable(fdrnbrcurve(plotvalues()$all_vals)))
          return(NULL)
        if ("points" %in% input$plottype &
            !is_plottable(fdrnbr(plotvalues()$all_vals)))
          return(NULL)
        if (!any(c("curve", "points") %in% input$plottype))
          return(NULL)
        plot_fdrnbrcurve(cobraplot = plotvalues()$all_vals,
                         title = plotvalues()$title,
                         stripsize = input$stripsize, titlecol = "white",
                         pointsize = input$pointsize,
                         xaxisrange = input$xrange_fdrnbr,
                         plottype = input$plottype,
                         linewidth = 1)
      })
    })

    output$fdrnbrcurve_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)))
        return(NULL)
      if ("curve" %in% input$plottype &
          !is_plottable(fdrnbrcurve(plotvalues()$all_vals)))
        return(NULL)
      if ("points" %in% input$plottype &
          !is_plottable(fdrnbr(plotvalues()$all_vals)))
        return(NULL)
      if ("curve" %in% input$plottype)
        tab_data <- isolate(fdrnbrcurve(plotvalues()$all_vals))
      else tab_data <- isolate(fdrnbr(plotvalues()$all_vals))
      if ("split" %in% input$facet_opt) {
        res <- nearPoints(tab_data, input$fdrnbrcurve_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          xvar = "FDR", yvar = "NBR")
      } else {
        res <- nearPoints(tab_data, input$fdrnbrcurve_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, xvar = "FDR", yvar = "NBR")
      }
      methodcol <- "fullmethod"
      fix_res(res, methodcol = methodcol,
              aspcts = c("FDR", "Number of detections"), tabtype = "large")
    })

    ## -------------------------- FSRNBR ------------------------------- ##
    output$plot.fsrnbrcurve <- renderUI({
      plotOutput("fsrnbrcurve", width = "100%",
                 height = paste0(input$plotheight, "px"),
                 hover = "fsrnbrcurve_plot_click")
    })
    
    ## Figure for exporting
    output$export.fsrnbrcurve <- downloadHandler(
      filename = "shiny-plot.pdf",
      content = function(file) {
        grDevices::pdf(file, width = 12, height = input$plotheight/67)
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)) ||
            !is_plottable(fsrnbrcurve(plotvalues()$all_vals)))
          return(NULL)
        print(plot_fsrnbrcurve(cobraplot = plotvalues()$all_vals,
                               title = plotvalues()$title,
                               stripsize = input$stripsize, titlecol = "white",
                               pointsize = input$pointsize,
                               xaxisrange = input$xrange_fsrnbr,
                               plottype = input$plottype,
                               linewidth = 1))
        grDevices::dev.off()
      })
    
    ## Data for exporting
    output$export.fsrnbr.df.rdata <- downloadHandler(
      filename = "fsrnbr-data.Rdata",
      content = function(file) {
        cobraplot <- isolate(plotvalues()$all_vals)
        save(cobraplot, file = file)
      })
    
    output$fsrnbr.df.tsv.button <- renderUI({
      if ("curve" %in% input$plottype)
        downloadButton("export.fsrnbr.df.tsv", label = "Download tsv (curve)")
      else
        downloadButton("export.fsrnbr.df.tsv", label = "Download tsv (points)")
    })
    
    output$export.fsrnbr.df.tsv <- downloadHandler(
      filename = "fsrnbr-data.tsv",
      content = function(file) {
        fsrnbr_curve_df <- isolate(fsrnbrcurve(plotvalues()$all_vals))
        fsrnbr_point_df <- isolate(fsrnbr(plotvalues()$all_vals))
        if ("curve" %in% input$plottype)
          utils::write.table(fsrnbr_curve_df, file = file, quote = FALSE,
                             row.names = FALSE, col.names = TRUE, sep = "\t")
        else if ("points" %in% input$plottype)
          utils::write.table(fsrnbr_point_df, file = file, quote = FALSE,
                             row.names = FALSE, col.names = TRUE, sep = "\t")
      })
    
    
    output$fsrnbrcurve <- renderPlot({
      ## Generate error messages if data not available.
      validate(
        need(length(values$all_methods) > 0 & length(input$cols) > 0 &
               (input$goButton > 0 | isTRUE(autorun)),
             paste0("No input data provided, or 'Start calculation!' has ",
                    "not been clicked in order to launch calculations."))
      )
      validate(need(any(c("points", "curve") %in% input$plottype),
                    "No plot type selected.")
      )
      if ("curve" %in% input$plottype)
        validate(
          need(is_plottable(fsrnbrcurve(plotvalues()$all_vals)),
               paste0("FSR/NBR curves can not be displayed. Check that ",
                      "cont_truth is not 'none' and that s-values ", 
                      "are provided."))
        )
      if ("points" %in% input$plottype)
        validate(
          need(is_plottable(fsrnbr(plotvalues()$all_vals)),
               paste0("FSR/NBR at specified adjusted p-value thresholds ",
                      "can not be displayed. Check that cont_truth is not ",
                      "'none' and that s-values are provided."))
        )
      
      withProgress(message = "Updating plot...", value = 0, {
        if (length(values$all_methods) == 0 | length(input$cols) == 0 |
            (input$goButton == 0 & !isTRUE(autorun)))
          return(NULL)
        if ("curve" %in% input$plottype &
            !is_plottable(fsrnbrcurve(plotvalues()$all_vals)))
          return(NULL)
        if ("points" %in% input$plottype &
            !is_plottable(fsrnbr(plotvalues()$all_vals)))
          return(NULL)
        if (!any(c("curve", "points") %in% input$plottype))
          return(NULL)
        plot_fsrnbrcurve(cobraplot = plotvalues()$all_vals,
                         title = plotvalues()$title,
                         stripsize = input$stripsize, titlecol = "white",
                         pointsize = input$pointsize,
                         xaxisrange = input$xrange_fsrnbr,
                         plottype = input$plottype,
                         linewidth = 1)
      })
    })
    
    output$fsrnbrcurve_click_info <- DT::renderDataTable({
      if (length(values$all_methods) == 0 | length(input$cols) == 0 |
          (input$goButton == 0 & !isTRUE(autorun)))
        return(NULL)
      if ("curve" %in% input$plottype &
          !is_plottable(fsrnbrcurve(plotvalues()$all_vals)))
        return(NULL)
      if ("points" %in% input$plottype &
          !is_plottable(fsrnbr(plotvalues()$all_vals)))
        return(NULL)
      if ("curve" %in% input$plottype)
        tab_data <- isolate(fsrnbrcurve(plotvalues()$all_vals))
      else tab_data <- isolate(fsrnbr(plotvalues()$all_vals))
      if ("split" %in% input$facet_opt) {
        res <- nearPoints(tab_data, input$fsrnbrcurve_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, panelvar1 = "splitval",
                          xvar = "FSR", yvar = "NBR")
      } else {
        res <- nearPoints(tab_data, input$fsrnbrcurve_plot_click,
                          threshold = 5, maxpoints = 100,
                          addDist = TRUE, xvar = "FSR", yvar = "NBR")
      }
      methodcol <- "fullmethod"
      fix_res(res, methodcol = methodcol,
              aspcts = c("FSR", "Number of detections"), tabtype = "fsrnbr")
    })
    
    ## Close app
    output$close_app_ui <- shiny::renderUI({
      if (addStopButton) {
        shiny::actionButton("close_app", "Close app")
      } else {
        NULL
      }
    })
    shiny::observeEvent(input$close_app, {
      shiny::stopApp()
    })
    
  } ## end of server_function
  #nocov end
  
  shinyApp(ui = p_layout, server = server_function)
}

## Test shiny app

library(iCOBRA)
context("Check that shiny app is generated")

test_that("Shiny app is generated", {
  expect_warning({
    expect_is(COBRAapp(), "shiny.appobj")
  }, "expect a collection of")
  expect_warning({
    expect_is(COBRAapp(cobradata_example), "shiny.appobj")
  }, "expect a collection of")

  expect_warning({
    expect_is(COBRAapp(autorun = TRUE), "shiny.appobj")
  }, "expect a collection of")
  expect_warning({
    expect_is(COBRAapp(cobradata_example, autorun = TRUE), "shiny.appobj")
  }, "expect a collection of")
})

gen_thr_vec <- function(thresholds) {
  v <- rep(0, length(thresholds))
  names(v) <- paste0("thr", thresholds)
  v
}

## conttruth = true logFC
## estvals = estimated logFC
## svals = s-values, to rank and cutoff based on
get_curve_cont <- function(conttruth, estvals, svals, aspc) {
  if (aspc == "fsrnbr") {
    ## Get total number of features with positive, negative and zero true sign
    possign <- length(which(sign(conttruth) == 1))
    negsign <- length(which(sign(conttruth) == -1))
    zerosign <- length(which(sign(conttruth) == 0))
    
    ## subset to features with existing s-value
    kg <- intersect(names(conttruth), names(svals[!is.na(svals)]))
    svals <- sort(svals[match(kg, names(svals))])
    conttruth <- conttruth[match(names(svals), names(conttruth))]
    estvals <- estvals[match(names(svals), names(estvals))]
    
    if (any(sign(conttruth) != 0)) {
      ## Make sure that genes without logFC estimate are counted as having
      ## mismatching signs
      if (any(is.na(estvals))) {
        estvals[is.na(estvals)] <- -conttruth[is.na(estvals)]
      }
      
      ## Calculate the FSR at each s-value threshold
      unique_svals <- sort(unique(svals))
      fsrnbr <- as.data.frame(t(sapply(unique_svals, function(s) {
        ids <- which(svals <= s)
        nbr <- length(ids)
        fs <- length(which(abs(sign(conttruth[ids]) - sign(estvals[ids])) == 2))
        fsr <- fs/nbr
        ts <- length(which(sign(conttruth[ids]) == sign(estvals[ids])))
        tot_called <- length(which(!is.na(svals)))
        c(SVAL_CUTOFF = s, NBR = nbr, FS = fs, FSR = fsr, TS = ts, 
          TOT_CALLED = tot_called, POSSIGN = possign, NEGSIGN = negsign,
          ZEROSIGN = zerosign)
      })))
      return(fsrnbr)
    } else {
      return(NULL)
    }
  }
}

#' @importFrom ROCR prediction performance
get_curve <- function(bintruth, vals, revr, aspc, rank_by_abs) {
  kg <- intersect(names(bintruth), names(vals[!is.na(vals)]))
  if (any(bintruth[match(kg, names(bintruth))] == 0) &
      any(bintruth[match(kg, names(bintruth))] == 1)) {
    if (aspc %in% c("roc", "fdrtpr")) {
      tpcorr <- length(which(bintruth == 1)) /
        length(intersect(names(bintruth)[which(bintruth == 1)], kg))
      fpcorr <- length(which(bintruth == 0)) /
        length(intersect(names(bintruth)[which(bintruth == 0)], kg))
    }

    if (isTRUE(rank_by_abs))
      vals <- abs(vals)
    
    if (isTRUE(revr))
      inpvals <- 1 - vals[match(kg, names(vals))]
    else
      inpvals <- vals[match(kg, names(vals))]
    preds <- ROCR::prediction(predictions = inpvals,
                              labels = bintruth[match(kg, names(bintruth))],
                              label.ordering = c(0, 1))
    if (aspc == "roc")
      mes <- ROCR::performance(preds, measure = "tpr", x.measure = "fpr")
    else if (aspc == "fpc")
      mes <- ROCR::performance(preds, measure = "fpr", x.measure = "rpp")
    else if (aspc == "fdrtpr") {
      mes <- ROCR::performance(preds, measure = "tpr", x.measure = "ppv")
      mes2 <- ROCR::performance(preds, measure = "fpr", x.measure = "rpp")
      stopifnot(all(unlist(mes@alpha.values) == unlist(mes2@alpha.values)))
    }

    if (isTRUE(revr))
      alphas <- 1 - unlist(mes@alpha.values)
    else
      alphas <- unlist(mes@alpha.values)

    if (aspc == "roc") {
      roc = cbind(FPR = unlist(mes@x.values) / fpcorr,
                  TPR = unlist(mes@y.values) / tpcorr,
                  ROC_CUTOFF = alphas)
      return(roc)
    } else if (aspc == "fpc") {
      fpc = cbind(topN = unlist(mes@x.values) * length(kg),
                  FP = unlist(mes@y.values) *
                    length(which(bintruth[match(kg, names(bintruth))] == 0)),
                  FPC_CUTOFF = alphas)
      return(fpc)
    } else if (aspc == "fdrtpr") {
      fdrtpr = cbind(FDR = 1 - unlist(mes@x.values),
                     TPR = unlist(mes@y.values) / tpcorr,
                     NBR = round(unlist(mes2@x.values) * length(kg)),
                     CUTOFF = alphas,
                     FP = unlist(mes2@y.values) *
                       length(which(bintruth[match(kg, names(bintruth))] == 0)),
                     TOT_CALLED = length(kg))
      fdrtpr = cbind(fdrtpr,
                     TP = fdrtpr[, "TPR"] * length(which(bintruth == 1)))
      fdrtpr = cbind(fdrtpr, FN = length(
        which(bintruth[match(kg, names(bintruth))] == 1)) - fdrtpr[, "TP"])
      fdrtpr = cbind(fdrtpr, TN = fdrtpr[, "TOT_CALLED"] - fdrtpr[, "TP"] -
                       fdrtpr[, "FN"] - fdrtpr[, "FP"])
      fdrtpr = cbind(fdrtpr, DIFF = length(which(bintruth == 1)))
      fdrtpr = cbind(fdrtpr, NONDIFF = length(which(bintruth == 0)))
      return(fdrtpr)
    }
  } else {
    return(NULL)
  }
}

#' Calculate performance measures
#'
#' Calculate performance measures from a given collection of p-values, adjusted
#' p-values and scores provided in a \code{COBRAData} object.
#'
#' Depending on the collection of observations that are available for a given
#' method, the appropriate one will be chosen for each performance measure. For
#' \code{fpr}, \code{tpr}, \code{fdrtpr}, \code{fdrnbr} and \code{overlap}
#' aspects, results will only be calculated for methods where adjusted p-values
#' are included in the \code{COBRAData} object, since these calculations make
#' use of specific adjusted p-value cutoffs. For \code{fdrtprcurve} and
#' \code{fdrnbrcurve} aspects, the \code{score} observations will be
#' preferentially used, given that they are monotonically associated with the
#' adjusted p-values (if provided). If the \code{score} is not provided, the
#' nominal p-values will be used, given that they are monotonically associated
#' with the adjusted p-values (if provided). In other cases, the adjusted
#' p-values will be used also for these aspects. For \code{roc} and \code{fpc},
#' the \code{score} observations will be used if they are provided, otherwise
#' p-values and, as a last instance, adjusted p-values. Finally, for the
#' \code{fsrnbr}, \code{corr}, \code{scatter} and \code{deviation} aspects, the
#' \code{score} observations will be used if they are provided, otherwise no
#' results will be calculated.
#'
#' @param cobradata A \code{COBRAData} object.
#' @param binary_truth A character string giving the name of the column of
#'   truth(cobradata) that contains the binary truth (true assignment of
#'   variables into two classes, represented by 0/1).
#' @param cont_truth A character string giving the name of the column of
#'   truth(cobradata) that contains the continuous truth (a continuous value
#'   that the observations can be compared to).
#' @param aspects A character vector giving the types of performance measures 
#'   to calculate. Must be a subset of c("fdrtpr", "fdrtprcurve", "fdrnbr",
#'   "fdrnbrcurve", "tpr", "fpr", "roc", "fpc", "overlap", "corr", "scatter",
#'   "deviation", "fsrnbr", "fsrnbrcurve").
#' @param thrs A numeric vector of adjusted p-value thresholds for which to
#'   calculate the performance measures. Affects "fdrtpr", "fdrnbr", "tpr" and
#'   "fpr".
#' @param svalthrs A numeric vector of s-value thresholds for which to calculate
#'   the FSR. Affects "fsrnbr".
#' @param splv A character string giving the name of the column of
#'   truth(cobradata) that will be used to stratify the results. The default
#'   value is "none", indicating no stratification.
#' @param maxsplit A numeric value giving the maximal number of categories to
#'   keep in the stratification. The largest categories containing both positive
#'   and negative features will be retained. By setting this argument to `Inf`
#'   or `NA_integer_`, all categories (as well as the order of categories) will
#'   be retained.
#' @param onlyshared A logical, indicating whether to only consider features for
#'   which both the true assignment and a result (p-value, adjusted p-value or
#'   score) is given. If FALSE, all features contained in the truth table are
#'   used.
#' @param thr_venn A numeric value giving the adjusted p-value threshold to use
#'   to create Venn diagrams (if \code{type_venn} is "adjp").
#' @param type_venn Either "adjp" or "rank", indicating whether Venn diagrams
#'   should be constructed based on features with adjusted p-values below a
#'   certain threshold, or based on the same number of top-ranked features by
#'   different methods.
#' @param topn_venn A numeric value giving the number of top-ranked features to
#'   compare between methods (if \code{type_venn} is "rank").
#' @param rank_by_abs Whether to take the absolute value of the score before
#'   using it to rank the variables  for ROC, FPC, FDR/NBR and FDR/TPR curves.
#' @param prefer_pval Whether to preferentially rank variables by p-values or
#'   adjusted p-values rather than score for ROC and FPC calculations. From
#'   version 1.5.5, this is the default behaviour. To obtain the behaviour of
#'   previous versions, set to \code{FALSE}.
#'
#' @return A \code{COBRAPerformance} object
#'
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = c("fdrtpr", "fdrtprcurve",
#'                                                "tpr", "roc"),
#'                                    thrs = c(0.01, 0.05, 0.1), splv = "none")
calculate_performance <- function(cobradata, binary_truth = NULL,
                                  cont_truth = NULL,
                                  aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr",
                                              "fdrnbrcurve", "tpr", "fpr",
                                              "roc", "fpc", "overlap",
                                              "corr", "scatter", "deviation",
                                              "fsrnbr", "fsrnbrcurve"),
                                  thrs = c(0.01, 0.05, 0.1), 
                                  svalthrs = c(0.01, 0.05, 0.1), splv = "none",
                                  maxsplit = 3, onlyshared = FALSE,
                                  thr_venn = 0.05, type_venn = "adjp",
                                  topn_venn = 100, rank_by_abs = TRUE, 
                                  prefer_pval = TRUE) {

  ## Get all methods represented in the test result object
  ## (with at least one type of result)
  all_methods <- unique(c(colnames(pval(cobradata)), colnames(padj(cobradata)),
                          colnames(sval(cobradata)), 
                          colnames(score(cobradata))))
  
  ## If the binary_truth column is logical, convert to 0/1 values
  if (is.logical(truth(cobradata)[, binary_truth])) {
    message("binary_truth column is logical, converting to numeric (0/1)")
    truth(cobradata)[, binary_truth] <- 
      as.numeric(truth(cobradata)[, binary_truth])
  }

  ## ------------------- NBR, TP, FP etc (always calculated) ------------ ##
  if (any(c("tpr", "fdr", "fdrtpr", "fpr", "fdrnbr") %in% aspects) &&
      !is.null(binary_truth)) {
    outNBR <- outFP <- outTP <- outFN <- outTN <- outTOT_CALLED <-
      outDS <- outNONDS <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "fpr", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = binary_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = binary_truth,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        if (inpcol == "padj") {
          NBR <- list()
          nbr <- gen_thr_vec(thrs)
          for (thr in thrs) {
            nbr[paste0("thr", thr)] <- length(which(tmp[i] <= thr))
          }
          NBR[[paste0(i, "_overall", "__", inpcol)]] <-
            list(basemethod = i, meas_type = inpcol, nbr = nbr)
          if (splv != "none") {
            for (j in kltmp) {
              nbr <- gen_thr_vec(thrs)
              for (thr in thrs) {
                g <- rownames(truth)[which(truth[[splv]] == j)]
                nbr[paste0("thr", thr)] <-
                  length(intersect(g, rownames(tmp)[which(tmp[i] <= thr)]))
              }
              NBR[[paste0(i, "_", splv, ":", j, "__",
                          inpcol)]] <- list(basemethod = i,
                                            meas_type = inpcol, nbr = nbr)
            }
          }

          FP <- TP <- FN <- TN <- TOT_CALLED <- DS <- NONDS <- list()
          fp <- tp <- fn <- tn <- tot_called <- ds <- nonds <- gen_thr_vec(thrs)

          for (thr in thrs) {
            signif <- rownames(tmp)[which(tmp[i] <= thr)]
            nonsignif <- rownames(tmp)[which(tmp[i] > thr)]
            fp[paste0("thr", thr)] <-
              length(intersect(
                signif, rownames(truth)[which(truth[, binary_truth] == 0)]))
            tp[paste0("thr", thr)] <-
              length(intersect(
                signif, rownames(truth)[which(truth[, binary_truth] == 1)]))
            fn[paste0("thr", thr)] <-
              length(intersect(
                nonsignif, rownames(truth)[which(truth[, binary_truth] == 1)]))
            tn[paste0("thr", thr)] <-
              length(intersect(
                nonsignif, rownames(truth)[which(truth[, binary_truth] == 0)]))
            tot_called[paste0("thr", thr)] <-
              length(intersect(rownames(tmp)[which(!is.na(tmp[i]))],
                               rownames(truth)))
            ds[paste0("thr", thr)] <- length(which(truth[, binary_truth] == 1))
            nonds[paste0("thr", thr)] <-
              length(which(truth[, binary_truth] == 0))
          }
          idx <- paste0(i, "_overall", "__", inpcol)
          FP[[idx]] <- list(basemethod = i, meas_type = inpcol, fp = fp)
          TP[[idx]] <- list(basemethod = i, meas_type = inpcol, tp = tp)
          FN[[idx]] <- list(basemethod = i, meas_type = inpcol, fn = fn)
          TN[[idx]] <- list(basemethod = i, meas_type = inpcol, tn = tn)
          TOT_CALLED[[idx]] <- list(basemethod = i, meas_type = inpcol,
                                    tot_called = tot_called)
          DS[[idx]] <- list(basemethod = i, meas_type = inpcol, ds = ds)
          NONDS[[idx]] <- list(basemethod = i,
                               meas_type = inpcol, nonds = nonds)

          if (splv != "none" & any(truth[, binary_truth] == 0)) {
            for (j in kltmp) {
              fp <- tp <- fn <- tn <- tot_called <- ds <- nonds <-
                gen_thr_vec(thrs)
              for (thr in thrs) {
                signif <- rownames(tmp)[which(tmp[i] <= thr)]
                nonsignif <- rownames(tmp)[which(tmp[i] > thr)]
                g <- rownames(truth)[which(truth[[splv]] == j)]
                gt <- intersect(
                  g, rownames(truth)[which(truth[, binary_truth] == 0)])
                gtt <- intersect(
                  g, rownames(truth)[which(truth[, binary_truth] == 1)])
                gf <- intersect(g, signif)
                gnf <- intersect(g, nonsignif)
                fp[paste0("thr", thr)] <- length(intersect(gf, gt))
                tp[paste0("thr", thr)] <- length(intersect(gf, gtt))
                fn[paste0("thr", thr)] <- length(intersect(gnf, gtt))
                tn[paste0("thr", thr)] <- length(intersect(gnf, gt))
                tot_called[paste0("thr", thr)] <-
                  length(intersect(rownames(tmp)[which(!is.na(tmp[i]))], g))
                ds[paste0("thr", thr)] <- length(gtt)
                nonds[paste0("thr", thr)] <- length(gt)
              }
              idx <- paste0(i, "_", splv, ":", j, "__", inpcol)
              FP[[idx]] <- list(basemethod = i, meas_type = inpcol, fp = fp)
              TP[[idx]] <- list(basemethod = i, meas_type = inpcol, tp = tp)
              FN[[idx]] <- list(basemethod = i, meas_type = inpcol, fn = fn)
              TN[[idx]] <- list(basemethod = i, meas_type = inpcol, tn = tn)
              TOT_CALLED[[idx]] <- list(basemethod = i, meas_type = inpcol,
                                        tot_called = tot_called)
              DS[[idx]] <- list(basemethod = i, meas_type = inpcol, ds = ds)
              NONDS[[idx]] <- list(basemethod = i, meas_type = inpcol,
                                   nonds = nonds)
            }
          }

          outNBR <- c(outNBR, NBR)
          outFP <- c(outFP, FP)
          outTP <- c(outTP, TP)
          outFN <- c(outFN, FN)
          outTN <- c(outTN, TN)
          outTOT_CALLED <- c(outTOT_CALLED, TOT_CALLED)
          outDS <- c(outDS, DS)
          outNONDS <- c(outNONDS, NONDS)
        }
      } else {
        message("column ", i, " is being ignored for NBRS calculations")
      }
    }
    if (any(sapply(lapply(outNBR, function(w) w$nbr), length) > 0)) {
      nbrs <- t(do.call(rbind, lapply(outNBR, function(w) w$nbr)))
      vn <- sapply(outNBR, function(w) w$basemethod)
      tps <- t(do.call(rbind, lapply(outTP, function(w) w$tp)))
      vtp <- sapply(outTP, function(w) w$basemethod)
      fps <- t(do.call(rbind, lapply(outFP, function(w) w$fp)))
      vfp <- sapply(outFP, function(w) w$basemethod)
      tns <- t(do.call(rbind, lapply(outTN, function(w) w$tn)))
      vtn <- sapply(outTN, function(w) w$basemethod)
      fns <- t(do.call(rbind, lapply(outFN, function(w) w$fn)))
      vfn <- sapply(outFN, function(w) w$basemethod)
      tcs <- t(do.call(rbind, lapply(outTOT_CALLED, function(w) w$tot_called)))
      vtc <- sapply(outTOT_CALLED, function(w) w$basemethod)
      dss <- t(do.call(rbind, lapply(outDS, function(w) w$ds)))
      vds <- sapply(outDS, function(w) w$basemethod)
      nds <- t(do.call(rbind, lapply(outNONDS, function(w) w$nonds)))
      vnds <- sapply(outNONDS, function(w) w$basemethod)

      nbrs <- extend_resulttable(nbrs, splv, keeplevels, "NBR",
                                 vn, domelt = TRUE)
      tps <- extend_resulttable(tps, splv, keeplevels, "TP",
                                vtp, domelt = TRUE)
      fps <- extend_resulttable(fps, splv, keeplevels, "FP",
                                vfp, domelt = TRUE)
      fns <- extend_resulttable(fns, splv, keeplevels, "FN",
                                vfn, domelt = TRUE)
      tns <- extend_resulttable(tns, splv, keeplevels, "TN",
                                vtn, domelt = TRUE)
      tcs <- extend_resulttable(tcs, splv, keeplevels, "TOT_CALLED",
                                vtc, domelt = TRUE)
      dss <- extend_resulttable(dss, splv, keeplevels, "DIFF",
                                vds, domelt = TRUE)
      nds <- extend_resulttable(nds, splv, keeplevels, "NONDIFF",
                                vnds, domelt = TRUE)

      all_nbr <- Reduce(function(...) merge(..., all = TRUE),
                        list(nbrs, tps, fps, tns, fns, tcs, dss, nds))
    } else {
      all_nbr <- data.frame()
    }
  }

  ## ----------------------------- CORR --------------------------------- ##
  if ("corr" %in% aspects & !is.null(cont_truth)) {
    outPEARSON <- list()
    outSPEARMAN <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "corr", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = cont_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = NULL,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        PEARSON <- list()
        SPEARMAN <- list()
        isf <- intersect(which(is.finite(tmp[[i]])),
                         which(is.finite(truth[, cont_truth])))
        pearson <- stats::cor(tmp[[i]][isf], truth[, cont_truth][isf],
                              use = "complete.obs", method = "pearson")
        spearman <- stats::cor(tmp[[i]][isf], truth[, cont_truth][isf],
                               use = "complete.obs", method = "spearman")
        PEARSON[[paste0(i, "_overall", "__", inpcol)]] <-
          list(basemethod = i, meas_type = inpcol, pearson = pearson)
        SPEARMAN[[paste0(i, "_overall", "__", inpcol)]] <-
          list(basemethod = i, meas_type = inpcol, spearman = spearman)
        if (splv != "none") {
          for (j in kltmp) {
            g <- rownames(truth)[which(truth[[splv]] == j)]
            isf <- intersect(which(is.finite(tmp[match(g, rownames(tmp)), i])),
                             which(is.finite(truth[match(g, rownames(truth)),
                                                   cont_truth])))
            pearson <- stats::cor(tmp[match(g, rownames(tmp)), i][isf],
                                  truth[match(g, rownames(truth)),
                                        cont_truth][isf],
                                  use = "complete.obs", method = "pearson")
            spearman <- stats::cor(tmp[match(g, rownames(tmp)), i][isf],
                                   truth[match(g, rownames(truth)),
                                         cont_truth][isf],
                                   use = "complete.obs", method = "spearman")
            PEARSON[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
              list(basemethod = i, meas_type = inpcol, pearson = pearson)
            SPEARMAN[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
              list(basemethod = i, meas_type = inpcol, spearman = spearman)
          }
        }
        outPEARSON <- c(outPEARSON, PEARSON)
        outSPEARMAN <- c(outSPEARMAN, SPEARMAN)
      } else {
        message("column ", i, " is being ignored for correlation calculations")
      }
    }

    if (any(sapply(lapply(outPEARSON, function(w) w$pearson), length) > 0)) {
      pearsons <- t(do.call(rbind, lapply(outPEARSON, function(w) w$pearson)))
      spearmans <- t(do.call(rbind,
                             lapply(outSPEARMAN, function(w) w$spearman)))
      vp <- sapply(outPEARSON, function(w) w$basemethod)
      vs <- sapply(outSPEARMAN, function(w) w$basemethod)
      pearsons <- extend_resulttable(pearsons, splv, keeplevels, "PEARSON",
                                     vp, domelt = TRUE)
      spearmans <- extend_resulttable(spearmans, splv, keeplevels, "SPEARMAN",
                                      vs, domelt = TRUE)
      corrs <- merge(pearsons, spearmans)
    } else {
      corrs <- data.frame()
    }
  } else {
    corrs <- data.frame()
  }

  ## ----------------------------- TPR ---------------------------------- ##
  if (any(c("tpr", "fdrtpr", "fdrnbr") %in% aspects) & !is.null(binary_truth)) {
    outTPR <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "tpr", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = binary_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = binary_truth,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        if (inpcol == "padj") {
          TPR <- list()
          tpr <- gen_thr_vec(thrs)
          for (thr in thrs) {
            signif <- rownames(tmp)[which(tmp[i] <= thr)]
            tpr[paste0("thr", thr)] <-
              length(intersect(
                signif, rownames(truth)[which(truth[, binary_truth] == 1)]))/
              length(which(truth[, binary_truth] == 1))
          }
          TPR[[paste0(i, "_overall", "__", inpcol)]] <- list(basemethod = i,
                                                             meas_type = inpcol,
                                                             tpr = tpr)

          if (splv != "none") {
            for (j in kltmp) {
              tpr <- gen_thr_vec(thrs)
              for (thr in thrs) {
                g <- rownames(truth)[which(truth[[splv]] == j)]
                signif <- intersect(g, rownames(tmp)[which(tmp[i] <= thr)])
                gt <- intersect(
                  g, rownames(truth)[which(truth[, binary_truth] == 1)])
                gf <- intersect(g, signif)
                tpr[paste0("thr", thr)] <- length(intersect(gf, gt))/length(gt)
              }
              TPR[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
                list(basemethod = i, meas_type = inpcol, tpr = tpr)
            }
          }
          outTPR <- c(outTPR, TPR)
        }
      } else {
        message("column ", i, " is being ignored for TPR calculations")
      }
    }
    if (any(sapply(lapply(outTPR, function(w) w$tpr), length) > 0)) {
      tprs <- t(do.call(rbind, lapply(outTPR, function(w) w$tpr)))
      vt <- sapply(outTPR, function(w) w$basemethod)
      tprs <- extend_resulttable(tprs, splv, keeplevels, "TPR",
                                 vt, domelt = TRUE)
    } else {
      tprs <- data.frame()
    }
  } else {
    tprs <- data.frame()
  }

  ## ----------------------------- FDR ---------------------------------- ##
  if (any(c("fdrtpr", "fdrnbr") %in% aspects) & !is.null(binary_truth)) {
    outFDR <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "fdr", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = binary_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = binary_truth,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        if (inpcol == "padj") {
          FDR <- list()
          fdr <- gen_thr_vec(thrs)
          for (thr in thrs) {
            signif <- rownames(tmp)[which(tmp[i] <= thr)]
            if (length(signif) == 0) {
              fdr[paste0("thr", thr)] <- 0
            } else {
              fdr[paste0("thr", thr)] <-
                length(setdiff(
                  signif, rownames(truth)[which(truth[, binary_truth] == 1)])) /
                length(signif)
            }
          }
          FDR[[paste0(i, "_overall", "__", inpcol)]] <- list(basemethod = i,
                                                             meas_type = inpcol,
                                                             fdr = fdr)

          if (splv != "none" & any(truth[, binary_truth] == 0)) {
            for (j in kltmp) {
              fdr <- gen_thr_vec(thrs)
              for (thr in thrs) {
                g <- rownames(truth)[which(truth[[splv]] == j)]
                signif <- intersect(g, rownames(tmp)[which(tmp[i] <= thr)])
                gt <- intersect(
                  g, rownames(truth)[which(truth[, binary_truth] == 0)])
                gf <- intersect(g, signif)
                if (length(gf) == 0) {
                  fdr[paste0("thr", thr)] <- 0
                } else {
                  fdr[paste0("thr", thr)] <-
                    length(intersect(gf, gt))/length(gf)
                }
              }
              FDR[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
                list(basemethod = i, meas_type = inpcol, fdr = fdr)
            }
          }
          outFDR <- c(outFDR, FDR)
        }
      } else {
        message("column ", i, " is being ignored for FDR calculations")
      }
    }
    if (any(sapply(lapply(outFDR, function(w) w$fdr), length) > 0)) {
      fdrs <- t(do.call(rbind, lapply(outFDR, function(w) w$fdr)))
      vf <- sapply(outFDR, function(w) w$basemethod)

      fdrs <- extend_resulttable(fdrs, splv, keeplevels, "FDR",
                                 vf, domelt = TRUE)
      fdrs$satis <- ifelse(fdrs$FDR <= as.numeric(gsub("thr", "", fdrs$thr)),
                           "yes", "no")
      fdrs$method.satis <- paste0(fdrs$fullmethod, fdrs$satis)
    } else {
      fdrs <- data.frame()
    }
  } else {
    fdrs <- data.frame()
  }

  ## ----------------------------- FPR ---------------------------------- ##
  if ("fpr" %in% aspects & !is.null(binary_truth)) {
    outFPR <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "fpr", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = binary_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = binary_truth,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        if (inpcol == "padj") {
          FPR <- list()
          fpr <- gen_thr_vec(thrs)
          for (thr in thrs) {
            signif <- rownames(tmp)[which(tmp[i] <= thr)]
            fpr[paste0("thr", thr)] <-
              length(intersect(
                signif, rownames(truth)[which(truth[, binary_truth] == 0)])) /
              length(which(truth[, binary_truth] == 0))
          }
          FPR[[paste0(i, "_overall", "__", inpcol)]] <- list(basemethod = i,
                                                             meas_type = inpcol,
                                                             fpr = fpr)

          if (splv != "none" & any(truth[, binary_truth] == 0)) {
            for (j in kltmp) {
              fpr <- gen_thr_vec(thrs)
              for (thr in thrs) {
                g <- rownames(truth)[which(truth[[splv]] == j)]
                signif <- intersect(g, rownames(tmp)[which(tmp[i] <= thr)])
                gt <- intersect(
                  g, rownames(truth)[which(truth[, binary_truth] == 0)])
                gf <- intersect(g, signif)
                fpr[paste0("thr", thr)] <- length(intersect(gf, gt))/length(gt)
              }
              FPR[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
                list(basemethod = i, meas_type = inpcol, fpr = fpr)
            }
          }
          outFPR <- c(outFPR, FPR)
        }
      } else {
        message("column ", i, " is being ignored for FPR calculations")
      }
    }
    if (any(sapply(lapply(outFPR, function(w) w$fpr), length) > 0)) {
      fprs <- t(do.call(rbind, lapply(outFPR, function(w) w$fpr)))
      vff <- sapply(outFPR, function(w) w$basemethod)

      fprs <- extend_resulttable(fprs, splv, keeplevels, "FPR",
                                 vff, domelt = TRUE)
    } else {
      fprs <- data.frame()
    }
  } else {
    fprs <- data.frame()
  }

  ## ----------------------------- FSRNBR ------------------------------- ##
  if ("fsrnbr" %in% aspects && !is.null(cont_truth)) {
    outFSR <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "fsrnbr", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        svl <- slot(cobradata, inpcol)[i]
        scr <- slot(cobradata, "score")[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = svl, method = i,
                                 colm = cont_truth, onlyshared = onlyshared)
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                  drop = FALSE]
        
        svlv <- structure(svl[match(allg, rownames(svl)), ], names = allg)
        scrv <- structure(scr[match(allg, rownames(scr)), ], names = allg)
        trtv <- structure(truth[, cont_truth], names = rownames(truth))
        stopifnot(all(names(svlv) == names(scrv)))
        stopifnot(all(names(svlv) == names(trtv)))
        
        ## Make sure that genes without estimated logFCs are counted as having
        ## mismatching signs
        if (any(is.na(scrv))) {
          scrv[is.na(scrv)] <- -trtv[is.na(scrv)]
        }
        
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = NULL,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)
        
        FSR <- list()
        fsr <- gen_thr_vec(svalthrs)
        fs <- gen_thr_vec(svalthrs)
        ts <- gen_thr_vec(svalthrs)
        possign <- gen_thr_vec(svalthrs)
        negsign <- gen_thr_vec(svalthrs)
        zerosign <- gen_thr_vec(svalthrs)
        totcalled <- gen_thr_vec(svalthrs)
        nbr <- gen_thr_vec(svalthrs)
        for (thr in svalthrs) {
          passed <- which(svlv <= thr)
          tested <- which(!is.na(svlv))
          nbr[paste0("thr", thr)] <- length(passed)
          fs[paste0("thr", thr)] <- 
            length(which(abs(sign(trtv[passed]) - sign(scrv[passed])) == 2))
          ts[paste0("thr", thr)] <- 
            length(which(sign(trtv[passed]) == sign(scrv[passed])))
          possign[paste0("thr", thr)] <- length(which(sign(trtv) == 1))
          negsign[paste0("thr", thr)] <- length(which(sign(trtv) == -1))
          zerosign[paste0("thr", thr)] <- length(which(sign(trtv) == 0))
          totcalled[paste0("thr", thr)] <- length(tested)
          if (length(passed) == 0) {
            fsr[paste0("thr", thr)] <- 0
          } else {
            fsr[paste0("thr", thr)] <- fs[paste0("thr", thr)]/length(passed)
          }
        }
        FSR[[paste0(i, "_overall", "__", inpcol)]] <- 
          list(basemethod = i,
               meas_type = inpcol,
               fsr = fsr,
               nbr = nbr,
               fs = fs, 
               ts = ts,
               possign = possign,
               negsign = negsign,
               zerosign = zerosign,
               totcalled = totcalled)
        
        if (splv != "none") {
          for (j in kltmp) {
            fsr <- gen_thr_vec(svalthrs)
            fs <- gen_thr_vec(svalthrs)
            ts <- gen_thr_vec(svalthrs)
            possign <- gen_thr_vec(svalthrs)
            negsign <- gen_thr_vec(svalthrs)
            zerosign <- gen_thr_vec(svalthrs)
            totcalled <- gen_thr_vec(svalthrs)
            nbr <- gen_thr_vec(svalthrs)
            g <- rownames(truth)[which(truth[[splv]] == j)]
            allg2 <- intersect(allg, g)
            svlv2 <- svlv[match(allg2, names(svlv))]
            scrv2 <- scrv[match(allg2, names(scrv))]
            trtv2 <- trtv[match(allg2, names(scrv))]
            for (thr in svalthrs) {
              passed <- which(svlv2 <= thr)
              tested <- which(!is.na(svlv2))
              nbr[paste0("thr", thr)] <- length(passed)
              fs[paste0("thr", thr)] <- 
                length(which(abs(sign(trtv2[passed]) - 
                                   sign(scrv2[passed])) == 2))
              ts[paste0("thr", thr)] <- 
                length(which(sign(trtv2[passed]) == sign(scrv2[passed])))
              possign[paste0("thr", thr)] <- length(which(sign(trtv2) == 1))
              negsign[paste0("thr", thr)] <- length(which(sign(trtv2) == -1))
              zerosign[paste0("thr", thr)] <- length(which(sign(trtv2) == 0))
              totcalled[paste0("thr", thr)] <- length(tested)
              if (length(passed) == 0) {
                fsr[paste0("thr", thr)] <- 0
              } else {
                fsr[paste0("thr", thr)] <- fs[paste0("thr", thr)]/length(passed)
              }
            }
            FSR[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
              list(basemethod = i, meas_type = inpcol, fsr = fsr, nbr = nbr,
                   fs = fs, ts = ts, possign = possign, negsign = negsign,
                   zerosign = zerosign, totcalled = totcalled)
          }
        }
        outFSR <- c(outFSR, FSR)
      } else {
        message("column ", i, " is being ignored for FSR/NBR calculations")
      }
    }
    if (any(sapply(lapply(outFSR, function(w) w$fsr), length) > 0)) {
      fsrs <- t(do.call(rbind, lapply(outFSR, function(w) w$fsr)))
      nbrs <- t(do.call(rbind, lapply(outFSR, function(w) w$nbr)))
      tss <- t(do.call(rbind, lapply(outFSR, function(w) w$ts)))
      fss <- t(do.call(rbind, lapply(outFSR, function(w) w$fs)))
      possigns <- t(do.call(rbind, lapply(outFSR, function(w) w$possign)))
      negsigns <- t(do.call(rbind, lapply(outFSR, function(w) w$negsign)))
      zerosigns <- t(do.call(rbind, lapply(outFSR, function(w) w$zerosign)))
      totcalleds <- t(do.call(rbind, lapply(outFSR, function(w) w$totcalled)))
      vf <- sapply(outFSR, function(w) w$basemethod)
      
      fsrs <- extend_resulttable(fsrs, splv, keeplevels, "FSR",
                                 vf, domelt = TRUE)
      fsrs$satis <- ifelse(fsrs$FSR <= as.numeric(gsub("thr", "", fsrs$thr)),
                           "yes", "no")
      fsrs$method.satis <- paste0(fsrs$fullmethod, fsrs$satis)
      nbrs <- extend_resulttable(nbrs, splv, keeplevels, "NBR",
                                 vf, domelt = TRUE)
      tss <- extend_resulttable(tss, splv, keeplevels, "TS", vf, domelt = TRUE)
      fss <- extend_resulttable(fss, splv, keeplevels, "FS", vf, domelt = TRUE)
      possigns <- extend_resulttable(possigns, splv, keeplevels, "POSSIGN", 
                                     vf, domelt = TRUE)
      negsigns <- extend_resulttable(negsigns, splv, keeplevels, "NEGSIGN", 
                                     vf, domelt = TRUE)
      zerosigns <- extend_resulttable(zerosigns, splv, keeplevels, "ZEROSIGN", 
                                      vf, domelt = TRUE)
      totcalleds <- extend_resulttable(totcalleds, splv, keeplevels, 
                                       "TOT_CALLED", vf, domelt = TRUE)
      
      fsrs <- Reduce(function(...) merge(..., all = TRUE), 
                     list(fsrs = fsrs, nbrs = nbrs, tss = tss, fss = fss, 
                          possigns = possigns, negsigns = negsigns,
                          zerosigns = zerosigns, totcalleds = totcalleds))
    } else {
      fsrs <- data.frame()
    }
  } else {
    fsrs <- data.frame()
  }
  
  ## ------------------------- FSRNBRcurve ------------------------------ ##
  if ("fsrnbrcurve" %in% aspects & !is.null(cont_truth)) {
    outFSRNBR <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "fsrnbr", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        scr <- slot(cobradata, "score")[i]
        allg <- get_keepfeatures(truth = truth(cobradata), 
                                 df = tmp, method = i, 
                                 colm = cont_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), , 
                                  drop = FALSE]
        scr <- scr[match(allg, rownames(scr)), , drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv, 
                                binary_truth = NULL, maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)
        
        FSRNBR <- list()
        conttruth <- truth[, cont_truth]
        names(conttruth) <- rownames(truth)
        vals <- tmp[[i]]
        names(vals) <- rownames(tmp)
        estvals <- scr[, i]
        names(estvals) <- rownames(scr)
        fsrnbr <- get_curve_cont(conttruth = conttruth, estvals = estvals,
                                 svals = vals, aspc = "fsrnbr")
        FSRNBR[[paste0(i, "_overall", "__", inpcol)]] <- 
          list(basemethod = i, meas_type = inpcol, fsrnbr = fsrnbr)
        if (splv != "none") {
          for (j in kltmp) {
            conttruth <- truth[which(truth[[splv]] == j), cont_truth]
            names(conttruth) <- rownames(truth)[which(truth[[splv]] == j)]
            fsrnbr <- get_curve_cont(conttruth = conttruth, estvals = estvals,
                                     svals = vals, aspc = "fsrnbr")
            if (!is.null(fsrnbr)) {
              FSRNBR[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
                list(basemethod = i, meas_type = inpcol, fsrnbr = fsrnbr)
            }
          }
        }
        outFSRNBR <- c(outFSRNBR, FSRNBR)
      } else {
        message("column ", i, " is being ignored for FSRNBR calculations")
      }
    }
    if (any(sapply(lapply(outFSRNBR, function(w) w$fsrnbr),
                   length) > 0)) {
      vfsn <- sapply(outFSRNBR, function(w) w$basemethod)
      fsrnbrs <- do.call(rbind, lapply(names(outFSRNBR), function(s) {
        data.frame(FSR = outFSRNBR[[s]]$fsrnbr[, "FSR"],
                   NBR = outFSRNBR[[s]]$fsrnbr[, "NBR"],
                   CUTOFF = outFSRNBR[[s]]$fsrnbr[, "SVAL_CUTOFF"],
                   TS = outFSRNBR[[s]]$fsrnbr[, "TS"],
                   FS = outFSRNBR[[s]]$fsrnbr[, "FS"],
                   TOT_CALLED = outFSRNBR[[s]]$fsrnbr[, "TOT_CALLED"],
                   POSSIGN = outFSRNBR[[s]]$fsrnbr[, "POSSIGN"],
                   NEGSIGN = outFSRNBR[[s]]$fsrnbr[, "NEGSIGN"],
                   ZEROSIGN = outFSRNBR[[s]]$fsrnbr[, "ZEROSIGN"],
                   method = s)
      }))
      fsrnbrs <- extend_resulttable(fsrnbrs, splv, keeplevels, NULL,
                                    vfsn, domelt = FALSE)
    } else {
      fsrnbrs <- data.frame()
    }
  } else {
    fsrnbrs <- data.frame()
  }
  
  ## ------------------------- FDRTPRcurve ------------------------------ ##
  if (any(c("fdrtprcurve", "fdrnbrcurve") %in% aspects) &
      !is.null(binary_truth)) {
    outFDRTPR <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "fdrtpr", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = binary_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = binary_truth,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        if (!is.null(inpcol)) {
          if (any(truth[, binary_truth] == 0) &
              any(truth[, binary_truth] == 1)) {
            FDRTPR <- list()
            bintruth <- truth[, binary_truth]
            names(bintruth) <- rownames(truth)
            vals <- tmp[[i]]
            names(vals) <- rownames(tmp)
            fdrtpr <- get_curve(bintruth = bintruth, vals = vals,
                                revr = ifelse(inpcol == "score", FALSE, TRUE),
                                aspc = "fdrtpr", rank_by_abs = rank_by_abs)
            FDRTPR[[paste0(i, "_overall", "__", inpcol)]] <-
              list(basemethod = i, meas_type = inpcol, fdrtpr = fdrtpr)
            if (splv != "none") {
              for (j in kltmp) {
                bintruth <- truth[which(truth[[splv]] == j), binary_truth]
                names(bintruth) <- rownames(truth)[which(truth[[splv]] ==  j)]
                fdrtpr <- get_curve(bintruth = bintruth, vals = vals,
                                    revr = ifelse(inpcol == "score",
                                                  FALSE, TRUE),
                                    aspc = "fdrtpr", rank_by_abs = rank_by_abs)
                if (!is.null(fdrtpr)) {
                  FDRTPR[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
                    list(basemethod = i, meas_type = inpcol, fdrtpr = fdrtpr)
                }
              }
            }
          } else {
            FDRTPR <- NULL
          }
          outFDRTPR <- c(outFDRTPR, FDRTPR)
        }
      } else {
        message("column ", i, " is being ignored for FDRTPR calculations")
      }
    }
    if (any(sapply(lapply(outFDRTPR, function(w) w$fdrtpr),
                   length) > 0)) {
      vftp <- sapply(outFDRTPR, function(w) w$basemethod)
      fdrtprs <- do.call(rbind, lapply(names(outFDRTPR), function(s) {
        data.frame(FDR = outFDRTPR[[s]]$fdrtpr[, "FDR"],
                   TPR = outFDRTPR[[s]]$fdrtpr[, "TPR"],
                   NBR = outFDRTPR[[s]]$fdrtpr[, "NBR"],
                   CUTOFF = outFDRTPR[[s]]$fdrtpr[, "CUTOFF"],
                   TP = outFDRTPR[[s]]$fdrtpr[, "TP"],
                   FP = outFDRTPR[[s]]$fdrtpr[, "FP"],
                   TN = outFDRTPR[[s]]$fdrtpr[, "TN"],
                   FN = outFDRTPR[[s]]$fdrtpr[, "FN"],
                   TOT_CALLED = outFDRTPR[[s]]$fdrtpr[, "TOT_CALLED"],
                   DIFF = outFDRTPR[[s]]$fdrtpr[, "DIFF"],
                   NONDIFF = outFDRTPR[[s]]$fdrtpr[, "NONDIFF"],
                   method = s)
      }))
      fdrtprs <- extend_resulttable(fdrtprs, splv, keeplevels, NULL,
                                    vftp, domelt = FALSE)
    } else {
      fdrtprs <- data.frame()
    }
  } else {
    fdrtprs <- data.frame()
  }
  fdrnbrs <- fdrtprs

  ## ----------------------------- ROC ---------------------------------- ##
  if ("roc" %in% aspects & !is.null(binary_truth)) {
    outROC <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "roc", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = binary_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = binary_truth,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        if (any(truth[, binary_truth] == 0) &
            any(truth[, binary_truth] == 1)) {
          ROC <- list()
          bintruth <- truth[, binary_truth]
          names(bintruth) <- rownames(truth)
          vals <- tmp[[i]]
          names(vals) <- rownames(tmp)
          roc <- get_curve(bintruth = bintruth, vals = vals,
                           revr = ifelse(inpcol == "score", FALSE, TRUE),
                           aspc = "roc", rank_by_abs = rank_by_abs)
          ROC[[paste0(i, "_overall", "__", inpcol)]] <-
            list(basemethod = i, meas_type = inpcol, roc = roc)
          if (splv != "none") {
            for (j in kltmp) {
              bintruth <- truth[which(truth[[splv]] == j), binary_truth]
              names(bintruth) <- rownames(truth)[which(truth[[splv]] == j)]
              roc <- get_curve(bintruth = bintruth, vals = vals,
                               revr = ifelse(inpcol == "score", FALSE, TRUE),
                               aspc = "roc", rank_by_abs = rank_by_abs)
              if (!is.null(roc)) {
                ROC[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
                  list(basemethod = i, meas_type = inpcol, roc = roc)
              }
            }
          }
        }
        outROC <- c(outROC, ROC)
      } else {
        message("column ", i, " is being ignored for ROC calculations")
      }
    }

    if (any(sapply(lapply(outROC, function(w) w$roc), length) > 0)) {
      vr <- sapply(outROC, function(w) w$basemethod)
      rocs <- do.call(rbind, lapply(names(outROC), function(s) {
        data.frame(FPR = outROC[[s]]$roc[, "FPR"],
                   TPR = outROC[[s]]$roc[, "TPR"],
                   ROC_CUTOFF = outROC[[s]]$roc[, "ROC_CUTOFF"],
                   method = s)
      }))
      rocs <- extend_resulttable(rocs, splv, keeplevels, NULL,
                                 vr, domelt = FALSE)
    } else {
      rocs <- data.frame()
    }
  } else {
    rocs <- data.frame()
  }

  ## ---------------------------- SCATTER -------------------------------- ##
  if ("scatter" %in% aspects & !is.null(cont_truth)) {
    outSCATTER <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "scatter", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = cont_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = NULL,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        SCATTER <- list()
        SCATTER[[paste0(i, "_overall", "__", inpcol)]] <-
          list(basemethod = i, meas_type = inpcol,
               scatter = data.frame(vals = tmp[[i]],
                                    truth = truth[, cont_truth],
                                    row.names = rownames(tmp),
                                    stringsAsFactors = FALSE))
        if (splv != "none") {
          for (j in kltmp) {
            s <- which(truth[[splv]] == j)
            SCATTER[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
              list(basemethod = i, meas_type = inpcol,
                   scatter = data.frame(vals = tmp[[i]][s],
                                        truth = truth[s, cont_truth],
                                        row.names = rownames(tmp)[s],
                                        stringsAsFactors = FALSE))
          }
        }
      } else {
        SCATTER <- NULL
      }
      outSCATTER <- c(outSCATTER, SCATTER)
    }

    if (any(sapply(lapply(outSCATTER, function(w) w$scatter), length) > 0)) {
      vsc <- sapply(outSCATTER, function(w) w$basemethod)
      scatters <- do.call(rbind, lapply(names(outSCATTER), function(s) {
        data.frame(OBSERVATION = outSCATTER[[s]]$scatter[, "vals"],
                   TRUTH = outSCATTER[[s]]$scatter[, "truth"],
                   feature = rownames(outSCATTER[[s]]$scatter),
                   method = s)
      }))
      scatters <- extend_resulttable(scatters, splv, keeplevels, NULL,
                                     vsc, domelt = FALSE)
    } else {
      scatters <- data.frame()
    }
  } else {
    scatters <- data.frame()
  }

  ## --------------------------- DEVIATION ------------------------------- ##
  if ("deviation" %in% aspects & !is.null(cont_truth)) {
    outDEVIATION <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "deviation", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = cont_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = NULL,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        DEVIATION <- list()
        DEVIATION[[paste0(i, "_overall", "__", inpcol)]] <-
          list(basemethod = i, meas_type = inpcol,
               deviation = data.frame(vals = tmp[[i]] - truth[, cont_truth],
                                      row.names = rownames(tmp),
                                      stringsAsFactors = FALSE))
        if (splv != "none") {
          for (j in kltmp) {
            s <- which(truth[[splv]] == j)
            DEVIATION[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
              list(basemethod = i, meas_type = inpcol,
                   deviation = data.frame(vals = tmp[[i]][s] -
                                            truth[s, cont_truth],
                                          row.names = rownames(tmp)[s],
                                          stringsAsFactors = FALSE))
          }
        }
      } else {
        DEVIATION <- NULL
      }
      outDEVIATION <- c(outDEVIATION, DEVIATION)
    }

    if (any(sapply(lapply(outDEVIATION,
                          function(w) w$deviation), length) > 0)) {
      vdv <- sapply(outDEVIATION, function(w) w$basemethod)
      deviations <- do.call(rbind, lapply(names(outDEVIATION), function(s) {
        data.frame(DEVIATION = outDEVIATION[[s]]$deviation[, "vals"],
                   feature = rownames(outDEVIATION[[s]]$deviation),
                   method = s)
      }))
      deviations <- extend_resulttable(deviations, splv, keeplevels, NULL,
                                       vdv, domelt = FALSE)
    } else {
      deviations <- data.frame()
    }
  } else {
    deviations <- data.frame()
  }

  ## ----------------------------- FPC ---------------------------------- ##
  if ("fpc" %in% aspects & !is.null(binary_truth)) {
    outFPC <- list()
    keeplevels <- c()
    for (i in all_methods) {
      inpcol <- select_measure(cobradata, i, asp = "fpc", 
                               prefer_pval = prefer_pval)
      if (!is.null(inpcol)) {
        tmp <- slot(cobradata, inpcol)[i]
        allg <- get_keepfeatures(truth = truth(cobradata),
                                 df = tmp,  method = i,
                                 colm = binary_truth, onlyshared = onlyshared)
        tmp <- tmp[match(allg, rownames(tmp)), , drop = FALSE]
        truth <- truth(cobradata)[match(allg, rownames(truth(cobradata))), ,
                                 drop = FALSE]
        kltmp <- get_keeplevels(truth = truth, splv = splv,
                                binary_truth = binary_truth,
                                maxsplit = maxsplit)
        keeplevels <- union(keeplevels, kltmp)

        if (any(truth[, binary_truth] == 0) &
            any(truth[, binary_truth] == 1)) {
          FPC <- list()
          bintruth <- truth[, binary_truth]
          names(bintruth) <- rownames(truth)
          vals <- tmp[[i]]
          names(vals) <- rownames(tmp)
          fpc <- get_curve(bintruth = bintruth, vals = vals,
                           revr = ifelse(inpcol == "score", FALSE, TRUE),
                           aspc = "fpc", rank_by_abs = rank_by_abs)
          FPC[[paste0(i, "_overall", "__", inpcol)]] <-
            list(basemethod = i, meas_type = inpcol, fpc = fpc)
          if (splv != "none") {
            for (j in kltmp) {
              bintruth <- truth[which(truth[[splv]] == j), binary_truth]
              names(bintruth) <- rownames(truth)[which(truth[[splv]] == j)]
              fpc <- get_curve(bintruth = bintruth, vals = vals,
                               revr = ifelse(inpcol == "score", FALSE, TRUE),
                               aspc = "fpc", rank_by_abs = rank_by_abs)
              if (!is.null(fpc)) {
                FPC[[paste0(i, "_", splv, ":", j, "__", inpcol)]] <-
                  list(basemethod = i, meas_type = inpcol, fpc = fpc)
              }
            }
          }
        }
        outFPC <- c(outFPC, FPC)
      } else {
        message("column ", i, " is being ignored for FPC calculations")
      }
    }

    if (any(sapply(lapply(outFPC, function(w) w$fpc), length) > 0)) {
      vfc <- sapply(outFPC, function(w) w$basemethod)
      fpcs <- do.call(rbind, lapply(names(outFPC), function(s) {
        data.frame(topN = outFPC[[s]]$fpc[, "topN"],
                   FP = outFPC[[s]]$fpc[, "FP"],
                   FPC_CUTOFF = outFPC[[s]]$fpc[, "FPC_CUTOFF"],
                   method = s)
      }))
      fpcs <- extend_resulttable(fpcs, splv, keeplevels, NULL,
                                 vfc, domelt = FALSE)
    } else {
      fpcs <- data.frame()
    }
  } else {
    fpcs <- data.frame()
  }

  ## --------------------------- OVERLAP -------------------------------- ##
  if ("overlap" %in% aspects) {
    tmplist <- padj(cobradata)
    if (length(tmplist) > 0) {
      if (!is.null(binary_truth)) {
        ## Add 'truth' to list of results, if type_venn is not 'rank' since in
        ## that case, the 'true' genes can be ranked in any order
        if (type_venn == "rank") {
          message(paste0("Note that including truth with type_venn = 'rank' ", 
                         "is not supported, since true features may be ", 
                         "ranked in arbitrary order"))
        } else {
          tmp2 <- 1 - truth(cobradata)[, binary_truth, drop = FALSE]
          colnames(tmp2) <- "truth"
          missing_genes <- setdiff(rownames(tmp2), rownames(tmplist))
          tmpadd <- as.data.frame(matrix(NA, length(missing_genes), 
                                         ncol(tmplist),
                                         dimnames = list(missing_genes,
                                                         colnames(tmplist))))
          tmplist <- rbind(tmplist, tmpadd)
          tmplist$truth <- tmp2$truth[match(rownames(tmplist), rownames(tmp2))]
        }
      }

      ## Create matrix indicating "significant" features
      if (type_venn == "adjp") {
        overlap <- apply(tmplist, 2, function(w) {
          as.numeric(w <= thr_venn)
        })
      } else if (type_venn == "rank") {
        overlap <- apply(tmplist, 2, function(w) {
          as.numeric(w <= sort(w, decreasing = FALSE)[topn_venn])
        })
        if ("truth" %in% colnames(overlap)) {
          overlap[, "truth"] <- as.numeric(tmplist[, "truth"] == 0)
        }
      }
      common_genes <- rownames(tmplist)
      rownames(overlap) <- common_genes
      overlap <- overlap[, which(colSums(is.na(overlap)) != nrow(overlap))]

      ## Assume that if we don't have information about a gene, it is "negative"
      ##overlap[is.na(overlap)] <- 0
      overlap <- data.frame(overlap, stringsAsFactors = FALSE)

      ## If splv is not "none", split overlap matrix
      if (splv != "none") {
        overlap_overall <- overlap
        keep <- intersect(rownames(overlap), rownames(truth(cobradata)))
        tth <- truth(cobradata)[match(keep, rownames(truth(cobradata))), ]
        keeplevels <- get_keeplevels(truth = tth, splv = splv,
                                     binary_truth = NULL, maxsplit = maxsplit)
        overlap <- overlap[match(keep, rownames(overlap)), ]
        overlap <- split(overlap, tth[, splv])
        ## Keep only the top "maxsplit" categories
        overlap <- overlap[keeplevels]
        ## Add the overall category
        overlap$overall <- overlap_overall
      }
    } else {
      overlap <- data.frame()
    }
  } else {
    overlap <- data.frame()
  }

  ## --------------------------- RETURNS -------------------------------- ##
  ## Put data together
  if ("fdrtpr" %in% aspects & !is.null(binary_truth)) {
    fdrtpr <- Reduce(function(...) merge(..., all = TRUE),
                     list(all_nbr, tprs, fdrs))
  } else {
    fdrtpr <- data.frame()
  }
  if ("fdrnbr" %in% aspects & !is.null(binary_truth)) {
    fdrnbr <- Reduce(function(...) merge(..., all = TRUE),
                     list(all_nbr, tprs, fdrs))
  } else {
    fdrnbr <- data.frame()
  }
  if ("tpr" %in% aspects & !is.null(binary_truth)) {
    tprs <- Reduce(function(...) merge(..., all = TRUE),
                   list(all_nbr, tprs))
  }
  if ("fpr" %in% aspects & !is.null(binary_truth)) {
    fprs <- Reduce(function(...) merge(..., all = TRUE),
                   list(all_nbr, fprs))
  }

  if (!("tpr" %in% aspects)) tprs <- data.frame()
  if (!("fdrtprcurve" %in% aspects)) fdrtprcurve <- data.frame()
  if (!("fdrnbrcurve" %in% aspects)) fdrnbrcurve <- data.frame()
  if (!("fsrnbrcurve" %in% aspects)) fsrnbrcurve <- data.frame()
  
  COBRAPerformance(tpr = tprs, fpr = fprs, fdrtprcurve = fdrtprs,
                   fdrnbrcurve = fdrnbrs, deviation = deviations,
                   roc = rocs, fpc = fpcs, fdrtpr = fdrtpr, fdrnbr = fdrnbr,
                   maxsplit = maxsplit, overlap = overlap, splv = splv,
                   corr = corrs, scatter = scatters, onlyshared = onlyshared,
                   fsrnbrcurve = fsrnbrs, fsrnbr = fsrs)

}

#' Prepare data for plotting
#'
#' Prepare performance data provided in a \code{COBRAPerformance} object
#' (obtained by \code{\link{calculate_performance}}) for plotting.
#'
#' @param cobraperf A \code{COBRAPerformance} object.
#' @param keepmethods A character vector consisting of methods to retain for
#'   plotting (these should be a subset of \code{basemethods(cobraperf)}), or
#'   NULL (indicating that all methods represented in cobraperf should be
#'   retained).
#' @param incloverall A logical indicating whether the "overall" results should
#'   be included if the results are stratified by an annotation.
#' @param colorscheme Either a character string giving the color palette to use
#'   to define colors for the different methods, or a character vector with
#'   colors to use. The available pre-defined palettes depend on the number of
#'   different methods to distinguish. The choices are:
#'   \describe{
#'   \item{- \code{Accent}}{(max 8 methods)}
#'   \item{- \code{Dark2}}{(max 8 methods)}
#'   \item{- \code{Paired}}{(max 12 methods)}
#'   \item{- \code{Pastel1}}{(max 9 methods)}
#'   \item{- \code{Pastel2}}{(max 8 methods)}
#'   \item{- \code{Set1}}{(max 9 methods)}
#'   \item{- \code{Set2}}{(max 8 methods)}
#'   \item{- \code{Set3}}{(max 12 methods)}
#'   \item{- \code{hue_pal}}{}
#'   \item{- \code{rainbow}}{}
#'   \item{- \code{heat}}{}
#'   \item{- \code{terrain}}{}
#'   \item{- \code{topo}}{}
#'   \item{- \code{cm}}{}
#'   }
#'   If the number of allowed methods is exceeded, the colorscheme defaults to
#'   \code{hue_pal}.
#' @param facetted A logical indicating whether the results should be split into
#'   subpanels when stratified by an annotation (\code{TRUE}), or kept in the
#'   same panel but shown with different colors (\code{FALSE}).
#' @param incltruth A logical indicating whether the truth should be included in
#'   Venn diagrams.
#' @param conditionalfill A logical indicating whether the points (in FDR/TPR,
#'   FDR/NBR, FSR/NBR plots) should be filled conditional on whether they
#'   satisfy the imposed criterion (e.g., false discovery rate control at
#'   imposed threshold).
#'
#' @return A \code{COBRAPlot} object
#'
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    cont_truth = "none",
#'                                    aspects = c("fdrtpr", "fdrtprcurve",
#'                                                "tpr", "roc"),
#'                                    thrs = c(0.01, 0.05, 0.1), splv = "none")
#' cobraplot <- prepare_data_for_plot(cobraperf, keepmethods = NULL,
#'                                    colorscheme = "Dark2")
#'
#' ## User-specified colors
#' cobraplot2 <- prepare_data_for_plot(cobraperf, keepmethods = NULL,
#'                                    colorscheme = c("blue", "red", "green"))
prepare_data_for_plot <- function(cobraperf, keepmethods = NULL,
                                  incloverall = TRUE, colorscheme = "hue_pal",
                                  facetted = TRUE, incltruth = TRUE,
                                  conditionalfill = TRUE) {
  splitval <- NULL
  if (is.null(keepmethods)) {
    keepmethods <- basemethods(cobraperf)
  }

  if (length(intersect(basemethods(cobraperf), keepmethods)) == 0)
    return(COBRAPlot())

  ## Subset cobraperf object
  cobraperf <- cobraperf[, keepmethods]

  ## Exclude truth from overlap matrix
  ## If truth not included, keep all features and set NAs to 0
  if (!isTRUE(incltruth)) {
    if (length(overlap(cobraperf)) != 0) {
      if (is(overlap(cobraperf), "data.frame")) {
        overlap(cobraperf) <-
          overlap(cobraperf)[, setdiff(colnames(overlap(cobraperf)), "truth"),
                            drop = FALSE]
        overlap(cobraperf)[is.na(overlap(cobraperf))] <- 0
      } else {
        overlap(cobraperf) <- lapply(overlap(cobraperf), function(w) {
          w <- w[, setdiff(colnames(w), "truth"), drop = FALSE]
          w[is.na(w)] <- 0
          w
        })
      }
    }
  } else {
    ## If truth is included, what to do depends on onlyshared.
    ## If onlyshared = FALSE, remove all features where truth!=NA
    ## If onlyshared = TRUE, remove all features with any NA
    if (length(overlap(cobraperf)) != 0) {
      if (is(overlap(cobraperf), "data.frame")) {
        if ("truth" %in% colnames(overlap(cobraperf))) {
          if (isTRUE(onlyshared(cobraperf))) {
            overlap(cobraperf) <-
              overlap(cobraperf)[which(rowSums(is.na(overlap(cobraperf))) == 0),
                                , drop = FALSE]
          } else {
            overlap(cobraperf) <-
              overlap(cobraperf)[which(!is.na(overlap(cobraperf)$truth)),
                                , drop = FALSE]
          }
        }
        overlap(cobraperf)[is.na(overlap(cobraperf))] <- 0
      } else {
        overlap(cobraperf) <- lapply(overlap(cobraperf), function(w) {
          if ("truth" %in% colnames(w)) {
            if (isTRUE(onlyshared(cobraperf))) {
              w <- w[which(rowSums(is.na(w)) == 0), , drop = FALSE]
            } else {
              w <- w[which(!is.na(w$truth)), , drop = FALSE]
            }
          } else
            w <- w
          w[is.na(w)] <- 0
          w
        })
      }
    }
  }

  ## Define colors
  use_colors <- define_colors(cobraperf = cobraperf, palette = colorscheme,
                              facetted = facetted, incloverall = incloverall, 
                              conditionalfill = conditionalfill)

  ## Exclude overall level
  for (sl in c("tpr", "fpr", "corr", "roc", "fpc", "scatter", "deviation",
               "fdrtprcurve", "fdrtpr", "fdrnbrcurve", "fdrnbr", 
               "fsrnbrcurve")) {
    if (splv(cobraperf) != "none") {
      if (!(isTRUE(incloverall))) {
        if (length(slot(cobraperf, sl)) != 0)
          slot(cobraperf, sl) <- subset(slot(cobraperf, sl),
                                       splitval != "overall")
      }
    }
    if (length(slot(cobraperf, sl)) != 0) {
      if (isTRUE(facetted))
        slot(cobraperf, sl)$num_method <-
          as.numeric(as.factor(slot(cobraperf, sl)$method))
      else
        slot(cobraperf, sl)$num_method <-
          as.numeric(as.factor(slot(cobraperf, sl)$fullmethod))
    }
  }

  if (splv(cobraperf) != "none") {
    if (!(isTRUE(incloverall))) {
      if (length(overlap(cobraperf)) != 0)
        overlap(cobraperf)$overall <- NULL
    }
  }

  cobraperf <- as(cobraperf, "COBRAPlot")
  facetted(cobraperf) <- facetted
  plotcolors(cobraperf) <- use_colors
  cobraperf
}

#' Reorder levels in COBRAPlot object
#'
#' Reorder levels in COBRAPlot object to achieve desired ordering in figure
#' legends etc. If facetted(cobraplot) is TRUE, the releveling will be applied
#' to the "method" column. If facetted(cobraplot) is FALSE, it will be applied
#' to the "fullmethod" column.
#'
#' @param cobraplot A COBRAPlot object
#' @param levels A character vector giving the order of the levels. Any values
#'   not present in the COBRAPlot object will be removed. Any methods present in
#'   the COBRAPlot object but not contained in this vector will be added at the
#'   end.
#'
#' @return A COBRAPlot object
#' @author Charlotte Soneson
#'
#' @examples
#' data(cobradata_example_sval)
#' cobraperf <- calculate_performance(cobradata_example_sval,
#'                                    binary_truth = "status", aspects = "fpr")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' cobraplot <- reorder_levels(cobraplot, c("Method2", "Method1"))
#' 
#' @export
reorder_levels <- function(cobraplot, levels) {
  if (isTRUE(facetted(cobraplot))) column <- "method"
  else column <- "fullmethod"

  for (sl in c("tpr", "fpr", "corr", "roc", "fpc", "scatter", "deviation",
               "fdrtprcurve", "fdrtpr", "fdrnbrcurve", "fdrnbr",
               "fsrnbrcurve", "fsrnbr")) {
    if (length(slot(cobraplot, sl)) != 0) {
      levels_to_keep <- levels[which(levels %in% slot(cobraplot, sl)[, column])]
      if (length(setdiff(unique(slot(cobraplot, sl)[, column]),
                         levels_to_keep)) != 0) {
        message("The following methods have been added to the ", sl, " slot: ",
                paste0(setdiff(unique(as.character(slot(cobraplot,
                                                        sl)[, column])),
                               levels_to_keep), collapse = ", "))
        levels_to_keep <-
          c(levels_to_keep,
            sort(setdiff(unique(as.character(slot(cobraplot, sl)[, column])),
                         levels_to_keep)))
      }
      slot(cobraplot, sl)[, column] <-
        factor(as.character(slot(cobraplot, sl)[, column]),
               levels = levels_to_keep)

      slot(cobraplot, sl)$num_method <-
        as.numeric(slot(cobraplot, sl)[, column])
    }
  }
  cobraplot
}

## Test that calculate_performance is doing the right thing.

library(iCOBRA)
context("Check calculation of performance scores")

local({
  ## Remove some genes from the truth
  cobradata <- cobradata_example_sval
  truth(cobradata) <- truth(cobradata)[1:3000, , drop = FALSE]

  set.seed(123)
  ## Set some adjusted p-values to NA
  tmp <- padj(cobradata)$Method1
  tmp[sample(1:length(tmp), 100)] <- NA
  padj(cobradata)$Method1 <- tmp
  
  ## Set some s-values to NA
  tmp <- sval(cobradata)$Method1
  tmp[sample(1:length(tmp), 100)] <- NA
  sval(cobradata)$Method1 <- tmp

  ## Set some scores to NA
  tmp <- iCOBRA::score(cobradata)$Method1
  tmp[sample(1:length(tmp), 100)] <- NA
  iCOBRA::score(cobradata)$Method1 <- tmp
  
  ## Change order of rows in padj
  tmp <- iCOBRA::padj(cobradata)
  tmp <- tmp[sample(seq_len(nrow(tmp)), size = nrow(tmp)), ]
  iCOBRA::padj(cobradata) <- tmp
  
  ## Add a truth column represented with logical values instead of 0/1
  truth(cobradata)$statuslogical <- as.logical(truth(cobradata)$status)
  
  ## No stratification, onlyshared = FALSE
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr",
                                           "fdrnbrcurve", "tpr", "fpr",
                                           "roc", "fpc", "overlap", "corr",
                                           "scatter", "deviation", 
                                           "fsrnbr", "fsrnbrcurve"),
                               thrs = c(0.05), splv = "none",
                               onlyshared = FALSE, thr_venn = 0.05, 
                               rank_by_abs = TRUE)
  
  ## The same, but with the logical truth
  ib1logical <- calculate_performance(cobradata, binary_truth = "statuslogical",
                                      cont_truth = "logFC",
                                      aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr",
                                                  "fdrnbrcurve", "tpr", "fpr",
                                                  "roc", "fpc", "overlap", "corr",
                                                  "scatter", "deviation", 
                                                  "fsrnbr", "fsrnbrcurve"),
                                      thrs = c(0.05), splv = "none",
                                      onlyshared = FALSE, thr_venn = 0.05, 
                                      rank_by_abs = TRUE)

  ## Stratification, onlyshared = FALSE
  ib2 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr",
                                           "fdrnbrcurve", "tpr", "fpr",
                                           "roc", "fpc", "overlap", "corr",
                                           "scatter", "deviation", 
                                           "fsrnbr", "fsrnbrcurve"),
                               thrs = c(0.05), splv = "expr_cat",
                               onlyshared = FALSE, thr_venn = 0.05,
                               rank_by_abs = TRUE)

  ## No stratification, onlyshared = TRUE
  ib3 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr",
                                           "fdrnbrcurve", "tpr", "fpr",
                                           "roc", "fpc", "overlap", "corr",
                                           "scatter", "deviation", 
                                           "fsrnbr", "fsrnbrcurve"),
                               thrs = c(0.05), splv = "none", onlyshared = TRUE,
                               thr_venn = 0.05,
                               rank_by_abs = TRUE)

  pval <- pval(cobradata)
  padj <- padj(cobradata)
  sval <- sval(cobradata)
  score <- score(cobradata)
  truth <- truth(cobradata)

  pvalsub <- pval[rownames(subset(truth, expr_cat == "[   2.628,  17.148)")), ]
  padjsub <- padj[rownames(subset(truth, expr_cat == "[   2.628,  17.148)")), ]
  svalsub <- sval[rownames(subset(truth, expr_cat == "[   2.628,  17.148)")), ]
  scoresub <- score[rownames(subset(truth, expr_cat == "[   2.628,  17.148)")), ]
  truthsub <- truth[rownames(subset(truth, expr_cat == "[   2.628,  17.148)")), ]

  tp1 <- length(intersect(rownames(subset(padj, Method1 <= 0.05)),
                          rownames(subset(truth, status == 1))))
  fp1 <- length(intersect(rownames(subset(padj, Method1 <= 0.05)),
                          rownames(subset(truth, status == 0))))
  fn1 <- length(intersect(rownames(subset(padj, Method1 > 0.05)),
                          rownames(subset(truth, status == 1))))
  tn1 <- length(intersect(rownames(subset(padj, Method1 > 0.05)),
                          rownames(subset(truth, status == 0))))
  cc1 <- length(setdiff(rownames(subset(truth, status == 1)),
                        rownames(subset(padj, !is.na(Method1)))))
  dd1 <- length(setdiff(rownames(subset(truth, status == 0)),
                        rownames(subset(padj, !is.na(Method1)))))
  aa1 <- length(setdiff(rownames(subset(padj, Method1 <= 0.05)),
                        rownames(subset(truth, status %in% c(0, 1)))))
  bb1 <- length(setdiff(rownames(subset(padj, Method1 > 0.05)),
                        rownames(subset(truth, status %in% c(0, 1)))))
  ## True/false sign
  av <- sign(score[rownames(subset(sval, Method1 <= 0.05)), "Method1"])
  bv <- sign(truth[rownames(subset(sval, Method1 <= 0.05)), "logFC"])
  kp <- which(!is.na(bv))
  av <- av[kp]
  bv <- bv[kp]
  av[is.na(av)] <- -bv[is.na(av)]
  ts1 <- sum(av == bv, na.rm = TRUE)
  fs1 <- sum(abs(av - bv) == 2, na.rm = TRUE)
  fsr1 <- fs1/length(av)
  fnbr1 <- length(av)

  tp2 <- length(intersect(rownames(subset(padjsub, Method1 <= 0.05)),
                          rownames(subset(truthsub, status == 1))))
  fp2 <- length(intersect(rownames(subset(padjsub, Method1 <= 0.05)),
                          rownames(subset(truthsub, status == 0))))
  fn2 <- length(intersect(rownames(subset(padjsub, Method1 > 0.05)),
                          rownames(subset(truthsub, status == 1))))
  tn2 <- length(intersect(rownames(subset(padjsub, Method1 > 0.05)),
                          rownames(subset(truthsub, status == 0))))
  cc2 <- length(setdiff(rownames(subset(truthsub, status == 1)),
                        rownames(subset(padjsub, !is.na(Method1)))))
  dd2 <- length(setdiff(rownames(subset(truthsub, status == 0)),
                        rownames(subset(padjsub, !is.na(Method1)))))
  aa2 <- length(setdiff(rownames(subset(padjsub, Method1 <= 0.05)),
                        rownames(subset(truthsub, status %in% c(0, 1)))))
  bb2 <- length(setdiff(rownames(subset(padjsub, Method1 > 0.05)),
                        rownames(subset(truthsub, status %in% c(0, 1)))))
  ## True/false sign
  av <- sign(scoresub[rownames(subset(svalsub, Method1 <= 0.05)), "Method1"])
  bv <- sign(truthsub[rownames(subset(svalsub, Method1 <= 0.05)), "logFC"])
  kp <- which(!is.na(bv))
  av <- av[kp]
  bv <- bv[kp]
  av[is.na(av)] <- -bv[is.na(av)]
  ts2 <- sum(av == bv, na.rm = TRUE)
  fs2 <- sum(abs(av - bv) == 2, na.rm = TRUE)
  fsr2 <- fs2/length(av)
  fnbr2 <- length(av)
  
  test_that("Shared values in output objects are equal", {
    ibp1 <- prepare_data_for_plot(ib1, keepmethod = NULL, incloverall = TRUE,
                                  incltruth = TRUE)
    ibp1logical <- prepare_data_for_plot(ib1logical, keepmethod = NULL, 
                                         incloverall = TRUE, incltruth = TRUE)
    expect_equivalent(colSums(overlap(ibp1))["Method1"],
                      (subset(tpr(ib1), fullmethod == "Method1_overall" &
                                thr == "thr0.05"))[, "NBR"])
    expect_equivalent(colSums(overlap(ibp1logical))["Method1"],
                      (subset(tpr(ib1logical), fullmethod == "Method1_overall" &
                                thr == "thr0.05"))[, "NBR"])
    
    ## TP (fdrtpr, fdrnbr, tpr, fpr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(fpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    
    expect_equal((subset(fdrtpr(ib1logical), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(fdrnbr(ib1logical), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib1logical), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(tpr(ib1logical), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib1logical), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(fpr(ib1logical), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TP"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TP"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TP"],
                 (subset(fpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(fpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TP"])

    ## FP (fdrtpr, fdrnbr, tpr, fpr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"],
                 (subset(fpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FP"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FP"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FP"],
                 (subset(fpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"],
                 (subset(fpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FP"])

    ## NBR (fdrtpr, fdrnbr, tpr, fpr, overlap)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"],
                 (subset(fpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NBR"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NBR"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NBR"],
                 (subset(fpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"],
                 (subset(fpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NBR"])

    ## TN (fdrtpr, fdrnbr, tpr, fpr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"],
                 (subset(fpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TN"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TN"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TN"],
                 (subset(fpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"],
                 (subset(fpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TN"])

    ## FN (fdrtpr, fdrnbr, tpr, fpr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"],
                 (subset(fpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FN"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FN"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FN"],
                 (subset(fpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"],
                 (subset(fpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FN"])

    ## TOT_CALLED (fdrtpr, fdrnbr, tpr, fpr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(fpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(fpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(fpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TOT_CALLED"])

    ## DIFF (fdrtpr, fdrnbr, tpr, fpr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"],
                 (subset(fpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "DIFF"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "DIFF"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "DIFF"],
                 (subset(fpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"],
                 (subset(fpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "DIFF"])

    ## NONDIFF (fdrtpr, fdrnbr, tpr, fpr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"],
                 (subset(fpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NONDIFF"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NONDIFF"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NONDIFF"],
                 (subset(fpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"],
                 (subset(fpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "NONDIFF"])

    ## TPR (fdrtpr, fdrnbr, tpr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"],
                 (subset(tpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TPR"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TPR"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TPR"],
                 (subset(tpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "TPR"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"],
                 (subset(tpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "TPR"])

    ## FDR (fdrtpr, fdrnbr)
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FDR"],
                 (subset(fdrnbr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FDR"])
    expect_equal((subset(fdrtpr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FDR"],
                 (subset(fdrnbr(ib2), fullmethod == "Method2_expr_cat:[   0.000,   0.362)" &
                           thr == "thr0.05"))[, "FDR"])
    expect_equal((subset(fdrtpr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FDR"],
                 (subset(fdrnbr(ib3), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FDR"])
    expect_equal((subset(fdrtpr(ib1), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FDR"],
                 (subset(fdrtpr(ib2), fullmethod == "Method2_overall" & thr == "thr0.05"))[, "FDR"])

    ## fdrtpr vs fdrtprcurve
    nbr1 <- (subset(fdrtpr(ib1), fullmethod == "Method1_overall" & thr == "thr0.05"))[, "NBR"]
    expect_equivalent((subset(fdrtpr(ib1), fullmethod == "Method1_overall" &
                                NBR == nbr1))[, c("NBR", "TP", "FP", "TN", "FN",
                                                  "TOT_CALLED", "DIFF", "NONDIFF", "TPR", "FDR")],
                      (subset(fdrtprcurve(ib1), fullmethod == "Method1_overall" &
                                NBR == nbr1))[, c("NBR", "TP", "FP", "TN", "FN",
                                                  "TOT_CALLED", "DIFF", "NONDIFF", "TPR", "FDR")])

    nbr1 <- (subset(fdrtpr(ib2), fullmethod == "Method1_expr_cat:[   0.000,   0.362)" &
                      thr == "thr0.05"))[, "NBR"]
    fdrtprcurve(ib2)$NBR <- round(fdrtprcurve(ib2)$NBR)
    expect_equivalent((subset(fdrtpr(ib2), fullmethod == "Method1_expr_cat:[   0.000,   0.362)" &
                                NBR == nbr1))[, c("NBR", "TP", "FP", "TN", "FN",
                                                  "TOT_CALLED", "DIFF", "NONDIFF", "TPR", "FDR")],
                      (subset(fdrtprcurve(ib2), fullmethod == "Method1_expr_cat:[   0.000,   0.362)" &
                                NBR == nbr1))[, c("NBR", "TP", "FP", "TN", "FN",
                                                  "TOT_CALLED", "DIFF", "NONDIFF", "TPR", "FDR")])

    nbr1 <- (subset(fdrtpr(ib3), fullmethod == "Method1_overall" & thr == "thr0.05"))[, "NBR"]
    expect_equivalent((subset(fdrtpr(ib3), fullmethod == "Method1_overall" &
                                NBR == nbr1))[, c("NBR", "TP", "FP", "TN", "FN",
                                                  "TOT_CALLED", "DIFF", "NONDIFF", "TPR", "FDR")],
                      (subset(fdrtprcurve(ib3), fullmethod == "Method1_overall" &
                                NBR == nbr1))[, c("NBR", "TP", "FP", "TN", "FN",
                                                  "TOT_CALLED", "DIFF", "NONDIFF", "TPR", "FDR")])

    ## fdrtprcurve and fdrnbrcurve data frames should be equal
    expect_equal(fdrtprcurve(ib1), fdrnbrcurve(ib1))
    expect_equal(fdrtprcurve(ib2), fdrnbrcurve(ib2))
    expect_equal(fdrtprcurve(ib3), fdrnbrcurve(ib3))

    ## fsrnbr vs fsrnbrcurve
    nbr1 <- (subset(fsrnbr(ib1), fullmethod == "Method1_overall" & thr == "thr0.05"))[, "NBR"]
    expect_equivalent((subset(fsrnbr(ib1), fullmethod == "Method1_overall" &
                                NBR == nbr1))[, c("NBR", "FSR", "FS", "TS", "POSSIGN",
                                                  "NEGSIGN", "ZEROSIGN", "TOT_CALLED")],
                      (subset(fsrnbrcurve(ib1), fullmethod == "Method1_overall" &
                                NBR == nbr1))[, c("NBR", "FSR", "FS", "TS", "POSSIGN",
                                                  "NEGSIGN", "ZEROSIGN", "TOT_CALLED")])
    
    nbr1 <- (subset(fsrnbr(ib2), fullmethod == "Method1_expr_cat:[   0.000,   0.362)" &
                      thr == "thr0.05"))[, "NBR"]
    fsrnbrcurve(ib2)$NBR <- round(fsrnbrcurve(ib2)$NBR)
    expect_equivalent((subset(fsrnbr(ib2), fullmethod == "Method1_expr_cat:[   0.000,   0.362)" &
                                NBR == nbr1))[, c("NBR", "FSR", "FS", "TS", "POSSIGN",
                                                  "NEGSIGN", "ZEROSIGN", "TOT_CALLED")],
                      (subset(fsrnbrcurve(ib2), fullmethod == "Method1_expr_cat:[   0.000,   0.362)" &
                                NBR == nbr1))[, c("NBR", "FSR", "FS", "TS", "POSSIGN",
                                                  "NEGSIGN", "ZEROSIGN", "TOT_CALLED")])
    
    nbr1 <- (subset(fsrnbr(ib3), fullmethod == "Method1_overall" & thr == "thr0.05"))[, "NBR"]
    expect_equivalent((subset(fsrnbr(ib3), fullmethod == "Method1_overall" &
                                NBR == nbr1))[, c("NBR", "FSR", "FS", "TS", "POSSIGN",
                                                  "NEGSIGN", "ZEROSIGN", "TOT_CALLED")],
                      (subset(fsrnbrcurve(ib3), fullmethod == "Method1_overall" &
                                NBR == nbr1))[, c("NBR", "FSR", "FS", "TS", "POSSIGN",
                                                  "NEGSIGN", "ZEROSIGN", "TOT_CALLED")])

    ## FPC vs FP
    nbr1 <- (subset(fpc(ib1), fullmethod == "Method1_overall"))[, "topN"][150]
    cut1 <- (subset(fpc(ib1), fullmethod == "Method1_overall" & topN == nbr1))[, "FPC_CUTOFF"]
    fp4 <- length(intersect(rownames(subset(pval, abs(Method1) <= cut1)), 
                            rownames(subset(truth, status == 0))))
    expect_equal(fp4, (subset(fpc(ib1), fullmethod == "Method1_overall" & topN == nbr1))[, "FP"])

    ## FPC vs FP
    nbr1 <- (subset(fpc(ib2), fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "topN"][150]
    cut1 <- (subset(fpc(ib2), fullmethod == "Method1_expr_cat:[   2.628,  17.148)" &
                      topN == nbr1))[, "FPC_CUTOFF"]
    fp4 <- length(intersect(rownames(subset(pvalsub, abs(Method1) <= cut1)),
                            rownames(subset(truthsub, status == 0))))
    expect_equal(fp4, (subset(fpc(ib2), fullmethod == "Method1_expr_cat:[   2.628,  17.148)" &
                                topN == nbr1))[, "FP"])

    ## FPC vs FP
    nbr1 <- (subset(fpc(ib3), fullmethod == "Method1_overall"))[, "topN"][150]
    cut1 <- (subset(fpc(ib3), fullmethod == "Method1_overall" & topN == nbr1))[, "FPC_CUTOFF"]
    fp4 <- length(intersect(rownames(subset(pval, abs(Method1) <= cut1)), 
                            rownames(subset(truth, status == 0))))
    expect_equal(fp4, (subset(fpc(ib3), fullmethod == "Method1_overall" & topN == nbr1))[, "FP"])

    ## ROC vs TPR/FPR
    tpr1 <- (subset(roc(ib1), fullmethod == "Method1_overall"))[, "TPR"][155]
    fpr1 <- (subset(roc(ib1), fullmethod == "Method1_overall" & TPR == tpr1))[, "FPR"]
    cut1 <- (subset(roc(ib1), fullmethod == "Method1_overall" & TPR == tpr1))[, "ROC_CUTOFF"]
    tp4 <- length(intersect(rownames(subset(pval, abs(Method1) <= cut1)), 
                            rownames(subset(truth, status == 1))))
    fp4 <- length(intersect(rownames(subset(pval, abs(Method1) <= cut1)), 
                            rownames(subset(truth, status == 0))))
    fn4 <- length(intersect(rownames(subset(pval, abs(Method1) > cut1)), 
                            rownames(subset(truth, status == 1))))
    tn4 <- length(intersect(rownames(subset(pval, abs(Method1) > cut1)), 
                            rownames(subset(truth, status == 0))))
    cc4 <- length(setdiff(rownames(subset(truth, status == 1)), rownames(subset(pval, !is.na(Method1)))))
    dd4 <- length(setdiff(rownames(subset(truth, status == 0)), rownames(subset(pval, !is.na(Method1)))))
    expect_equal(fpr1, fp4/(fp4 + tn4 + dd4))
    expect_equal(tpr1, tp4/(tp4 + fn4 + cc4))

    ## ROC vs TPR/FPR
    tpr1 <- (subset(roc(ib2), fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "TPR"][465]
    fpr1 <- (subset(roc(ib2), fullmethod == "Method1_expr_cat:[   2.628,  17.148)" & TPR == tpr1))[, "FPR"]
    cut1 <- (subset(roc(ib2), fullmethod == "Method1_expr_cat:[   2.628,  17.148)" & TPR == tpr1))[, "ROC_CUTOFF"]
    tp4 <- length(intersect(rownames(subset(pvalsub, abs(Method1) <= cut1)), 
                            rownames(subset(truthsub, status == 1))))
    fp4 <- length(intersect(rownames(subset(pvalsub, abs(Method1) <= cut1)), 
                            rownames(subset(truthsub, status == 0))))
    fn4 <- length(intersect(rownames(subset(pvalsub, abs(Method1) > cut1)), 
                            rownames(subset(truthsub, status == 1))))
    tn4 <- length(intersect(rownames(subset(pvalsub, abs(Method1) > cut1)), 
                            rownames(subset(truthsub, status == 0))))
    cc4 <- length(setdiff(rownames(subset(truthsub, status == 1)), rownames(subset(pvalsub, !is.na(Method1)))))
    dd4 <- length(setdiff(rownames(subset(truthsub, status == 0)), rownames(subset(pvalsub, !is.na(Method1)))))
    expect_equal(fpr1, fp4/(fp4 + tn4 + dd4))
    expect_equal(tpr1, tp4/(tp4 + fn4 + cc4))

    ## ROC vs TPR/FPR
    tpr1 <- (subset(roc(ib3), fullmethod == "Method1_overall"))[, "TPR"][155]
    fpr1 <- (subset(roc(ib3), fullmethod == "Method1_overall" & TPR == tpr1))[, "FPR"]
    cut1 <- (subset(roc(ib3), fullmethod == "Method1_overall" & TPR == tpr1))[, "ROC_CUTOFF"]
    tp4 <- length(intersect(rownames(subset(pval, abs(Method1) <= cut1)), 
                            rownames(subset(truth, status == 1))))
    fp4 <- length(intersect(rownames(subset(pval, abs(Method1) <= cut1)), 
                            rownames(subset(truth, status == 0))))
    fn4 <- length(intersect(rownames(subset(pval, abs(Method1) > cut1)), 
                            rownames(subset(truth, status == 1))))
    tn4 <- length(intersect(rownames(subset(pval, abs(Method1) > cut1)), 
                            rownames(subset(truth, status == 0))))
    cc4 <- length(setdiff(rownames(subset(truth, status == 1)), rownames(subset(pval, !is.na(Method1)))))
    dd4 <- length(setdiff(rownames(subset(truth, status == 0)), rownames(subset(pval, !is.na(Method1)))))
    expect_equal(fpr1, fp4/(fp4 + tn4))
    expect_equal(tpr1, tp4/(tp4 + fn4))
  })

  test_that("FSR values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(fsr1, (subset(fsrnbr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "FSR"])
    expect_equal(fnbr1,  (subset(fsrnbr(ib1), thr == "thr0.05" &
                                                   basemethod == "Method1"))[, "NBR"])
    
    ## Stratification, onlyshared = FALSE
    expect_equal(fsr2, (subset(fsrnbr(ib2), thr == "thr0.05" &
                                fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "FSR"])
    expect_equal(fnbr2,
                 (subset(fsrnbr(ib2), thr == "thr0.05" &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "NBR"])
    
    ## No stratification, onlyshared = TRUE
    expect_equal(fsr1, (subset(fsrnbr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "FSR"])
    expect_equal(fnbr1,
                 (subset(fsrnbr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "NBR"])
  })
  
  test_that("TPR values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(tp1, (subset(tpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "TP"])
    expect_equal(tp1/(tp1 + fn1 + cc1),  (subset(tpr(ib1), thr == "thr0.05" &
                                                   basemethod == "Method1"))[, "TPR"])

    ## Stratification, onlyshared = FALSE
    expect_equal(tp2, (subset(tpr(ib2), thr == "thr0.05" &
                                fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "TP"])
    expect_equal(tp2/(tp2 + fn2 + cc2),
                 (subset(tpr(ib2), thr == "thr0.05" &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "TPR"])

    ## No stratification, onlyshared = TRUE
    expect_equal(tp1, (subset(tpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "TP"])
    expect_equal(tp1/(tp1 + fn1),
                 (subset(tpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "TPR"])
  })

  test_that("FPR values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(fp1, (subset(fpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "FP"])
    expect_equal(fp1/(fp1 + tn1 + dd1),
                 (subset(fpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "FPR"])

    ## Stratification, onlyshared = FALSE
    expect_equal(fp2, (subset(fpr(ib2), thr == "thr0.05" &
                                fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "FP"])
    expect_equal(fp2/(fp2 + tn2 + dd2),
                 (subset(fpr(ib2), thr == "thr0.05" &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "FPR"])

    ## No stratification, onlyshared = TRUE
    expect_equal(fp1, (subset(fpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "FP"])
    expect_equal(fp1/(fp1 + tn1),
                 (subset(fpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "FPR"])
  })

  test_that("FDR values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(fp1/(tp1 + fp1),
                 (subset(fdrtpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "FDR"])

    ## Stratification, onlyshared = FALSE
    expect_equal(fp2/(tp2 + fp2),
                 (subset(fdrtpr(ib2), thr == "thr0.05" &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "FDR"])

    ## No stratification, onlyshared = TRUE
    expect_equal(fp1/(tp1 + fp1),
                 (subset(fdrtpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "FDR"])
  })

  test_that("NBR values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(tp1 + fp1,
                 (subset(fdrnbr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "NBR"])

    ## Stratification, onlyshared = FALSE
    expect_equal(tp2 + fp2,
                 (subset(fdrnbr(ib2), thr == "thr0.05" &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "NBR"])

    ## No stratification, onlyshared = TRUE
    expect_equal(tp1 + fp1,
                 (subset(fdrtpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "NBR"])
  })

  test_that("TN values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(tn1,  (subset(fdrtpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "TN"])

    ## Stratification, onlyshared = FALSE
    expect_equal(tn2,  (subset(fdrtpr(ib2), thr == "thr0.05" &
                                 fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "TN"])

    ## No stratification, onlyshared = TRUE
    expect_equal(tn1,  (subset(fdrtpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "TN"])
  })

  test_that("FN values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(fn1,  (subset(fdrtpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "FN"])

    ## Stratification, onlyshared = FALSE
    expect_equal(fn2,  (subset(fdrtpr(ib2), thr == "thr0.05" &
                                 fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "FN"])

    ## No stratification, onlyshared = TRUE
    expect_equal(fn1,  (subset(fdrtpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "FN"])
  })

  test_that("TOT_CALLED values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(tp1 + fp1 + fn1 + tn1,
                 (subset(fdrtpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "TOT_CALLED"])

    ## Stratification, onlyshared = FALSE
    expect_equal(tp2 + fp2 + fn2 + tn2,
                 (subset(fdrtpr(ib2), thr == "thr0.05" &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "TOT_CALLED"])

    ## No stratification, onlyshared = TRUE
    expect_equal(tp1 + fp1 + fn1 + tn1,
                 (subset(fdrtpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "TOT_CALLED"])
  })

  test_that("DIFF values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(tp1 + fn1 + cc1,
                 (subset(fdrtpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "DIFF"])

    ## Stratification, onlyshared = FALSE
    expect_equal(tp2 + fn2 + cc2,
                 (subset(fdrtpr(ib2), thr == "thr0.05" &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "DIFF"])

    ## No stratification, onlyshared = TRUE
    expect_equal(tp1 + fn1,
                 (subset(fdrtpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "DIFF"])
  })

  test_that("NONDIFF values are correct", {
    ## No stratification, onlyshared = FALSE
    expect_equal(fp1 + tn1 + dd1,
                 (subset(fdrtpr(ib1), thr == "thr0.05" & basemethod == "Method1"))[, "NONDIFF"])

    ## Stratification, onlyshared = FALSE
    expect_equal(fp2 + tn2 + dd2,
                 (subset(fdrtpr(ib2), thr == "thr0.05" &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "NONDIFF"])

    ## No stratification, onlyshared = TRUE
    expect_equal(fp1 + tn1,
                 (subset(fdrtpr(ib3), thr == "thr0.05" & basemethod == "Method1"))[, "NONDIFF"])
  })

  test_that("CORR values are correct", {
    ## No stratification, onlyshared = FALSE
    s1 <- score$Method1[match(rownames(truth), rownames(score))]
    s2 <- truth$logFC
    expect_equal(cor(s1, s2, use = "complete.obs", method = "pearson"),
                 (subset(corr(ib1), fullmethod == "Method1_overall"))[, "PEARSON"])
    expect_equal(cor(s1, s2, use = "complete.obs", method = "spearman"),
                 (subset(corr(ib1), fullmethod == "Method1_overall"))[, "SPEARMAN"])

    ## Stratification, onlyshared = FALSE
    s1 <- scoresub$Method1[match(rownames(truthsub), rownames(scoresub))]
    s2 <- truthsub$logFC
    expect_equal(cor(s1, s2, use = "complete.obs", method = "pearson"),
                 (subset(corr(ib2), fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "PEARSON"])
    expect_equal(cor(s1, s2, use = "complete.obs", method = "spearman"),
                 (subset(corr(ib2), fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "SPEARMAN"])

    ## No stratification, onlyshared = TRUE
    kg <- intersect(rownames(truth), rownames(score))
    s1 <- score$Method1[match(kg, rownames(score))]
    s2 <- truth$logFC[match(kg, rownames(truth))]
    expect_equal(cor(s1, s2, use = "complete.obs", method = "pearson"),
                 (subset(corr(ib3), fullmethod == "Method1_overall"))[, "PEARSON"])
    expect_equal(cor(s1, s2, use = "complete.obs", method = "spearman"),
                 (subset(corr(ib3), fullmethod == "Method1_overall"))[, "SPEARMAN"])
  })

  test_that("DEVIATION values are correct", {
    ## No stratification, onlyshared = FALSE
    feat <- "ENSG00000001460"
    expect_equal(score[feat, "Method1"] - truth[feat, "logFC"],
                 (subset(deviation(ib1), feature == feat & fullmethod == "Method1_overall"))[, "DEVIATION"])

    ## Stratification, onlyshared = FALSE
    expect_equal(scoresub[feat, "Method1"] - truthsub[feat, "logFC"],
                 (subset(deviation(ib2), feature == feat &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "DEVIATION"])

    ## No stratification, onlyshared = TRUE
    expect_equal(score[feat, "Method1"] - truth[feat, "logFC"],
                 (subset(deviation(ib3), feature == feat & fullmethod == "Method1_overall"))[, "DEVIATION"])
  })

  test_that("SCATTER values are correct", {
    ## No stratification, onlyshared = FALSE
    feat <- "ENSG00000001460"
    expect_equal(score[feat, "Method1"],
                 (subset(scatter(ib1), feature == feat & fullmethod == "Method1_overall"))[, "OBSERVATION"])
    expect_equal(truth[feat, "logFC"],
                 (subset(scatter(ib1), feature == feat & fullmethod == "Method1_overall"))[, "TRUTH"])

    ## Stratification, onlyshared = FALSE
    expect_equal(scoresub[feat, "Method1"],
                 (subset(scatter(ib2), feature == feat &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "OBSERVATION"])
    expect_equal(truthsub[feat, "logFC"],
                 (subset(scatter(ib2), feature == feat &
                           fullmethod == "Method1_expr_cat:[   2.628,  17.148)"))[, "TRUTH"])

    ## No stratification, onlyshared = TRUE
    expect_equal(score[feat, "Method1"],
                 (subset(scatter(ib3), feature == feat & fullmethod == "Method1_overall"))[, "OBSERVATION"])
    expect_equal(truth[feat, "logFC"],
                 (subset(scatter(ib3), feature == feat & fullmethod == "Method1_overall"))[, "TRUTH"])
  })

  test_that("DEVIATION and SCATTER values correspond to each other", {
    ## No stratification, onlyshared = FALSE
    m <- merge(deviation(ib1), scatter(ib1))
    expect_equal(m$DEVIATION, m$OBSERVATION - m$TRUTH)

    ## Stratification, onlyshared = FALSE
    m <- merge(deviation(ib2), scatter(ib2))
    expect_equal(m$DEVIATION, m$OBSERVATION - m$TRUTH)

    ## No stratification, onlyshared = TRUE
    m <- merge(deviation(ib3), scatter(ib3))
    expect_equal(m$DEVIATION, m$OBSERVATION - m$TRUTH)
  })
})

test_that("getcurve returns NULL if not both positive and negative instances are present", {
  cobradata <- cobradata_example_sval
  truth(cobradata)$status <- 0
  cobraperf <- calculate_performance(cobradata, binary_truth <- "status",
                                     aspects = "fdrtprcurve")
  expect_equal(length(fdrtprcurve(cobraperf)), 0)
  cobraperf2 <- calculate_performance(cobradata, binary_truth = "status",
                                      aspects = "fdrtprcurve", splv = "expr_cat")
  expect_equal(length(fdrtprcurve(cobraperf2)), 0)
})

test_that("calculate_performance without score works", {
  cobradata <- cobradata_example_sval
  score(cobradata) <- data.frame()
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC",
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)
  expect_is(ib1, "COBRAPerformance")
})

test_that("calculate_performance without adj.p works", {
  cobradata <- cobradata_example_sval
  padj(cobradata) <- data.frame()
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC",
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)
  expect_is(ib1, "COBRAPerformance")
})

test_that("calculate_performance with only adj.p works", {
  cobradata <- cobradata_example_sval
  pval(cobradata) <- data.frame()
  score(cobradata) <- data.frame()
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC", aspects = "fdrtprcurve",
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)
  expect_is(ib1, "COBRAPerformance")
})

test_that("calculate_performance with empty input works", {
  cobradata <- cobradata_example_sval
  padj(cobradata) <- data.frame()
  pval(cobradata) <- data.frame()
  score(cobradata) <- data.frame()
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC",
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)
  expect_is(ib1, "COBRAPerformance")
  expect_equal(basemethods(ib1), character(0))
})

test_that("calculate_performance without significant features works", {
  cobradata <- cobradata_example_sval
  padj(cobradata) <- data.frame(Method1 = rep(1, nrow(padj(cobradata))),
                                Method2 = rep(1, nrow(padj(cobradata))),
                                Method3 = rep(1, nrow(padj(cobradata))),
                                row.names = rownames(padj(cobradata)))
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr"),
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)

  expect_is(ib1, "COBRAPerformance")
  expect_equal(fdrtpr(ib1)$FDR, rep(0, 3))

  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr"),
                               thrs = c(0.05), splv = "expr_cat",
                               onlyshared = FALSE,
                               thr_venn = 0.05)

  expect_is(ib1, "COBRAPerformance")
  expect_equal(fdrtpr(ib1)$FDR, rep(0, 12))
})

test_that("overlap calculations are correct", {
  cobradata <- cobradata_example_sval
  truth(cobradata) <- truth(cobradata)[1:3000, , drop = FALSE]
  ## Set some adjusted p-values to NA
  tmp <- padj(cobradata)$Method1
  tmp[sample(1:length(tmp), 5)] <- NA
  padj(cobradata)$Method1 <- tmp
  padj <- padj(cobradata)
  truth <- truth(cobradata)

  ## onlyshared = FALSE, incltruth = FALSE
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               aspects = "overlap", splv = "none",
                               onlyshared = FALSE, thr_venn = 0.05)
  ib1 <- prepare_data_for_plot(ib1, incltruth = FALSE)
  expect_equal(length(intersect(which(padj$Method1 <= 0.05),
                                which(padj$Method2 <= 0.05))),
               sum(overlap(ib1)$Method1 * overlap(ib1)$Method2))

  ## onlyshared = TRUE, incltruth = FALSE
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               aspects = "overlap", splv = "none",
                               onlyshared = TRUE, thr_venn = 0.05)
  ib1 <- prepare_data_for_plot(ib1, incltruth = FALSE)
  expect_equal(length(intersect(which(padj$Method1 <= 0.05),
                                which(padj$Method2 <= 0.05))),
               sum(overlap(ib1)$Method1 * overlap(ib1)$Method2))

  ## onlyshared = FALSE, incltruth = TRUE
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               aspects = "overlap", splv = "none",
                               onlyshared = FALSE, thr_venn = 0.05)
  ib1 <- prepare_data_for_plot(ib1, incltruth = TRUE)
  padj1 <- padj[match(rownames(truth)[which(!is.na(truth$status))],
                      rownames(padj)), ]
  rownames(padj1) <- rownames(truth)
  expect_equal(length(intersect(which(padj1$Method1 <= 0.05),
                                which(padj1$Method2 <= 0.05))),
               sum(overlap(ib1)$Method1 * overlap(ib1)$Method2))
  expect_equal(length(setdiff(which(padj1$Method2 <= 0.05), which(padj1$Method1 <= 0.05))),
               length(setdiff(which(overlap(ib1)$Method2 == 1), which(overlap(ib1)$Method1 == 1))))

  ## onlyshared = TRUE, incltruth = TRUE
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               aspects = "overlap", splv = "none",
                               onlyshared = TRUE, thr_venn = 0.05)
  ib1 <- prepare_data_for_plot(ib1, incltruth = TRUE)
  kp <- intersect(rownames(truth)[which(!is.na(truth$status))],
                  rownames(padj)[which(rowSums(is.na(padj)) == 0)])
  padj1 <- padj[match(kp, rownames(padj)), ]
  expect_equal(length(intersect(which(padj1$Method1 <= 0.05),
                                which(padj1$Method2 <= 0.05))),
               sum(overlap(ib1)$Method1 * overlap(ib1)$Method2))
  expect_equal(length(setdiff(which(padj1$Method2 <= 0.05), which(padj1$Method1 <= 0.05))),
               length(setdiff(which(overlap(ib1)$Method2 == 1), which(overlap(ib1)$Method1 == 1))))

  ## onlyshared = TRUE, incltruth = TRUE, stratify
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               aspects = c("tpr", "overlap"), splv = "expr_cat",
                               onlyshared = TRUE, thr_venn = 0.05)
  ib1 <- prepare_data_for_plot(ib1, incltruth = TRUE)
  kp <- intersect(rownames(truth)[which(!is.na(truth$status))],
                  rownames(padj)[which(rowSums(is.na(padj)) == 0)])
  padj1 <- padj[match(kp, rownames(padj)), ]
  expect_equal(length(intersect(which(padj1$Method1 <= 0.05),
                                which(padj1$Method2 <= 0.05))),
               sum(overlap(ib1)$overall$Method1 * overlap(ib1)$overall$Method2))
  expect_equal(length(setdiff(which(padj1$Method2 <= 0.05), which(padj1$Method1 <= 0.05))),
               length(setdiff(which(overlap(ib1)$overall$Method2 == 1), which(overlap(ib1)$overall$Method1 == 1))))

  ib2 <- prepare_data_for_plot(ib1, incltruth = TRUE, incloverall = FALSE)
  expect_is(ib2, "COBRAPlot")
})

test_that("overlap calculations based on rank are correct", {
  cobradata <- cobradata_example_sval
  truth(cobradata) <- truth(cobradata)[1:3000, , drop = FALSE]
  ## Set some adjusted p-values to NA
  tmp <- padj(cobradata)$Method1
  tmp[sample(1:length(tmp), 5)] <- NA
  padj(cobradata)$Method1 <- tmp
  padj <- padj(cobradata)
  truth <- truth(cobradata)
  
  ## onlyshared = FALSE, incltruth = FALSE
  ## all features used. missing adjp considered nonsignificant
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               aspects = "overlap", splv = "none",
                               onlyshared = FALSE, topn_venn = 100, 
                               type_venn = "rank")
  ib1 <- prepare_data_for_plot(ib1, incltruth = FALSE)
  expect_equal(length(intersect(order(padj$Method1)[1:100],
                                order(padj$Method2)[1:100])),
               sum(overlap(ib1)$Method1 * overlap(ib1)$Method2))
  
  ## onlyshared = TRUE, incltruth = FALSE
  ## all features used. missing adjp considered nonsignificant
  ib1 <- calculate_performance(cobradata, binary_truth = "status",
                               aspects = "overlap", splv = "none",
                               onlyshared = TRUE, topn_venn = 100,
                               type_venn = "rank")
  ib1 <- prepare_data_for_plot(ib1, incltruth = FALSE)
  expect_equal(length(intersect(order(padj$Method1)[1:100],
                                order(padj$Method2)[1:100])),
               sum(overlap(ib1)$Method1 * overlap(ib1)$Method2))
  
  ib2 <- prepare_data_for_plot(ib1, incltruth = FALSE, incloverall = FALSE)
  expect_is(ib2, "COBRAPlot")
})

#' @rdname COBRAPlot
#' @export
.COBRAPlot <- setClass("COBRAPlot",
                      slots = c(plotcolors = "character",
                                facetted = "logical"),
                      contains = "COBRAPerformance")

#' \code{COBRAPlot} object and constructor
#'
#' The \code{COBRAPlot} class is similar to the \code{COBRAPerformance} class in
#' that it holds various types of calculated performance measures. However, it
#' also contains other attributes that are necessary for plotting, such as color
#' assignments. Several \code{COBRAPlot} objects can be generated from the same
#' \code{COBRAPerformance} object, without having to go through the potentially
#' time consuming task of recalculating all performance measures. Objects from
#' this class are typically generated from a \code{COBRAPerformance} objects by
#' means of the function \code{\link{prepare_data_for_plot}}.
#'
#' @param plotcolors A character vector giving the color for each method (or
#'   method-stratification level combination).
#' @param facetted A logical indicating whether the data is prepared for a
#'   facetted plot (separating different stratification levels into different
#'   panels) or for displaying all values in one plot panel.
#'
#' @return A \code{COBRAPlot} object.
#' @inheritParams COBRAPerformance
#' @include COBRAPerformance.R
#'
#' @aliases COBRAPlot COBRAPlot-class
#'
#' @docType class
#'
#' @export
#' @rdname COBRAPlot
#' @author Charlotte Soneson
#' @examples
#' ## Empty COBRAPlot object
#' cobraplot <- COBRAPlot()
COBRAPlot <- function(fdrtpr = data.frame(), fdrtprcurve = data.frame(),
                      fsrnbr = data.frame(), fsrnbrcurve = data.frame(), 
                      fdrnbr = data.frame(), corr = data.frame(),
                      fdrnbrcurve = data.frame(), tpr = data.frame(),
                      fpr = data.frame(), roc = data.frame(),
                      scatter = data.frame(), onlyshared = NA,
                      fpc = data.frame(), overlap = data.frame(),
                      plotcolors = "", splv = "", deviation = data.frame(),
                      maxsplit = NA_integer_, facetted = NA) {

  .COBRAPlot(fdrtpr = fdrtpr, fdrtprcurve = fdrtprcurve,
             fsrnbr = fsrnbr, fsrnbrcurve = fsrnbrcurve, 
             onlyshared = onlyshared, fdrnbr = fdrnbr,
             fdrnbrcurve = fdrnbrcurve, deviation = deviation,
             tpr = tpr, fpr = fpr, roc = roc, fpc = fpc, scatter = scatter,
             overlap = overlap, plotcolors = plotcolors, corr = corr,
             splv = splv, maxsplit = maxsplit, facetted = facetted)
}

setMethod("show", "COBRAPlot", function(object) {
  cat("An object of class \"", class(object), "\"\n", sep = "")
  for (sl in slotNames(object)) {
    if (.hasSlot(object, sl)) {
      x <- slot(object, sl)
      cat("@", sl, "\n", sep = "")
      .printHead(x)
      cat("\n")
    }
  }
})

#' @rdname fdrtpr
#' @aliases fdrtpr fdrtpr,COBRAPlot-method fdrtpr<-,COBRAPlot,data.frame-method
setReplaceMethod("fdrtpr", signature(x = "COBRAPlot",
                                     value = "data.frame"),
                 function(x, value) {
                   x@fdrtpr <- value
                   if (validObject(x))
                     x
                 })

#' @rdname onlyshared
#' @aliases onlyshared onlyshared,COBRAPlot-method
#'   onlyshared<-,COBRAPlot,logical-method
setReplaceMethod("onlyshared", signature(x = "COBRAPlot",
                                         value = "logical"),
                 function(x, value) {
                   x@onlyshared <- value
                   if (validObject(x))
                     x
                 })

#' @rdname fdrtprcurve
#' @aliases fdrtprcurve fdrtprcurve,COBRAPlot-method
#'   fdrtprcurve<-,COBRAPlot,data.frame-method
setReplaceMethod("fdrtprcurve", signature(x = "COBRAPlot",
                                          value = "data.frame"),
                 function(x, value) {
                   x@fdrtprcurve <- value
                   if (validObject(x))
                     x
                 })

#' @rdname deviation
#' @aliases deviation deviation,COBRAPlot-method
#'   deviation<-,COBRAPlot,data.frame-method
setReplaceMethod("deviation", signature(x = "COBRAPlot",
                                        value = "data.frame"),
                 function(x, value) {
                   x@deviation <- value
                   if (validObject(x))
                     x
                 })

#' @rdname fdrnbr
#' @aliases fdrnbr fdrnbr,COBRAPlot-method fdrnbr<-,COBRAPlot,data.frame-method
setReplaceMethod("fdrnbr", signature(x = "COBRAPlot",
                                     value = "data.frame"),
                 function(x, value) {
                   x@fdrnbr <- value
                   if (validObject(x))
                     x
                 })

#' @rdname fdrnbrcurve
#' @aliases fdrnbrcurve fdrnbrcurve,COBRAPlot-method
#'   fdrnbrcurve<-,COBRAPlot,data.frame-method
setReplaceMethod("fdrnbrcurve", signature(x = "COBRAPlot",
                                          value = "data.frame"),
                 function(x, value) {
                   x@fdrnbrcurve <- value
                   if (validObject(x))
                     x
                 })

#' @rdname fsrnbr
#' @aliases fsrnbr fsrnbr,COBRAPlot-method fsrnbr<-,COBRAPlot,data.frame-method
setReplaceMethod("fsrnbr", signature(x = "COBRAPlot",
                                     value = "data.frame"),
                 function(x, value) {
                   x <- update_cobraperformance(x, quiet = FALSE)
                   x@fsrnbr <- value
                   if (validObject(x))
                     x
                 })

#' @rdname fsrnbrcurve
#' @aliases fsrnbrcurve fsrnbrcurve,COBRAPlot-method
#'   fsrnbrcurve<-,COBRAPlot,data.frame-method
setReplaceMethod("fsrnbrcurve", signature(x = "COBRAPlot",
                                          value = "data.frame"),
                 function(x, value) {
                   x <- update_cobraperformance(x, quiet = FALSE)
                   x@fsrnbrcurve <- value
                   if (validObject(x))
                     x
                 })

#' @rdname scatter
#' @aliases scatter scatter,COBRAPlot-method
#'   scatter<-,COBRAPlot,data.frame-method
setReplaceMethod("scatter", signature(x = "COBRAPlot",
                                      value = "data.frame"),
                 function(x, value) {
                   x@scatter <- value
                   if (validObject(x))
                     x
                 })

#' @rdname tpr
#' @aliases tpr tpr,COBRAPlot-method tpr<-,COBRAPlot,data.frame-method
setReplaceMethod("tpr", signature(x = "COBRAPlot", value = "data.frame"),
                 function(x, value) {
                   x@tpr <- value
                   if (validObject(x))
                     x
                 })

#' @rdname fpr
#' @aliases fpr fpr,COBRAPlot-method fpr<-,COBRAPlot,data.frame-method
setReplaceMethod("fpr", signature(x = "COBRAPlot", value = "data.frame"),
                 function(x, value) {
                   x@fpr <- value
                   if (validObject(x))
                     x
                 })

#' @rdname roc
#' @aliases roc roc,COBRAPlot-method roc<-,COBRAPlot,data.frame-method
setReplaceMethod("roc", signature(x = "COBRAPlot", value = "data.frame"),
                 function(x, value) {
                   x@roc <- value
                   if (validObject(x))
                     x
                 })

#' @rdname fpc
#' @aliases fpc fpc,COBRAPlot-method fpc<-,COBRAPlot,data.frame-method
setReplaceMethod("fpc", signature(x = "COBRAPlot", value = "data.frame"),
                 function(x, value) {
                   x@fpc <- value
                   if (validObject(x))
                     x
                 })

#' @rdname corr
#' @aliases corr corr,COBRAPlot-method corr<-,COBRAPlot,data.frame-method
setReplaceMethod("corr", signature(x = "COBRAPlot",
                                   value = "data.frame"),
                 function(x, value) {
                   x@corr <- value
                   if (validObject(x))
                     x
                 })

#' @rdname overlap
#' @aliases overlap overlap,COBRAPlot-method overlap<-,COBRAPlot,list_df-method
setReplaceMethod("overlap", signature(x = "COBRAPlot",
                                      value = "list_df"),
                 function(x, value) {
                   x@overlap <- value
                   if (validObject(x))
                     x
                 })

#' @rdname splv
#' @aliases splv splv,COBRAPlot-method splv<-,COBRAPlot,character-method
setReplaceMethod("splv", signature(x = "COBRAPlot", value = "character"),
                 function(x, value) {
                   x@splv <- value
                   if (validObject(x))
                     x
                 })

#' @rdname maxsplit
#' @aliases maxsplit maxsplit,COBRAPlot-method
#'   maxsplit<-,COBRAPlot,numeric-method
setReplaceMethod("maxsplit", signature(x = "COBRAPlot",
                                       value = "numeric"),
                 function(x, value) {
                   x@maxsplit <- value
                   if (validObject(x))
                     x
                 })



#' Accessor and replacement functions for \code{plotcolors} slot
#'
#' Accessor and replacement functions for the \code{plotcolors} slot in an
#' \code{COBRAPlot} object.
#'
#' @docType methods
#' @name plotcolors
#' @rdname plotcolors
#' @aliases plotcolors plotcolors,COBRAPlot-method
#'   plotcolors<-,COBRAPlot,character-method
#' @return The accessor function returns a character vector giving the colors
#'   assigned to each of the methods (or method/stratification level
#'   combinations) represented in the \code{COBRAPlot} object.
#'
#' @param x A \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A character vector giving the colors assigned to each of the
#'   methods (or method/stratification level combinations) represented in the
#'   \code{COBRAPlot} object.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr")
#' cobraplot <- prepare_data_for_plot(cobraperf)
#' plotcolors(cobraplot)
setMethod("plotcolors", "COBRAPlot", function(x) x@plotcolors)
#' @name plotcolors
#' @rdname plotcolors
#' @exportMethod "plotcolors<-"
setReplaceMethod("plotcolors", signature(x = "COBRAPlot", value = "character"),
                 function(x, value) {
                   x@plotcolors <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{facetted} slot
#'
#' Accessor and replacement functions for the \code{facetted} slot in an
#' \code{COBRAPlot} object.
#'
#' @docType methods
#' @name facetted
#' @rdname facetted
#' @aliases facetted facetted,COBRAPlot-method
#'   facetted<-,COBRAPlot,logical-method
#' @return The accessor function returns a logical value, indicating whether the
#'   object is formatted for facetted plots (visualizing each stratification
#'   level in a separate panel) or not.
#'
#' @param x A \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A logical value, indicating whether the object is formatted for
#'   facetted plots (visualizing each stratification level in a separate panel)
#'   or not.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr")
#' cobraplot <- prepare_data_for_plot(cobraperf)
#' facetted(cobraplot)
setMethod("facetted", "COBRAPlot", function(x) x@facetted)
#' @name facetted
#' @rdname facetted
#' @exportMethod "facetted<-"
setReplaceMethod("facetted", signature(x = "COBRAPlot", value = "logical"),
                 function(x, value) {
                   x@facetted <- value
                   if (validObject(x))
                     x
                 })

#' @docType methods
#' @name Extract
#' @rdname Extract
#' @aliases [ [,COBRAPlot-method \S4method{[}{COBRAPlot,ANY,ANY}
#'   \S4method{[}{COBRAPlot,ANY,ANY,ANY}
#' @export
#' @examples
#' data(cobradata_example)
#' cobradata_example[c("ENSG00000000457", "ENSG00000000971",
#'                     "ENSG00000000460"), ]
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr")
#' cobraperf[, c("voom")]
#' cobraplot <- prepare_data_for_plot(cobraperf)
#' cobraplot[, c("voom")]
setMethod("[", "COBRAPlot",
          function(x, i = "missing", j, drop = "missing") {
            x <- update_cobraperformance(x, quiet = FALSE)
            
            if (length(intersect(j, basemethods(x))) == 0)
              stop("none of the provided method found in the object, ",
                   "no subsetting done")
            if (length(x@plotcolors) != 0) {
              combs <-
                expand.grid(c(j, "truth"), c("", paste0("_", stratiflevels(x))),
                            c("", "yes", "no"))
              keepcols <- paste0(combs[, 1], combs[, 2], combs[, 3])
              x@plotcolors <- x@plotcolors[names(x@plotcolors) %in% keepcols]
            }
            if (length(x@tpr) != 0)
              x@tpr <- x@tpr[which(x@tpr$basemethod %in% j), ]
            if (length(x@fpr) != 0)
              x@fpr <- x@fpr[which(x@fpr$basemethod %in% j), ]
            if (length(x@roc) != 0)
              x@roc <- x@roc[which(x@roc$basemethod %in% j), ]
            if (length(x@fpc) != 0)
              x@fpc <- x@fpc[which(x@fpc$basemethod %in% j), ]
            if (length(x@fdrtpr) != 0)
              x@fdrtpr <- x@fdrtpr[which(x@fdrtpr$basemethod %in% j), ]
            if (length(x@fdrnbr) != 0)
              x@fdrnbr <- x@fdrnbr[which(x@fdrnbr$basemethod %in% j), ]
            if (length(x@fsrnbr) != 0)
              x@fsrnbr <- x@fsrnbr[which(x@fsrnbr$basemethod %in% j), ]
            if (length(x@deviation) != 0)
              x@deviation <- x@deviation[which(x@deviation$basemethod %in% j), ]
            if (length(x@fdrtprcurve) != 0)
              x@fdrtprcurve <-
                x@fdrtprcurve[which(x@fdrtprcurve$basemethod %in% j), ]
            if (length(x@fdrnbrcurve) != 0)
              x@fdrnbrcurve <-
                x@fdrnbrcurve[which(x@fdrnbrcurve$basemethod %in% j), ]
            if (length(x@fsrnbrcurve) != 0)
              x@fsrnbrcurve <-
                x@fsrnbrcurve[which(x@fsrnbrcurve$basemethod %in% j), ]
            if (length(x@corr) != 0)
              x@corr <- x@corr[which(x@corr$basemethod %in% j), ]
            if (length(x@scatter) != 0)
              x@scatter <- x@scatter[which(x@scatter$basemethod %in% j), ]
            if (length(x@overlap) != 0) {
              if (is(x@overlap, "data.frame")) {
                x@overlap <-
                  x@overlap[, which(colnames(x@overlap) %in% c(j, "truth")),
                            drop = FALSE]
              } else {
                x@overlap <-
                  lapply(x@overlap, function(w) {
                    w[, which(colnames(w) %in% c(j, "truth")), drop = FALSE]
                  })
              }
            }
            x
          })

setValidity("COBRAPlot",
            function(object) {
              msg <- NULL
              valid <- TRUE
              if (valid) TRUE else msg
            })

#' Convert an object to another class
#'
#' Convert object between \code{COBRAPerformance} and \code{COBRAPlot} classes.
#'
#' @docType methods
#' @name coerce
#' @rdname coerce
#' @aliases coerce coerce,COBRAPerformance,COBRAPlot-method coerce,
#'   COBRAPlot,COBRAPerformance-method
#'
#' @param from The object that is to be coerced into another class.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr")
#' cobraplot <- prepare_data_for_plot(cobraperf)
#'
#' ## Coerce COBRAPerformance object into COBRAPlot object
#' as(cobraperf, "COBRAPlot")
#'
#' ## Coerce COBRAPlot object into COBRAPerformance object
#' as(cobraplot, "COBRAPerformance")
setAs("COBRAPerformance", "COBRAPlot",
      function(from) {
        from <- update_cobraperformance(from, quiet = TRUE)
        .COBRAPlot(fdrtpr = from@fdrtpr, fdrtprcurve = from@fdrtprcurve,
                   fdrnbr = from@fdrnbr, fdrnbrcurve = from@fdrnbrcurve,
                   fsrnbr = from@fsrnbr, fsrnbrcurve = from@fsrnbrcurve, 
                   tpr = from@tpr, fpr = from@fpr, roc = from@roc,
                   fpc = from@fpc, onlyshared = from@onlyshared,
                   scatter = from@scatter, deviation = from@deviation,
                   overlap = from@overlap, plotcolors = "", corr = from@corr,
                   splv = from@splv, maxsplit = from@maxsplit, facetted = TRUE)
      })

#' @docType methods
#' @name coerce
#' @rdname coerce
#' @aliases coerce coerce,COBRAPlot,COBRAPerformance-method
#' @export
setAs("COBRAPlot", "COBRAPerformance",
      function(from) {
        from <- update_cobraperformance(from, quiet = TRUE)
        .COBRAPerformance(fdrtpr = from@fdrtpr, fdrtprcurve = from@fdrtprcurve,
                          fdrnbr = from@fdrnbr, fdrnbrcurve = from@fdrnbrcurve,
                          fsrnbr = from@fsrnbr, fsrnbrcurve = from@fsrnbrcurve, 
                          tpr = from@tpr, fpr = from@fpr, roc = from@roc,
                          fpc = from@fpc, deviation = from@deviation,
                          corr = from@corr, onlyshared = from@onlyshared,
                          scatter = from@scatter, overlap = from@overlap,
                          splv = from@splv, maxsplit = from@maxsplit)
      })

methods::setClassUnion("list_df", c("list", "data.frame"))

#' Update \code{COBRAPerformance} or \code{COBRAPlot} object to the current
#' version of the class format
#' 
#' Update a \code{COBRAPerformance} or \code{COBRAPlot} object generated by a
#' previous version of the package to the latest version.
#' 
#' @param object A \code{COBRAPerformance} or \code{COBRAPlot} object
#' @param quiet Set to TRUE to disable messages listing the modifications that
#'   are applied to the object
#' 
#' @return An updated \code{COBRAPerformance} or \code{COBRAPlot} object
#' 
#' @export
#' 
#' @author Charlotte Soneson
#' 
#' @examples
#' cobradata <- cobradata_example_sval
#' 
#' cobraperf <- calculate_performance(cobradata, 
#'                                    binary_truth = "status", 
#'                                    aspects = "fpr")
#' cobraperf <- update_cobraperformance(cobraperf)
update_cobraperformance <- function(object, quiet = FALSE) {
  mod <- FALSE
  if (!(.hasSlot(object, "fsrnbr"))) {
    object@fsrnbr <- data.frame()
    if (!quiet) message("Added empty fsrnbr slot to object")
    mod <- TRUE
  } 
  if (!(.hasSlot(object, "fsrnbrcurve"))) {
    object@fsrnbrcurve <- data.frame()
    if (!quiet) message("Adding empty fsrnbrcurve slot to object")
    mod <- TRUE
  } 
  if (!mod) {
    if (!quiet) message("Object up to date")
  }
  object
}

#' @rdname COBRAPerformance
#' @export
.COBRAPerformance <- setClass("COBRAPerformance",
                             slots = c(fdrtpr = "data.frame",
                                       fdrtprcurve = "data.frame",
                                       fdrnbr = "data.frame",
                                       fdrnbrcurve = "data.frame",
                                       fsrnbr = "data.frame",
                                       fsrnbrcurve = "data.frame",
                                       deviation = "data.frame",
                                       tpr = "data.frame", 
                                       fpr = "data.frame",
                                       roc = "data.frame",
                                       scatter = "data.frame",
                                       fpc = "data.frame", 
                                       overlap = "list_df",
                                       corr = "data.frame",
                                       maxsplit = "numeric",
                                       splv = "character",
                                       onlyshared = "logical"))

#' \code{COBRAPerformance} object and constructor
#'
#' The \code{COBRAPerformance} class holds various types of calculated
#' performance measures. Objects from this class are typically generated from
#' \code{COBRAData} objects by means of the function
#' \code{\link{calculate_performance}}.
#'
#' @param fdrtpr A data frame containing observed FDR and TPR values at various
#'   adjusted p-value thresholds.
#' @param fdrtprcurve A data frame containing observed FDR and TPR values for a
#'   (potentially large) number of cutoffs applied to a 'score' (that can be
#'   p-value, adjusted p-value or a more general score).
#' @param fdrnbr A data frame containing observed FDR and the number of features
#'   considered to be significant at various adjusted p-value thresholds.
#' @param fdrnbrcurve A data frame containing observed FDR and number of
#'   features considered to be significant for a (potentially large) number of
#'   cutoffs applied to a 'score' (that can be p-value, adjusted p-value or a
#'   more general score).
#' @param fsrnbr A data frame containing observed false sign rate (FSR) and the
#'   number of features considered to be significant at various s-value
#'   thresholds
#' @param fsrnbrcurve A data frame containing observed false sign rate (FSR) and
#'   number of features considered to be significant for a (potentially large)
#'   number of cutoffs applied to the s-value
#' @param tpr A data frame containing observed TPR values at various adjusted
#'   p-value thresholds.
#' @param fpr A data frame containing observed FPR values at various adjusted
#'   p-value thresholds.
#' @param roc A data frame containing observed FPR and TPR values  for a
#'   (potentially large) number of cutoffs applied to a 'score' (that can be
#'   p-value, adjusted p-value or a more general score), which can be used to
#'   generate a ROC curve.
#' @param scatter A data frame containing observed 'scores' (p-values, adjusted
#'   p-values or more general scores) and true scores, which can be used to
#'   generate scatter plots.
#' @param fpc A data frame containing observed numbers of false positive
#'   findings among the N top-ranked features (ranked by p-values, adjusted
#'   p-values or more general scores), for a (potentially large) number of Ns,
#'   which can be used to generate a false positive curve.
#' @param overlap A data frame or list of data frames with binary values
#'   indicating, for each of a number of methods and number of features, whether
#'   the method consider the feature 'positive' (significant, 1) or 'negative'
#'   (non-significant, 0). If it is a list of data frames, each list element
#'   corresponds to one level of a stratifying factor.
#' @param corr A data frame containing observed (Pearson and Spearman)
#'   correlation values between observed and true scores.
#' @param deviation A data frame containing deviations between observed scores
#'   and true scores.
#' @param maxsplit A numeric value indicating the largest number of levels to
#'   retain if the results have been stratified by an annotation.
#' @param splv A character string giving the name of the stratification factor,
#'   "none" if the results are not stratified.
#' @param onlyshared A logical value indicating whether only features shared
#'   between the results and the truth should be retained, or if all features
#'   present in the truth should be used.
#'
#' @include COBRAData.R
#' @return A \code{COBRAPerformance} object.
#'
#' @aliases COBRAPerformance COBRAPerformance-class
#'
#' @docType class
#'
#' @export
#' @rdname COBRAPerformance
#' @author Charlotte Soneson
#' @import methods
#' @examples
#' ## Empty COBRAPerformance object
#' COBRAPerformance()
COBRAPerformance <- function(fdrtpr = data.frame(), fdrtprcurve = data.frame(),
                             fdrnbr = data.frame(), fdrnbrcurve = data.frame(),
                             fsrnbr = data.frame(), fsrnbrcurve = data.frame(), 
                             tpr = data.frame(), fpr = data.frame(), splv = "",
                             roc = data.frame(), fpc = data.frame(),
                             deviation = data.frame(), onlyshared = NA,
                             overlap = data.frame(), maxsplit = NA_integer_,
                             corr = data.frame(), scatter = data.frame()) {
  ## TODO: add some checks of the input arguments

  .COBRAPerformance(fdrtpr = fdrtpr, fdrtprcurve = fdrtprcurve,
                    fsrnbr = fsrnbr, fsrnbrcurve = fsrnbrcurve, 
                    deviation = deviation, fdrnbr = fdrnbr,
                    fdrnbrcurve = fdrnbrcurve, scatter = scatter,
                    tpr = tpr, fpr = fpr, roc = roc, fpc = fpc, corr = corr,
                    overlap = overlap, splv = splv, maxsplit = maxsplit,
                    onlyshared = onlyshared)
}

setMethod("show", "COBRAPerformance", function(object) {
  cat("An object of class \"", class(object), "\"\n", sep = "")
  for (sl in slotNames(object)) {
    if (.hasSlot(object, sl)) {
      x <- slot(object, sl)
      cat("@", sl, "\n", sep = "")
      .printHead(x)
      cat("\n")
    }
  }
})

#' Accessor and replacement functions for \code{fdrtpr} slot
#'
#' Accessor and replacement functions for the \code{fdrtpr} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name fdrtpr
#' @rdname fdrtpr
#' @aliases fdrtpr fdrtpr,COBRAPerformance-method
#'   fdrtpr<-,COBRAPerformance,data.frame-method fdrtpr,COBRAPlot-method
#'   fdrtpr<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information about
#'   the observed FPR and TPR for each method and each stratification level, at
#'   various adjusted p-value thresholds.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information about the observed FPR and TPR
#'   for each method and each stratification level, at various adjusted p-value
#'   thresholds.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr")
#' head(fdrtpr(cobraperf))
setMethod("fdrtpr", signature(x = "COBRAPerformance"), function(x) x@fdrtpr)
#' @name fdrtpr
#' @rdname fdrtpr
#' @exportMethod "fdrtpr<-"
setReplaceMethod("fdrtpr", signature(x = "COBRAPerformance",
                                     value = "data.frame"),
                 function(x, value) {
                   x@fdrtpr <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{onlyshared} slot
#'
#' Accessor and replacement functions for the \code{onlyshared} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name onlyshared
#' @rdname onlyshared
#' @aliases onlyshared onlyshared,COBRAPerformance-method
#'   onlyshared<-,COBRAPerformance,logical-method onlyshared,COBRAPlot-method
#'   onlyshared<-,COBRAPlot,logical-method
#' @return The accessor function returns a logical indicating whether only
#'   features that are shared between result and truth are retained, or if all
#'   features in the truth are used.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A logical indicating whether only features that are shared
#'   between result and truth are retained, or if all features in the truth are
#'   used.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr")
#' head(onlyshared(cobraperf))
setMethod("onlyshared", signature(x = "COBRAPerformance"),
          function(x) x@onlyshared)
#' @name onlyshared
#' @rdname onlyshared
#' @exportMethod "onlyshared<-"
setReplaceMethod("onlyshared", signature(x = "COBRAPerformance",
                                         value = "logical"),
                 function(x, value) {
                   x@onlyshared <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{fdrtprcurve} slot
#'
#' Accessor and replacement functions for the \code{fdrtprcurve} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name fdrtprcurve
#' @rdname fdrtprcurve
#' @aliases fdrtprcurve fdrtprcurve,COBRAPerformance-method
#'   fdrtprcurve<-,COBRAPerformance,data.frame-method
#'   fdrtprcurve,COBRAPlot-method fdrtprcurve<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information
#'   necessary to generate curves of observed FDR vs TPR for each method and
#'   each stratification level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information necessary to generate curves of
#'   observed FDR vs TPR for each method and each stratification level.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtprcurve")
#' head(fdrtprcurve(cobraperf))
setMethod("fdrtprcurve", "COBRAPerformance", function(x) x@fdrtprcurve)
#' @name fdrtprcurve
#' @rdname fdrtprcurve
#' @exportMethod "fdrtprcurve<-"
setReplaceMethod("fdrtprcurve", signature(x = "COBRAPerformance",
                                          value = "data.frame"),
                 function(x, value) {
                   x@fdrtprcurve <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{deviation} slot
#'
#' Accessor and replacement functions for the \code{deviation} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name deviation
#' @rdname deviation
#' @aliases deviation deviation,COBRAPerformance-method
#'   deviation<-,COBRAPerformance,data.frame-method deviation,COBRAPlot-method
#'   deviation<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information
#'   necessary to plots of deviations between observed and true scores for each
#'   method and each stratification level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information necessary to plots of deviations
#'   between observed and true scores for each method and each stratification
#'   level.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
#'                                    aspects = "deviation")
#' head(deviation(cobraperf))
setMethod("deviation", "COBRAPerformance", function(x) x@deviation)
#' @name deviation
#' @rdname deviation
#' @exportMethod "deviation<-"
setReplaceMethod("deviation", signature(x = "COBRAPerformance",
                                        value = "data.frame"),
                 function(x, value) {
                   x@deviation <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{fdrnbr} slot
#'
#' Accessor and replacement functions for the \code{fdrnbr} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name fdrnbr
#' @rdname fdrnbr
#' @aliases fdrnbr fdrnbr,COBRAPerformance-method
#'   fdrnbr<-,COBRAPerformance,data.frame-method fdrnbr,COBRAPlot-method
#'   fdrnbr<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information about
#'   the observed FPR and the number of features called positive for each method
#'   and each stratification level, at various adjusted p-value thresholds.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information about the observed FPR and the
#'   number of features called positive for each method and each stratification
#'   level, at various adjusted p-value thresholds.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrnbr")
#' head(fdrnbr(cobraperf))
setMethod("fdrnbr", "COBRAPerformance", function(x) x@fdrnbr)
#' @name fdrnbr
#' @rdname fdrnbr
#' @exportMethod "fdrnbr<-"
setReplaceMethod("fdrnbr", signature(x = "COBRAPerformance",
                                     value = "data.frame"),
                 function(x, value) {
                   x@fdrnbr <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{fdrnbrcurve} slot
#'
#' Accessor and replacement functions for the \code{fdrnbrcurve} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name fdrnbrcurve
#' @rdname fdrnbrcurve
#' @aliases fdrnbrcurve fdrnbrcurve,COBRAPerformance-method
#'   fdrnbrcurve<-,COBRAPerformance,data.frame-method
#'   fdrnbrcurve,COBRAPlot-method fdrnbrcurve<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information
#'   necessary to generate curves of observed FDR vs number of features called
#'   positive for each method and each stratification level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information necessary to generate curves of
#'   observed FDR vs number of features called positive for each method and each
#'   stratification level.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrnbrcurve")
#' head(fdrnbrcurve(cobraperf))
setMethod("fdrnbrcurve", "COBRAPerformance", function(x) x@fdrnbrcurve)
#' @name fdrnbrcurve
#' @rdname fdrnbrcurve
#' @exportMethod "fdrnbrcurve<-"
setReplaceMethod("fdrnbrcurve", signature(x = "COBRAPerformance",
                                          value = "data.frame"),
                 function(x, value) {
                   x@fdrnbrcurve <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{fsrnbr} slot
#'
#' Accessor and replacement functions for the \code{fsrnbr} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name fsrnbr
#' @rdname fsrnbr
#' @aliases fsrnbr fsrnbr,COBRAPerformance-method
#'   fsrnbr<-,COBRAPerformance,data.frame-method fsrnbr,COBRAPlot-method
#'   fsrnbr<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information about
#'   the observed FSR and the number of features called positive for each method
#'   and each stratification level, at various s-value thresholds.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information about the observed FSR and the 
#'   number of features called positive for each method and each stratification 
#'   level, at various s-value thresholds. If the object does not have an fsrnbr
#'   slot (older versions of the class did not have this slot), an empty data
#'   frame is returned for simplicity.
#' @author Charlotte Soneson
#' @export
#' @examples
#' cobradata <- cobradata_example_sval
#' 
#' cobraperf <- calculate_performance(cobradata, 
#'                                    cont_truth = "logFC", 
#'                                    aspects = "fsrnbr")
#' head(fsrnbr(cobraperf))
setMethod("fsrnbr", "COBRAPerformance", function(x) {
  if (.hasSlot(x, "fsrnbr")) x@fsrnbr
  else {
    warning(paste0("Object doesn't have a slot fsrnbr. Please run ", 
                   "update_cobraperformance(). For consistency, I will ", 
                   "return an empty data.frame"))
    data.frame()
  }
})
#' @name fsrnbr
#' @rdname fsrnbr
#' @exportMethod "fsrnbr<-"
setReplaceMethod("fsrnbr", signature(x = "COBRAPerformance",
                                     value = "data.frame"),
                 function(x, value) {
                   x <- update_cobraperformance(x, quiet = FALSE)
                   x@fsrnbr <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{fsrnbrcurve} slot
#'
#' Accessor and replacement functions for the \code{fsrnbrcurve} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name fsrnbrcurve
#' @rdname fsrnbrcurve
#' @aliases fsrnbrcurve fsrnbrcurve,COBRAPerformance-method
#'   fsrnbrcurve<-,COBRAPerformance,data.frame-method
#'   fsrnbrcurve,COBRAPlot-method fsrnbrcurve<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information
#'   necessary to generate curves of observed FSR vs number of features called
#'   positive for each method and each stratification level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information necessary to generate curves of 
#'   observed FSR vs number of features called positive for each method and each
#'   stratification level. If the object does not have an fsrnbrcurve slot
#'   (older versions of the class did not have this slot), an empty data frame
#'   is returned for simplicity.
#' @author Charlotte Soneson
#' @export
#' @examples
#' cobradata <- cobradata_example_sval
#' 
#' cobraperf <- calculate_performance(cobradata, 
#'                                    cont_truth = "logFC", 
#'                                    aspects = "fsrnbrcurve")
#' head(fsrnbrcurve(cobraperf))
setMethod("fsrnbrcurve", "COBRAPerformance", function(x) {
  if (.hasSlot(x, "fsrnbrcurve")) x@fsrnbrcurve
  else {
    warning(paste0("Object doesn't have a slot fsrnbrcurve. Please run ", 
                   "update_cobraperformance(). For consistency, I will ", 
                   "return an empty data.frame"))
    data.frame()
  }
})
#' @name fsrnbrcurve
#' @rdname fsrnbrcurve
#' @exportMethod "fsrnbrcurve<-"
setReplaceMethod("fsrnbrcurve", signature(x = "COBRAPerformance",
                                          value = "data.frame"),
                 function(x, value) {
                   x <- update_cobraperformance(x, quiet = FALSE)
                   x@fsrnbrcurve <- value
                   if (validObject(x))
                     x
                 })


#' Accessor and replacement functions for \code{scatter} slot
#'
#' Accessor and replacement functions for the \code{scatter} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name scatter
#' @rdname scatter
#' @aliases scatter scatter,COBRAPerformance-method
#'   scatter<-,COBRAPerformance,data.frame-method scatter,COBRAPlot-method
#'   scatter<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information
#'   necessary to generate scatter plots of observed vs true values for each
#'   method and each stratification level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information necessary to generate scatter
#'   plots of observed vs true values for each method and each stratification
#'   level.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
#'                                    aspects = "scatter")
#' head(scatter(cobraperf))
setMethod("scatter", "COBRAPerformance", function(x) x@scatter)
#' @name scatter
#' @rdname scatter
#' @exportMethod "scatter<-"
setReplaceMethod("scatter", signature(x = "COBRAPerformance",
                                      value = "data.frame"),
                 function(x, value) {
                   x@scatter <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{tpr} slot
#'
#' Accessor and replacement functions for the \code{tpr} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name tpr
#' @rdname tpr
#' @aliases tpr tpr,COBRAPerformance-method
#'   tpr<-,COBRAPerformance,data.frame-method tpr,COBRAPlot-method
#'   tpr<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information about
#'   the observed TPR for each method and each stratification level, at various
#'   adjusted p-value thresholds.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information about the observed TPR for each
#'   method and each stratification level, at various adjusted p-value
#'   thresholds.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status", aspects = "tpr")
#' head(tpr(cobraperf))
setMethod("tpr", "COBRAPerformance", function(x) x@tpr)
#' @name tpr
#' @rdname tpr
#' @exportMethod "tpr<-"
setReplaceMethod("tpr", signature(x = "COBRAPerformance", value = "data.frame"),
                 function(x, value) {
                   x@tpr <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{fpr} slot
#'
#' Accessor and replacement functions for the \code{fpr} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name fpr
#' @rdname fpr
#' @aliases fpr fpr,COBRAPerformance-method
#'   fpr<-,COBRAPerformance,data.frame-method fpr,COBRAPlot-method
#'   fpr<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information about
#'   the observed FPR for each method and each stratification level, at various
#'   adjusted p-value thresholds.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information about the observed FPR for each
#'   method and each stratification level, at various adjusted p-value
#'   thresholds.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status", aspects = "fpr")
#' head(fpr(cobraperf))
setMethod("fpr", "COBRAPerformance", function(x) x@fpr)
#' @name fpr
#' @rdname fpr
#' @exportMethod fpr
setReplaceMethod("fpr", signature(x = "COBRAPerformance", value = "data.frame"),
                 function(x, value) {
                   x@fpr <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{roc} slot
#'
#' Accessor and replacement functions for the \code{roc} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name roc
#' @rdname roc
#' @aliases roc roc,COBRAPerformance-method
#'   roc<-,COBRAPerformance,data.frame-method roc,COBRAPlot-method
#'   roc<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information
#'   necessary to generate ROC curves for each method and each stratification
#'   level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information necessary to generate ROC curves
#'   for each method and each stratification level.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status", aspects = "roc")
#' head(roc(cobraperf))
setMethod("roc", "COBRAPerformance", function(x) x@roc)
#' @name roc
#' @rdname roc
#' @exportMethod "roc<-"
setReplaceMethod("roc", signature(x = "COBRAPerformance", value = "data.frame"),
                 function(x, value) {
                   x@roc <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{fpc} slot
#'
#' Accessor and replacement functions for the \code{fpc} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name fpc
#' @rdname fpc
#' @aliases fpc fpc,COBRAPerformance-method
#'   fpc<-,COBRAPerformance,data.frame-method fpc,COBRAPlot-method
#'   fpc<-,COBRAPlot,data.frame-method
#' @return The accessor function returns a data frame giving information
#'   necessary to generate false positive curves for each method and each
#'   stratification level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving information necessary to generate false
#'   positive curves for each method and each stratification level.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status", aspects = "fpc")
#' head(fpc(cobraperf))
setMethod("fpc", "COBRAPerformance", function(x) x@fpc)
#' @name fpc
#' @rdname fpc
#' @exportMethod "fpc<-"
setReplaceMethod("fpc", signature(x = "COBRAPerformance", value = "data.frame"),
                 function(x, value) {
                   x@fpc <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{corr} slot
#'
#' Accessor and replacement functions for the \code{corr} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name corr
#' @rdname corr
#' @aliases corr corr,COBRAPerformance-method
#'   corr<-,COBRAPerformance,data.frame-method
#' @return The accessor function returns a data frame giving correlation values
#'   for each method and each stratification level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame giving correlation values for each method and each
#'   stratification level.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
#'                                    aspects = "corr")
#' head(corr(cobraperf))
setMethod("corr", "COBRAPerformance", function(x) x@corr)
#' @name corr
#' @rdname corr
#' @exportMethod "corr<-"
setReplaceMethod("corr", signature(x = "COBRAPerformance",
                                   value = "data.frame"),
                 function(x, value) {
                   x@corr <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{overlap} slot
#'
#' Accessor and replacement functions for the \code{overlap} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name overlap
#' @rdname overlap
#' @aliases overlap overlap,COBRAPerformance-method
#'   overlap<-,COBRAPerformance,list_df-method overlap,COBRAPlot-method
#'   overlap<-,COBRAPlot,list_df-method
#' @return The accessor function returns a data frame or a list, giving
#'   information about which feature that are classified as 'positive' by each
#'   method and for each stratification level.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A data frame or a list, giving information about which feature
#'   that are classified as 'positive' by each method and for each
#'   stratification level.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "overlap")
#' head(overlap(cobraperf))
setMethod("overlap", "COBRAPerformance", function(x) x@overlap)
#' @name overlap
#' @rdname overlap
#' @exportMethod "overlap<-"
setReplaceMethod("overlap", signature(x = "COBRAPerformance",
                                      value = "list_df"),
                 function(x, value) {
                   x@overlap <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{splv} slot
#'
#' Accessor and replacement functions for the \code{splv} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name splv
#' @rdname splv
#' @aliases splv splv,COBRAPerformance-method
#'   splv<-,COBRAPerformance,character-method splv,COBRAPlot-method
#'   splv<-,COBRAPlot,character-method
#' @return The accessor function returns a character string giving the name of a
#'   feature annotation to use for stratification.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A character string giving the name of a feature annotation to
#'   use for stratification.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr", splv = "expr_cat")
#' splv(cobraperf)
setMethod("splv", "COBRAPerformance", function(x) x@splv)
#' @name splv
#' @rdname splv
#' @exportMethod "splv<-"
setReplaceMethod("splv", signature(x = "COBRAPerformance", value = "character"),
                 function(x, value) {
                   x@splv <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{maxsplit} slot
#'
#' Accessor and replacement functions for the \code{maxsplit} slot in a
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name maxsplit
#' @rdname maxsplit
#' @aliases maxsplit maxsplit,COBRAPerformance-method
#'   maxsplit<-,COBRAPerformance,numeric-method maxsplit,COBRAPlot-method
#'   maxsplit<-,COBRAPlot,numeric-method
#' @return The accessor function returns a numeric value giving the maximal
#'   number of strata to retain.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @param value A numeric value giving the maximal number of strata to retain.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr", splv = "expr_cat",
#'                                    maxsplit = 3)
#' maxsplit(cobraperf)
setMethod("maxsplit", "COBRAPerformance", function(x) x@maxsplit)
#' @name maxsplit
#' @rdname maxsplit
#' @exportMethod "maxsplit<-"
setReplaceMethod("maxsplit", signature(x = "COBRAPerformance",
                                       value = "numeric"),
                 function(x, value) {
                   x@maxsplit <- value
                   if (validObject(x))
                     x
                 })

#' Accessor function for basemethods
#'
#' Accessor function to extract the methods that are represented in an
#' \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name basemethods
#' @rdname basemethods
#' @aliases basemethods basemethods,COBRAPerformance-method
#'   basemethods,COBRAPlot-method
#' @return A character vector of all methods represented in the object.
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object.
#' @param ... Additional arguments.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtprcurve")
#' basemethods(cobraperf)
setMethod("basemethods", "COBRAPerformance", function(x) {
  x1 <- unlist(lapply(slotNames(x), function(w) {
    tryCatch(slot(x, w)$basemethod,
             error = function(e) NULL)
  }))
  if (is(x@overlap, "list"))
    x2 <- unlist(lapply(x@overlap, function(w) colnames(w)))
  else
    x2 <- colnames(x@overlap)
  unique(c(x1, x2))
})

#' Accessor function for stratification levels
#'
#' Accessor function to extract the stratification levels that are represented
#' in a \code{COBRAPerformance} or \code{COBRAPlot} object.
#'
#' @docType methods
#' @name stratiflevels
#' @rdname stratiflevels
#' @aliases stratiflevels stratiflevels,COBRAPerformance-method
#'   stratiflevels,COBRAPlot-method
#' @return A character vector of all stratification levels represented in the
#'   object
#'
#' @param x A \code{COBRAPerformance} or \code{COBRAPlot} object
#' @param ... Additional arguments
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "fdrtpr", splv = "expr_cat",
#'                                    maxsplit = 4)
#' stratiflevels(cobraperf)
setMethod("stratiflevels", "COBRAPerformance", function(x) {
  x1 <- unlist(lapply(slotNames(x), function(w) {
    tryCatch(as.character(slot(x, w)$splitval),
             error = function(e) NULL)
  }))
  if (is(x@overlap, "list"))
    x2 <- paste0(x@splv, ":", names(x@overlap))
  else
    x2 <- "overall"
  unique(c(x1, x2))
})

#' @docType methods
#' @name Extract
#' @rdname Extract
#' @aliases [ [,COBRAPerformance-method \S4method{[}{COBRAPerformance,ANY,ANY}
#'   \S4method{[}{COBRAPerformance,ANY,ANY,ANY}
#' @return A subset of the original object, of the same class
#' @export
setMethod("[", "COBRAPerformance",
          function(x, i = "missing", j, drop = "missing") {
            ## Update object if needed
            x <- update_cobraperformance(x, quiet = TRUE)
            
            if (length(intersect(j, basemethods(x))) == 0)
              stop("none of the provided method found in the object, ",
                   "no subsetting done")
            if (length(x@tpr) != 0 && length(intersect(j,x@tpr$basemethod)) > 0)
              x@tpr <- x@tpr[which(x@tpr$basemethod %in% j), ]
            else
              x@tpr <- data.frame()

            if (length(x@fpr) != 0 && length(intersect(j,x@fpr$basemethod)) > 0)
              x@fpr <- x@fpr[which(x@fpr$basemethod %in% j), ]
            else
              x@fpr <- data.frame()

            if (length(x@roc) != 0 && length(intersect(j,x@roc$basemethod)) > 0)
              x@roc <- x@roc[which(x@roc$basemethod %in% j), ]
            else
              x@roc <- data.frame()

            if (length(x@fpc) != 0 && length(intersect(j,x@fpc$basemethod)) > 0)
              x@fpc <- x@fpc[which(x@fpc$basemethod %in% j), ]
            else
              x@fpc <- data.frame()

            if (length(x@fdrtpr) != 0 &&
                length(intersect(j, x@fdrtpr$basemethod)) > 0)
              x@fdrtpr <- x@fdrtpr[which(x@fdrtpr$basemethod %in% j), ]
            else
              x@fdrtpr <- data.frame()

            if (length(x@fdrnbr) != 0 &&
                length(intersect(j, x@fdrnbr$basemethod)) > 0)
              x@fdrnbr <- x@fdrnbr[which(x@fdrnbr$basemethod %in% j), ]
            else
              x@fdrnbr <- data.frame()

            if (length(x@fsrnbr) != 0 &&
                length(intersect(j, x@fsrnbr$basemethod)) > 0)
              x@fsrnbr <- x@fsrnbr[which(x@fsrnbr$basemethod %in% j), ]
            else
              x@fsrnbr <- data.frame()

            if (length(x@deviation) != 0 &&
                length(intersect(j, x@deviation$basemethod)) > 0)
              x@deviation <- x@deviation[which(x@deviation$basemethod %in% j), ]
            else
              x@deviation <- data.frame()

            if (length(x@fdrtprcurve) != 0 &&
                length(intersect(j, x@fdrtprcurve$basemethod)) > 0)
              x@fdrtprcurve <-
                x@fdrtprcurve[which(x@fdrtprcurve$basemethod %in% j), ]
            else
              x@fdrtprcurve <- data.frame()

            if (length(x@fdrnbrcurve) != 0 &&
                length(intersect(j,x@fdrnbrcurve$basemethod)) > 0)
              x@fdrnbrcurve <-
                x@fdrnbrcurve[which(x@fdrnbrcurve$basemethod %in% j), ]
            else
              x@fdrnbrcurve <- data.frame()

            if (length(x@fsrnbrcurve) != 0 &&
                length(intersect(j,x@fsrnbrcurve$basemethod)) > 0)
              x@fsrnbrcurve <-
                x@fsrnbrcurve[which(x@fsrnbrcurve$basemethod %in% j), ]
            else
              x@fsrnbrcurve <- data.frame()

            if (length(x@corr) != 0 &&
                length(intersect(j, x@corr$basemethod)) > 0)
              x@corr <- x@corr[which(x@corr$basemethod %in% j), ]
            else
              x@corr <- data.frame()

            if (length(x@scatter) != 0 &&
                length(intersect(j, x@scatter$basemethod)) > 0)
              x@scatter <- x@scatter[which(x@scatter$basemethod %in% j), ]
            else
              x@scatter <- data.frame()

            if (length(x@overlap) != 0) {
              if (is(x@overlap, "data.frame")) {
                if (length(intersect(c(j, "truth"), colnames(x@overlap))) > 0) {
                  x@overlap <-
                    x@overlap[, which(colnames(x@overlap) %in% c(j, "truth")),
                              drop = FALSE]
                } else {
                  x@overlap <- data.frame()
                }
              } else {
                if (length(intersect(c(j, "truth"),
                                     colnames(x@overlap[[1]]))) > 0) {
                  x@overlap <-
                    lapply(x@overlap, function(w) {
                      w[, which(colnames(w) %in% c(j, "truth")), drop = FALSE]
                    })
                } else {
                  x@overlap <- list()
                }
              }
            }
            x
          })

## Validity
setValidity("COBRAPerformance",
            function(object) {
              msg <- NULL
              valid <- TRUE
              if (valid) TRUE else msg
            })

#' @rdname pval
#' @export
setGeneric("pval", function(x, ...) standardGeneric("pval"))
#' @rdname pval
#' @export
setGeneric("pval<-", function(x, ..., value) standardGeneric("pval<-"))

#' @rdname padj
#' @export
setGeneric("padj", function(x, ...) standardGeneric("padj"))
#' @rdname padj
#' @export
setGeneric("padj<-", function(x, ..., value) standardGeneric("padj<-"))

#' @rdname sval
#' @export
setGeneric("sval", function(x, ...) standardGeneric("sval"))
#' @rdname sval
#' @export
setGeneric("sval<-", function(x, ..., value) standardGeneric("sval<-"))

#' @rdname score
#' @export
setGeneric("score", function(x, ...) standardGeneric("score"))
#' @rdname score
#' @export
setGeneric("score<-", function(x, ..., value) standardGeneric("score<-"))

#' @rdname truth
#' @export
setGeneric("truth", function(x, ...) standardGeneric("truth"))
#' @rdname truth
#' @export
setGeneric("truth<-", function(x, ..., value) standardGeneric("truth<-"))


#' @rdname fdrtpr
#' @export
setGeneric("fdrtpr", function(x, ...) standardGeneric("fdrtpr"))
#' @rdname fdrtpr
#' @export
setGeneric("fdrtpr<-", function(x, ..., value) standardGeneric("fdrtpr<-"))

#' @rdname onlyshared
#' @export
setGeneric("onlyshared", function(x, ...) standardGeneric("onlyshared"))
#' @rdname onlyshared
#' @export
setGeneric("onlyshared<-", function(x, ..., value)
  standardGeneric("onlyshared<-"))

#' @rdname fdrtprcurve
#' @export
setGeneric("fdrtprcurve", function(x, ...) standardGeneric("fdrtprcurve"))
#' @rdname fdrtprcurve
#' @export
setGeneric("fdrtprcurve<-", function(x, ..., value)
  standardGeneric("fdrtprcurve<-"))

#' @rdname fdrnbr
#' @export
setGeneric("fdrnbr", function(x, ...) standardGeneric("fdrnbr"))
#' @rdname fdrnbr
#' @export
setGeneric("fdrnbr<-", function(x, ..., value) standardGeneric("fdrnbr<-"))

#' @rdname fdrnbrcurve
#' @export
setGeneric("fdrnbrcurve", function(x, ...) standardGeneric("fdrnbrcurve"))
#' @rdname fdrnbrcurve
#' @export
setGeneric("fdrnbrcurve<-", function(x, ..., value)
  standardGeneric("fdrnbrcurve<-"))

#' @rdname fsrnbr
#' @export
setGeneric("fsrnbr", function(x, ...) standardGeneric("fsrnbr"))
#' @rdname fsrnbr
#' @export
setGeneric("fsrnbr<-", function(x, ..., value) standardGeneric("fsrnbr<-"))

#' @rdname fsrnbrcurve
#' @export
setGeneric("fsrnbrcurve", function(x, ...) standardGeneric("fsrnbrcurve"))
#' @rdname fsrnbrcurve
#' @export
setGeneric("fsrnbrcurve<-", function(x, ..., value)
  standardGeneric("fsrnbrcurve<-"))

#' @rdname scatter
#' @export
setGeneric("scatter", function(x, ...) standardGeneric("scatter"))
#' @rdname scatter
#' @export
setGeneric("scatter<-", function(x, ..., value) standardGeneric("scatter<-"))

#' @rdname tpr
#' @export
setGeneric("tpr", function(x, ...) standardGeneric("tpr"))
#' @rdname tpr
#' @export
setGeneric("tpr<-", function(x, ..., value) standardGeneric("tpr<-"))

#' @rdname fpr
#' @export
setGeneric("fpr", function(x, ...) standardGeneric("fpr"))
#' @rdname fpr
#' @export
setGeneric("fpr<-", function(x, ..., value) standardGeneric("fpr<-"))

#' @rdname roc
#' @export
setGeneric("roc", function(x, ...) standardGeneric("roc"))
#' @rdname roc
#' @export
setGeneric("roc<-", function(x, ..., value) standardGeneric("roc<-"))

#' @rdname fpc
#' @export
setGeneric("fpc", function(x, ...) standardGeneric("fpc"))
#' @rdname fpc
#' @export
setGeneric("fpc<-", function(x, ..., value) standardGeneric("fpc<-"))

#' @rdname corr
#' @export
setGeneric("corr", function(x, ...) standardGeneric("corr"))
#' @rdname corr
#' @export
setGeneric("corr<-", function(x, ..., value) standardGeneric("corr<-"))

#' @rdname overlap
#' @export
setGeneric("overlap", function(x, ...) standardGeneric("overlap"))
#' @rdname overlap
#' @export
setGeneric("overlap<-", function(x, ..., value) standardGeneric("overlap<-"))

#' @rdname deviation
#' @export
setGeneric("deviation", function(x, ...) standardGeneric("deviation"))
#' @rdname deviation
#' @export
setGeneric("deviation<-", function(x, ..., value)
  standardGeneric("deviation<-"))

#' @rdname splv
#' @export
setGeneric("splv", function(x, ...) standardGeneric("splv"))
#' @rdname splv
#' @export
setGeneric("splv<-", function(x, ..., value) standardGeneric("splv<-"))

#' @rdname maxsplit
#' @export
setGeneric("maxsplit", function(x, ...) standardGeneric("maxsplit"))
#' @rdname maxsplit
#' @export
setGeneric("maxsplit<-", function(x, ..., value) standardGeneric("maxsplit<-"))

#' @rdname basemethods
#' @export
setGeneric("basemethods", function(x, ...) standardGeneric("basemethods"))

#' @rdname stratiflevels
#' @export
setGeneric("stratiflevels", function(x, ...) standardGeneric("stratiflevels"))


#' @rdname plotcolors
#' @export
setGeneric("plotcolors", function(x, ...) standardGeneric("plotcolors"))

#' @rdname plotcolors
#' @export
setGeneric("plotcolors<-", function(x, ..., value)
  standardGeneric("plotcolors<-"))

#' @rdname facetted
#' @export
setGeneric("facetted", function(x, ...) standardGeneric("facetted"))

#' @rdname facetted
#' @export
setGeneric("facetted<-", function(x, ..., value) standardGeneric("facetted<-"))

## ------------------- FDR or TPR or CORR ----------------------------- ##
#' @import ggplot2
#' @importFrom rlang .data
plot_fpr_tpr <- function(cobraplot, title, stripsize, titlecol, pointsize,
                         xaxisrange, aspc) {
  if (aspc == "FPR")
    plot_data <- fpr(cobraplot)
  else if (aspc == "TPR")
    plot_data <- tpr(cobraplot)
  else if (aspc %in% c("SPEARMAN", "PEARSON")) {
    plot_data <- corr(cobraplot)
    plot_data$thr <- as.character(plot_data$thr)
  }

  if (!(isTRUE(facetted(cobraplot)))) {
    plot_data$method <- plot_data$fullmethod
  }

  ## Number of thresholds
  nthr <- length(unique(plot_data$thr))
  
  pp <- 
    ggplot(plot_data, aes(x = .data[[aspc]], y = .data$method, 
                          group = .data$method)) +
    geom_point(size = pointsize + 1,
               aes(colour = .data$method, shape = .data$thr)) +
    scale_shape_manual(values = rep(19, nthr), guide = "none") + 
    scale_color_manual(values = plotcolors(cobraplot), name = "", limits = force) +
    xlim(xaxisrange[1], xaxisrange[2]) +
    plot_theme(stripsize = stripsize, titlecol = titlecol) +
    ggtitle(title)
  if (isTRUE(facetted(cobraplot))) {
    if (!is.finite(maxsplit(cobraplot)))
      msp <- length(unique(plot_data$splitval))
    else
      msp <- maxsplit(cobraplot)
    pp + facet_wrap(~ splitval, nrow = ceiling((msp + 1)/3))
  } else {
    pp
  }
}

#' Plot FPR
#'
#' Plot observed false positive rate (FPR) for given adjusted p-value
#' thresholds.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param pointsize A numeric value giving the size of the plot characters.
#' @param xaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the x-axis, respectively.
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status", aspects = "fpr")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_fpr(cobraplot, xaxisrange = c(0, 0.25))
plot_fpr <- function(cobraplot, title = "", stripsize = 15, titlecol = "black",
                     pointsize = 5, xaxisrange = c(0, 1)) {
  plot_fpr_tpr(cobraplot = cobraplot, title = title, stripsize = stripsize,
               titlecol = titlecol, pointsize = pointsize,
               xaxisrange = xaxisrange, aspc = "FPR")
}

#' Plot TPR
#'
#' Plot observed true positive rate (TPR) for given adjusted p-value thresholds.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param pointsize A numeric value giving the size of the plot characters.
#' @param xaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the x-axis, respectively.
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status", aspects = "tpr")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_tpr(cobraplot)
plot_tpr <- function(cobraplot, title = "", stripsize = 15, titlecol = "black",
                     pointsize = 5, xaxisrange = c(0, 1)) {
  plot_fpr_tpr(cobraplot = cobraplot, title = title, stripsize = stripsize,
               titlecol = titlecol, pointsize = pointsize,
               xaxisrange = xaxisrange, aspc = "TPR")
}

#' Plot correlations
#'
#' Plot correlations between observations and a continuous truth value.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param pointsize A numeric value giving the size of the plot characters.
#' @param xaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the x-axis, respectively.
#' @param corrtype A character string giving the type of correlation to show.
#'   Either "pearson" or "spearman".
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
#'                                    aspects = "corr")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_corr(cobraplot, corrtype = "spearman")
plot_corr <- function(cobraplot, title = "", stripsize = 15, titlecol = "black",
                      pointsize = 5, xaxisrange = c(-1, 1),
                      corrtype = "pearson") {
  plot_fpr_tpr(cobraplot = cobraplot, title = title, stripsize = stripsize,
               titlecol = titlecol, pointsize = pointsize,
               xaxisrange = xaxisrange, aspc = toupper(corrtype))
}

## ---------------------- ROC or FPC --------------------------------- ##
#' @import ggplot2
plot_roc_fpc <- function(cobraplot, title, stripsize, titlecol, xaxisrange,
                         yaxisrange, maxnfdc, aspc, linewidth) {
  if (aspc == "roc")
    plot_data <- roc(cobraplot)
  else if (aspc == "fpc")
    plot_data <- fpc(cobraplot)
  if (!(isTRUE(facetted(cobraplot)))) {
    plot_data$method <- plot_data$fullmethod
  }

  ## Number of colors/linetypes
  nlevs <- length(unique(plot_data$method))
  
  plotxx <- ifelse(aspc == "roc", "FPR", "topN")
  plotyy <- ifelse(aspc == "roc", "TPR", "FP")
  pp <- ggplot(plot_data, aes(x = .data[[plotxx]],
                              y = .data[[plotyy]],
                              group = .data$method, colour = .data$method)) +
    geom_path(linewidth = linewidth, aes(linetype = .data$method)) +
    scale_linetype_manual(values = rep("solid", nlevs), guide = "none") + 
    scale_color_manual(values = plotcolors(cobraplot), name = "", limits = force) +
    plot_theme(stripsize = stripsize, titlecol = titlecol) +
    xlim(ifelse(aspc == "roc", xaxisrange[1], 0),
         ifelse(aspc == "roc", xaxisrange[2], maxnfdc)) +
    ylim(ifelse(aspc == "roc", yaxisrange[1], 0),
         ifelse(aspc == "roc", yaxisrange[2],
                max(plot_data$FP[plot_data$topN <= maxnfdc]))) +
    ggtitle(title)
  if (isTRUE(facetted(cobraplot))) {
    if (!is.finite(maxsplit(cobraplot)))
      msp <- length(unique(plot_data$splitval))
    else
      msp <- maxsplit(cobraplot)
    pp + facet_wrap(~ splitval, nrow = ceiling((msp + 1)/3))
  } else {
    pp
  }
}

#' Plot ROC curves
#'
#' Plot receiver operating characteristics (ROC) curves.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param xaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the x-axis, respectively.
#' @param yaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the y-axis, respectively.
#' @param linewidth The line width used for plotting
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status", aspects = "roc")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_roc(cobraplot)
plot_roc <- function(cobraplot, title = "", stripsize = 15, titlecol = "black",
                     xaxisrange = c(0, 1), yaxisrange = c(0, 1), 
                     linewidth = 1) {
  plot_roc_fpc(cobraplot = cobraplot, title = title, stripsize = stripsize,
               titlecol = titlecol, xaxisrange = xaxisrange,
               yaxisrange = yaxisrange, maxnfdc = NULL, aspc = "roc",
               linewidth = linewidth)
}

#' Plot FP curves
#'
#' Plot false positive curves, indicating the number of false positives among
#' the top-ranked N variables, for varying values of N.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param maxnfdc A numeric value giving the largest N to consider.
#' @param linewidth The line width used for plotting
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status", aspects = "fpc")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_fpc(cobraplot, maxnfdc = 750)
plot_fpc <- function(cobraplot, title = "", stripsize = 15, titlecol = "black",
                     maxnfdc = 500, linewidth = 1) {
  plot_roc_fpc(cobraplot = cobraplot, title = title, stripsize = stripsize,
               titlecol = titlecol, xaxisrange = NULL, yaxisrange = NULL,
               maxnfdc = maxnfdc, aspc = "fpc", linewidth = linewidth)
}

## ------------------------- SCATTER --------------------------------- ##
#' Plot scatter plots
#'
#' Plot scatter plots, indicating the relationship between observed values and a
#' continuous truth.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param pointsize A numeric value giving the size of the plot characters.
#' @param doflip A logical indicating whether to flip the axes when results are
#'   stratified by an annotation. By default (\code{doflip = FALSE}),
#'   stratification levels are shown as columns and methods as rows in the plot.
#' @param dolog A logical indicating whether to log10-transform values before
#'   plotting.
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
#'                                    aspects = "scatter")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_scatter(cobraplot)
plot_scatter <- function(cobraplot, title = "", stripsize = 10,
                         titlecol = "black", pointsize = 3, doflip = FALSE,
                         dolog = FALSE) {
  plot_data <- scatter(cobraplot)

  if (isTRUE(dolog)) {
    plot_data$OBSERVATION[which(plot_data$OBSERVATION < 1e-305)] <- 0
    plot_data$TRUTH[which(plot_data$TRUTH < 1e-305)] <- 0
  }

  if (isTRUE(facetted(cobraplot))) {
    plot_data$fullmethod <- plot_data$method
  }
  pp <- ggplot(plot_data, aes(x = .data$OBSERVATION, y = .data$TRUTH,
                              colour = .data$fullmethod)) +
    geom_point(size = pointsize) +
    scale_color_manual(values = plotcolors(cobraplot), name = "", limits = force) +
    plot_theme(stripsize = stripsize, titlecol = titlecol) +
    ggtitle(title)
  if (isTRUE(facetted(cobraplot))) {
    if (isTRUE(doflip))
      pp <- pp + facet_grid(splitval ~ method)
    else
      pp <- pp + facet_grid(method ~ splitval)
  } else {
    pp <- pp + facet_wrap(~ method)
  }
  if (isTRUE(dolog))
    pp <- pp + scale_x_log10() + scale_y_log10()
  pp
}


## ------------------- FDRTPR, FDRNBR or FSRNBR ---------------------- ##
#' @import ggplot2
plot_fdrcurve <- function(cobraplot, title, stripsize, titlecol, pointsize,
                          xaxisrange, yaxisrange, plottype, aspc, linewidth) {
  if (aspc == "TPR") {
    plot_data_lines <- fdrtprcurve(cobraplot)
    plot_data_points <- fdrtpr(cobraplot)
    xasp <- "FDR"
    yasp <- aspc
  } else if (aspc == "NBR") {
    plot_data_lines <- fdrnbrcurve(cobraplot)
    plot_data_points <- fdrnbr(cobraplot)
    xasp <- "FDR"
    yasp <- aspc
  } else if (aspc == "FSR") {
    plot_data_lines <- fsrnbrcurve(cobraplot)
    plot_data_points <- fsrnbr(cobraplot)
    xasp <- "FSR"
    yasp <- "NBR"
  }
  
  if ("curve" %in% plottype && length(plot_data_lines) == 0) {
    message(paste0("The provided 'plottype' argument includes 'curve' but ", 
                   "the required values are not calculated. Please include ", 
                   "the appropriate aspect in calculate_performance()."))
    plottype <- setdiff(plottype, "curve")
  }
  if ("points" %in% plottype && length(plot_data_points) == 0) {
    message(paste0("The provided 'plottype' argument includes 'points' but ", 
                   "the required values are not calculated. Please include ", 
                   "the appropriate aspect in calculate_performance()."))
    plottype <- setdiff(plottype, "points")
  }
  pp <- ggplot()

  thresholds <- sort(unique(as.numeric(gsub("thr", "", plot_data_points$thr))))
  plot_data_points$method2.satis <- paste0(plot_data_points$method,
                                           plot_data_points$satis)

  if (!(isTRUE(facetted(cobraplot)))) {
    plot_data_points$method <- plot_data_points$fullmethod
    plot_data_lines$method <- plot_data_lines$fullmethod
    plot_data_points$method2.satis <- paste0(plot_data_points$fullmethod,
                                             plot_data_points$satis)
  }

  ## Number of colors/linetypes
  nlevs <- length(unique(plot_data_lines$method))
  ## Number of thresholds
  nthr <- length(unique(plot_data_points$thr))
  
  if ("curve" %in% plottype && "points" %in% plottype) {
    pp <- ggplot(plot_data_lines, aes(x = .data[[xasp]], y = .data[[yasp]],
                                      group = .data$method,
                                      colour = .data$method)) +
      geom_vline(xintercept = seq(0, xaxisrange[2], 0.1),
                 colour = "lightgrey", linetype = "dashed") +
      geom_vline(xintercept = thresholds, linetype = "dashed") +
      geom_path(linewidth = linewidth, aes(linetype = .data$method)) +
      scale_linetype_manual(values = rep("solid", nlevs), guide = "none") + 
      geom_point(data = plot_data_points, size = pointsize,
                 aes(fill = .data$method2.satis, colour = .data$method, 
                     shape = .data$thr),
                 stroke = 1) +
      scale_shape_manual(values = rep(21, nthr), guide = "none") + 
      scale_fill_manual(values = plotcolors(cobraplot), guide = "none",
                        name = "") +
      scale_color_manual(values = plotcolors(cobraplot), name = "", limits = force) +
      ylim(ifelse(yasp == "TPR", yaxisrange[1], 0),
           ifelse(yasp == "TPR", yaxisrange[2],
                  max(c(0, plot_data_lines[[yasp]][plot_data_lines[[xasp]] <=
                                                     xaxisrange[2]])))) +
      scale_x_continuous(breaks = c(thresholds,
                                    seq(0, xaxisrange[2], 0.1)),
                         labels = c(thresholds, "", 
                                    seq(0, xaxisrange[2], 0.1)[-1]),
                         limits = c(xaxisrange[1], xaxisrange[2])) +
      plot_theme(stripsize = stripsize, titlecol = titlecol) +
      ggtitle(title)
  } else if ("curve" %in% plottype) {
    pp <- ggplot(plot_data_lines,
                 aes(x = .data[[xasp]], y = .data[[yasp]],
                     group = .data$method, colour = .data$method)) +
      geom_path(linewidth = linewidth, aes(linetype = .data$method)) +
      scale_linetype_manual(values = rep("solid", nlevs), guide = "none") + 
      xlim(xaxisrange[1], xaxisrange[2]) +
      ylim(ifelse(yasp == "TPR", yaxisrange[1], 0),
           ifelse(yasp == "TPR", yaxisrange[2],
                  max(c(0, plot_data_lines[[yasp]][plot_data_lines[[xasp]] <=
                                                     xaxisrange[2]])))) +
      scale_color_manual(values = plotcolors(cobraplot), name = "", limits = force) +
      plot_theme(stripsize = stripsize, titlecol = titlecol) +
      ggtitle(title)
  } else if ("points" %in% plottype) {
    pp <- ggplot(plot_data_points, aes(x = .data[[xasp]], y = .data[[yasp]],
                                       group = .data$method)) +
      geom_vline(xintercept = seq(0, xaxisrange[2], 0.1),
                 colour = "lightgrey", linetype = "dashed") +
      geom_vline(xintercept = thresholds, linetype = "dashed") +
      geom_path(linewidth = linewidth, 
                aes(colour = .data$method, linetype = .data$method)) +
      scale_linetype_manual(values = rep("solid", nlevs), guide = "none") + 
      geom_point(size = pointsize,
                 aes(fill = .data$method2.satis, colour = .data$method,
                     shape = .data$thr),
                 stroke = 1) +
      scale_shape_manual(values = rep(21, nthr), guide = "none") + 
      scale_fill_manual(values = plotcolors(cobraplot), guide = "none",
                        name = "") +
      scale_color_manual(values = plotcolors(cobraplot), name = "", limits = force) +
      ylim(ifelse(yasp == "TPR", yaxisrange[1], 0),
           ifelse(yasp == "TPR", yaxisrange[2],
                  max(c(0, plot_data_lines[[yasp]][plot_data_lines[[xasp]] <=
                                                     xaxisrange[2]])))) +
      scale_x_continuous(breaks = c(thresholds,
                                    seq(0, xaxisrange[2], 0.1)),
                         labels = c(thresholds, "", 
                                    seq(0, xaxisrange[2], 0.1)[-1]),
                         limits = c(xaxisrange[1], xaxisrange[2])) +
      plot_theme(stripsize = stripsize, titlecol = titlecol) +
      ggtitle(title)
  }
  if (isTRUE(facetted(cobraplot))) {
    if (!is.finite(maxsplit(cobraplot))) {
      if (length(plot_data_lines) != 0)
        msp <- length(unique(plot_data_lines$splitval))
      else
        msp <- length(unique(plot_data_points$splitval))
    } else {
      msp <- maxsplit(cobraplot)
    }
    pp + facet_wrap(~ splitval, nrow = ceiling((msp + 1)/3))
  } else {
    pp
  }
}

#' Plot TPR vs FDR
#'
#' Plot observed true positive rate (TPR) vs observed false discovery rate
#' (FDR), for given adjusted p-value thresholds and/or as curves traced out by
#' considering all threshold values.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param pointsize A numeric value giving the size of the plot characters.
#' @param xaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the x-axis, respectively.
#' @param yaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the y-axis, respectively.
#' @param plottype A character vector giving the type of plot to construct. Can
#'   be any combination of the two elements "curve" and "points".
#' @param linewidth The line width used for plotting
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = c("fdrtpr", "fdrtprcurve"))
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_fdrtprcurve(cobraplot, plottype = c("curve", "points"))
plot_fdrtprcurve <- function(cobraplot, title = "", stripsize = 15,
                             titlecol = "black", pointsize = 5,
                             xaxisrange = c(0, 1), yaxisrange = c(0, 1),
                             plottype = c("curve", "points"),
                             linewidth = 1) {
  plot_fdrcurve(cobraplot = cobraplot, title = title, stripsize = stripsize,
                titlecol = titlecol, pointsize = pointsize,
                xaxisrange = xaxisrange, yaxisrange = yaxisrange,
                plottype = plottype, aspc = "TPR", linewidth = linewidth)
}

#' Plot number of significant features vs FDR
#'
#' Plot the number of features considered significant vs observed false
#' discovery rate (FDR), for given adjusted p-value thresholds and/or as curves
#' traced out by considering all threshold values.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param pointsize A numeric value giving the size of the plot characters.
#' @param xaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the x-axis, respectively.
#' @param plottype A character vector giving the type of plot to construct. Can
#'   be any combination of the two elements "curve" and "points".
#' @param linewidth The line width used for plotting
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = c("fdrnbr", "fdrnbrcurve"))
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_fdrnbrcurve(cobraplot, plottype = c("curve", "points"))
plot_fdrnbrcurve <- function(cobraplot, title = "", stripsize = 15,
                             titlecol = "black", pointsize = 5,
                             xaxisrange = c(0, 1),
                             plottype = c("curve", "points"),
                             linewidth = 1) {
  plot_fdrcurve(cobraplot = cobraplot, title = title, stripsize = stripsize,
                titlecol = titlecol, pointsize = pointsize,
                xaxisrange = xaxisrange, yaxisrange = NULL,
                plottype = plottype, aspc = "NBR",
                linewidth = linewidth)
}

#' Plot number of features with s-value below threshold vs FSR
#'
#' Plot the number of features with an s-value below a threshold vs the observed
#' false sign rate (FSR), for given adjusted p-value thresholds and/or as curves
#' traced out by considering all threshold values.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param pointsize A numeric value giving the size of the plot characters.
#' @param xaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the x-axis, respectively.
#' @param plottype A character vector giving the type of plot to construct. Can
#'   be any combination of the two elements "curve" and "points".
#' @param linewidth The line width used for plotting
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example_sval)
#' cobraperf <- calculate_performance(cobradata_example_sval,
#'                                    cont_truth = "logFC",
#'                                    aspects = c("fsrnbr", "fsrnbrcurve"))
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_fsrnbrcurve(cobraplot, plottype = c("curve", "points"))
plot_fsrnbrcurve <- function(cobraplot, title = "", stripsize = 15,
                             titlecol = "black", pointsize = 5,
                             xaxisrange = c(0, 1),
                             plottype = c("curve", "points"),
                             linewidth = 1) {
  plot_fdrcurve(cobraplot = cobraplot, title = title, stripsize = stripsize,
                titlecol = titlecol, pointsize = pointsize,
                xaxisrange = xaxisrange, yaxisrange = NULL,
                plottype = plottype, aspc = "FSR",
                linewidth = linewidth)
}

## ------------------------ OVERLAP ---------------------------------- ##
#' Plot Venn diagram
#'
#' Plot a Venn diagram showing the overlaps among sets of significant feature
#' for a given adjusted p-value threshold (see \code{thr_venn} argument of
#' \code{calculate_performance}). Optionally, the truth can be included
#' as a "perfect" method. Note that maximally five methods (including the truth,
#' if applicable) can be compared.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param ... Additional arguments to \code{limma::vennDiagram}.
#'
#' @return Nothing, displays a graph
#'
#' @export
#' @import limma
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "overlap")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_overlap(cobraplot)
plot_overlap <- function(cobraplot, ...) {
  overlap_table <- overlap(cobraplot)
  circle.col <- plotcolors(cobraplot)
  if (length(overlap_table) == 0)
    return(NULL)

  if (!is(overlap_table, "list")) {
    if (ncol(overlap_table) < 6) {
      if (ncol(overlap_table) == 5)
        cols <- rep(circle.col[colnames(overlap_table)], 2)[2:6]
      else
        cols <- circle.col[colnames(overlap_table)]
      vennDiagram(overlap_table, circle.col = cols, ...)
    } else {
      NULL
    }
  } else {
    ncl <- ceiling(sqrt(length(overlap_table)))
    nrw <- ceiling(length(overlap_table)/ncl)
    graphics::par(mfrow = c(nrw, ncl), mar = c(4, 1, 1, 1))
    for (i in seq_len(length(overlap_table))) {
      if (ncol(overlap_table[[i]]) < 6) {
        if (ncol(overlap_table[[i]]) == 5)
          cols <- rep(circle.col[colnames(overlap_table[[i]])], 2)[2:6]
        else
          cols <- circle.col[colnames(overlap_table[[i]])]
        vennDiagram(overlap_table[[i]], circle.col = cols,
                    main = paste0(splv(cobraplot), ":",
                                  names(overlap_table)[i]), ...)
      } else {
        NULL
      }
    }
    graphics::par(mfrow = c(1, 1), mar = c(5, 4, 4, 2))
  }
}

#' Create UpSet plots
#'
#' Generate UpSet plots showing the overlaps among sets of significant feature
#' for a given adjusted p-value threshold (see \code{thr_venn} argument of
#' \code{calculate_performance}). Optionally, the truth can be included
#' as a "perfect" method. Note that if the results are stratified, only one 
#' category at a time can be displayed.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param stratum If results are stratified, the category to plot results for.
#'   Can be numeric or categorical (the name of the category).
#' @param nsets The number of methods to include. By default, it is determined
#'   automatically from the \code{cobraplot} object.
#' @param nintersects The number of set intersections to display. By default, it
#'   is determined automatically from the \code{cobraplot} object.
#' @param sets.bar.color The colors to use for the bars in the UpSet plot. By
#'   default, they are extracted from the \code{plotcolors} slot of the
#'   \code{cobraplot} object.
#' @param ... Additional arguments to \code{UpSetR::upset}.
#'
#' @return Nothing, displays a graph
#' @references 
#' Lex and Gehlenborg (2014): Points of view: Sets and intersections. Nature
#' Methods 11, 779.
#' 
#' Lex et al (2014): UpSet: Visualization of intersecting sets. IEEE
#' Transactions on Visualization and Computer Graphics 20(12), 1983-1992.
#' @export
#' @import UpSetR
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example,
#'                                    binary_truth = "status",
#'                                    aspects = "overlap")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_upset(cobraplot)
#' plot_upset(cobraplot, order.by = "freq", decreasing = TRUE)
#' 
#' cobraperf <- calculate_performance(cobradata_example, 
#'                                    binary_truth = "status", 
#'                                    aspects = "overlap",
#'                                    splv = "expr_cat")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2", 
#'                                    incltruth = TRUE)
#' plot_upset(cobraplot, stratum = "[2.85e+00,1.45e+01)")
plot_upset <- function(cobraplot, stratum = NULL, nsets = NULL, 
                       nintersects = NULL, sets.bar.color = NULL, ...) {
  overlap_table <- overlap(cobraplot)
  if (length(overlap_table) == 0)
    return(NULL)
  
  if (!is(overlap_table, "list")) {
    plotorder <- 
      colnames(overlap_table)[order(colSums(overlap_table), 
                                    seq(seq_len(ncol(overlap_table))),
                                    decreasing = "true")]
    if (all(colSums(overlap_table) == 0)) return(NULL)
    if (is.null(nsets)) nsets <- ncol(overlap_table)
    if (is.null(nintersects)) nintersects <- 2^(ncol(overlap_table)) - 1
    if (is.null(sets.bar.color)) 
      sets.bar.color <- plotcolors(cobraplot)[plotorder]
    upset(overlap_table, nsets = nsets, nintersects = nintersects, 
          sets.bar.color = sets.bar.color, ...)
  } else {
    if (is.null(stratum)) stop("You must provide a stratum")
    plotorder <- 
      colnames(overlap_table[[stratum]])[
        order(colSums(overlap_table[[stratum]]), 
              seq(seq_len(ncol(overlap_table[[stratum]]))),
              decreasing = "true")]
    if (all(colSums(overlap_table[[stratum]]) == 0)) return(NULL)
    if (is.null(nsets)) nsets <- ncol(overlap_table[[stratum]])
    if (is.null(nintersects)) 
      nintersects <- 2^(ncol(overlap_table[[stratum]])) - 1
    if (is.null(sets.bar.color)) 
      sets.bar.color <- plotcolors(cobraplot)[plotorder]
    upset(overlap_table[[stratum]], nsets = nsets, nintersects = nintersects,
          sets.bar.color = sets.bar.color, ...)
  }
}

## -------------------------- Deviation ------------------------------ ##
#' Plot deviations
#'
#' Plot the deviations between observed scores and the continuous truth
#' variable.
#'
#' @param cobraplot A \code{COBRAPlot} object.
#' @param title A character string giving the title of the plot.
#' @param stripsize A numeric value giving the size of the strip text, when the
#'   results are stratified by an annotation.
#' @param titlecol A character string giving the color of the title.
#' @param xaxisrange A numeric vector with two elements, giving the lower and
#'   upper boundary of the x-axis, respectively.
#' @param plottype Either "boxplot" or "violinplot", indicating what type of
#'   plot to make.
#' @param dojitter A logical indicating whether to include jittered data points
#'   or not.
#' @param transf A character indicating the transformation to apply to the
#'   deviations before plotting. Must be one of "raw", "absolute" or "squared"
#'
#' @return A ggplot object
#'
#' @import ggplot2
#' @export
#' @author Charlotte Soneson
#' @examples
#' data(cobradata_example)
#' cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
#'                                    aspects = "deviation")
#' cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
#'                                    incltruth = TRUE)
#' plot_deviation(cobraplot)
plot_deviation <- function(cobraplot, title = "", stripsize = 15,
                           titlecol = "black", xaxisrange = NULL,
                           plottype = "boxplot",
                           dojitter = TRUE, transf = "raw") {
  stopifnot(transf %in% c("raw", "absolute", "squared"))
  plot_data <- deviation(cobraplot)
  if (transf == "absolute")
    plot_data$absDEVIATION <- abs(plot_data$DEVIATION)
  else if (transf == "squared")
    plot_data$sqDEVIATION <- plot_data$DEVIATION^2

  if (!(isTRUE(facetted(cobraplot)))) {
    plot_data$method <- plot_data$fullmethod
  }

  plotyy <- ifelse(transf == "raw", "DEVIATION",
                   ifelse(transf == "absolute",
                          "absDEVIATION", "sqDEVIATION"))
  pp <- ggplot(
    plot_data, aes(x = .data$method,
                   y = .data[[plotyy]],
                   group = .data$method, colour = .data$method)) +
    coord_flip() +
    scale_color_manual(values = plotcolors(cobraplot), name = "", limits = force) +
    plot_theme(stripsize = stripsize, titlecol = titlecol) +
    ggtitle(title)
  if (plottype == "boxplot") {
    if (isTRUE(dojitter))
      pp <- pp + geom_boxplot(outlier.size = 0)
    else
      pp <- pp + geom_boxplot()
  }
  else if (plottype == "violinplot")
    pp <- pp + geom_violin()
  if (isTRUE(dojitter))
    pp <- pp + geom_jitter(position = position_jitter(width = 0.1, height = 0),
                           size = 1.5)
  if (isTRUE(facetted(cobraplot))) {
    if (!is.finite(maxsplit(cobraplot)))
      msp <- length(unique(plot_data$splitval))
    else
      msp <- maxsplit(cobraplot)
    pp <- pp + facet_wrap(~ splitval,
                          nrow = ceiling((msp + 1)/3))
  }
  if (!is.null(xaxisrange))
    pp <- pp + ylim(xaxisrange[1], xaxisrange[2])
  pp
}

## Test that plot functions do the right thing

library(iCOBRA)
context("Check plot functions")

local({
  ## Test that overlap plotting with too many methods give NULL
  tmp <- cobradata_example_sval
  padj(tmp) <- cbind(padj(tmp), padj(tmp), padj(tmp))
  ib1 <- calculate_performance(tmp, aspects = "overlap")
  ib1 <- prepare_data_for_plot(ib1)
  expect_is(plot_overlap(ib1), "NULL")

  ib1 <- calculate_performance(cobradata_example_sval, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr",
                                           "fdrnbrcurve", "tpr", "fpr",
                                           "roc", "fpc", "overlap", "corr",
                                           "scatter", "deviation", 
                                           "fsrnbr", "fsrnbrcurve"),
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)

  ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                colorscheme = "hue_pal", facetted = TRUE,
                                incltruth = TRUE)

  test_that("Plot functions return ggplot objects", {
    expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_deviation(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fdrnbrcurve(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fsrnbrcurve(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fpc(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fpr(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_roc(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_scatter(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_scatter(ibp1, dolog = TRUE)))
    expect_is(plot_overlap(ibp1), "NULL")
    expect_is(plot_upset(ibp1), "upset")
  })

  ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = FALSE,
                                colorscheme = "hue_pal", facetted = FALSE,
                                incltruth = TRUE)

  test_that("Plot functions return ggplot objects if not facetted", {
    expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_deviation(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fdrnbrcurve(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fsrnbrcurve(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fpc(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_fpr(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_roc(ibp1)))
    expect_true(ggplot2::is_ggplot(plot_scatter(ibp1)))
    expect_is(plot_overlap(ibp1), "NULL")
    expect_is(plot_upset(ibp1), "upset")
  })
})

test_that("Plot functions return ggplot objects for empty input", {
  ib1 <- calculate_performance(cobradata_example_sval, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("overlap", "tpr"),
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)

  ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                colorscheme = c("blue", "green"), facetted = TRUE,
                                incltruth = FALSE)

  expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1, plottype = "curve")))
  expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1, plottype = "points")))
  expect_true(ggplot2::is_ggplot(plot_fsrnbrcurve(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fsrnbrcurve(ibp1, plottype = "curve")))
  expect_true(ggplot2::is_ggplot(plot_fsrnbrcurve(ibp1, plottype = "points")))
  expect_true(ggplot2::is_ggplot(plot_fpr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_roc(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_scatter(ibp1)))
  expect_is(plot_overlap(ibp1), "NULL")
  expect_is(plot_upset(ibp1), "upset")
})

local({
  ib1 <- calculate_performance(cobradata_example_sval, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("tpr"),
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)

  expect_warning({
    ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                  colorscheme = c("blue"), facetted = TRUE,
                                  incltruth = TRUE)
  }, "too few colors provided, 1 random colors will be added")

  test_that("Plot functions return ggplot objects with only one color", {
    expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
    expect_equal(length(plotcolors(ibp1)), 12)
    expect_equal(length(unique(plotcolors(ibp1))), 3)
  })

  expect_warning({
    ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                  colorscheme = c("blue", "green", "yellow", "red"),
                                  facetted = TRUE, incltruth = TRUE)
  }, "too many colors supplied, only a subset will be used")

  test_that("Plot functions return ggplot objects with too many colors", {
    expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
    expect_equal(length(plotcolors(ibp1)), 12)
    expect_equal(length(unique(plotcolors(ibp1))), 3)
  })
})

local({
  ib1 <- calculate_performance(cobradata_example_sval, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtprcurve"),
                               thrs = c(0.05), splv = "none", onlyshared = FALSE,
                               thr_venn = 0.05)

  expect_warning({
    ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                  colorscheme = c("blue", "green"), facetted = TRUE,
                                  incltruth = TRUE)
  }, "too few colors provided, 1 random colors will be added")

  test_that("Plot functions return ggplot objects with too few colors", {
    expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
    expect_equal(length(plotcolors(ibp1)), 18)
    expect_equal(length(unique(plotcolors(ibp1))), 4)
  })

  ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                colorscheme = c("blue", "green", "red"), facetted = TRUE,
                                incltruth = TRUE)

  test_that("Plot functions return ggplot objects with right nbr colors", {
    expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
    expect_equal(length(plotcolors(ibp1)), 18)
    expect_equal(length(unique(plotcolors(ibp1))), 4)
  })
})

test_that("Plot functions return ggplot objects after stratification", {
  ## Facetted, stratified
  ib1 <- calculate_performance(cobradata_example_sval, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr",
                                           "fdrnbrcurve", "tpr", "fpr",
                                           "roc", "fpc", "overlap", "corr",
                                           "scatter", "deviation"),
                               thrs = c(0.05), splv = "expr_cat", onlyshared = FALSE,
                               thr_venn = 0.05, maxsplit = 3)

  ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                colorscheme = "rainbow", facetted = TRUE,
                                incltruth = FALSE)

  ibp2 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                colorscheme = "rainbow", facetted = TRUE,
                                incltruth = TRUE)

  expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1, transf = "absolute")))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1, transf = "squared")))
  expect_error(plot_deviation(ibp1, transf = "square"))
  expect_true(ggplot2::is_ggplot(plot_fdrnbrcurve(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fsrnbrcurve(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fpc(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fpr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_roc(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_scatter(ibp1)))
  expect_is(plot_overlap(ibp1), "list")
  expect_is(plot_overlap(ibp2), "list")
  expect_is(plot_upset(ibp2, stratum = "[   0.000,   0.362)"), "upset")
})

test_that("Plot functions return ggplot objects after stratification", {
  ## Facetted, stratified
  ib1 <- calculate_performance(cobradata_example_sval, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr",
                                           "fdrnbrcurve", "tpr", "fpr",
                                           "roc", "fpc", "overlap", "corr",
                                           "scatter", "deviation"),
                               thrs = c(0.05), splv = "expr_cat", onlyshared = FALSE,
                               thr_venn = 0.05, maxsplit = Inf)

  ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                colorscheme = "rainbow", facetted = TRUE,
                                incltruth = FALSE)

  expect_true(ggplot2::is_ggplot(plot_tpr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1, transf = "absolute")))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1, transf = "squared")))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1, transf = "squared", dojitter = FALSE)))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1, transf = "squared", plottype = "violinplot")))
  expect_true(ggplot2::is_ggplot(plot_deviation(ibp1, transf = "squared", xaxisrange = c(0.2, 0.5))))
  expect_error(plot_deviation(ibp1, transf = "square"))
  expect_true(ggplot2::is_ggplot(plot_fsrnbrcurve(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fdrnbrcurve(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1, plottype = "curve")))
  expect_true(ggplot2::is_ggplot(plot_fdrtprcurve(ibp1, plottype = "points")))
  expect_true(ggplot2::is_ggplot(plot_fpc(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_fpr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_corr(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_roc(ibp1)))
  expect_true(ggplot2::is_ggplot(plot_scatter(ibp1)))
  expect_is(plot_overlap(ibp1), "list")
})

test_that("maxsplit=Inf works as expected", {
  ## Facetted, stratified
  cde <- cobradata_example_sval
  truth(cde)$expr_cat <- factor(truth(cde)$expr_cat)
  ib1 <- calculate_performance(cde, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr"),
                               thrs = 0.05, splv = "expr_cat", onlyshared = FALSE,
                               thr_venn = 0.05, maxsplit = Inf)
  expect_equal(c("overall", paste0("expr_cat:", levels(truth(cde)$expr_cat))),
               levels(fdrtpr(ib1)$splitval))
  ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                colorscheme = "rainbow", facetted = TRUE,
                                incltruth = FALSE)
  expect_equal(c("overall", paste0("expr_cat:", levels(truth(cde)$expr_cat))),
               levels(fdrtpr(ibp1)$splitval))

  truth(cde)$expr_cat <- factor(truth(cde)$expr_cat,
                                levels = levels(truth(cde)$expr_cat)[c(3, 4, 2, 1)])
  ib1 <- calculate_performance(cde, binary_truth = "status",
                               cont_truth = "logFC",
                               aspects = c("fdrtpr"),
                               thrs = 0.05, splv = "expr_cat", onlyshared = FALSE,
                               thr_venn = 0.05, maxsplit = Inf)
  expect_equal(c("overall", paste0("expr_cat:", levels(truth(cde)$expr_cat))),
               levels(fdrtpr(ib1)$splitval))
  ibp1 <- prepare_data_for_plot(ib1, keepmethods = NULL, incloverall = TRUE,
                                colorscheme = "rainbow", facetted = TRUE,
                                incltruth = FALSE)
  expect_equal(c("overall", paste0("expr_cat:", levels(truth(cde)$expr_cat))),
               levels(fdrtpr(ibp1)$splitval))
})

#' Update \code{COBRAData} object to the current version of the class 
#' format
#' 
#' Update a \code{COBRAData} object generated by a previous version of the
#' package to the latest version.
#' 
#' @param object A \code{COBRAData} object
#' @param quiet Set to TRUE to disable messages listing the modifications that
#'   are applied to the object
#' 
#' @return An updated \code{COBRAData} object
#' 
#' @export
#' 
#' @author Charlotte Soneson
#' 
#' @examples
#' ## Generate COBRAData object
#' set.seed(123)
#' pval <- data.frame(m1 = runif(100), m2 = runif(100),
#'                    row.names = paste0("F", 1:100))
#' truth <- data.frame(status = round(runif(100)),
#'                     row.names = paste0("F", 1:100))
#' cobradata <- COBRAData(pval = pval, truth = truth)
#' 
#' ## Update object if needed
#' cobradata <- update_cobradata(cobradata)
update_cobradata <- function(object, quiet = FALSE) {
  mod <- FALSE
  if (!(.hasSlot(object, "sval"))) {
    object@sval <- data.frame()
    if (!quiet) message("Adding empty sval slot to object")
    mod <- TRUE
  } 
  if (!mod) {
    if (!quiet) message("Object up to date")
  }
  object
}

#' @rdname COBRAData
#' @export
.COBRAData <- setClass("COBRAData",
                      slots = c(pval = "data.frame", padj = "data.frame",
                                sval = "data.frame", score = "data.frame", 
                                truth = "data.frame"))

#' \code{COBRAData} object and constructor
#'
#' The \code{COBRAData} class contains slots to hold calculated p-values,
#' adjusted p-values and general 'scores' for a set of features, as well as 
#' s-values (see Stephens (2017)). The slots can contain values from multiple
#' methods, and each method can contribute to one or more slots. The class also
#' contains a slot giving the 'truth' (a binary assignment and/or a continuous
#' score) for each feature, as well as additional annotations that can be used
#' to stratify the performance calculations.
#'
#' If adjusted p-values are missing for some methods, for which nominal p-values
#' are available, the adjusted p-values can be calculated using the
#' \code{\link{calculate_adjp}} function.
#'
#' The text files generated by \code{COBRAData_to_text} can be used as input to
#' \code{iCOBRAapp}, when it is called without an input argument.
#'
#' @param pval A data frame with features as rows and methods as columns,
#'   containing nominal p-values. Missing values (\code{NA}s) are allowed. The
#'   row names should be feature names.
#' @param padj A data frame with features as rows and methods as columns,
#'   containing adjusted p-values. Missing values (\code{NA}s) are allowed. The
#'   row names should be feature names.
#' @param score A data frame with features as rows and methods as columns,
#'   containing generic scores. In case of comparison to a binary truth, larger
#'   values of the scores should correspond to 'more significant' features.
#'   Missing values (\code{NA}s) are allowed. The row names should be feature
#'   names.
#' @param sval A data frame with features as rows and methods as columns,
#'   containing s-values (analogous to q-values, but for sign errors, see
#'   Stephens (2017)). Missing values (\code{NA}s) are allowed. The row names
#'   should be feature names.
#' @param truth A data frame with features as rows columns containing feature
#'   annotations such as, e.g., binary and continuous truths and additional
#'   annotations that can be used to stratify the performance calculations. The
#'   row names should be feature names.
#' @param object_to_extend A \code{COBRAData} object to extend with the
#'   provided information.
#'
#' @aliases COBRAData COBRAData-class
#' @return \code{COBRAData} and \code{COBRAData_from_text} return a
#'   \code{COBRAData} object.
#'
#' @docType class
#'
#' @export
#' @rdname COBRAData
#' @author Charlotte Soneson
#' @examples
#' ## Empty COBRAData object:
#' COBRAData()
#'
#' ## COBRAData object from individual data frames
#' set.seed(123)
#' pval <- data.frame(m1 = runif(100), m2 = runif(100),
#'                    row.names = paste0("F", 1:100))
#' truth <- data.frame(status = round(runif(100)),
#'                     row.names = paste0("F", 1:100))
#' cobradata <- COBRAData(pval = pval, truth = truth)
COBRAData <- function(pval = data.frame(), padj = data.frame(),
                      score = data.frame(), sval = data.frame(), 
                      truth = data.frame(), object_to_extend = NULL) {

  ## All objects should have sensible row names
  if (nrow(pval) > 0 && is.numeric(attr(pval, "row.names"))) 
    warning("'pval' appears to have non-informative (numeric) row names.", 
            "Note that row names will be used to match values across ",
            "the slots in the COBRAData object, so they should be ",
            "provided in a consistent form across all the slots.")
  if (nrow(padj) > 0 && is.numeric(attr(padj, "row.names"))) 
    warning("'padj' appears to have non-informative (numeric) row names.", 
            "Note that row names will be used to match values across ",
            "the slots in the COBRAData object, so they should be ",
            "provided in a consistent form across all the slots.")
  if (nrow(score) > 0 && is.numeric(attr(score, "row.names"))) 
    warning("'score' appears to have non-informative (numeric) row names.", 
            "Note that row names will be used to match values across ",
            "the slots in the COBRAData object, so they should be ",
            "provided in a consistent form across all the slots.")
  if (nrow(sval) > 0 && is.numeric(attr(sval, "row.names"))) 
    warning("'sval' appears to have non-informative (numeric) row names.", 
            "Note that row names will be used to match values across ",
            "the slots in the COBRAData object, so they should be ",
            "provided in a consistent form across all the slots.")
  if (nrow(truth) > 0 && is.numeric(attr(truth, "row.names"))) 
    warning("'truth' appears to have non-informative (numeric) row names.", 
            "Note that row names will be used to match values across ",
            "the slots in the COBRAData object, so they should be ",
            "provided in a consistent form across all the slots.")
  
  if (!(is.null(object_to_extend))) {
    if (!(is(object_to_extend, "COBRAData"))) {
      stop("object_to_extend must be a COBRAData object")
    } else {
      ## Update object if needed
      object_to_extend <- update_cobradata(object_to_extend, quiet = TRUE)
      
      ## Merge provided pval data frame with existing pval data frame
      if (length(object_to_extend@pval) != 0) {
        if (length(pval) != 0) {
          sds <- setdiff(colnames(pval),
                         colnames(object_to_extend@pval))
          if (length(sds) > 0) {
            pval <- pval[, setdiff(colnames(pval),
                                   colnames(object_to_extend@pval)),
                         drop = FALSE]
            object_to_extend@pval$feature_names_tmp <-
              rownames(object_to_extend@pval)
            pval$feature_names_tmp <- rownames(pval)
            pval <- as.data.frame(dplyr::full_join(object_to_extend@pval, pval,
                                                   by = "feature_names_tmp"))
            rownames(pval) <- pval$feature_names_tmp
            pval$feature_names_tmp <- NULL
          } else {
            pval <- object_to_extend@pval
          }
          message(length(setdiff(rownames(pval),
                                 rownames(object_to_extend@pval))),
                  " new features and ",
                  length(sds), " new methods added to pval table")
        } else {
          pval <- object_to_extend@pval
        }
      } else {
        message(nrow(pval), " new features and ",
                ncol(pval), " new methods added to pval table")
      }

      ## Merge provided padj data frame with existing padj data frame
      if (length(object_to_extend@padj) != 0) {
        if (length(padj) != 0) {
          sds <- setdiff(colnames(padj),
                         colnames(object_to_extend@padj))
          if (length(sds) > 0) {
            padj <- padj[, setdiff(colnames(padj),
                                   colnames(object_to_extend@padj)),
                         drop = FALSE]
            object_to_extend@padj$feature_names_tmp <-
              rownames(object_to_extend@padj)
            padj$feature_names_tmp <- rownames(padj)
            padj <- as.data.frame(dplyr::full_join(object_to_extend@padj, padj,
                                                   by = "feature_names_tmp"))
            rownames(padj) <- padj$feature_names_tmp
            padj$feature_names_tmp <- NULL
          } else {
            padj <- object_to_extend@padj
          }
          message(length(setdiff(rownames(padj),
                                 rownames(object_to_extend@padj))),
                  " new features and ",
                  length(sds), " new methods added to padj table")
        } else {
          padj <- object_to_extend@padj
        }
      } else {
        message(nrow(padj), " new features and ",
                ncol(padj), " new methods added to padj table")
      }

      ## Merge provided sval data frame with existing sval data frame
      if (length(object_to_extend@sval) != 0) {
        if (length(sval) != 0) {
          sds <- setdiff(colnames(sval),
                         colnames(object_to_extend@sval))
          if (length(sds) > 0) {
            sval <- sval[, setdiff(colnames(sval),
                                   colnames(object_to_extend@sval)),
                         drop = FALSE]
            object_to_extend@sval$feature_names_tmp <-
              rownames(object_to_extend@sval)
            sval$feature_names_tmp <- rownames(sval)
            sval <- as.data.frame(dplyr::full_join(object_to_extend@sval, sval,
                                                   by = "feature_names_tmp"))
            rownames(sval) <- sval$feature_names_tmp
            sval$feature_names_tmp <- NULL
          } else {
            sval <- object_to_extend@sval
          }
          message(length(setdiff(rownames(sval),
                                 rownames(object_to_extend@sval))),
                  " new features and ",
                  length(sds), " new methods added to sval table")
        } else {
          sval <- object_to_extend@sval
        }
      } else {
        message(nrow(sval), " new features and ",
                ncol(sval), " new methods added to sval table")
      }
      
      ## Merge provided score data frame with existing score data frame
      if (length(object_to_extend@score) != 0) {
        if (length(score) != 0) {
          sds <- setdiff(colnames(score),
                         colnames(object_to_extend@score))
          if (length(sds) > 0) {
            score <- score[, setdiff(colnames(score),
                                     colnames(object_to_extend@score)),
                           drop = FALSE]
            object_to_extend@score$feature_names_tmp <-
              rownames(object_to_extend@score)
            score$feature_names_tmp <- rownames(score)
            score <- as.data.frame(dplyr::full_join(object_to_extend@score,
                                                    score,
                                                    by = "feature_names_tmp"))
            rownames(score) <- score$feature_names_tmp
            score$feature_names_tmp <- NULL
          } else {
            score <- object_to_extend@score
          }
          message(length(setdiff(rownames(score),
                                 rownames(object_to_extend@score))),
                  " new features and ",
                  length(sds), " new methods added to score table")
        } else {
          score <- object_to_extend@score
        }
      } else {
        message(nrow(score), " new features and ",
                ncol(score), " new methods added to score table")
      }

      ## Merge provided truth data frame with existing truth data frame
      if (length(object_to_extend@truth) != 0) {
        if (length(truth) != 0) {
          truth$feature_names_tmp <- rownames(truth)
          object_to_extend@truth$feature_names_tmp <-
            rownames(object_to_extend@truth)
          tm <- as.data.frame(dplyr::full_join(object_to_extend@truth, truth))
          if (any(duplicated(tm$feature_names_tmp)))
            stop("problem merging truth tables, likely due to ",
                 "inconsistent annotations for one or more features")
          truth <- data.frame(tm, stringsAsFactors = FALSE)
          rownames(truth) <- truth$feature_names_tmp
          truth$feature_names_tmp <- NULL
        } else {
          truth <- object_to_extend@truth
        }
        message(length(setdiff(rownames(truth),
                               rownames(object_to_extend@truth))),
                " new features and ",
                length(setdiff(colnames(truth),
                               colnames(object_to_extend@truth))),
                " new annotations added to truth table")
      } else {
        message(nrow(truth), " new features and ",
                ncol(truth), " new annotations added to truth table")
      }
    }
  }
  .COBRAData(pval = pval, padj = padj, score = score, 
             sval = sval, truth = truth)
}

#' @rdname COBRAData
#' @param truth_file A character string giving the path to a file with true
#'   labels and other feature annotations.
#' @param result_files A character vector giving path(s) to file(s) with results
#'   (p-values, adjusted p-values, s-values, scores) for one or more methods.
#'   The column names of these files must be of the form "method:measure", where
#'   measure is one of P, adjP, S or score, depending on what is given in the
#'   column.
#' @param feature_id A character string giving the name of the column in the
#'   truth and result files that encodes the feature identifier.
#' @export
COBRAData_from_text <- function(truth_file, result_files, feature_id) {
  truth <- utils::read.delim(truth_file, header = TRUE, as.is = TRUE)
  rownames(truth) <- truth[, feature_id]
  RF <- lapply(result_files, function(f) {
    f <- utils::read.delim(f, header = TRUE, as.is = TRUE, check.names = FALSE)
    if (any(duplicated(f[, feature_id])))
      stop("Duplicate feature IDs found in result file. Please fix.")
    f
  })
  RF <- Reduce(function(...) merge(..., by = feature_id, all = TRUE), RF)
  pval <- RF[, c(feature_id,
                 grep(":P$", colnames(RF), value = TRUE)), drop = FALSE]
  padj <- RF[, c(feature_id,
                 grep(":adjP$", colnames(RF), value = TRUE)), drop = FALSE]
  sval <- RF[, c(feature_id,
                 grep(":S$", colnames(RF), value = TRUE)), drop = FALSE]
  score <- RF[, c(feature_id,
                  grep(":score$", colnames(RF), value = TRUE)), drop = FALSE]

  if (ncol(pval) == 1) {
    pval <- data.frame()
  } else {
    rownames(pval) <- pval[, feature_id]
    pval <- pval[, grep(":P$", colnames(pval), value = TRUE), drop = FALSE]
    colnames(pval) <- gsub(":P$", "", colnames(pval))
  }

  if (ncol(padj) == 1) {
    padj <- data.frame()
  } else {
    rownames(padj) <- padj[, feature_id]
    padj <- padj[, grep(":adjP$", colnames(padj), value = TRUE), drop = FALSE]
    colnames(padj) <- gsub(":adjP$", "", colnames(padj))
  }

  if (ncol(sval) == 1) {
    sval <- data.frame()
  } else {
    rownames(sval) <- sval[, feature_id]
    sval <- sval[, grep(":S$", colnames(sval), value = TRUE), drop = FALSE]
    colnames(sval) <- gsub(":S$", "", colnames(sval))
  }

  if (ncol(score) == 1) {
    score <- data.frame()
  } else {
    rownames(score) <- score[, feature_id]
    score <- score[, grep(":score$", colnames(score), value = TRUE),
                   drop = FALSE]
    colnames(score) <- gsub(":score$", "", colnames(score))
  }

  COBRAData(pval = pval, padj = padj, sval = sval, score = score, truth = truth)
}

#' @rdname COBRAData
#' @param cobradata A \code{COBRAData} object
#' @export
COBRAData_to_text <- function(cobradata, truth_file, result_files, feature_id) {
  ## Update object if needed
  cobradata <- update_cobradata(cobradata, quiet = TRUE)
  
  ## Write truth to file
  truth <- truth(cobradata)
  truth[, feature_id] <- rownames(truth)
  truth <- truth[, c(feature_id,
                     setdiff(colnames(truth), feature_id)), drop = FALSE]
  utils::write.table(truth, file = truth_file, quote = FALSE,
                     sep = "\t", row.names = FALSE, col.names = TRUE)

  ## Merge results and write to file
  pval <- pval(cobradata)
  if (!(length(pval)) == 0)
    colnames(pval) <- paste0(colnames(pval), ":P")
  pval[, feature_id] <- rownames(pval)

  padj <- padj(cobradata)
  if (!(length(padj)) == 0)
    colnames(padj) <- paste0(colnames(padj), ":adjP")
  padj[, feature_id] <- rownames(padj)

  sval <- sval(cobradata)
  if (!(length(sval)) == 0)
    colnames(sval) <- paste0(colnames(sval), ":S")
  sval[, feature_id] <- rownames(sval)

  score <- score(cobradata)
  if (!(length(score)) == 0)
    colnames(score) <- paste0(colnames(score), ":score")
  score[, feature_id] <- rownames(score)

  results <- Reduce(function(...) merge(..., by = feature_id, all = TRUE),
                    list(pval, padj, sval, score))

  utils::write.table(results, file = result_files, quote = FALSE,
                     sep = "\t", row.names = FALSE, col.names = TRUE)
}

setMethod("show", "COBRAData", function(object) {
  cat("An object of class \"", class(object), "\"\n", sep = "")
  for (sl in slotNames(object)) {
    if (.hasSlot(object, sl)) {
      x <- slot(object, sl)
      cat("@", sl, "\n", sep = "")
      .printHead(x)
      cat("\n")
    }
  }
})

#' Accessor and replacement functions for \code{pval} slot
#'
#' Accessor and replacement functions for the \code{pval} slot in a
#' \code{COBRAData} object.
#'
#' @docType methods
#' @name pval
#' @rdname pval
#' @aliases pval pval,COBRAData-method pval<-,COBRAData,data.frame-method
#' @return The accessor function returns a data frame containing p-values for
#'   each feature and each method.
#' @param x A \code{COBRAData} object.
#' @param ... Additional arguments.
#' @param value A data frame containing p-values for each feature and each
#'   method.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' head(pval(cobradata_example))
setMethod("pval", "COBRAData", function(x) x@pval)
#' @name pval
#' @rdname pval
#' @exportMethod "pval<-"
setReplaceMethod("pval", signature(x = "COBRAData", value = "data.frame"),
                 function(x, value) {
                   x <- update_cobradata(x, quiet = FALSE)
                   x@pval <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{padj} slot
#'
#' Accessor and replacement functions for the \code{padj} slot in a
#' \code{COBRAData} object.
#'
#' @docType methods
#' @name padj
#' @rdname padj
#' @aliases padj padj,COBRAData-method padj<-,COBRAData,data.frame-method
#' @return The accessor function returns a data frame containing adjusted
#'   p-values for each feature and each method.
#' @param x A \code{COBRAData} object.
#' @param ... Additional arguments.
#' @param value A data frame containing adjusted p-values for each feature and
#'   each method.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' head(padj(cobradata_example))
setMethod("padj", "COBRAData", function(x) x@padj)
#' @name padj
#' @rdname padj
#' @exportMethod "padj<-"
setReplaceMethod("padj", signature(x = "COBRAData", value = "data.frame"),
                 function(x, value) {
                   x <- update_cobradata(x, quiet = FALSE)
                   x@padj <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{sval} slot
#'
#' Accessor and replacement functions for the \code{sval} slot in a
#' \code{COBRAData} object.
#'
#' @docType methods
#' @name sval
#' @rdname sval
#' @aliases sval sval,COBRAData-method sval<-,COBRAData,data.frame-method
#' @return The accessor function returns a data frame containing s-values for 
#'   each feature and each method. 
#' @param x A \code{COBRAData} object.
#' @param ... Additional arguments.
#' @param value A data frame containing s-values for each feature and each 
#'   method. If the object does not have an s-value slot (older versions of the
#'   class did not have this slot), an empty data frame is returned for
#'   simplicity.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' head(sval(cobradata_example))
setMethod("sval", "COBRAData", function(x) {
  if (.hasSlot(x, "sval")) x@sval
  else {
    warning(paste0("Object doesn't have a slot sval. Please run ", 
                   "update_cobradata(). For consistency, I will return an " ,
                   "empty data.frame"))
    data.frame()
  }
})
#' @name sval
#' @rdname sval
#' @exportMethod "sval<-"
setReplaceMethod("sval", signature(x = "COBRAData", value = "data.frame"),
                 function(x, value) {
                   x <- update_cobradata(x, quiet = FALSE)
                   x@sval <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{score} slot
#'
#' Accessor and replacement functions for the \code{score} slot in a
#' \code{COBRAData} object.
#'
#' @docType methods
#' @name score
#' @rdname score
#' @aliases score score,COBRAData-method score<-,COBRAData,data.frame-method
#' @return The accessor function regurns a data frame containing scores for each
#'   feature and each method.
#' @param x A \code{COBRAData} object.
#' @param ... Additional arguments.
#' @param value A data frame containing scores for each feature and each method.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' head(score(cobradata_example))
setMethod("score", "COBRAData", function(x) x@score)
#' @name score
#' @rdname score
#' @exportMethod "score<-"
setReplaceMethod("score", signature(x = "COBRAData", value = "data.frame"),
                 function(x, value) {
                   x <- update_cobradata(x, quiet = FALSE)
                   x@score <- value
                   if (validObject(x))
                     x
                 })

#' Accessor and replacement functions for \code{truth} slot
#'
#' Accessor and replacement functions for the \code{truth} slot in a
#' \code{COBRAData} object.
#'
#' @docType methods
#' @name truth
#' @rdname truth
#' @aliases truth truth,COBRAData-method truth<-,COBRAData,data.frame-method
#' @return The accessor function returns a data frame containing true
#'   assignments and/or scores for features, together with other feature
#'   annotations to use for stratification of performance calculations.
#' @param x A \code{COBRAData} object.
#' @param ... Additional arguments.
#' @param value A data frame containing true assignments and/or scores for
#'   features, together with other feature annotations to use for stratification
#'   of performance calculations.
#' @author Charlotte Soneson
#' @export
#' @examples
#' data(cobradata_example)
#' head(truth(cobradata_example))
setMethod("truth", "COBRAData", function(x) x@truth)
#' @name truth
#' @rdname truth
#' @exportMethod "truth<-"
setReplaceMethod("truth", signature(x = "COBRAData", value = "data.frame"),
                 function(x, value) {
                   x <- update_cobradata(x, quiet = FALSE)
                   x@truth <- value
                   if (validObject(x))
                     x
                 })

#' Subsetting \code{COBRAData}, \code{COBRAPerformance} or \code{COBRAPlot}
#' objects
#'
#' Functions to subset \code{COBRAData}, \code{COBRAPerformance} or
#' \code{COBRAPlot} objects. \code{COBRAData} objects are subset by features
#' (rows), while \code{COBRAPerformance} and \code{COBRAPlot} objects are subset
#' by methods (columns). Numeric indices are not allowed, since not all slots
#' may be arranged in the same order.
#'
#' @docType methods
#' @name Extract
#' @rdname Extract
#' @aliases \S4method{[}{COBRAData,ANY,ANY} \S4method{[}{COBRAData,ANY,ANY,ANY}
#'   [ [,COBRAData-method
#' @param x A \code{COBRAData}, \code{COBRAPerformance} or \code{COBRAPlot}
#'   object.
#' @param i For \code{COBRAData} objects, a character vector of feature names to
#'   retain.
#' @param j For \code{COBRAPerformance} and \code{COBRAPlot} objects, a
#'   character vector with method names to retain.
#' @param drop not used.
#' @export
setMethod("[", "COBRAData",
          function(x, i, j = "missing", drop = "missing") {
            ## Update object if needed
            x <- update_cobradata(x, quiet = TRUE)
            
            if (length(x@pval) != 0 &&
                length(intersect(rownames(x@pval), i)) == 0)
              stop("none of the provided features found in the pval slot")
            if (length(x@padj) != 0 &&
                length(intersect(rownames(x@padj), i)) == 0)
              stop("none of the provided features found in the padj slot")
            if (length(x@sval) != 0 && 
                length(intersect(rownames(x@sval), i)) == 0)
              stop("none of the provided features found in the sval slot")
            if (length(x@score) != 0 &&
                length(intersect(rownames(x@score), i)) == 0)
              stop("none of the provided features found in the score slot")
            if (length(x@truth) != 0 &&
                length(intersect(rownames(x@truth), i)) == 0)
              stop("none of the provided features found in the truth slot")
            .pval <- x@pval[match(i, rownames(x@pval)), , drop = FALSE]
            .padj <- x@padj[match(i, rownames(x@padj)), , drop = FALSE]
            .sval <- x@sval[match(i, rownames(x@sval)), , drop = FALSE]
            .score <- x@score[match(i, rownames(x@score)), , drop = FALSE]
            .truth <- x@truth[match(i, rownames(x@truth)), , drop = FALSE]
            .COBRAData(pval = .pval, padj = .padj, score = .score,
                      sval = .sval, truth = .truth)
          })

## Validity
setValidity("COBRAData",
            function(object) {
              msg <- NULL
              valid <- TRUE
              if (length(object@pval) != 0 &&
                  !all(sapply(object@pval, is.numeric))) {
                valid <- FALSE
                msg <- c(msg, paste0("pval slot is not numeric"))
              }
              if (length(object@pval) != 0 &&
                  any(object@pval[!is.na(object@pval)] < 0)) {
                valid <- FALSE
                msg <- c(msg, paste0("pval slot contains negative values"))
              }
              if (length(object@pval) != 0 &&
                  any(object@pval[!is.na(object@pval)] > 1)) {
                valid <- FALSE
                msg <- c(msg, paste0("pval slot contains values larger than 1"))
              }
              if (length(object@padj) != 0 &&
                  !all(sapply(object@padj, is.numeric))) {
                valid <- FALSE
                msg <- c(msg, paste0("padj slot is not numeric"))
              }
              if (length(object@padj) != 0 &&
                  any(object@padj[!is.na(object@padj)] < 0)) {
                valid <- FALSE
                msg <- c(msg, paste0("padj slot contains negative values"))
              }
              if (length(object@padj) != 0 &&
                  any(object@padj[!is.na(object@padj)] > 1)) {
                valid <- FALSE
                msg <- c(msg, paste0("padj slot contains values larger than 1"))
              }
              if (.hasSlot(object, "sval") && length(object@sval) != 0 &&
                  !all(sapply(object@sval, is.numeric))) {
                valid <- FALSE
                msg <- c(msg, paste0("sval slot is not numeric"))
              }
              if (length(object@score) != 0 &&
                  !all(sapply(object@score, is.numeric))) {
                valid <- FALSE
                msg <- c(msg, paste0("score slot is not numeric"))
              }
              if (length(object@pval) != 0 && length(object@truth) != 0 &&
                  length(intersect(rownames(object@pval),
                                   rownames(object@truth))) == 0) {
                valid <- FALSE
                msg <- c(msg, paste0("pval slot does not share any features",
                                     " with truth slot"))
              }
              if (length(object@padj) != 0 && length(object@truth) != 0 &&
                  length(intersect(rownames(object@padj),
                                   rownames(object@truth))) == 0) {
                valid <- FALSE
                msg <- c(msg, paste0("padj slot does not share any features",
                                     " with truth slot"))
              }
              if (.hasSlot(object, "sval") && length(object@sval) != 0 && 
                  length(object@truth) != 0 &&
                  length(intersect(rownames(object@sval),
                                   rownames(object@truth))) == 0) {
                valid <- FALSE
                msg <- c(msg, paste0("sval slot does not share any features",
                                     " with truth slot"))
              }
              if (length(object@score) != 0 && length(object@truth) != 0 &&
                  length(intersect(rownames(object@score),
                                   rownames(object@truth))) == 0) {
                valid <- FALSE
                msg <- c(msg, paste0("score slot does not share any features",
                                     " with truth slot"))
              }
              if (valid) TRUE else msg
            })

## Test that the constructors generate objects of the correct type

library(iCOBRA)
context("Check that constructors generate objects of the correct type")

test_that("COBRAData constructors generate COBRAData objects", {
  expect_is(COBRAData(), "COBRAData")
  expect_is(COBRAData_from_text(
    truth_file = system.file("extdata", "cobradata_example_truth.txt",
                             package = "iCOBRA"),
    result_files = system.file("extdata", "cobradata_example_results_4.txt",
                               package = "iCOBRA"),
    feature_id = "feature"), "COBRAData")

  expect_is(COBRAData_from_text(
    truth_file = system.file("extdata", "cobradata_example_truth.txt",
                             package = "iCOBRA"),
    result_files = system.file("extdata", "cobradata_example_results_2.txt",
                               package = "iCOBRA"),
    feature_id = "feature"), "COBRAData")

  ibt <- COBRAData_from_text(
    truth_file = system.file("extdata", "cobradata_example_truth.txt",
                             package = "iCOBRA"),
    result_files = system.file("extdata", "cobradata_example_results_3.txt",
                               package = "iCOBRA"),
    feature_id = "feature")
  expect_is(ibt, "COBRAData")
  expect_equal(length(pval(ibt)), 0)
  expect_equal(length(padj(ibt)), 0)
  expect_equal(length(score(ibt)), 0)

  pval <- c(0.0058, 0.771, 0.024, 0.741, 0.247)
  padj <- p.adjust(pval, method = "BH")
  score <- 1:5
  ib <- COBRAData(pval = data.frame(m1 = pval, row.names = paste0("F", 1:5)),
                  padj = data.frame(m1 = padj, row.names = paste0("F", 1:5)),
                  score = data.frame(m1 = score, row.names = paste0("F", 1:5)),
                  truth = data.frame(status = c(0, 1, 0, 1, 0),
                                     row.names = paste0("F", 1:5)))
  ib2 <- COBRAData(pval = data.frame(m2 = pval, row.names = paste0("F", 1:5)),
                   padj = data.frame(m2 = padj, row.names = paste0("F", 1:5)),
                   score = data.frame(m2 = score, row.names = paste0("F", 1:5)),
                   object_to_extend = ib)
  expect_equal(pval(ib2)$m1, pval(ib2)$m2)
  expect_equal(padj(ib2)$m1, padj(ib2)$m2)
  expect_equal(score(ib2)$m1, score(ib2)$m2)
})

test_that("extending empty COBRAData object works as expected", {
  ibA <- COBRAData(pval = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                     row.names = paste0("F", 1:5)),
                   padj = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                     row.names = paste0("F", 1:5)),
                   sval = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                     row.names = paste0("F", 1:5)),
                   score = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                      row.names = paste0("F", 1:5)),
                   truth = data.frame(status = c(1, 0, 1, 0, 1),
                                      row.names = paste0("F", 1:5)))
  ibB <- COBRAData()
  ibC <- COBRAData(pval = pval(ibA), padj = padj(ibA), sval = sval(ibA), 
                   score = score(ibA), truth = truth(ibA),
                   object_to_extend = ibB)
  expect_equal(pval(ibA), pval(ibC))
  expect_equal(padj(ibA), padj(ibC))
  expect_equal(sval(ibA), sval(ibC))
  expect_equal(score(ibA), score(ibC))
  expect_equal(truth(ibA), truth(ibC))
})

test_that("extending object with object without pval slot works", {
  ibA <- COBRAData(pval = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                     row.names = paste0("F", 1:5)),
                   padj = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                     row.names = paste0("F", 1:5)),
                   sval = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                     row.names = paste0("F", 1:5)),
                   score = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                      row.names = paste0("F", 1:5)),
                   truth = data.frame(status = c(1, 0, 1, 0, 1),
                                      row.names = paste0("F", 1:5)))
  ibB <- COBRAData(score = score(ibA),
                   object_to_extend = ibA)
  expect_equal(pval(ibA), pval(ibB))
  expect_equal(padj(ibA), padj(ibB))
  expect_equal(sval(ibA), sval(ibB))
  expect_equal(score(ibA), score(ibB))
  expect_equal(truth(ibA), truth(ibB))
})

test_that("extending COBRAData objects works as expected", {
  ibA <- COBRAData(pval = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                    row.names = paste0("F", 1:5)),
                  padj = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                    row.names = paste0("F", 1:5)),
                  score = data.frame(mA = c(0.1, 0.2, 0.3, 0.4, 0.5),
                                     row.names = paste0("F", 1:5)),
                  truth = data.frame(status = c(1, 0, 1, 0, 1),
                                     row.names = paste0("F", 1:5)))
  ibAp <- ibA
  truth(ibAp)$expr <- 1:5
  truth(ibAp)$status <- c(0, 1, 0, 1, 0)
  ibB <- COBRAData(pval = data.frame(mB = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6),
                                    row.names = paste0("F", 1:6)),
                  padj = data.frame(mA = c(0.6, 0.7, 0.8, 0.9, 1.0),
                                    row.names = paste0("F", 1:5)),
                  sval = data.frame(mA = c(0.6, 0.7, 0.8, 0.9, 1.0),
                                    row.names = paste0("F", 1:5)),
                  score = data.frame(mB = c(0.6, 0.7, 0.8),
                                     row.names = paste0("F", 1:3)),
                  truth = data.frame(status = c(1, 0, 1, 0, 1, 0),
                                     expr = 1:6,
                                     row.names = paste0("F", 1:6)))
  ibBp <- ibB
  truth(ibBp)$expr <- NULL
  truth(ibBp)$status <- c(0, 1, 0, 1, 0, 1)

  ibtest <- COBRAData(pval = pval(ibA), padj = padj(ibA), score = score(ibA),
                      sval = sval(ibA), truth = truth(ibA), object_to_extend = ibB)
  expect_equal(nrow(pval(ibtest)), 6)
  expect_equal(pval(ibtest)[paste0("F", 1:6), "mB"], c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6))
  expect_equal(nrow(padj(ibtest)), 5)
  expect_equal(ncol(padj(ibtest)), 1)
  expect_equal(padj(ibtest)[paste0("F", 1:5), "mA"], c(0.6, 0.7, 0.8, 0.9, 1.0))
  expect_equal(sval(ibtest)[paste0("F", 1:5), "mA"], c(0.6, 0.7, 0.8, 0.9, 1.0))
  expect_equal(score(ibtest)[paste0("F", 1:5), "mB"], c(0.6, 0.7, 0.8, NA, NA))
  expect_equal(truth(ibtest)[paste0("F", 1:6), "expr"], 1:6)
  expect_equal(ncol(truth(ibtest)), 2)

  expect_error(COBRAData(pval = pval(ibA), padj = padj(ibA), score = score(ibA),
                         sval = sval(ibA), truth = truth(ibA), object_to_extend = ibBp))

  ibtestp <- COBRAData(pval = pval(ibBp), padj = padj(ibBp), score = score(ibBp),
                       sval = sval(ibBp), truth = truth(ibBp), object_to_extend = ibAp)
  expect_equal(pval(ibtest)[paste0("F", 1:6), c("mA", "mB")],
               pval(ibtestp)[paste0("F", 1:6), c("mA", "mB")])
  expect_equal(padj(ibtestp)[paste0("F", 1:5), "mA"], c(0.1, 0.2, 0.3, 0.4, 0.5))
  expect_equal(score(ibtest)[paste0("F", 1:5), c("mA", "mB")],
               score(ibtestp)[paste0("F", 1:5), c("mA", "mB")])
  expect_equal(truth(ibtestp)[paste0("F", 1:6), "expr"], c(1:5, NA))
})

test_that("export functions return correct class", {
  expect_is(COBRAData_to_text(cobradata_example_sval, feature_id = "feature",
                             truth_file = "test_truth.txt",
                             result_files = "test_results.txt"), "NULL")
})

test_that(paste0("COBRAPerformance constructor and calculate_performance ",
                 "generate COBRAPerformance objects"), {
                   expect_is(COBRAPerformance(), "COBRAPerformance")
                   expect_is(calculate_performance(cobradata_example_sval,
                                                   binary_truth = "status",
                                                   aspects = "fpr"),
                             "COBRAPerformance")
                   ip <- COBRAPerformance()
                   onlyshared(ip) <- TRUE
                   expect_is(ip, "COBRAPerformance")
})

test_that(paste0("COBRAPlot constructor and prepare_data_for_plot ",
                 "generate COBRAPlot objects"), {
                   expect_is(COBRAPlot(), "COBRAPlot")
                   expect_is(
                     prepare_data_for_plot(
                       calculate_performance(cobradata_example_sval,
                                             binary_truth = "status",
                                             aspects = "fpr")), "COBRAPlot")
})

test_that("replacement still returns valid objects", {
  cobradata <- cobradata_example_sval
  pval(cobradata) <- pval(cobradata)[1:500, ]
  padj(cobradata) <- padj(cobradata)[1:1000, ]
  sval(cobradata) <- sval(cobradata)[1:1000, ]
  score(cobradata) <- score(cobradata)[1:750, ]
  truth(cobradata) <- truth(cobradata)[1:500, ]
  expect_is(cobradata, "COBRAData")

  cobraperf <- calculate_performance(cobradata, binary_truth = "status",
                                    cont_truth = "logFC")
  fdrtpr(cobraperf) <- fdrtpr(cobraperf)[1:2, ]
  fdrtprcurve(cobraperf) <- fdrtprcurve(cobraperf)[1:2, ]
  fdrnbr(cobraperf) <- fdrnbr(cobraperf)[1:2, ]
  fdrnbrcurve(cobraperf) <- fdrnbrcurve(cobraperf)[1:2, ]
  fsrnbr(cobraperf) <- fsrnbr(cobraperf)[1:2, ]
  fsrnbrcurve(cobraperf) <- fsrnbrcurve(cobraperf)[1:2, ]
  fpr(cobraperf) <- fpr(cobraperf)[1:2, ]
  tpr(cobraperf) <- tpr(cobraperf)[1:2, ]
  roc(cobraperf) <- roc(cobraperf)[1:2, ]
  fpc(cobraperf) <- fpc(cobraperf)[1:2, ]
  deviation(cobraperf) <- deviation(cobraperf)[1:2, ]
  scatter(cobraperf) <- scatter(cobraperf)[1:2, ]
  fdrtpr(cobraperf) <- fdrtpr(cobraperf)[1:2, ]
  overlap(cobraperf) <- overlap(cobraperf)[1:2, ]
  corr(cobraperf) <- corr(cobraperf)[1:2, ]
  maxsplit(cobraperf) <- 4
  splv(cobraperf) <- "none"
  expect_is(cobraperf, "COBRAPerformance")

  cobraplot <- prepare_data_for_plot(cobraperf)
  plotcolors(cobraplot) <- plotcolors(cobraplot)[1:5]
  facetted(cobraplot) <- FALSE
  expect_true(iCOBRA:::is_plottable(tpr(cobraplot)))

  fdrtpr(cobraplot) <- fdrtpr(cobraplot)[1:2, ]
  fdrtprcurve(cobraplot) <- fdrtprcurve(cobraplot)[1:2, ]
  fdrnbr(cobraplot) <- fdrnbr(cobraplot)[1:2, ]
  fdrnbrcurve(cobraplot) <- fdrnbrcurve(cobraplot)[1:2, ]
  fsrnbr(cobraplot) <- fsrnbr(cobraplot)[1:2, ]
  fsrnbrcurve(cobraplot) <- fsrnbrcurve(cobraplot)[1:2, ]
  fpr(cobraplot) <- fpr(cobraplot)[1:2, ]
  tpr(cobraplot) <- tpr(cobraplot)[1:2, ]
  roc(cobraplot) <- roc(cobraplot)[1:2, ]
  fpc(cobraplot) <- fpc(cobraplot)[1:2, ]
  deviation(cobraplot) <- deviation(cobraplot)[1:2, ]
  scatter(cobraplot) <- scatter(cobraplot)[1:2, ]
  fdrtpr(cobraplot) <- fdrtpr(cobraplot)[1:2, ]
  overlap(cobraplot) <- overlap(cobraplot)[1:2, ]
  corr(cobraplot) <- corr(cobraplot)[1:2, ]
  maxsplit(cobraplot) <- 4
  splv(cobraplot) <- "none"
  onlyshared(cobraplot) <- TRUE
  expect_is(cobraplot, "COBRAPlot")
})

test_that("show returns NULL", {
  cobradata <- cobradata_example_sval
  cobraperf <- calculate_performance(cobradata, binary_truth = "status",
                                    cont_truth = "logFC", aspects = "tpr")
  cobraperf2 <- calculate_performance(cobradata, binary_truth = "status",
                                     cont_truth = "logFC",
                                     aspects = c("tpr", "overlap"),
                                     splv = "expr_cat", maxsplit = 4)
  cobraplot <- prepare_data_for_plot(cobraperf2, colorscheme = "Set3",
                                    facetted = FALSE)
  expect_is(show(cobradata), "NULL")
  expect_is(show(cobraperf), "NULL")
  expect_is(show(cobraperf2), "NULL")
  expect_is(show(cobraplot), "NULL")
})

test_that("subsetting of objects works", {
  cobradata <- cobradata_example_sval
  cobraperf <- calculate_performance(cobradata, binary_truth = "status",
                                     cont_truth = "logFC", aspects = "tpr")
  cobraplot <- prepare_data_for_plot(cobraperf)
  expect_error(cobraplot[, "nonexisting_method"])
  expect_is(cobraplot[, "Method2"], "COBRAPlot")
  expect_equal(tpr(cobraplot[, "Method2"])$basemethod, rep("Method2", 3))

  cobraperf <- calculate_performance(cobradata, binary_truth = "status",
                                    cont_truth = "logFC", aspects = "overlap",
                                    splv = "none")
  cobraplot <- prepare_data_for_plot(cobraperf)
  expect_error(cobraperf[, "nonexisting_method"])
  expect_is(overlap(cobraperf[, "Method2"]), "data.frame")
  expect_is(overlap(cobraplot[, "Method2"]), "data.frame")

  cobraperf <- calculate_performance(cobradata, binary_truth = "status",
                                    cont_truth = "logFC", aspects = "overlap",
                                    splv = "expr_cat")
  cobraplot <- prepare_data_for_plot(cobraperf)
  expect_error(cobraperf[, "nonexisting_method"])
  expect_is(cobraperf[, "Method2"], "COBRAPerformance")
  expect_is(overlap(cobraperf[, "Method2"]), "list")
  expect_equal(colnames(overlap(cobraperf[, "Method2"])[[1]]), c("Method2", "truth"))

  expect_error(cobraplot[, "nonexisting_method"])
  expect_is(cobraplot[, "Method2"], "COBRAPlot")
  expect_is(overlap(cobraplot[, "Method2"]), "list")
  expect_equal(colnames(overlap(cobraplot[, "Method2"])[[1]]), c("Method2", "truth"))

  kp <- rownames(pval(cobradata))[1:50]
  expect_error(cobradata["nonexisting_variable", ])
  expect_is(cobradata[kp, ], "COBRAData")
  expect_error(cobraplot[kp, ])
  expect_equal(nrow(pval(cobradata[kp, ])), length(kp))
})

test_that("extending an object still returns valid objects", {
  cobradata <- cobradata_example_sval
  cobradata <- COBRAData(pval = pval(cobradata), object_to_extend = cobradata)
  cobradata <- COBRAData(sval = sval(cobradata), object_to_extend = cobradata)
  expect_is(cobradata, "COBRAData")
})

test_that("extending an object of the wrong class doesn't work", {
  cobradata <- cobradata_example_sval
  cobraperf <- calculate_performance(cobradata, binary_truth = "status",
                                     cont_truth = "logFC", aspects = "tpr")
  expect_error(COBRAData(pval = pval(cobradata), object_to_extend = cobraperf))
})

test_that("updating COBRAData object works", {
  expect_warning({
    expect_is(sval(cobradata_example), "data.frame")
  }, "Object doesn't have a slot sval")
  cobradata <- update_cobradata(cobradata_example)
  expect_is(cobradata, "COBRAData")
  expect_is(cobradata@sval, "data.frame")
})

test_that("updating COBRAPerformance object works", {
  cobradata <- cobradata_example_sval
  cobraperf <- calculate_performance(cobradata, binary_truth = "status",
                                     cont_truth = "logFC", aspects = "tpr")
  expect_is(fsrnbr(cobraperf), "data.frame")
  cobraperf <- update_cobraperformance(cobraperf)
  expect_is(cobraperf, "COBRAPerformance")
})

test_that("reordering levels in COBRAPlot object works", {
  cobraperf <- calculate_performance(calculate_adjp(cobradata_example_sval),
                                     binary_truth = "status", aspects = "fpr")
  cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                     incltruth = TRUE)
  cobraplot <- reorder_levels(cobraplot, c("Method3", "Method1"))
  expect_is(cobraplot, "COBRAPlot")
  expect_equal(levels(fpr(cobraplot)$method), c("Method3", "Method1", "Method2"))
})

test_that("creating objects from data frames without row names generates warning", {
  pval <- c(0.0058, 0.771, 0.024, 0.741, 0.247)
  padj <- p.adjust(pval, method = "BH")
  score <- 1:5
  expect_warning({
    ib <- COBRAData(pval = data.frame(m1 = pval, row.names = NULL),
                    padj = data.frame(m1 = padj, row.names = NULL),
                    score = data.frame(m1 = score, row.names = NULL),
                    truth = data.frame(status = c(0, 1, 0, 1, 0),
                                       row.names = NULL))
  })
})

test_that(paste0("creating objects from data frames with mixed present/missing", 
                 "row names generates error"), {
  pval <- c(0.0058, 0.771, 0.024, 0.741, 0.247)
  padj <- p.adjust(pval, method = "BH")
  score <- 1:5
  expect_error({
    suppressWarnings({
      ib <- COBRAData(pval = data.frame(m1 = pval, row.names = paste0("F", 1:5)),
                      padj = data.frame(m1 = padj, row.names = NULL),
                      score = data.frame(m1 = score, row.names = NULL),
                      truth = data.frame(status = c(0, 1, 0, 1, 0),
                                         row.names = NULL))
    })
  })
  expect_error({
    suppressWarnings({
      ib <- COBRAData(pval = data.frame(m1 = pval, row.names = NULL),
                      padj = data.frame(m1 = padj, row.names = paste0("F", 1:5)),
                      score = data.frame(m1 = score, row.names = NULL),
                      truth = data.frame(status = c(0, 1, 0, 1, 0),
                                         row.names = NULL))
    })
  })
  expect_error({
    suppressWarnings({
      ib <- COBRAData(pval = data.frame(m1 = pval, row.names = NULL),
                      padj = data.frame(m1 = padj, row.names = NULL),
                      score = data.frame(m1 = score, row.names = paste0("F", 1:5)),
                      truth = data.frame(status = c(0, 1, 0, 1, 0),
                                         row.names = NULL))
    })
  })
  expect_error({
    suppressWarnings({
      ib <- COBRAData(pval = data.frame(m1 = pval, row.names = NULL),
                      padj = data.frame(m1 = padj, row.names = NULL),
                      score = data.frame(m1 = score, row.names = NULL),
                      truth = data.frame(status = c(0, 1, 0, 1, 0),
                                         row.names = paste0("F", 1:5)))
    })
  })
})

## Introduction
The `iCOBRA` interactive shiny application provides an easy-to-use, 
general-purpose benchmarking interface for comparing multiple methods 
in terms of their ability to correctly classify features in a (high-throughput) 
dataset as "positive" or "negative", as well as in terms of their ability to 
correctly estimate a continuous target. As an example, `iCOBRA` can be 
used to evaluate the performance of methods aimed at finding differentially 
expressed or differentially spliced genes between conditions, or to 
evaluate how well methods manage to estimate the (known) expression of a 
set of genes. The only formal requirements are that each evaluated method 
has assigned either (adjusted) p-values or a more general "score" (for 
example, the estimated expression level that will be compared to the true 
one) to each feature, and that the true status of the features to which 
the evaluated methods have been applied is known. The results can be 
visualized from different perspectives, using different metrics, and can 
also be stratified by feature attributes.

## Launching the application
The `iCOBRA` app can be launched in two modes:

- With an input object of the type `COBRAData`, generated by the `iCOBRA` R
package. This object contains data frames with calculated p-values, adjusted
p-values and/or scores, as well as a table of true values. When the application
is launched with an input object, all evaluations will be performed using the
data in that object. For more information about the `COBRAData` object type,
consult the corresponding help page in the `iCOBRA` R package.

- Without an input object, either from the `iCOBRA` R package or from the server
([http://imlspenticton.uzh.ch:3838/iCOBRA](http://imlspenticton.uzh.ch:3838/iCOBRA)).
In this case, all input data is uploaded in the form of text files containing
the truth and the results, respectively. These text files are described in more
detail in the next section. Note that the `iCOBRA` R package can be used to
convert between `COBRAData` objects and  correctly formatted text files (see
help pages for functions `COBRAData_to_text()` and `COBRAData_from_text()`)

## Input text files
If the `iCOBRA` app is launched without a data object, two types of input text 
files are necessary for the calculations to be performed:

- A "truth" file
- One or more "result" files for the evaluated methods

#### The truth file
The truth file is a tab-delimited text file (with a header line), listing all the 
features that were investigated (in rows), together with one or more attributes 
(in columns). The columns are of different types, and are used by the app for 
different purposes:

- One column contains the feature identifiers. By default, the application uses
the first column as identifiers, but this can be modified using the input
controls.
- (optional) One column containing a binary truth variable, encoding the true
classification of features into "positive" (encoded as 1) and "negative"
(encoded as 0). This can be, for example, the true differential expression
status if the features are genes. This column is only necessary to calculate the
metrics evaluating the binary classification of features (FDR/TPR, FDR/NBR, TPR,
FPR, ROC, FPC, overlap). By default, it is chosen as the first binary column in
the truth file, but this can be modified using the input controls. If binary
classification should not be evaluated, this can be set to "none".
- (optional) One column containing a continuous truth variable, encoding a
characteristic of the data that will be compared to the provided scores. This
can be, for example, the true expression of genes. This column is only necessary
to calculate the metrics based on comparing continuous scores (correlation,
scatter plots, deviations). By default, it is set to "none", since these metrics
may take slightly longer to calculate than the binary classification metrics.
This can be modified using the input controls.
- (optional) Additional columns containing categorical variables that can be
used to stratify the performance evaluation.

The table below shows the first lines of an example truth file. It contains the
columns **feature** (indicating the feature identifier), **status** (the true
binary assignment), **logFC** (the true continuous variable corresponding to the
scores calculated by the evaluated methods), as well as additional columns
representing stratification annotations.

<img src="screenshot_truth.png" style="width:800px;"/>

#### The result files
The result files contain the p-values, adjusted p-values and scores for the
evaluated features. Each file can contain results obtained by one or multiple
methods. It is also possible to load multiple result files into the app. Each
result file must have a column corresponding to the feature identifier. This
column **must** have the same name as in the truth file. In order to correctly
interpret the other columns, each column header must be of the form
**method:type**, where **type** is either **adjP** (if the column contains
adjusted p-values or FDR estimates), **P** (if the column contains nominal 
p-values), or **score** (if the column contains a general score).

Nominal p-values will be adjusted by the app, using the Benjamini-Hochberg
correction method, as long as the adjusted p-values for the same method have not
been previously loaded or are part of the same result file as the p-values. The
part of the column name preceding the ":adjP", ":P" or ":score" will be
considered the "name" of the method. Please make sure that this name is unique
for each evaluated method. However, note that one or several types of columns
can be provided for the same method (e.g., we can provide both p-values,
adjusted p-values and a score with the same method name, and they will be
considered as different representations corresponding to the same method).

The table below shows the first lines of an example result file, containing
nominal p-values, adjusted p-values and scores for several methods. Missing
values (NA) are allowed.

<img src="screenshot_results.png" style="width:800px;"/>

### Use of different measure types
The combination of measures that are provided for a given method (P, adjP,
score) affects how the performance evaluation will be performed:

- Calculations of FDR, TPR, number of detections and FPRs at given adjusted
p-value thresholds are always calculated based on adjusted p-value inputs
(recall that these are calculated from p-values if they are not provided).
Methods for which adjusted p-values are not available will be excluded from
these evaluations.
- The full FDR/TPR and FDR/NBR curves, as well as ROC and FD curves, are
calculated from scores or p-values if provided (since these often have a higher
resolution than adjusted p-values). If not, adjusted p-values will be used. Note
that scores that are used for this purpose should be such that a higher score
corresponds to "more significant" features. For FDR/TPR and FDR/NBR curves, if
both scores/p-values and adjusted p-values are provided, it is necessary that
the scores/p-values and the adjusted p-values are monotonically related to each
other, otherwise the adjusted p-values will be used also for the curves. This is
to avoid discrepancies between the full curves and the values calculated at
specific adjusted p-value thresholds.
- The continuous evaluations (correlations, scatter plots and deviation plots)
are always based on provided scores. Methods for which a score column is not
provided are excluded from these evaluations.

### Handling missing features

Sometimes, not all features are assigned a score by each of the evaluated
methods. For example, some methods filter out variables for which they can not
perform reliable inference. Similarly, some features may not be present in the
truth table. This can be due to, for example, some inference methods generating
"new" features by combining original features. In this case, the true status for
the new features is not known. The table below tabulates the possible sets of
features in a data set. Features that are neither present in the result tables
nor in the truth table will not be considered.

<img src="screenshot_missing.png" style="width:400px;"/>

The default settings of `iCOBRA` is to consider all features that are present
(with non-missing status) in the truth file. Thus, A = B = 0. In this case, all
features that are not called (i.e., where there are missing values in the result
table) will be considered negative ("not significant") and will thus be added to
the FN and TN, respectively, in the calculations of TPR and FPR. This is
motivated by the assumption that features are left out of the result table
because there was not enough evidence to call them significant. However, in some
circumstances it may mean that the given TPR and FPR values are slight
underestimates of the true values. Choosing to consider only features shared
between the truth and result table (by checking the box in the input controls)
will disregard features with missing values in the truth table such that also C
= D = 0. This will be done separately for each evaluated method. One exception
is made for the overlap plots (Venn diagrams and UpSet plots), which is the only
aspect that is interpretable even without a given truth (to evaluate agreement
between methods). For these, the following feature collections are used:

- When the checkbox for considering only shared features is not checked and the
truth is included in the Venn diagrams, all features where the (binary) truth is
available are used. Features with missing adjusted p-values are considered
non-significant.
- When the checkbox for considering only shared features is checked and the
truth is included, all features where the (binary) truth *as well* as the
adjusted p-value for *all* included methods are available are used.
- When the truth is not included, all features will be used. Features with
missing adjusted p-values are considered non-significant.

Note that this means that, for example, the value represented in the `Number of
detections` column in the FDR/NBR plots may differ from the total number of
calls in the Venn diagrams.

To summarise, the different columns shown in the information boxes below the
plots when hovering over a displayed point are defined as follows:

- Number of detections = TP + FP
- TPR = TP/(TP + FN + C)
- FPR = FP/(FP + TN + D)
- FDR = FP/(TP + FP)
- FP = FP
- TP = FP
- TN = TN
- FN = FN
- TOT_CALLED = TP + FP + FN + TN
- DIFF = TP + FN + C
- NONDIFF = FP + TN + D

## Comparison and evaluation methods
The `iCOBRA` application calculates several different types of comparison and
evaluation metrics, each represented in a separate tab. The available methods
are described briefly below.

- **TPR vs FDR plot**: This plot shows the observed false discovery rate vs the
observed true positive rate. If adjusted p-values are provided, it can be
calculated for a given set of adjusted p-value cutoffs. It can also be
calculated for each possible cutoff to generate an "FDR/TPR curve". Each point
in the plot represents one method and one cutoff, and points corresponding to
the same method are joined together. If "points" is selected to be displayed in
the input panel, and a method controls the FDR (that is, if the observed false
discovery rate is lower than or equal to the imposed cutoff), the corresponding
point is filled, otherwise it is open. The user can change the imposed adjusted
p-value cutoffs using the "FDR thresholds" input in the control panel in the
sidebar.
- **NBR vs FDR plot**: This plot is similar to the FDR vs TPR plot, but instead
of the TPR it shows the number of features classified as positive.
- **TPR plot**: This plot shows the observed true positive rate for one or more
adjusted p-value cutoffs. Each circle represents one method and one cutoff. The
user can change the imposed adjusted p-value cutoffs using the "FDR thresholds"
input in the control panel in the sidebar.
- **FPR plot**: This plot shows the observed false positive rate for one or more
adjusted p-value cutoffs. Each circle represents one method and one cutoff. The
user can change the imposed adjusted p-value cutoffs using the "FDR thresholds"
input in the control panel in the sidebar.
- **ROC curves**: This plot shows the ROC (receiver operating characteristic)
curves obtained by ranking features according to one of the provided measures
(scores if available, otherwise p-values, otherwise adjusted p-values), varying
the significance cutoff and calculating the true and false positive rate for
each cutoff value. A good method has a ROC curve that passes close to the upper
left corner, while a poorly performing method has a ROC curve that lies close to
the diagonal line. The user can zoom in to different parts of the ROC curve by
modifying the axis limits using the sliders below the plot.
- **False discovery curves**: This plot is obtained by ranking the features by
one of the provided measures (same order of prioritization as for ROC curves),
and counting the number of false positives among the top N features, for varying
values of N. The maximal value of N can be changed by the user.
- **Correlation**: This plot shows the correlation between the continuous truth
variable (defined by the user in the left-hand sidebar) and the scores for each
method. The user can choose between Pearson and Spearman correlation.
- **Scatter plots**: This plot depicts the observed scores against the
continuous truth variable (defined by the user in the left-hand sidebar), for
each method separately. Axes can be represented on a log-scale.
- **Deviation plots**: This plot shows the distribution of the deviation of the
observed scores from the continuous truth defined by the user. The distributions
can be shown as violin plots or regular box plots, with or without the
individual points overlaid. There is also an option to square the deviations or
take the absolute value before plotting. The use can zoom in by changing the
slider below the plot.
- **Venn diagram**: The sets of features classified as "positive" for up to five
methods (one of which can be the truth) can be compared using a Venn diagram.
The adjusted p-value cutoff for determining significance can be set by the user.
The user can also choose whether or not to include the truth in the Venn
diagram. If the truth is included, it will be considered as a "perfect" method,
assigning an adjusted p-value of 0 to all truly positive features, and an
adjusted p-value of 1 to all other features.
- **UpSet plots**: This type of plots also show the overlap among sets of genes,
but in a different way than the Venn diagrams. For more information see 
[Lex et al: UpSet: Visualization of Intersecting sets (IEEE Transactions on Visualization and Computer Graphics 20, 1983-1992 (2014))](http://ieeexplore.ieee.org/stamp/stamp.jsp?tp=&arnumber=6876017) or
[Lex & Gehlenborg: Points of view: Sets and intersections (Nature Methods 11, 779 (2014))](http://www.nature.com/nmeth/journal/v11/n8/full/nmeth.3033.html).

For all plots except Venn diagrams and UpSet plots, more information about a
given point can be obtained by hovering over the point in the plot. The
information will be displayed in a table below the plot. In this table it is
also possible to see which type of input measure (p-value, adjusted p-value or
score) that was used for each calculation.


## Input controls
Input controls are located in the sidebar as well as in the individual tabs. By
changing one or more of the input parameters, the user can get more precise
control of what is displayed. The following general parameters are available:

### Truth-related
- **Select column containing feature identifiers**: The name of the column in
truth and result text files that corresponds to the feature identifiers.
- **Select column containing binary truth**: The name of the column in the truth
text file or the provided `COBRAData` object that contains the binary truth
(classification) for the features.
- **Select column containing continuous truth**: The name of the column in the
truth text file or the provided `COBRAData` object that contains the continuous
truth for the features.
- **Select variable to stratify by**: If the truth table contains columns in
addition to the feature identifiers and the truth columns, these will be
considered as categorical annotations based on which the results can be
stratified. This dropdown menu will automatically appear and be populated with
the available columns from the truth table.
- **Maximum number of levels to show when stratifying**: The maximum number of
categories to include if the results are stratified by a variable annotation.
The most frequent categories containing both positive and negative instances
will be included.
- **Include 'overall' class when stratifying**: Whether or not the overall
results are retained  as one category when the results are stratified by a
variable annotation.
- **Show curve and/or points in FDR/TPR plots**: Whether to include full curves
and/or points (for given adjusted p-value thresholds) in FDR/TPR and FDR/NBR
plots.

### Result-related

- **Select methods**: A list of methods for which the result files provide
results. This list will be automatically populated, and augmented if new result
files are loaded. Methods can be selected or deselected to include or exclude
them from the result visualizations.
- **Calculate performance based only on features shared between truth and result
tables**: In some applications, some methods may not return results for all the
features for which we know the true status (e.g., due to filtering). Similarly,
the result tables may contain "new" features for which we don't know the true
status (e.g., due to merging of original features into complexes). By default,
all features available in the truth table will be included in the evaluations.
If this box is checked, only features shared between the truth table and each
method will be retained (see also above).

### General plot-related
- **Select color palette**: Select the color palette that will be used to define
colors for the different methods. Some palettes are only applicable if the
number of required colors is below a certain threshold. If this threshold is
exceeded, the `hue_pal` palette will be used. Note that the number of required
colors may be larger than the number of methods, e.g., if results are stratified
by an annotation but all strata are shown in the same panel (and thus each
method/stratum combination needs a unique color), or if the truth is included as
a method in the Venn diagram.
- **Display full curve and/or points in FDR/TPR and FDR/NBR plots**: In the
FDR/TPR and FDR/NBR plots, the user can choose to include one or both of a
limited number of points (results calculated at given adjusted p-value
thresholds), or to show the entire curve (considering all thresholds). Note that
the points will only be calculated if adjusted p-values are provided.
- **Split plots into panels by stratifying variable**: Whether to split the
plots into panels based on the stratifying variable, or to show all results in
the same panel.
- **FDR thresholds**: The adjusted p-value cutoffs that are used for the
FDR/TPR, FDR/NBR, FPR and TPR plots. One or more thresholds (in the interval [0,
1]) can be chosen. Multiple thresholds must be separated by commas.
- **Plot height**: The height of the plots (a numeric value, giving the height
in pixels).
- **Point size**: The size of the plot characters.
- **Font size for panel headers**: If the plots are split into panels, this
number gives the font size used in the panel headers.

### Specific plot-related
- **Axis limits**: In most plot types, the axis limits can be set using sliders.
- **Download plot**: Download the current plot in pdf format.
- **Download Rdata**: Download the `COBRAPlot` object containing all results
needed for plotting (see the `iCOBRA` R package for details).
- **Download tsv**: Download a tab-separated file with the values shown in the
plot.
- **Maximal rank to display** (False discovery curves): The largest number of
"top features" shown in the false discovery plots (that is, the upper x-axis
limit).
- **Correlation measure** (Correlation): Either Pearson or Spearman, the type of
correlation shown in the plot.
- **Flip axes** (Scatter plots): Flip the x- and y-axes.
- **Log-transform** (Scatter plots): Log-transform the x- and y-axes.
- **Include jittered points** (Deviations): In addition to showing the overall
distribution of values, include the individual points, with jittering to avoid
overplotting.
- **Transformation** (Deviations): How to transform the deviations before
plotting. Either raw, absolute or squared.
- **Plot type** (Deviations): Whether to show the distribution of deviations as
a violin plot or a regular box plot.
- **Include truth** (Venn diagram, applies also to UpSet plots): Whether or not
the truth should be considered as a (perfect) method in the Venn diagrams. If
yes, the "truth" method will be considered to assign an adjusted p-value of 0 to
all truly "positive" features, and an adjusted p-value of 1 to the truly
"negative" ones.
- **Type of threshold** (Venn diagram, applies also to UpSet plots): Whether to
define "significant features" by thresholding the adjusted p-value or the rank
(e.g., considering the top-ranked N features by each method, ranked by adjusted
p-value).
- **Adjusted p-value threshold** (Venn diagram, applies also to UpSet plots):
The adjusted p-value threshold that will be used to classify features as
positive or negative when threshold type is "adjp". The collections of positive
features from different methods are compared in the Venn diagrams and UpSet
plots.
- **Rank threshold** (Venn diagram, applies also to UpSet plots): The rank
threshold that will be used to classify features as positive or negative when
threshold type is "rank". The collections of positive features from different
methods are compared in the Venn diagrams and UpSet plots.
- **Select category** (UpSet plots): If the results are stratified by some
variable, these plots can only be generated for one level at a time. With this
input control, you select which level is displayed.
- **Order intersections by** (UpSet plots): You can choose whether to order the
intersections by degree (number of sets involved in the intersection) or by
frequency (number of elements in the intersection).
- **Order intersections** (UpSet plots): Choose whether to order the
intersections by increasing or decreasing values of the degree or frequency.

## Trouble-shooting

If the plots are not displayed, it can be for one of the following reasons:

- The app has been called with `autorun = FALSE` (default), and the button
"Start calculation!" has not been pressed.
- The input files are not correctly formatted (e.g., the column headers of the
result file do not end with `:P`, `:adjP` or `:score`). Check the "Data preview"
tab to see the data that `iCOBRA` has extracted from the input files.
- No methods are selected (check under the "Results" subsection in the left-hand
sidebar), or none of the selected methods contain the type of input needed for
the current plot (e.g., adjusted p-values for FDR/TPR, FDR/NBR, TPR and FPR
plots, scores for correlation, deviation and scatter plots). Check the "Data
preview" tab to see the data that `iCOBRA` has extracted from the input files.
- The right type of truth (binary or continuous) is unavailable or not selected.
Check under the "Truth" subsection in the left-hand sidebar that the truth is
not set to "none", and check the "Data preview" tab to see the data that
`iCOBRA` has extracted from the input files.
- More than five methods (including the truth, if applicable) can not be
displayed using Venn diagrams.
- For stratification, only strata with both truly "positive" and truly
"negative" instances will be included (except for Venn diagrams).
- If UpSet plots are not showing it may be because all intersections (for a
specific level of the stratification factor) are empty.

If not all input controls are visible in the left-hand sidebar, either "fold"
one of the three sections (Truth, Results, Plot settings) by clicking on the
corresponding title, or change the size of the window slightly.

The sidebar can be hidden by clicking on the three lines next to the main title.

If the colors of the plots do not change when a new color palette is chosen in
the left-hand sidebar, most likely the number of colors in the chosen palette is
not enough. Note that the number of required colors depends not only on the
number of different methods in the evaluation, but also on whether the plots are
facetted or not, and whether the truth is included or not. Also note that not
all methods may be included in each plot, so the number of methods that need to 
be assigned a unique color may exceed the number of methods displayed in any
given plot. `iCOBRA` will attempt to keep the colors for a given method
consistent throughout the different visualizations.

## About iCOBRA

`iCOBRA` was developed in the group of Mark Robinson at 
the Institute for Molecular Life Sciences of the 
University of Zurich. 

### Developers

Charlotte Soneson [email](mailto:charlotte.soneson@imls.uzh.ch)

Mark Robinson [email](mailto:mark.robinson@imls.uzh.ch)

### Instruction video

An instruction video is available on 
[YouTube](https://www.youtube.com/watch?v=KpYCI31iQd4&feature=youtu.be)

### Benchmarking data sets

We have collected a set of benchmarking data sets, each 
consisting of (raw and/or processed) data and the true status 
for each feature. The collection contains benchmarking data sets 
for both statistical inference (differential isoform usage, 
differential gene expression) and target value estimation 
(transcript abundance estimation), and can be reached 
<a href="http://imlspenticton.uzh.ch/robinson_lab/benchmark_collection/" target="_blank">here</a>. 

### Code

All code for `iCOBRA` is available on 
<a href="https://github.com/markrobinsonuzh/iCOBRA" target="_blank">GitHub</a> and via <a href="http://bioconductor.org/packages/iCOBRA/" target="_blank">Bioconductor</a>.

### Citation

C Soneson & MD Robinson: iCOBRA: open, reproducible, standardized and live method benchmarking. *Nature Methods* 13:283 (2016).

---
title: "iCOBRA User Guide"
author: "Charlotte Soneson"
date: "`r Sys.Date()`"
output:
  rmarkdown::html_vignette:
    fig_caption: yes
vignette: >
  %\VignetteIndexEntry{iCOBRA User Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, echo = FALSE}
options(width = 100)
```

iCOBRA is a package to calculate and visualize performance metrics for ranking 
and binary assignment methods. A typical use case could be, for example, 
comparing methods calling differential expression in gene expression 
experiments, which could be seen as either a ranking problem (estimating the 
correct effect size and ordering the genes by significance) or a binary 
assignment problem (classifying the genes into differentially expressed and 
non-differentially expressed).

For more detailed information regarding the calculations, handling of missing
values etc, please consult the "Instructions" tab of the shiny application,
available via the function ```COBRAapp()```.

# Basic workflow
## Creating a data set
The starting point for an evaluation is typically a collection of feature
'scores' (p-values, adjusted p-values or any other ranking score) and the truth
(information about the true status, or the true scores for the features). These
values are stored in an `COBRAData` object. Here, we will start from an example
`COBRAData` object provided with the package. Each slot in the `COBRAData` 
object should be a `data.frame`, with row names representing feature IDs. The 
row names will be used to match features across slots. 

```{r}
library(iCOBRA)
data(cobradata_example)
class(cobradata_example)
cobradata_example
```

For the methods that don't have adjusted p-values, we will calculate adjusted
p-values from nominal p-values.

```{r}
cobradata_example <- calculate_adjp(cobradata_example)
```


## Calculating performance scores
Next, we calculate the performance scores using the COBRAData object. We can
perform both assignment-based and correlation-based evaluations, and thus we
have to provide information about which columns in the truth data frame that are
to be interpreted as the binary (assignment) truth and continuous truth score,
respectively. Here, we let the "status" column (indicating the true differential
expression status of the genes) represent the true classification, and the true
log-fold change represent the true score, to which the observed scores will be
compared.

```{r}
cobraperf <- calculate_performance(cobradata_example, binary_truth = "status", 
                                  cont_truth = "logFC", splv = "none",
                                  maxsplit = 4)
slotNames(cobraperf)
```

## Preparing performance object for plotting
After calculating the performance scores, we prepare the object for plotting. In
this step, we will assign colors to each of the methods, and we can decide to
keep only a subset of the methods. We also decide whether the plotting should be
done with or without facetting (i.e., splitting the plots into panels based on
the stratification variable). This choice will determine the number of colors
that are needed to distinguish between all method/stratification level
combinations.

```{r}
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2", 
                                  facetted = TRUE)
```

## Plotting
When the data has been prepared for plotting, we can apply any of the provided
plot functions to visualize different aspects of the results.

```{r, fig.width = 7, fig.height = 4, fig.cap = "", warning = FALSE}
plot_tpr(cobraplot)
```

```{r, fig.width = 7, fig.height = 4, fig.cap = "", warning = FALSE}
plot_fdrtprcurve(cobraplot)
```

```{r, fig.width = 7, fig.height = 4, fig.cap = "", warning = FALSE}
plot_overlap(cobraplot)
```

# Modifications to the basic workflow
## Stratification
All results can be stratified by any annotation provided in the truth slot of
the COBRAData object. Note that only the largest categories will be retained
(the number of retained categories is determined by the maxsplit argument), and
thus stratifying by continuous annotations is not supported (each category would
likely contain just one observation). Here, we show how the stratification is 
performed, and how it changes the plots generated above.

```{r, fig.width = 7, fig.height = 5, warning = FALSE}
cobraperf <- calculate_performance(cobradata_example, binary_truth = "status", 
                                  cont_truth = "status", splv = "expr_cat")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2", 
                                  facetted = TRUE)
plot_tpr(cobraplot)
```

## Modification of plots

The plotting functions (except plot_overlap) return ggplot objects. Some basic
settings (such as the size of the plot characters, the size of the text in the
panel headers and the axis ranges) can be set directly in the plot functions.
Others can be modified manually. For example, to reduce the size of the axis
labels, an appropriate theme can be added to the ggplot object returned by the
plot functions:

```{r, fig.width = 7, fig.height = 5, warning = FALSE}
library(ggplot2)
pp <- plot_tpr(cobraplot, stripsize = 7.5, pointsize = 3)
pp + theme(axis.text.x = element_text(angle = 90, vjust = 0.5,
                                      hjust = 1, size = 10),
           axis.text.y = element_text(size = 10),
           axis.title.x = element_text(size = 10),
           axis.title.y = element_text(size = 10))
```

It is also possible to modify the facetting and/or label position by adding a
facet_wrap statement (note that the stratification variable is denoted
"splitval"):

```{r, fig.width = 7, fig.height = 4, warning = FALSE}
pp + theme(axis.text.x = element_text(angle = 90, vjust = 0.5,
                                      hjust = 1, size = 10),
           axis.text.y = element_text(size = 10),
           axis.title.x = element_text(size = 10),
           axis.title.y = element_text(size = 10),
           legend.position = "bottom") +
  facet_wrap(~splitval, nrow = 1)
```

The plot_overlap function generates venn diagram(s), using the vennDiagram
function from the limma package. It can be provided with any additional
arguments that are accepted by limma::vennDiagram, e.g. to change the size of
the text:

```{r, fig.width = 7, fig.height = 5, warning = FALSE}
plot_overlap(cobraplot)
plot_overlap(cobraplot, cex = c(1, 0.7, 0.7))
```

## Custom color assignment
The colors can either be chosen by one of the pre-defined color palettes, or
provided by the user as a character vector. Note that the number of colors that
are required depends on the facetting, and that not all methods may be shown in
all plots (depending on the provided score types). If the wrong number of colors
is provided, iCOBRA will automatically add or remove colors.

```{r, fig.width = 7, fig.height = 5, warning = FALSE}
cobraplot <- prepare_data_for_plot(cobraperf, 
                                  colorscheme = c("blue", "green", "pink"), 
                                  facetted = TRUE)
pp <- plot_tpr(cobraplot, stripsize = 7.5, pointsize = 3)
pp + theme(axis.text.x = element_text(angle = 90, vjust = 0.5,
                                      hjust = 1, size = 10),
           axis.text.y = element_text(size = 10),
           axis.title.x = element_text(size = 10),
           axis.title.y = element_text(size = 10))
```

## Interactive exploration
iCOBRA contains an interactive shiny application, for interactive exploration of
results. It can be called with an COBRAData object, or with no arguments. In the
latter case, results can be loaded into the app from text files. 

```{r, eval = FALSE}
COBRAapp(cobradata_example)
COBRAapp()
```

Formatting instructions for the text files can be found in the Instructions tab.
To generate text files from an existing COBRAData object, use the
`COBRAData_to_text()` function:

```{r}
COBRAData_to_text(cobradata = cobradata_example, 
                  truth_file = "cobradata_truth.txt",
                  result_files = "cobradata_results.txt", 
                  feature_id = "feature")
```

Similarly, a COBRAData object can be generated from a set of text files, using
the `COBRAData_from_text()` function:

```{r}
cobra <- COBRAData_from_text(truth_file = "cobradata_truth.txt", 
                             result_files = "cobradata_results.txt", 
                             feature_id = "feature")
cobra
```


The screenshot below shows the appearance of the iCOBRA shiny app, using the 
example data set.

<img src="screenshot_COBRAapp.png" style="width:800px;"/>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/COBRAData.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{Extract}
\alias{Extract}
\alias{\S4method{[}{COBRAData,ANY,ANY}}
\alias{\S4method{[}{COBRAData,ANY,ANY,ANY}}
\alias{[}
\alias{[,COBRAData-method}
\alias{[,COBRAPerformance-method}
\alias{\S4method{[}{COBRAPerformance,ANY,ANY}}
\alias{\S4method{[}{COBRAPerformance,ANY,ANY,ANY}}
\alias{[,COBRAPlot-method}
\alias{\S4method{[}{COBRAPlot,ANY,ANY}}
\alias{\S4method{[}{COBRAPlot,ANY,ANY,ANY}}
\title{Subsetting \code{COBRAData}, \code{COBRAPerformance} or \code{COBRAPlot}
objects}
\usage{
\S4method{[}{COBRAData,ANY,ANY,ANY}(x, i, j = "missing", drop = "missing")

\S4method{[}{COBRAPerformance,ANY,ANY,ANY}(x, i = "missing", j, drop = "missing")

\S4method{[}{COBRAPlot,ANY,ANY,ANY}(x, i = "missing", j, drop = "missing")
}
\arguments{
\item{x}{A \code{COBRAData}, \code{COBRAPerformance} or \code{COBRAPlot}
object.}

\item{i}{For \code{COBRAData} objects, a character vector of feature names to
retain.}

\item{j}{For \code{COBRAPerformance} and \code{COBRAPlot} objects, a
character vector with method names to retain.}

\item{drop}{not used.}
}
\value{
A subset of the original object, of the same class
}
\description{
Functions to subset \code{COBRAData}, \code{COBRAPerformance} or
\code{COBRAPlot} objects. \code{COBRAData} objects are subset by features
(rows), while \code{COBRAPerformance} and \code{COBRAPlot} objects are subset
by methods (columns). Numeric indices are not allowed, since not all slots
may be arranged in the same order.
}
\examples{
data(cobradata_example)
cobradata_example[c("ENSG00000000457", "ENSG00000000971",
                    "ENSG00000000460"), ]
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr")
cobraperf[, c("voom")]
cobraplot <- prepare_data_for_plot(cobraperf)
cobraplot[, c("voom")]
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_performance.R
\name{reorder_levels}
\alias{reorder_levels}
\title{Reorder levels in COBRAPlot object}
\usage{
reorder_levels(cobraplot, levels)
}
\arguments{
\item{cobraplot}{A COBRAPlot object}

\item{levels}{A character vector giving the order of the levels. Any values
not present in the COBRAPlot object will be removed. Any methods present in
the COBRAPlot object but not contained in this vector will be added at the
end.}
}
\value{
A COBRAPlot object
}
\description{
Reorder levels in COBRAPlot object to achieve desired ordering in figure
legends etc. If facetted(cobraplot) is TRUE, the releveling will be applied
to the "method" column. If facetted(cobraplot) is FALSE, it will be applied
to the "fullmethod" column.
}
\examples{
data(cobradata_example_sval)
cobraperf <- calculate_performance(cobradata_example_sval,
                                   binary_truth = "status", aspects = "fpr")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
cobraplot <- reorder_levels(cobraplot, c("Method2", "Method1"))

}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/COBRAData.R
\name{update_cobradata}
\alias{update_cobradata}
\title{Update \code{COBRAData} object to the current version of the class 
format}
\usage{
update_cobradata(object, quiet = FALSE)
}
\arguments{
\item{object}{A \code{COBRAData} object}

\item{quiet}{Set to TRUE to disable messages listing the modifications that
are applied to the object}
}
\value{
An updated \code{COBRAData} object
}
\description{
Update a \code{COBRAData} object generated by a previous version of the
package to the latest version.
}
\examples{
## Generate COBRAData object
set.seed(123)
pval <- data.frame(m1 = runif(100), m2 = runif(100),
                   row.names = paste0("F", 1:100))
truth <- data.frame(status = round(runif(100)),
                    row.names = paste0("F", 1:100))
cobradata <- COBRAData(pval = pval, truth = truth)

## Update object if needed
cobradata <- update_cobradata(cobradata)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{deviation}
\alias{deviation}
\alias{deviation<-}
\alias{deviation,COBRAPerformance-method}
\alias{deviation<-,COBRAPerformance,data.frame-method}
\alias{deviation,COBRAPlot-method}
\alias{deviation<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{deviation} slot}
\usage{
deviation(x, ...)

deviation(x, ...) <- value

\S4method{deviation}{COBRAPerformance}(x)

\S4method{deviation}{COBRAPerformance,data.frame}(x) <- value

\S4method{deviation}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information necessary to plots of deviations
between observed and true scores for each method and each stratification
level.}
}
\value{
The accessor function returns a data frame giving information
  necessary to plots of deviations between observed and true scores for each
  method and each stratification level.
}
\description{
Accessor and replacement functions for the \code{deviation} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
                                   aspects = "deviation")
head(deviation(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{onlyshared}
\alias{onlyshared}
\alias{onlyshared<-}
\alias{onlyshared,COBRAPerformance-method}
\alias{onlyshared<-,COBRAPerformance,logical-method}
\alias{onlyshared,COBRAPlot-method}
\alias{onlyshared<-,COBRAPlot,logical-method}
\title{Accessor and replacement functions for \code{onlyshared} slot}
\usage{
onlyshared(x, ...)

onlyshared(x, ...) <- value

\S4method{onlyshared}{COBRAPerformance}(x)

\S4method{onlyshared}{COBRAPerformance,logical}(x) <- value

\S4method{onlyshared}{COBRAPlot,logical}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A logical indicating whether only features that are shared
between result and truth are retained, or if all features in the truth are
used.}
}
\value{
The accessor function returns a logical indicating whether only
  features that are shared between result and truth are retained, or if all
  features in the truth are used.
}
\description{
Accessor and replacement functions for the \code{onlyshared} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr")
head(onlyshared(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAData.R
\docType{methods}
\name{padj}
\alias{padj}
\alias{padj<-}
\alias{padj,COBRAData-method}
\alias{padj<-,COBRAData,data.frame-method}
\title{Accessor and replacement functions for \code{padj} slot}
\usage{
padj(x, ...)

padj(x, ...) <- value

\S4method{padj}{COBRAData}(x)

\S4method{padj}{COBRAData,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAData} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame containing adjusted p-values for each feature and
each method.}
}
\value{
The accessor function returns a data frame containing adjusted
  p-values for each feature and each method.
}
\description{
Accessor and replacement functions for the \code{padj} slot in a
\code{COBRAData} object.
}
\examples{
data(cobradata_example)
head(padj(cobradata_example))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_deviation}
\alias{plot_deviation}
\title{Plot deviations}
\usage{
plot_deviation(
  cobraplot,
  title = "",
  stripsize = 15,
  titlecol = "black",
  xaxisrange = NULL,
  plottype = "boxplot",
  dojitter = TRUE,
  transf = "raw"
)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{title}{A character string giving the title of the plot.}

\item{stripsize}{A numeric value giving the size of the strip text, when the
results are stratified by an annotation.}

\item{titlecol}{A character string giving the color of the title.}

\item{xaxisrange}{A numeric vector with two elements, giving the lower and
upper boundary of the x-axis, respectively.}

\item{plottype}{Either "boxplot" or "violinplot", indicating what type of
plot to make.}

\item{dojitter}{A logical indicating whether to include jittered data points
or not.}

\item{transf}{A character indicating the transformation to apply to the
deviations before plotting. Must be one of "raw", "absolute" or "squared"}
}
\value{
A ggplot object
}
\description{
Plot the deviations between observed scores and the continuous truth
variable.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
                                   aspects = "deviation")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_deviation(cobraplot)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{fdrnbrcurve}
\alias{fdrnbrcurve}
\alias{fdrnbrcurve<-}
\alias{fdrnbrcurve,COBRAPerformance-method}
\alias{fdrnbrcurve<-,COBRAPerformance,data.frame-method}
\alias{fdrnbrcurve,COBRAPlot-method}
\alias{fdrnbrcurve<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{fdrnbrcurve} slot}
\usage{
fdrnbrcurve(x, ...)

fdrnbrcurve(x, ...) <- value

\S4method{fdrnbrcurve}{COBRAPerformance}(x)

\S4method{fdrnbrcurve}{COBRAPerformance,data.frame}(x) <- value

\S4method{fdrnbrcurve}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information necessary to generate curves of
observed FDR vs number of features called positive for each method and each
stratification level.}
}
\value{
The accessor function returns a data frame giving information
  necessary to generate curves of observed FDR vs number of features called
  positive for each method and each stratification level.
}
\description{
Accessor and replacement functions for the \code{fdrnbrcurve} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrnbrcurve")
head(fdrnbrcurve(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_upset}
\alias{plot_upset}
\title{Create UpSet plots}
\usage{
plot_upset(
  cobraplot,
  stratum = NULL,
  nsets = NULL,
  nintersects = NULL,
  sets.bar.color = NULL,
  ...
)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{stratum}{If results are stratified, the category to plot results for.
Can be numeric or categorical (the name of the category).}

\item{nsets}{The number of methods to include. By default, it is determined
automatically from the \code{cobraplot} object.}

\item{nintersects}{The number of set intersections to display. By default, it
is determined automatically from the \code{cobraplot} object.}

\item{sets.bar.color}{The colors to use for the bars in the UpSet plot. By
default, they are extracted from the \code{plotcolors} slot of the
\code{cobraplot} object.}

\item{...}{Additional arguments to \code{UpSetR::upset}.}
}
\value{
Nothing, displays a graph
}
\description{
Generate UpSet plots showing the overlaps among sets of significant feature
for a given adjusted p-value threshold (see \code{thr_venn} argument of
\code{calculate_performance}). Optionally, the truth can be included
as a "perfect" method. Note that if the results are stratified, only one 
category at a time can be displayed.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "overlap")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_upset(cobraplot)
plot_upset(cobraplot, order.by = "freq", decreasing = TRUE)

cobraperf <- calculate_performance(cobradata_example, 
                                   binary_truth = "status", 
                                   aspects = "overlap",
                                   splv = "expr_cat")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2", 
                                   incltruth = TRUE)
plot_upset(cobraplot, stratum = "[2.85e+00,1.45e+01)")
}
\references{
Lex and Gehlenborg (2014): Points of view: Sets and intersections. Nature
Methods 11, 779.

Lex et al (2014): UpSet: Visualization of intersecting sets. IEEE
Transactions on Visualization and Computer Graphics 20(12), 1983-1992.
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{corr}
\alias{corr}
\alias{corr<-}
\alias{corr,COBRAPerformance-method}
\alias{corr<-,COBRAPerformance,data.frame-method}
\alias{corr<-,COBRAPlot,data.frame-method}
\alias{corr,COBRAPlot-method}
\title{Accessor and replacement functions for \code{corr} slot}
\usage{
corr(x, ...)

corr(x, ...) <- value

\S4method{corr}{COBRAPerformance}(x)

\S4method{corr}{COBRAPerformance,data.frame}(x) <- value

\S4method{corr}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving correlation values for each method and each
stratification level.}
}
\value{
The accessor function returns a data frame giving correlation values
  for each method and each stratification level.
}
\description{
Accessor and replacement functions for the \code{corr} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
                                   aspects = "corr")
head(corr(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{roc}
\alias{roc}
\alias{roc<-}
\alias{roc,COBRAPerformance-method}
\alias{roc<-,COBRAPerformance,data.frame-method}
\alias{roc,COBRAPlot-method}
\alias{roc<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{roc} slot}
\usage{
roc(x, ...)

roc(x, ...) <- value

\S4method{roc}{COBRAPerformance}(x)

\S4method{roc}{COBRAPerformance,data.frame}(x) <- value

\S4method{roc}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information necessary to generate ROC curves
for each method and each stratification level.}
}
\value{
The accessor function returns a data frame giving information
  necessary to generate ROC curves for each method and each stratification
  level.
}
\description{
Accessor and replacement functions for the \code{roc} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status", aspects = "roc")
head(roc(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_performance.R
\name{prepare_data_for_plot}
\alias{prepare_data_for_plot}
\title{Prepare data for plotting}
\usage{
prepare_data_for_plot(
  cobraperf,
  keepmethods = NULL,
  incloverall = TRUE,
  colorscheme = "hue_pal",
  facetted = TRUE,
  incltruth = TRUE,
  conditionalfill = TRUE
)
}
\arguments{
\item{cobraperf}{A \code{COBRAPerformance} object.}

\item{keepmethods}{A character vector consisting of methods to retain for
plotting (these should be a subset of \code{basemethods(cobraperf)}), or
NULL (indicating that all methods represented in cobraperf should be
retained).}

\item{incloverall}{A logical indicating whether the "overall" results should
be included if the results are stratified by an annotation.}

\item{colorscheme}{Either a character string giving the color palette to use
to define colors for the different methods, or a character vector with
colors to use. The available pre-defined palettes depend on the number of
different methods to distinguish. The choices are:
\describe{
\item{- \code{Accent}}{(max 8 methods)}
\item{- \code{Dark2}}{(max 8 methods)}
\item{- \code{Paired}}{(max 12 methods)}
\item{- \code{Pastel1}}{(max 9 methods)}
\item{- \code{Pastel2}}{(max 8 methods)}
\item{- \code{Set1}}{(max 9 methods)}
\item{- \code{Set2}}{(max 8 methods)}
\item{- \code{Set3}}{(max 12 methods)}
\item{- \code{hue_pal}}{}
\item{- \code{rainbow}}{}
\item{- \code{heat}}{}
\item{- \code{terrain}}{}
\item{- \code{topo}}{}
\item{- \code{cm}}{}
}
If the number of allowed methods is exceeded, the colorscheme defaults to
\code{hue_pal}.}

\item{facetted}{A logical indicating whether the results should be split into
subpanels when stratified by an annotation (\code{TRUE}), or kept in the
same panel but shown with different colors (\code{FALSE}).}

\item{incltruth}{A logical indicating whether the truth should be included in
Venn diagrams.}

\item{conditionalfill}{A logical indicating whether the points (in FDR/TPR,
FDR/NBR, FSR/NBR plots) should be filled conditional on whether they
satisfy the imposed criterion (e.g., false discovery rate control at
imposed threshold).}
}
\value{
A \code{COBRAPlot} object
}
\description{
Prepare performance data provided in a \code{COBRAPerformance} object
(obtained by \code{\link{calculate_performance}}) for plotting.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   cont_truth = "none",
                                   aspects = c("fdrtpr", "fdrtprcurve",
                                               "tpr", "roc"),
                                   thrs = c(0.01, 0.05, 0.1), splv = "none")
cobraplot <- prepare_data_for_plot(cobraperf, keepmethods = NULL,
                                   colorscheme = "Dark2")

## User-specified colors
cobraplot2 <- prepare_data_for_plot(cobraperf, keepmethods = NULL,
                                   colorscheme = c("blue", "red", "green"))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/COBRAPerformance.R
\name{update_cobraperformance}
\alias{update_cobraperformance}
\title{Update \code{COBRAPerformance} or \code{COBRAPlot} object to the current
version of the class format}
\usage{
update_cobraperformance(object, quiet = FALSE)
}
\arguments{
\item{object}{A \code{COBRAPerformance} or \code{COBRAPlot} object}

\item{quiet}{Set to TRUE to disable messages listing the modifications that
are applied to the object}
}
\value{
An updated \code{COBRAPerformance} or \code{COBRAPlot} object
}
\description{
Update a \code{COBRAPerformance} or \code{COBRAPlot} object generated by a
previous version of the package to the latest version.
}
\examples{
cobradata <- cobradata_example_sval

cobraperf <- calculate_performance(cobradata, 
                                   binary_truth = "status", 
                                   aspects = "fpr")
cobraperf <- update_cobraperformance(cobraperf)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{tpr}
\alias{tpr}
\alias{tpr<-}
\alias{tpr,COBRAPerformance-method}
\alias{tpr<-,COBRAPerformance,data.frame-method}
\alias{tpr,COBRAPlot-method}
\alias{tpr<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{tpr} slot}
\usage{
tpr(x, ...)

tpr(x, ...) <- value

\S4method{tpr}{COBRAPerformance}(x)

\S4method{tpr}{COBRAPerformance,data.frame}(x) <- value

\S4method{tpr}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information about the observed TPR for each
method and each stratification level, at various adjusted p-value
thresholds.}
}
\value{
The accessor function returns a data frame giving information about
  the observed TPR for each method and each stratification level, at various
  adjusted p-value thresholds.
}
\description{
Accessor and replacement functions for the \code{tpr} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status", aspects = "tpr")
head(tpr(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R
\docType{methods}
\name{basemethods}
\alias{basemethods}
\alias{basemethods,COBRAPerformance-method}
\alias{basemethods,COBRAPlot-method}
\title{Accessor function for basemethods}
\usage{
basemethods(x, ...)

\S4method{basemethods}{COBRAPerformance}(x)
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}
}
\value{
A character vector of all methods represented in the object.
}
\description{
Accessor function to extract the methods that are represented in an
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtprcurve")
basemethods(cobraperf)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{fsrnbr}
\alias{fsrnbr}
\alias{fsrnbr<-}
\alias{fsrnbr,COBRAPerformance-method}
\alias{fsrnbr<-,COBRAPerformance,data.frame-method}
\alias{fsrnbr,COBRAPlot-method}
\alias{fsrnbr<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{fsrnbr} slot}
\usage{
fsrnbr(x, ...)

fsrnbr(x, ...) <- value

\S4method{fsrnbr}{COBRAPerformance}(x)

\S4method{fsrnbr}{COBRAPerformance,data.frame}(x) <- value

\S4method{fsrnbr}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information about the observed FSR and the 
number of features called positive for each method and each stratification 
level, at various s-value thresholds. If the object does not have an fsrnbr
slot (older versions of the class did not have this slot), an empty data
frame is returned for simplicity.}
}
\value{
The accessor function returns a data frame giving information about
  the observed FSR and the number of features called positive for each method
  and each stratification level, at various s-value thresholds.
}
\description{
Accessor and replacement functions for the \code{fsrnbr} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
cobradata <- cobradata_example_sval

cobraperf <- calculate_performance(cobradata, 
                                   cont_truth = "logFC", 
                                   aspects = "fsrnbr")
head(fsrnbr(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{fdrnbr}
\alias{fdrnbr}
\alias{fdrnbr<-}
\alias{fdrnbr,COBRAPerformance-method}
\alias{fdrnbr<-,COBRAPerformance,data.frame-method}
\alias{fdrnbr,COBRAPlot-method}
\alias{fdrnbr<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{fdrnbr} slot}
\usage{
fdrnbr(x, ...)

fdrnbr(x, ...) <- value

\S4method{fdrnbr}{COBRAPerformance}(x)

\S4method{fdrnbr}{COBRAPerformance,data.frame}(x) <- value

\S4method{fdrnbr}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information about the observed FPR and the
number of features called positive for each method and each stratification
level, at various adjusted p-value thresholds.}
}
\value{
The accessor function returns a data frame giving information about
  the observed FPR and the number of features called positive for each method
  and each stratification level, at various adjusted p-value thresholds.
}
\description{
Accessor and replacement functions for the \code{fdrnbr} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrnbr")
head(fdrnbr(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_overlap}
\alias{plot_overlap}
\title{Plot Venn diagram}
\usage{
plot_overlap(cobraplot, ...)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{...}{Additional arguments to \code{limma::vennDiagram}.}
}
\value{
Nothing, displays a graph
}
\description{
Plot a Venn diagram showing the overlaps among sets of significant feature
for a given adjusted p-value threshold (see \code{thr_venn} argument of
\code{calculate_performance}). Optionally, the truth can be included
as a "perfect" method. Note that maximally five methods (including the truth,
if applicable) can be compared.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "overlap")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_overlap(cobraplot)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{maxsplit}
\alias{maxsplit}
\alias{maxsplit<-}
\alias{maxsplit,COBRAPerformance-method}
\alias{maxsplit<-,COBRAPerformance,numeric-method}
\alias{maxsplit,COBRAPlot-method}
\alias{maxsplit<-,COBRAPlot,numeric-method}
\title{Accessor and replacement functions for \code{maxsplit} slot}
\usage{
maxsplit(x, ...)

maxsplit(x, ...) <- value

\S4method{maxsplit}{COBRAPerformance}(x)

\S4method{maxsplit}{COBRAPerformance,numeric}(x) <- value

\S4method{maxsplit}{COBRAPlot,numeric}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A numeric value giving the maximal number of strata to retain.}
}
\value{
The accessor function returns a numeric value giving the maximal
  number of strata to retain.
}
\description{
Accessor and replacement functions for the \code{maxsplit} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr", splv = "expr_cat",
                                   maxsplit = 3)
maxsplit(cobraperf)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calculate_performance.R
\name{calculate_performance}
\alias{calculate_performance}
\title{Calculate performance measures}
\usage{
calculate_performance(
  cobradata,
  binary_truth = NULL,
  cont_truth = NULL,
  aspects = c("fdrtpr", "fdrtprcurve", "fdrnbr", "fdrnbrcurve", "tpr", "fpr", "roc",
    "fpc", "overlap", "corr", "scatter", "deviation", "fsrnbr", "fsrnbrcurve"),
  thrs = c(0.01, 0.05, 0.1),
  svalthrs = c(0.01, 0.05, 0.1),
  splv = "none",
  maxsplit = 3,
  onlyshared = FALSE,
  thr_venn = 0.05,
  type_venn = "adjp",
  topn_venn = 100,
  rank_by_abs = TRUE,
  prefer_pval = TRUE
)
}
\arguments{
\item{cobradata}{A \code{COBRAData} object.}

\item{binary_truth}{A character string giving the name of the column of
truth(cobradata) that contains the binary truth (true assignment of
variables into two classes, represented by 0/1).}

\item{cont_truth}{A character string giving the name of the column of
truth(cobradata) that contains the continuous truth (a continuous value
that the observations can be compared to).}

\item{aspects}{A character vector giving the types of performance measures 
to calculate. Must be a subset of c("fdrtpr", "fdrtprcurve", "fdrnbr",
"fdrnbrcurve", "tpr", "fpr", "roc", "fpc", "overlap", "corr", "scatter",
"deviation", "fsrnbr", "fsrnbrcurve").}

\item{thrs}{A numeric vector of adjusted p-value thresholds for which to
calculate the performance measures. Affects "fdrtpr", "fdrnbr", "tpr" and
"fpr".}

\item{svalthrs}{A numeric vector of s-value thresholds for which to calculate
the FSR. Affects "fsrnbr".}

\item{splv}{A character string giving the name of the column of
truth(cobradata) that will be used to stratify the results. The default
value is "none", indicating no stratification.}

\item{maxsplit}{A numeric value giving the maximal number of categories to
keep in the stratification. The largest categories containing both positive
and negative features will be retained. By setting this argument to `Inf`
or `NA_integer_`, all categories (as well as the order of categories) will
be retained.}

\item{onlyshared}{A logical, indicating whether to only consider features for
which both the true assignment and a result (p-value, adjusted p-value or
score) is given. If FALSE, all features contained in the truth table are
used.}

\item{thr_venn}{A numeric value giving the adjusted p-value threshold to use
to create Venn diagrams (if \code{type_venn} is "adjp").}

\item{type_venn}{Either "adjp" or "rank", indicating whether Venn diagrams
should be constructed based on features with adjusted p-values below a
certain threshold, or based on the same number of top-ranked features by
different methods.}

\item{topn_venn}{A numeric value giving the number of top-ranked features to
compare between methods (if \code{type_venn} is "rank").}

\item{rank_by_abs}{Whether to take the absolute value of the score before
using it to rank the variables  for ROC, FPC, FDR/NBR and FDR/TPR curves.}

\item{prefer_pval}{Whether to preferentially rank variables by p-values or
adjusted p-values rather than score for ROC and FPC calculations. From
version 1.5.5, this is the default behaviour. To obtain the behaviour of
previous versions, set to \code{FALSE}.}
}
\value{
A \code{COBRAPerformance} object
}
\description{
Calculate performance measures from a given collection of p-values, adjusted
p-values and scores provided in a \code{COBRAData} object.
}
\details{
Depending on the collection of observations that are available for a given
method, the appropriate one will be chosen for each performance measure. For
\code{fpr}, \code{tpr}, \code{fdrtpr}, \code{fdrnbr} and \code{overlap}
aspects, results will only be calculated for methods where adjusted p-values
are included in the \code{COBRAData} object, since these calculations make
use of specific adjusted p-value cutoffs. For \code{fdrtprcurve} and
\code{fdrnbrcurve} aspects, the \code{score} observations will be
preferentially used, given that they are monotonically associated with the
adjusted p-values (if provided). If the \code{score} is not provided, the
nominal p-values will be used, given that they are monotonically associated
with the adjusted p-values (if provided). In other cases, the adjusted
p-values will be used also for these aspects. For \code{roc} and \code{fpc},
the \code{score} observations will be used if they are provided, otherwise
p-values and, as a last instance, adjusted p-values. Finally, for the
\code{fsrnbr}, \code{corr}, \code{scatter} and \code{deviation} aspects, the
\code{score} observations will be used if they are provided, otherwise no
results will be calculated.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = c("fdrtpr", "fdrtprcurve",
                                               "tpr", "roc"),
                                   thrs = c(0.01, 0.05, 0.1), splv = "none")
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_fdrtprcurve}
\alias{plot_fdrtprcurve}
\title{Plot TPR vs FDR}
\usage{
plot_fdrtprcurve(
  cobraplot,
  title = "",
  stripsize = 15,
  titlecol = "black",
  pointsize = 5,
  xaxisrange = c(0, 1),
  yaxisrange = c(0, 1),
  plottype = c("curve", "points"),
  linewidth = 1
)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{title}{A character string giving the title of the plot.}

\item{stripsize}{A numeric value giving the size of the strip text, when the
results are stratified by an annotation.}

\item{titlecol}{A character string giving the color of the title.}

\item{pointsize}{A numeric value giving the size of the plot characters.}

\item{xaxisrange}{A numeric vector with two elements, giving the lower and
upper boundary of the x-axis, respectively.}

\item{yaxisrange}{A numeric vector with two elements, giving the lower and
upper boundary of the y-axis, respectively.}

\item{plottype}{A character vector giving the type of plot to construct. Can
be any combination of the two elements "curve" and "points".}

\item{linewidth}{The line width used for plotting}
}
\value{
A ggplot object
}
\description{
Plot observed true positive rate (TPR) vs observed false discovery rate
(FDR), for given adjusted p-value thresholds and/or as curves traced out by
considering all threshold values.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = c("fdrtpr", "fdrtprcurve"))
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_fdrtprcurve(cobraplot, plottype = c("curve", "points"))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/COBRAPlot.R
\docType{methods}
\name{coerce}
\alias{coerce}
\alias{coerce,COBRAPerformance,COBRAPlot-method}
\alias{coerce,}
\alias{COBRAPlot,COBRAPerformance-method}
\alias{coerce,COBRAPlot,COBRAPerformance-method}
\title{Convert an object to another class}
\arguments{
\item{from}{The object that is to be coerced into another class.}
}
\description{
Convert object between \code{COBRAPerformance} and \code{COBRAPlot} classes.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr")
cobraplot <- prepare_data_for_plot(cobraperf)

## Coerce COBRAPerformance object into COBRAPlot object
as(cobraperf, "COBRAPlot")

## Coerce COBRAPlot object into COBRAPerformance object
as(cobraplot, "COBRAPerformance")
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPlot.R
\docType{methods}
\name{facetted}
\alias{facetted}
\alias{facetted<-}
\alias{facetted,COBRAPlot-method}
\alias{facetted<-,COBRAPlot,logical-method}
\title{Accessor and replacement functions for \code{facetted} slot}
\usage{
facetted(x, ...)

facetted(x, ...) <- value

\S4method{facetted}{COBRAPlot}(x)

\S4method{facetted}{COBRAPlot,logical}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A logical value, indicating whether the object is formatted for
facetted plots (visualizing each stratification level in a separate panel)
or not.}
}
\value{
The accessor function returns a logical value, indicating whether the
  object is formatted for facetted plots (visualizing each stratification
  level in a separate panel) or not.
}
\description{
Accessor and replacement functions for the \code{facetted} slot in an
\code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr")
cobraplot <- prepare_data_for_plot(cobraperf)
facetted(cobraplot)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_corr}
\alias{plot_corr}
\title{Plot correlations}
\usage{
plot_corr(
  cobraplot,
  title = "",
  stripsize = 15,
  titlecol = "black",
  pointsize = 5,
  xaxisrange = c(-1, 1),
  corrtype = "pearson"
)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{title}{A character string giving the title of the plot.}

\item{stripsize}{A numeric value giving the size of the strip text, when the
results are stratified by an annotation.}

\item{titlecol}{A character string giving the color of the title.}

\item{pointsize}{A numeric value giving the size of the plot characters.}

\item{xaxisrange}{A numeric vector with two elements, giving the lower and
upper boundary of the x-axis, respectively.}

\item{corrtype}{A character string giving the type of correlation to show.
Either "pearson" or "spearman".}
}
\value{
A ggplot object
}
\description{
Plot correlations between observations and a continuous truth value.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
                                   aspects = "corr")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_corr(cobraplot, corrtype = "spearman")
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAData.R
\docType{methods}
\name{pval}
\alias{pval}
\alias{pval<-}
\alias{pval,COBRAData-method}
\alias{pval<-,COBRAData,data.frame-method}
\title{Accessor and replacement functions for \code{pval} slot}
\usage{
pval(x, ...)

pval(x, ...) <- value

\S4method{pval}{COBRAData}(x)

\S4method{pval}{COBRAData,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAData} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame containing p-values for each feature and each
method.}
}
\value{
The accessor function returns a data frame containing p-values for
  each feature and each method.
}
\description{
Accessor and replacement functions for the \code{pval} slot in a
\code{COBRAData} object.
}
\examples{
data(cobradata_example)
head(pval(cobradata_example))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAData.R
\docType{methods}
\name{sval}
\alias{sval}
\alias{sval<-}
\alias{sval,COBRAData-method}
\alias{sval<-,COBRAData,data.frame-method}
\title{Accessor and replacement functions for \code{sval} slot}
\usage{
sval(x, ...)

sval(x, ...) <- value

\S4method{sval}{COBRAData}(x)

\S4method{sval}{COBRAData,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAData} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame containing s-values for each feature and each 
method. If the object does not have an s-value slot (older versions of the
class did not have this slot), an empty data frame is returned for
simplicity.}
}
\value{
The accessor function returns a data frame containing s-values for 
  each feature and each method.
}
\description{
Accessor and replacement functions for the \code{sval} slot in a
\code{COBRAData} object.
}
\examples{
data(cobradata_example)
head(sval(cobradata_example))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/COBRAData.R
\docType{class}
\name{COBRAData-class}
\alias{COBRAData-class}
\alias{.COBRAData}
\alias{COBRAData}
\alias{COBRAData_from_text}
\alias{COBRAData_to_text}
\title{\code{COBRAData} object and constructor}
\usage{
COBRAData(
  pval = data.frame(),
  padj = data.frame(),
  score = data.frame(),
  sval = data.frame(),
  truth = data.frame(),
  object_to_extend = NULL
)

COBRAData_from_text(truth_file, result_files, feature_id)

COBRAData_to_text(cobradata, truth_file, result_files, feature_id)
}
\arguments{
\item{pval}{A data frame with features as rows and methods as columns,
containing nominal p-values. Missing values (\code{NA}s) are allowed. The
row names should be feature names.}

\item{padj}{A data frame with features as rows and methods as columns,
containing adjusted p-values. Missing values (\code{NA}s) are allowed. The
row names should be feature names.}

\item{score}{A data frame with features as rows and methods as columns,
containing generic scores. In case of comparison to a binary truth, larger
values of the scores should correspond to 'more significant' features.
Missing values (\code{NA}s) are allowed. The row names should be feature
names.}

\item{sval}{A data frame with features as rows and methods as columns,
containing s-values (analogous to q-values, but for sign errors, see
Stephens (2017)). Missing values (\code{NA}s) are allowed. The row names
should be feature names.}

\item{truth}{A data frame with features as rows columns containing feature
annotations such as, e.g., binary and continuous truths and additional
annotations that can be used to stratify the performance calculations. The
row names should be feature names.}

\item{object_to_extend}{A \code{COBRAData} object to extend with the
provided information.}

\item{truth_file}{A character string giving the path to a file with true
labels and other feature annotations.}

\item{result_files}{A character vector giving path(s) to file(s) with results
(p-values, adjusted p-values, s-values, scores) for one or more methods.
The column names of these files must be of the form "method:measure", where
measure is one of P, adjP, S or score, depending on what is given in the
column.}

\item{feature_id}{A character string giving the name of the column in the
truth and result files that encodes the feature identifier.}

\item{cobradata}{A \code{COBRAData} object}
}
\value{
\code{COBRAData} and \code{COBRAData_from_text} return a
  \code{COBRAData} object.
}
\description{
The \code{COBRAData} class contains slots to hold calculated p-values,
adjusted p-values and general 'scores' for a set of features, as well as 
s-values (see Stephens (2017)). The slots can contain values from multiple
methods, and each method can contribute to one or more slots. The class also
contains a slot giving the 'truth' (a binary assignment and/or a continuous
score) for each feature, as well as additional annotations that can be used
to stratify the performance calculations.
}
\details{
If adjusted p-values are missing for some methods, for which nominal p-values
are available, the adjusted p-values can be calculated using the
\code{\link{calculate_adjp}} function.

The text files generated by \code{COBRAData_to_text} can be used as input to
\code{iCOBRAapp}, when it is called without an input argument.
}
\examples{
## Empty COBRAData object:
COBRAData()

## COBRAData object from individual data frames
set.seed(123)
pval <- data.frame(m1 = runif(100), m2 = runif(100),
                   row.names = paste0("F", 1:100))
truth <- data.frame(status = round(runif(100)),
                    row.names = paste0("F", 1:100))
cobradata <- COBRAData(pval = pval, truth = truth)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAData.R
\docType{methods}
\name{truth}
\alias{truth}
\alias{truth<-}
\alias{truth,COBRAData-method}
\alias{truth<-,COBRAData,data.frame-method}
\title{Accessor and replacement functions for \code{truth} slot}
\usage{
truth(x, ...)

truth(x, ...) <- value

\S4method{truth}{COBRAData}(x)

\S4method{truth}{COBRAData,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAData} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame containing true assignments and/or scores for
features, together with other feature annotations to use for stratification
of performance calculations.}
}
\value{
The accessor function returns a data frame containing true
  assignments and/or scores for features, together with other feature
  annotations to use for stratification of performance calculations.
}
\description{
Accessor and replacement functions for the \code{truth} slot in a
\code{COBRAData} object.
}
\examples{
data(cobradata_example)
head(truth(cobradata_example))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{fdrtpr}
\alias{fdrtpr}
\alias{fdrtpr<-}
\alias{fdrtpr,COBRAPerformance-method}
\alias{fdrtpr<-,COBRAPerformance,data.frame-method}
\alias{fdrtpr,COBRAPlot-method}
\alias{fdrtpr<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{fdrtpr} slot}
\usage{
fdrtpr(x, ...)

fdrtpr(x, ...) <- value

\S4method{fdrtpr}{COBRAPerformance}(x)

\S4method{fdrtpr}{COBRAPerformance,data.frame}(x) <- value

\S4method{fdrtpr}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information about the observed FPR and TPR
for each method and each stratification level, at various adjusted p-value
thresholds.}
}
\value{
The accessor function returns a data frame giving information about
  the observed FPR and TPR for each method and each stratification level, at
  various adjusted p-value thresholds.
}
\description{
Accessor and replacement functions for the \code{fdrtpr} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr")
head(fdrtpr(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{fpc}
\alias{fpc}
\alias{fpc<-}
\alias{fpc,COBRAPerformance-method}
\alias{fpc<-,COBRAPerformance,data.frame-method}
\alias{fpc,COBRAPlot-method}
\alias{fpc<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{fpc} slot}
\usage{
fpc(x, ...)

fpc(x, ...) <- value

\S4method{fpc}{COBRAPerformance}(x)

\S4method{fpc}{COBRAPerformance,data.frame}(x) <- value

\S4method{fpc}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information necessary to generate false
positive curves for each method and each stratification level.}
}
\value{
The accessor function returns a data frame giving information
  necessary to generate false positive curves for each method and each
  stratification level.
}
\description{
Accessor and replacement functions for the \code{fpc} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status", aspects = "fpc")
head(fpc(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPlot.R
\docType{methods}
\name{plotcolors}
\alias{plotcolors}
\alias{plotcolors<-}
\alias{plotcolors,COBRAPlot-method}
\alias{plotcolors<-,COBRAPlot,character-method}
\title{Accessor and replacement functions for \code{plotcolors} slot}
\usage{
plotcolors(x, ...)

plotcolors(x, ...) <- value

\S4method{plotcolors}{COBRAPlot}(x)

\S4method{plotcolors}{COBRAPlot,character}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A character vector giving the colors assigned to each of the
methods (or method/stratification level combinations) represented in the
\code{COBRAPlot} object.}
}
\value{
The accessor function returns a character vector giving the colors
  assigned to each of the methods (or method/stratification level
  combinations) represented in the \code{COBRAPlot} object.
}
\description{
Accessor and replacement functions for the \code{plotcolors} slot in an
\code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr")
cobraplot <- prepare_data_for_plot(cobraperf)
plotcolors(cobraplot)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_tpr}
\alias{plot_tpr}
\title{Plot TPR}
\usage{
plot_tpr(
  cobraplot,
  title = "",
  stripsize = 15,
  titlecol = "black",
  pointsize = 5,
  xaxisrange = c(0, 1)
)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{title}{A character string giving the title of the plot.}

\item{stripsize}{A numeric value giving the size of the strip text, when the
results are stratified by an annotation.}

\item{titlecol}{A character string giving the color of the title.}

\item{pointsize}{A numeric value giving the size of the plot characters.}

\item{xaxisrange}{A numeric vector with two elements, giving the lower and
upper boundary of the x-axis, respectively.}
}
\value{
A ggplot object
}
\description{
Plot observed true positive rate (TPR) for given adjusted p-value thresholds.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status", aspects = "tpr")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_tpr(cobraplot)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_fpc}
\alias{plot_fpc}
\title{Plot FP curves}
\usage{
plot_fpc(
  cobraplot,
  title = "",
  stripsize = 15,
  titlecol = "black",
  maxnfdc = 500,
  linewidth = 1
)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{title}{A character string giving the title of the plot.}

\item{stripsize}{A numeric value giving the size of the strip text, when the
results are stratified by an annotation.}

\item{titlecol}{A character string giving the color of the title.}

\item{maxnfdc}{A numeric value giving the largest N to consider.}

\item{linewidth}{The line width used for plotting}
}
\value{
A ggplot object
}
\description{
Plot false positive curves, indicating the number of false positives among
the top-ranked N variables, for varying values of N.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status", aspects = "fpc")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_fpc(cobraplot, maxnfdc = 750)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{fdrtprcurve}
\alias{fdrtprcurve}
\alias{fdrtprcurve<-}
\alias{fdrtprcurve,COBRAPerformance-method}
\alias{fdrtprcurve<-,COBRAPerformance,data.frame-method}
\alias{fdrtprcurve,COBRAPlot-method}
\alias{fdrtprcurve<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{fdrtprcurve} slot}
\usage{
fdrtprcurve(x, ...)

fdrtprcurve(x, ...) <- value

\S4method{fdrtprcurve}{COBRAPerformance}(x)

\S4method{fdrtprcurve}{COBRAPerformance,data.frame}(x) <- value

\S4method{fdrtprcurve}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information necessary to generate curves of
observed FDR vs TPR for each method and each stratification level.}
}
\value{
The accessor function returns a data frame giving information
  necessary to generate curves of observed FDR vs TPR for each method and
  each stratification level.
}
\description{
Accessor and replacement functions for the \code{fdrtprcurve} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtprcurve")
head(fdrtprcurve(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{overlap}
\alias{overlap}
\alias{overlap<-}
\alias{overlap,COBRAPerformance-method}
\alias{overlap<-,COBRAPerformance,list_df-method}
\alias{overlap,COBRAPlot-method}
\alias{overlap<-,COBRAPlot,list_df-method}
\title{Accessor and replacement functions for \code{overlap} slot}
\usage{
overlap(x, ...)

overlap(x, ...) <- value

\S4method{overlap}{COBRAPerformance}(x)

\S4method{overlap}{COBRAPerformance,list_df}(x) <- value

\S4method{overlap}{COBRAPlot,list_df}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame or a list, giving information about which feature
that are classified as 'positive' by each method and for each
stratification level.}
}
\value{
The accessor function returns a data frame or a list, giving
  information about which feature that are classified as 'positive' by each
  method and for each stratification level.
}
\description{
Accessor and replacement functions for the \code{overlap} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "overlap")
head(overlap(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{scatter}
\alias{scatter}
\alias{scatter<-}
\alias{scatter,COBRAPerformance-method}
\alias{scatter<-,COBRAPerformance,data.frame-method}
\alias{scatter,COBRAPlot-method}
\alias{scatter<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{scatter} slot}
\usage{
scatter(x, ...)

scatter(x, ...) <- value

\S4method{scatter}{COBRAPerformance}(x)

\S4method{scatter}{COBRAPerformance,data.frame}(x) <- value

\S4method{scatter}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information necessary to generate scatter
plots of observed vs true values for each method and each stratification
level.}
}
\value{
The accessor function returns a data frame giving information
  necessary to generate scatter plots of observed vs true values for each
  method and each stratification level.
}
\description{
Accessor and replacement functions for the \code{scatter} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
                                   aspects = "scatter")
head(scatter(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cobradata_example_sval.R
\docType{data}
\name{cobradata_example_sval}
\alias{cobradata_example_sval}
\title{Example data set with three differential gene expression methods}
\format{
A \code{COBRAData} object with five slots:
\describe{
  \item{pval}{data frame with p-values for 2,399 genes, from three different
  methods.}
  \item{padj}{data frame with adjusted p-values for 2,399 genes, from two
  different methods.}
  \item{sval}{data frame representing s-values for 2,399 genes, from two
  different methods. Since the purpose of this data set is only to
  demonstrate functionality, the s-values were obtained by copying the
  adjusted p-value slot, and thus these values do not represent true
  s-values.}
  \item{score}{data frame with estimated log fold changes for 2,399 genes,
  from three different methods.}
  \item{truth}{data frame with true differential expression status
  (status column), the number of isoforms (n_isoforms column), the true
  log fold change (logFC and logFC_cat columns) and average expression level
  (expr and expr_cat columns) for 3,858 genes.}
  }
}
\usage{
cobradata_example_sval
}
\value{
A \code{COBRAData} object.
}
\description{
A data set consisting of p-values, adjusted p-values, s-values and estimated
log fold changes (in the 'scores' slot) obtained by three methods for
differential expression analysis of RNA-seq data, applied to a small
synthetic data set of 3,858 human genes. The values are stored in a
\code{COBRAData} object together with a 'truth' data frame containing the
true differential expression status for each gene as well as various
additional annotations such as the true log fold change, the number of
isoforms of the gene and the average expression level.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{fpr}
\alias{fpr}
\alias{fpr<-}
\alias{fpr,COBRAPerformance-method}
\alias{fpr<-,COBRAPerformance,data.frame-method}
\alias{fpr,COBRAPlot-method}
\alias{fpr<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{fpr} slot}
\usage{
fpr(x, ...)

fpr(x, ...) <- value

\S4method{fpr}{COBRAPerformance}(x)

\S4method{fpr}{COBRAPerformance,data.frame}(x) <- value

\S4method{fpr}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information about the observed FPR for each
method and each stratification level, at various adjusted p-value
thresholds.}
}
\value{
The accessor function returns a data frame giving information about
  the observed FPR for each method and each stratification level, at various
  adjusted p-value thresholds.
}
\description{
Accessor and replacement functions for the \code{fpr} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status", aspects = "fpr")
head(fpr(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helpers_general.R
\name{calculate_adjp}
\alias{calculate_adjp}
\title{Calculate adjusted p-values}
\usage{
calculate_adjp(cobradata, method = "BH")
}
\arguments{
\item{cobradata}{A \code{COBRAData} object.}

\item{method}{A character string giving the method (selected from
\code{p.adjust.methods()}) that will be used to perform the adjustment.}
}
\value{
A \code{COBRAData} object, extended with the calculated adjusted
  p-values.
}
\description{
Calculate adjusted p-values for methods where only nominal p-values are
available in a \code{COBRAData} object.
}
\examples{
data(cobradata_example)
cobradata_example <- calculate_adjp(cobradata_example, method = "BH")
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_scatter}
\alias{plot_scatter}
\title{Plot scatter plots}
\usage{
plot_scatter(
  cobraplot,
  title = "",
  stripsize = 10,
  titlecol = "black",
  pointsize = 3,
  doflip = FALSE,
  dolog = FALSE
)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{title}{A character string giving the title of the plot.}

\item{stripsize}{A numeric value giving the size of the strip text, when the
results are stratified by an annotation.}

\item{titlecol}{A character string giving the color of the title.}

\item{pointsize}{A numeric value giving the size of the plot characters.}

\item{doflip}{A logical indicating whether to flip the axes when results are
stratified by an annotation. By default (\code{doflip = FALSE}),
stratification levels are shown as columns and methods as rows in the plot.}

\item{dolog}{A logical indicating whether to log10-transform values before
plotting.}
}
\value{
A ggplot object
}
\description{
Plot scatter plots, indicating the relationship between observed values and a
continuous truth.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example, cont_truth = "logFC",
                                   aspects = "scatter")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_scatter(cobraplot)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R
\docType{methods}
\name{stratiflevels}
\alias{stratiflevels}
\alias{stratiflevels,COBRAPerformance-method}
\alias{stratiflevels,COBRAPlot-method}
\title{Accessor function for stratification levels}
\usage{
stratiflevels(x, ...)

\S4method{stratiflevels}{COBRAPerformance}(x)
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object}

\item{...}{Additional arguments}
}
\value{
A character vector of all stratification levels represented in the
  object
}
\description{
Accessor function to extract the stratification levels that are represented
in a \code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr", splv = "expr_cat",
                                   maxsplit = 4)
stratiflevels(cobraperf)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{fsrnbrcurve}
\alias{fsrnbrcurve}
\alias{fsrnbrcurve<-}
\alias{fsrnbrcurve,COBRAPerformance-method}
\alias{fsrnbrcurve<-,COBRAPerformance,data.frame-method}
\alias{fsrnbrcurve,COBRAPlot-method}
\alias{fsrnbrcurve<-,COBRAPlot,data.frame-method}
\title{Accessor and replacement functions for \code{fsrnbrcurve} slot}
\usage{
fsrnbrcurve(x, ...)

fsrnbrcurve(x, ...) <- value

\S4method{fsrnbrcurve}{COBRAPerformance}(x)

\S4method{fsrnbrcurve}{COBRAPerformance,data.frame}(x) <- value

\S4method{fsrnbrcurve}{COBRAPlot,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame giving information necessary to generate curves of 
observed FSR vs number of features called positive for each method and each
stratification level. If the object does not have an fsrnbrcurve slot
(older versions of the class did not have this slot), an empty data frame
is returned for simplicity.}
}
\value{
The accessor function returns a data frame giving information
  necessary to generate curves of observed FSR vs number of features called
  positive for each method and each stratification level.
}
\description{
Accessor and replacement functions for the \code{fsrnbrcurve} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
cobradata <- cobradata_example_sval

cobraperf <- calculate_performance(cobradata, 
                                   cont_truth = "logFC", 
                                   aspects = "fsrnbrcurve")
head(fsrnbrcurve(cobraperf))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny.R
\name{COBRAapp}
\alias{COBRAapp}
\title{Interactive shiny app to visualize results}
\usage{
COBRAapp(cobradata = NULL, autorun = FALSE, addStopButton = TRUE)
}
\arguments{
\item{cobradata}{An (optional) \code{COBRAData} object. If not given, the
user can load results from text files.}

\item{autorun}{A logical indicating whether the app calculations should start
automatically on launch, or wait for the user to press the 'Start
calculation!' button.}

\item{addStopButton}{Logical scalar. If \code{TRUE} (default), will add a
button to stop the app (by calling \code{shiny::stopApp}).}
}
\value{
Returns (and runs) an object representing the shiny app.
}
\description{
Interactive shiny app for visualization of results. The app can be
initialized with a \code{COBRAData} object. If no object is provided, truth
and results are loaded into the app from text files (see the Instructions tab
of the app for formatting instructions). Properly formatted text files can
also be obtained using the function \code{\link{COBRAData_to_text}}.
}
\examples{
data(cobradata_example)
app <- COBRAapp(cobradata_example)
if (interactive()) {
  shiny::runApp(app)
}
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAData.R
\docType{methods}
\name{score}
\alias{score}
\alias{score<-}
\alias{score,COBRAData-method}
\alias{score<-,COBRAData,data.frame-method}
\title{Accessor and replacement functions for \code{score} slot}
\usage{
score(x, ...)

score(x, ...) <- value

\S4method{score}{COBRAData}(x)

\S4method{score}{COBRAData,data.frame}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAData} object.}

\item{...}{Additional arguments.}

\item{value}{A data frame containing scores for each feature and each method.}
}
\value{
The accessor function regurns a data frame containing scores for each
  feature and each method.
}
\description{
Accessor and replacement functions for the \code{score} slot in a
\code{COBRAData} object.
}
\examples{
data(cobradata_example)
head(score(cobradata_example))
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/COBRAPerformance.R,
%   R/COBRAPlot.R
\docType{methods}
\name{splv}
\alias{splv}
\alias{splv<-}
\alias{splv,COBRAPerformance-method}
\alias{splv<-,COBRAPerformance,character-method}
\alias{splv,COBRAPlot-method}
\alias{splv<-,COBRAPlot,character-method}
\title{Accessor and replacement functions for \code{splv} slot}
\usage{
splv(x, ...)

splv(x, ...) <- value

\S4method{splv}{COBRAPerformance}(x)

\S4method{splv}{COBRAPerformance,character}(x) <- value

\S4method{splv}{COBRAPlot,character}(x) <- value
}
\arguments{
\item{x}{A \code{COBRAPerformance} or \code{COBRAPlot} object.}

\item{...}{Additional arguments.}

\item{value}{A character string giving the name of a feature annotation to
use for stratification.}
}
\value{
The accessor function returns a character string giving the name of a
  feature annotation to use for stratification.
}
\description{
Accessor and replacement functions for the \code{splv} slot in a
\code{COBRAPerformance} or \code{COBRAPlot} object.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status",
                                   aspects = "fdrtpr", splv = "expr_cat")
splv(cobraperf)
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/COBRAPerformance.R
\docType{class}
\name{COBRAPerformance-class}
\alias{COBRAPerformance-class}
\alias{.COBRAPerformance}
\alias{COBRAPerformance}
\title{\code{COBRAPerformance} object and constructor}
\usage{
COBRAPerformance(
  fdrtpr = data.frame(),
  fdrtprcurve = data.frame(),
  fdrnbr = data.frame(),
  fdrnbrcurve = data.frame(),
  fsrnbr = data.frame(),
  fsrnbrcurve = data.frame(),
  tpr = data.frame(),
  fpr = data.frame(),
  splv = "",
  roc = data.frame(),
  fpc = data.frame(),
  deviation = data.frame(),
  onlyshared = NA,
  overlap = data.frame(),
  maxsplit = NA_integer_,
  corr = data.frame(),
  scatter = data.frame()
)
}
\arguments{
\item{fdrtpr}{A data frame containing observed FDR and TPR values at various
adjusted p-value thresholds.}

\item{fdrtprcurve}{A data frame containing observed FDR and TPR values for a
(potentially large) number of cutoffs applied to a 'score' (that can be
p-value, adjusted p-value or a more general score).}

\item{fdrnbr}{A data frame containing observed FDR and the number of features
considered to be significant at various adjusted p-value thresholds.}

\item{fdrnbrcurve}{A data frame containing observed FDR and number of
features considered to be significant for a (potentially large) number of
cutoffs applied to a 'score' (that can be p-value, adjusted p-value or a
more general score).}

\item{fsrnbr}{A data frame containing observed false sign rate (FSR) and the
number of features considered to be significant at various s-value
thresholds}

\item{fsrnbrcurve}{A data frame containing observed false sign rate (FSR) and
number of features considered to be significant for a (potentially large)
number of cutoffs applied to the s-value}

\item{tpr}{A data frame containing observed TPR values at various adjusted
p-value thresholds.}

\item{fpr}{A data frame containing observed FPR values at various adjusted
p-value thresholds.}

\item{splv}{A character string giving the name of the stratification factor,
"none" if the results are not stratified.}

\item{roc}{A data frame containing observed FPR and TPR values  for a
(potentially large) number of cutoffs applied to a 'score' (that can be
p-value, adjusted p-value or a more general score), which can be used to
generate a ROC curve.}

\item{fpc}{A data frame containing observed numbers of false positive
findings among the N top-ranked features (ranked by p-values, adjusted
p-values or more general scores), for a (potentially large) number of Ns,
which can be used to generate a false positive curve.}

\item{deviation}{A data frame containing deviations between observed scores
and true scores.}

\item{onlyshared}{A logical value indicating whether only features shared
between the results and the truth should be retained, or if all features
present in the truth should be used.}

\item{overlap}{A data frame or list of data frames with binary values
indicating, for each of a number of methods and number of features, whether
the method consider the feature 'positive' (significant, 1) or 'negative'
(non-significant, 0). If it is a list of data frames, each list element
corresponds to one level of a stratifying factor.}

\item{maxsplit}{A numeric value indicating the largest number of levels to
retain if the results have been stratified by an annotation.}

\item{corr}{A data frame containing observed (Pearson and Spearman)
correlation values between observed and true scores.}

\item{scatter}{A data frame containing observed 'scores' (p-values, adjusted
p-values or more general scores) and true scores, which can be used to
generate scatter plots.}
}
\value{
A \code{COBRAPerformance} object.
}
\description{
The \code{COBRAPerformance} class holds various types of calculated
performance measures. Objects from this class are typically generated from
\code{COBRAData} objects by means of the function
\code{\link{calculate_performance}}.
}
\examples{
## Empty COBRAPerformance object
COBRAPerformance()
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_methods.R
\name{plot_roc}
\alias{plot_roc}
\title{Plot ROC curves}
\usage{
plot_roc(
  cobraplot,
  title = "",
  stripsize = 15,
  titlecol = "black",
  xaxisrange = c(0, 1),
  yaxisrange = c(0, 1),
  linewidth = 1
)
}
\arguments{
\item{cobraplot}{A \code{COBRAPlot} object.}

\item{title}{A character string giving the title of the plot.}

\item{stripsize}{A numeric value giving the size of the strip text, when the
results are stratified by an annotation.}

\item{titlecol}{A character string giving the color of the title.}

\item{xaxisrange}{A numeric vector with two elements, giving the lower and
upper boundary of the x-axis, respectively.}

\item{yaxisrange}{A numeric vector with two elements, giving the lower and
upper boundary of the y-axis, respectively.}

\item{linewidth}{The line width used for plotting}
}
\value{
A ggplot object
}
\description{
Plot receiver operating characteristics (ROC) curves.
}
\examples{
data(cobradata_example)
cobraperf <- calculate_performance(cobradata_example,
                                   binary_truth = "status", aspects = "roc")
cobraplot <- prepare_data_for_plot(cobraperf, colorscheme = "Dark2",
                                   incltruth = TRUE)
plot_roc(cobraplot)
}
\author{
Charlotte Soneson
}
